@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -25,7 +25,8 @@
 #include "log.h"
 #include "parse.h"
 #include "timing.h"
-#include "io_defs.h"
+#include "serialdefs.h"
+#include "ascii.h"
 
 typedef enum {
   PARM_PROTOCOLS,
@@ -60,29 +61,29 @@ BEGIN_KEY_NAME_TABLE(command)
 END_KEY_NAME_TABLE
 
 BEGIN_KEY_NAME_TABLE(front)
-  KEY_NAME_ENTRY(BM_KEY_FRONT+0, "Front10"),
-  KEY_NAME_ENTRY(BM_KEY_FRONT+1, "Front9"),
-  KEY_NAME_ENTRY(BM_KEY_FRONT+2, "Front8"),
-  KEY_NAME_ENTRY(BM_KEY_FRONT+3, "Front7"),
-  KEY_NAME_ENTRY(BM_KEY_FRONT+4, "Front6"),
-  KEY_NAME_ENTRY(BM_KEY_FRONT+5, "Front5"),
-  KEY_NAME_ENTRY(BM_KEY_FRONT+6, "Front4"),
-  KEY_NAME_ENTRY(BM_KEY_FRONT+7, "Front3"),
-  KEY_NAME_ENTRY(BM_KEY_FRONT+8, "Front2"),
-  KEY_NAME_ENTRY(BM_KEY_FRONT+9, "Front1"),
+  KEY_NAME_ENTRY(BM_KEY_FRONT+0, "Front1"),
+  KEY_NAME_ENTRY(BM_KEY_FRONT+1, "Front2"),
+  KEY_NAME_ENTRY(BM_KEY_FRONT+2, "Front3"),
+  KEY_NAME_ENTRY(BM_KEY_FRONT+3, "Front4"),
+  KEY_NAME_ENTRY(BM_KEY_FRONT+4, "Front5"),
+  KEY_NAME_ENTRY(BM_KEY_FRONT+5, "Front6"),
+  KEY_NAME_ENTRY(BM_KEY_FRONT+6, "Front7"),
+  KEY_NAME_ENTRY(BM_KEY_FRONT+7, "Front8"),
+  KEY_NAME_ENTRY(BM_KEY_FRONT+8, "Front9"),
+  KEY_NAME_ENTRY(BM_KEY_FRONT+9, "Front10"),
 END_KEY_NAME_TABLE
 
 BEGIN_KEY_NAME_TABLE(back)
-  KEY_NAME_ENTRY(BM_KEY_BACK+0, "Back10"),
-  KEY_NAME_ENTRY(BM_KEY_BACK+1, "Back9"),
-  KEY_NAME_ENTRY(BM_KEY_BACK+2, "Back8"),
-  KEY_NAME_ENTRY(BM_KEY_BACK+3, "Back7"),
-  KEY_NAME_ENTRY(BM_KEY_BACK+4, "Back6"),
-  KEY_NAME_ENTRY(BM_KEY_BACK+5, "Back5"),
-  KEY_NAME_ENTRY(BM_KEY_BACK+6, "Back4"),
-  KEY_NAME_ENTRY(BM_KEY_BACK+7, "Back3"),
-  KEY_NAME_ENTRY(BM_KEY_BACK+8, "Back2"),
-  KEY_NAME_ENTRY(BM_KEY_BACK+9, "Back1"),
+  KEY_NAME_ENTRY(BM_KEY_BACK+0, "Back1"),
+  KEY_NAME_ENTRY(BM_KEY_BACK+1, "Back2"),
+  KEY_NAME_ENTRY(BM_KEY_BACK+2, "Back3"),
+  KEY_NAME_ENTRY(BM_KEY_BACK+3, "Back4"),
+  KEY_NAME_ENTRY(BM_KEY_BACK+4, "Back5"),
+  KEY_NAME_ENTRY(BM_KEY_BACK+5, "Back6"),
+  KEY_NAME_ENTRY(BM_KEY_BACK+6, "Back7"),
+  KEY_NAME_ENTRY(BM_KEY_BACK+7, "Back8"),
+  KEY_NAME_ENTRY(BM_KEY_BACK+8, "Back9"),
+  KEY_NAME_ENTRY(BM_KEY_BACK+9, "Back10"),
 END_KEY_NAME_TABLE
 
 BEGIN_KEY_NAME_TABLE(entry)
@@ -153,6 +154,8 @@ END_KEY_NAME_TABLE
 BEGIN_KEY_NAME_TABLE(vertical)
   KEY_SET_ENTRY(BM_SET_LeftSensors, "LeftSensor"),
   KEY_SET_ENTRY(BM_SET_RightSensors, "RightSensor"),
+  KEY_SET_ENTRY(BM_SET_ScaledLeftSensors, "ScaledLeftSensor"),
+  KEY_SET_ENTRY(BM_SET_ScaledRightSensors, "ScaledRightSensor"),
 END_KEY_NAME_TABLE
 
 BEGIN_KEY_NAME_TABLES(inka)
@@ -166,6 +169,14 @@ BEGIN_KEY_NAME_TABLES(dm80p)
   KEY_NAME_TABLE(routing),
 END_KEY_NAME_TABLES
 
+BEGIN_KEY_NAME_TABLES(vario80)
+  KEY_NAME_SUBTABLE(display,6),
+  KEY_NAME_TABLE(command),
+  KEY_NAME_TABLE(front),
+  KEY_NAME_TABLE(back),
+  KEY_NAME_TABLE(routing),
+END_KEY_NAME_TABLES
+
 BEGIN_KEY_NAME_TABLES(pro)
   KEY_NAME_SUBTABLE(display,6),
   KEY_NAME_TABLE(wheels),
@@ -193,6 +204,7 @@ END_KEY_NAME_TABLES
 
 DEFINE_KEY_TABLE(inka)
 DEFINE_KEY_TABLE(dm80p)
+DEFINE_KEY_TABLE(vario80)
 DEFINE_KEY_TABLE(pro)
 DEFINE_KEY_TABLE(default)
 DEFINE_KEY_TABLE(vario)
@@ -200,6 +212,7 @@ DEFINE_KEY_TABLE(vario)
 BEGIN_KEY_TABLE_LIST
   &KEY_TABLE_DEFINITION(inka),
   &KEY_TABLE_DEFINITION(dm80p),
+  &KEY_TABLE_DEFINITION(vario80),
   &KEY_TABLE_DEFINITION(pro),
   &KEY_TABLE_DEFINITION(default),
   &KEY_TABLE_DEFINITION(vario),
@@ -219,7 +232,6 @@ static int cellCount;
 static int cellsUpdated;
 static unsigned char internalCells[MAXIMUM_CELL_COUNT];
 static unsigned char externalCells[MAXIMUM_CELL_COUNT];
-static TranslationTable outputTable;
 
 typedef struct {
   unsigned char navigationKeys[KEY_GROUP_SIZE(BM_KEY_COUNT)];
@@ -234,15 +246,15 @@ static unsigned char switchSettings;
 
 typedef struct {
   const char *name;
-  int serialBaud;
+  unsigned int serialBaud;
   SerialParity serialParity;
-  DotsTable dotsTable;
+  const DotsTable *dotsTable;
   int (*readPacket) (BrailleDisplay *brl, unsigned char *packet, int size);
   int (*writePacket) (BrailleDisplay *brl, const unsigned char *packet, int length);
   int (*probeDisplay) (BrailleDisplay *brl);
   void (*updateKeys) (BrailleDisplay *brl);
   int (*writeCells) (BrailleDisplay *brl);
-  int (*writeCellRange) (BrailleDisplay *brl, int start, int count);
+  int (*writeCellRange) (BrailleDisplay *brl, unsigned int start, unsigned int count);
 } ProtocolOperations;
 
 typedef struct {
@@ -257,7 +269,7 @@ typedef struct {
 
 static const InputOutputOperations *io;
 static const ProtocolOperations *protocol;
-static int charactersPerSecond;
+static unsigned int charactersPerSecond;
 
 /* Internal Routines */
 
@@ -268,7 +280,7 @@ logTextField (const char *name, const char *address, int length) {
     if (byte && (byte != ' ')) break;
     --length;
   }
-  LogPrint(LOG_INFO, "%s: %.*s", name, length, address);
+  logMessage(LOG_INFO, "%s: %.*s", name, length, address);
 }
 
 static int
@@ -321,7 +333,7 @@ resetKeyGroup (unsigned char *group, unsigned char count, unsigned char key) {
 static void
 updateKeyGroup (
   unsigned char *group, const unsigned char *new,
-  unsigned char set, unsigned char base, unsigned char count
+  unsigned char set, unsigned char base, unsigned char count, int scaled
 ) {
   unsigned char pressTable[count];
   unsigned char pressCount = 0;
@@ -332,6 +344,8 @@ updateKeyGroup (
     int press = (new[offset / 8] & (1 << (offset % 8))) != 0;
 
     if (setGroupedKey(group, key, press)) {
+      if (scaled) key = rescaleInteger(key, count-1, BRL_MSK_ARG);
+
       if (press) {
         pressTable[pressCount++] = key;
       } else {
@@ -345,7 +359,7 @@ updateKeyGroup (
 
 static void
 updateNavigationKeys (const unsigned char *new, unsigned char base, unsigned char count) {
-  updateKeyGroup(keysState.navigationKeys, new, BM_SET_NavigationKeys, base, count);
+  updateKeyGroup(keysState.navigationKeys, new, BM_SET_NavigationKeys, base, count, 0);
 }
 
 static void
@@ -355,7 +369,7 @@ updateDisplayKeys (unsigned char new) {
 
 static void
 updateRoutingKeys (const unsigned char *new, unsigned char count) {
-  updateKeyGroup(keysState.routingKeys, new, BM_SET_RoutingKeys, 0, count);
+  updateKeyGroup(keysState.routingKeys, new, BM_SET_RoutingKeys, 0, count, 0);
 }
 
 static int
@@ -368,16 +382,9 @@ updateCells (BrailleDisplay *brl) {
 }
 
 static int
-updateCellRange (BrailleDisplay *brl, int start, int count) {
+updateCellRange (BrailleDisplay *brl, unsigned int start, unsigned int count) {
   if (count) {
-    const unsigned char *source = &internalCells[start];
-    unsigned char *target = &externalCells[start];
-    const unsigned char *end = source + count;
-
-    do {
-      *target++ = outputTable[*source++];
-    } while (source != end);
-
+    translateOutputCells(&externalCells[start], &internalCells[start], count);
     cellsUpdated = 1;
     if (!protocol->writeCellRange(brl, start, count)) return 0;
   }
@@ -386,11 +393,23 @@ updateCellRange (BrailleDisplay *brl, int start, int count) {
 }
 
 static int
-clearCellRange (BrailleDisplay *brl, int start, int count) {
+clearCellRange (BrailleDisplay *brl, unsigned int start, unsigned int count) {
   memset(&internalCells[start], 0, count);
   return updateCellRange(brl, start, count);
 }
 
+static int
+putCells (BrailleDisplay *brl, const unsigned char *cells, unsigned int start, unsigned int count) {
+  unsigned int from;
+  unsigned int to;
+
+  if (cellsHaveChanged(&internalCells[start], cells, count, &from, &to, NULL)) {
+    if (!updateCellRange(brl, start+from, to-from)) return 0;
+  }
+
+  return 1;
+}
+
 static void
 logCellCount (BrailleDisplay *brl) {
   switch ((brl->textColumns = cellCount)) {
@@ -407,8 +426,8 @@ logCellCount (BrailleDisplay *brl) {
   brl->textRows = 1;
   brl->statusRows = (brl->statusColumns = cellCount - brl->textColumns)? 1: 0;
 
-  LogPrint(LOG_INFO, "Cell Count: %d (%d text, %d status)",
-           cellCount, brl->textColumns, brl->statusColumns);
+  logMessage(LOG_INFO, "Cell Count: %d (%d text, %d status)",
+             cellCount, brl->textColumns, brl->statusColumns);
 }
 
 static int
@@ -438,8 +457,6 @@ changeCellCount (BrailleDisplay *brl, int count) {
 
 /* Baum Protocol */
 
-#define ESCAPE 0X1B
-
 typedef unsigned char BaumInteger[2];
 #define MAKE_BAUM_INTEGER_FIRST(i) ((i) & 0XFF)
 #define MAKE_BAUM_INTEGER_SECOND(i) (((i) >> 8) & 0XFF)
@@ -721,6 +738,7 @@ typedef union {
 typedef enum {
   BAUM_DEVICE_Inka,
   BAUM_DEVICE_DM80P,
+  BAUM_DEVICE_Vario80,
   BAUM_DEVICE_Modular,
   BAUM_DEVICE_Default
 } BaumDeviceType;
@@ -805,7 +823,7 @@ getBaumModuleDescription (uint16_t identifier) {
     bmd += 1;
   }
 
-  LogPrint(LOG_DEBUG, "unknown module identifier: %04X", identifier);
+  logMessage(LOG_DEBUG, "unknown module identifier: %04X", identifier);
   return NULL;
 }
 
@@ -865,7 +883,7 @@ getBaumModuleCellCount (void) {
 
 static void
 assumeBaumDeviceIdentity (const char *identity) {
-  LogPrint(LOG_INFO, "Baum Device Identity: %s", identity);
+  logMessage(LOG_INFO, "Baum Device Identity: %s", identity);
 }
 
 static void
@@ -901,17 +919,20 @@ logBaumPowerdownReason (BaumPowerdownReason reason) {
   int length = 0;
   char delimiter = ':';
 
-  sprintf(&buffer[length], "Baum Powerdown%n", &length);
-  for (entry=reasonTable; entry->bit; ++entry) {
+  STR_BEGIN(buffer, sizeof(buffer));
+  STR_PRINTF("Baum Powerdown");
+
+  for (entry=reasonTable; entry->bit; entry+=1) {
     if (reason & entry->bit) {
-      int count;
-      sprintf(&buffer[length], "%c %s%n", delimiter, entry->explanation, &count);
-      length += count;
+      STR_PRINTF("%c %s", delimiter, entry->explanation);
       delimiter = ',';
     }
   }
 
-  LogPrint(LOG_WARNING, "%.*s", length, buffer);
+  length = STR_LENGTH;
+  STR_END;
+
+  logMessage(LOG_WARNING, "%.*s", length, buffer);
   return 1;
 }
 
@@ -930,7 +951,7 @@ readBaumPacket (BrailleDisplay *brl, unsigned char *packet, int size) {
       return 0;
     }
 
-    if (byte == ESCAPE) {
+    if (byte == ESC) {
       if ((escape = !escape)) continue;
     } else if (escape) {
       escape = 0;
@@ -1067,13 +1088,13 @@ static int
 writeBaumPacket (BrailleDisplay *brl, const unsigned char *packet, int length) {
   unsigned char buffer[1 + (length * 2)];
   unsigned char *byte = buffer;
-  *byte++ = ESCAPE;
+  *byte++ = ESC;
 
   {
     int index = 0;
     while (index < length)
-      if ((*byte++ = packet[index++]) == ESCAPE)
-        *byte++ = ESCAPE;
+      if ((*byte++ = packet[index++]) == ESC)
+        *byte++ = ESC;
   }
 
   {
@@ -1133,9 +1154,7 @@ writeBaumDataRegisters (
       *byte++ = BAUM_DRC_Write;
       *byte++ = start;
       *byte++ = count;
-
-      memcpy(byte, registers, count);
-      byte += count;
+      byte = mempcpy(byte, registers, count);
 
       if (!writeBaumPacket(brl, packet, byte-packet)) return 0;
     }
@@ -1147,7 +1166,7 @@ writeBaumDataRegisters (
 typedef struct {
   const KeyTableDefinition *keyTableDefinition;
   int (*writeAllCells) (BrailleDisplay *brl);
-  int (*writeCellRange) (BrailleDisplay *brl, int start, int count);
+  int (*writeCellRange) (BrailleDisplay *brl, unsigned int start, unsigned int count);
 } BaumDeviceOperations;
 
 static int
@@ -1156,9 +1175,7 @@ writeBaumCells_all (BrailleDisplay *brl) {
   unsigned char *byte = packet;
 
   *byte++ = BAUM_REQ_DisplayData;
-
-  memcpy(byte, externalCells, cellCount);
-  byte += cellCount;
+  byte = mempcpy(byte, externalCells, cellCount);
 
   return writeBaumPacket(brl, packet, byte-packet);
 }
@@ -1170,17 +1187,15 @@ writeBaumCells_start (BrailleDisplay *brl) {
 
   *byte++ = BAUM_REQ_DisplayData;
   *byte++ = 0;
-
-  memcpy(byte, externalCells, cellCount);
-  byte += cellCount;
+  byte = mempcpy(byte, externalCells, cellCount);
 
   return writeBaumPacket(brl, packet, byte-packet);
 }
 
 static int
-writeBaumCells_modular (BrailleDisplay *brl, int start, int count) {
+writeBaumCells_modular (BrailleDisplay *brl, unsigned int start, unsigned int count) {
   if (start < brl->textColumns) {
-    int amount = MIN(count, brl->textColumns-start);
+    unsigned int amount = MIN(count, brl->textColumns-start);
 
     if (amount > 0) {
       if (!writeBaumDataRegisters(brl, &baumDisplayModule, &externalCells[start], start, amount)) return 0;
@@ -1207,6 +1222,11 @@ static const BaumDeviceOperations baumDeviceOperations[] = {
     .writeAllCells = writeBaumCells_start
   }
   ,
+  [BAUM_DEVICE_Vario80] = {
+    .keyTableDefinition = &KEY_TABLE_DEFINITION(vario80),
+    .writeAllCells = writeBaumCells_all
+  }
+  ,
   [BAUM_DEVICE_Modular] = {
     .keyTableDefinition = &KEY_TABLE_DEFINITION(pro),
     .writeCellRange = writeBaumCells_modular
@@ -1309,7 +1329,7 @@ handleBaumDataRegistersEvent (BrailleDisplay *brl, const BaumResponsePacket *pac
     switch (bmd->type) {
       {
         unsigned char flags;
-        unsigned char errors;
+        unsigned char UNUSED errors;
         const signed char *wheel;
         unsigned char wheels;
         unsigned char buttons;
@@ -1387,7 +1407,7 @@ handleBaumDataRegistersEvent (BrailleDisplay *brl, const BaumResponsePacket *pac
         break;
 
       default:
-        LogPrint(LOG_WARNING, "unsupported data register configuration: %u", bmd->type);
+        logMessage(LOG_WARNING, "unsupported data register configuration: %u", bmd->type);
         break;
     }
   }
@@ -1457,7 +1477,7 @@ probeBaumDisplay (BrailleDisplay *brl) {
               return 1;
             }
 
-            LogPrint(LOG_DEBUG, "unexpected cell count: %u", count);
+            logMessage(LOG_DEBUG, "unexpected cell count: %u", count);
             continue;
           }
 
@@ -1495,7 +1515,7 @@ probeBaumDisplay (BrailleDisplay *brl) {
 
           case BAUM_RSP_ErrorCode:
             if (response.data.values.errorCode != BAUM_ERR_PacketType) goto unexpectedPacket;
-            LogPrint(LOG_DEBUG, "unsupported request");
+            logMessage(LOG_DEBUG, "unsupported request");
             continue;
 
           default:
@@ -1510,7 +1530,16 @@ probeBaumDisplay (BrailleDisplay *brl) {
     if (errno != EAGAIN) break;
 
     if (identityCellCount) {
-      cellCount = identityCellCount;
+      /* Older models don't provide the actual cell count
+       * so it must be derived from the identity string.
+       */
+      switch ((cellCount = identityCellCount)) {
+        case 80: /* probably a Vario 80 */
+          baumDeviceType = BAUM_DEVICE_Vario80;
+          cellCount += 4;
+          break;
+      }
+
       return 1;
     }
   } while (++probes < probeLimit);
@@ -1547,7 +1576,7 @@ updateBaumKeys (BrailleDisplay *brl) {
 
       case BAUM_RSP_DisplayKeys: {
         unsigned char keys;
-        unsigned char count = 6;
+        unsigned char UNUSED count = 6;
 
         switch (baumDeviceType) {
           case BAUM_DEVICE_Inka:
@@ -1591,15 +1620,21 @@ updateBaumKeys (BrailleDisplay *brl) {
                              BM_KEY_BACK, 6);
         continue;
 
-      case BAUM_RSP_Front10:
-        updateNavigationKeys(packet.data.values.front10,
-                             BM_KEY_FRONT, 10);
+      case BAUM_RSP_Front10: {
+        unsigned char keys[2];
+        keys[0] = packet.data.values.front10[1];
+        keys[1] = packet.data.values.front10[0];
+        updateNavigationKeys(keys, BM_KEY_FRONT, 10);
         continue;
+      }
 
-      case BAUM_RSP_Back10:
-        updateNavigationKeys(packet.data.values.back10,
-                             BM_KEY_BACK, 10);
+      case BAUM_RSP_Back10: {
+        unsigned char keys[2];
+        keys[0] = packet.data.values.back10[1];
+        keys[1] = packet.data.values.back10[0];
+        updateNavigationKeys(keys, BM_KEY_BACK, 10);
         continue;
+      }
 
       case BAUM_RSP_EntryKeys:
         updateNavigationKeys(packet.data.values.entryKeys,
@@ -1616,7 +1651,7 @@ updateBaumKeys (BrailleDisplay *brl) {
       case BAUM_RSP_HorizontalSensors:
         if (!(switchSettings & BAUM_SWT_DisableSensors)) {
           updateKeyGroup(keysState.horizontalSensors, packet.data.values.horizontalSensors,
-                         BM_SET_HorizontalSensors, 0, brl->textColumns);
+                         BM_SET_HorizontalSensors, 0, brl->textColumns, 0);
         }
         continue;
 
@@ -1640,10 +1675,14 @@ updateBaumKeys (BrailleDisplay *brl) {
 
       case BAUM_RSP_VerticalSensors:
         if (!(switchSettings & BAUM_SWT_DisableSensors)) {
+          int scaled = (switchSettings & BAUM_SWT_ScaledVertical) != 0;
+
           updateKeyGroup(keysState.leftSensors, packet.data.values.verticalSensors.left,
-                         BM_SET_LeftSensors, 0, VERTICAL_SENSOR_COUNT);
+                         (scaled? BM_SET_ScaledLeftSensors: BM_SET_LeftSensors),
+                         0, VERTICAL_SENSOR_COUNT, scaled);
           updateKeyGroup(keysState.rightSensors, packet.data.values.verticalSensors.right,
-                         BM_SET_RightSensors, 0, VERTICAL_SENSOR_COUNT);
+                         (scaled? BM_SET_ScaledRightSensors: BM_SET_RightSensors),
+                         0, VERTICAL_SENSOR_COUNT, scaled);
         }
         continue;
 
@@ -1659,7 +1698,7 @@ updateBaumKeys (BrailleDisplay *brl) {
 
       doRoutingKeys:
         updateKeyGroup(keysState.routingKeys, packet.data.values.routingKeys,
-                       BM_SET_RoutingKeys, 0, cellCount);
+                       BM_SET_RoutingKeys, 0, cellCount, 0);
         continue;
 
       case BAUM_RSP_Switches:
@@ -1679,7 +1718,7 @@ updateBaumKeys (BrailleDisplay *brl) {
 
       case BAUM_RSP_ErrorCode:
         if (packet.data.values.errorCode != BAUM_ERR_PacketType) goto unexpectedPacket;
-        LogPrint(LOG_DEBUG, "unsupported request");
+        logMessage(LOG_DEBUG, "unsupported request");
         continue;
 
       default:
@@ -1698,7 +1737,7 @@ writeBaumCells (BrailleDisplay *brl) {
 }
 
 static int
-writeBaumCellRange (BrailleDisplay *brl, int start, int count) {
+writeBaumCellRange (BrailleDisplay *brl, unsigned int start, unsigned int count) {
   const BaumDeviceOperations *bdo = &baumDeviceOperations[baumDeviceType];
   if (!bdo->writeCellRange) return 1;
   return bdo->writeCellRange(brl, start, count);
@@ -1707,7 +1746,7 @@ writeBaumCellRange (BrailleDisplay *brl, int start, int count) {
 static const ProtocolOperations baumOperations = {
   "Baum",
   19200, SERIAL_PARITY_NONE,
-  {0X01, 0X02, 0X04, 0X08, 0X10, 0X20, 0X40, 0X80},
+  &dotsTable_ISO11548_1,
   readBaumPacket, writeBaumPacket,
   probeBaumDisplay, updateBaumKeys,
   writeBaumCells, writeBaumCellRange
@@ -1854,12 +1893,12 @@ findHandyTechModel (unsigned char identity) {
 
   for (model=handyTechModelTable; model->name; ++model) {
     if (identity == model->identity) {
-      LogPrint(LOG_INFO, "Baum emulation: HandyTech Model: %02X -> %s", identity, model->name);
+      logMessage(LOG_INFO, "Baum emulation: HandyTech Model: %02X -> %s", identity, model->name);
       return model;
     }
   }
 
-  LogPrint(LOG_WARNING, "Baum emulation: unknown HandyTech identity code: %02X", identity);
+  logMessage(LOG_WARNING, "Baum emulation: unknown HandyTech identity code: %02X", identity);
   return NULL;
 }
 
@@ -1954,21 +1993,20 @@ writeHandyTechCells (BrailleDisplay *brl) {
     while (count-- > 0) *byte++ = 0;
   }
 
-  memcpy(byte, externalCells, ht->textCount);
-  byte += ht->textCount;
+  byte = mempcpy(byte, externalCells, ht->textCount);
 
   return writeHandyTechPacket(brl, packet, byte-packet);
 }
 
 static int
-writeHandyTechCellRange (BrailleDisplay *brl, int start, int count) {
+writeHandyTechCellRange (BrailleDisplay *brl, unsigned int start, unsigned int count) {
   return 1;
 }
 
 static const ProtocolOperations handyTechOperations = {
   "HandyTech",
   19200, SERIAL_PARITY_ODD,
-  {0X01, 0X02, 0X04, 0X08, 0X10, 0X20, 0X40, 0X80},
+  &dotsTable_ISO11548_1,
   readHandyTechPacket, writeHandyTechPacket,
   probeHandyTechDisplay, updateHandyTechKeys,
   writeHandyTechCells, writeHandyTechCellRange
@@ -2112,9 +2150,7 @@ writePowerBraillePacket (BrailleDisplay *brl, const unsigned char *packet, int l
 
   *byte++ = 0XFF;
   *byte++ = 0XFF;
-
-  memcpy(byte, packet, length);
-  byte += length;
+  byte = mempcpy(byte, packet, length);
 
   {
     int count = byte - buffer;
@@ -2138,8 +2174,8 @@ probePowerBrailleDisplay (BrailleDisplay *brl) {
       if (getPowerBraillePacket(brl, &response)) {
         if (response.data.code == PB_RSP_IDENTITY) {
           const unsigned char *version = response.data.values.identity.version;
-          LogPrint(LOG_INFO, "Baum emulation: PowerBraille Version: %c%c%c%c",
-                   version[0], version[1], version[2], version[3]);
+          logMessage(LOG_INFO, "Baum emulation: PowerBraille Version: %c%c%c%c",
+                     version[0], version[1], version[2], version[3]);
           cellCount = response.data.values.identity.cells;
           return 1;
         }
@@ -2166,7 +2202,7 @@ updatePowerBrailleKeys (BrailleDisplay *brl) {
 
         case PB_RSP_SENSORS:
           updateKeyGroup(keysState.routingKeys, packet.data.values.sensors.horizontal,
-                         BM_SET_RoutingKeys, 0, brl->textColumns);
+                         BM_SET_RoutingKeys, 0, brl->textColumns, 0);
           continue;
 
         default:
@@ -2254,14 +2290,14 @@ writePowerBrailleCells (BrailleDisplay *brl) {
 }
 
 static int
-writePowerBrailleCellRange (BrailleDisplay *brl, int start, int count) {
+writePowerBrailleCellRange (BrailleDisplay *brl, unsigned int start, unsigned int count) {
   return 1;
 }
 
 static const ProtocolOperations powerBrailleOperations = {
   "PowerBraille",
   9600, SERIAL_PARITY_NONE,
-  {0X01, 0X02, 0X04, 0X08, 0X10, 0X20, 0X40, 0X80},
+  &dotsTable_ISO11548_1,
   readPowerBraillePacket, writePowerBraillePacket,
   probePowerBrailleDisplay, updatePowerBrailleKeys,
   writePowerBrailleCells, writePowerBrailleCellRange
@@ -2488,10 +2524,8 @@ openUsbPort (const char *device) {
 static int
 configureUsbPort (void) {
   const SerialParameters parameters = {
+    SERIAL_DEFAULT_PARAMETERS,
     .baud = protocol->serialBaud,
-    .flow = SERIAL_FLOW_NONE,
-    .data = 8,
-    .stop = 1,
     .parity = protocol->serialParity
   };
 
@@ -2578,9 +2612,9 @@ writeBluetoothBytes (const unsigned char *buffer, int length) {
   int count = bthWriteData(bluetoothConnection, buffer, length);
   if (count != length) {
     if (count == -1) {
-      LogError("Baum Bluetooth write");
+      logSystemError("Baum Bluetooth write");
     } else {
-      LogPrint(LOG_WARNING, "Trunccated bluetooth write: %d < %d", count, length);
+      logMessage(LOG_WARNING, "Trunccated bluetooth write: %d < %d", count, length);
     }
   }
   return count;
@@ -2608,13 +2642,13 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
     unsigned int index;
 
     if (!validateChoice(&index, parameters[PARM_PROTOCOLS], choices))
-      LogPrint(LOG_WARNING, "%s: %s", "invalid protocols setting", parameters[PARM_PROTOCOLS]);
+      logMessage(LOG_WARNING, "%s: %s", "invalid protocols setting", parameters[PARM_PROTOCOLS]);
     requestedProtocols = values[index];
   }
 
   useVarioKeys = 0;
   if (!validateYesNo(&useVarioKeys, parameters[PARM_VARIOKEYS]))
-    LogPrint(LOG_WARNING, "%s: %s", "invalid vario keys setting", parameters[PARM_VARIOKEYS]);
+    logMessage(LOG_WARNING, "%s: %s", "invalid vario keys setting", parameters[PARM_VARIOKEYS]);
 
   if (isSerialDevice(&device)) {
     io = &serialOperations;
@@ -2636,11 +2670,10 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
       if (!protocolAddress) protocolAddress = io->protocols;
 
       while ((protocol = *(protocolAddress++))) {
-        LogPrint(LOG_DEBUG, "probing with %s protocol", protocol->name);
-        makeOutputTable(protocol->dotsTable, outputTable);
+        logMessage(LOG_DEBUG, "probing with %s protocol", protocol->name);
 
         {
-          int bits = 10;
+          unsigned int bits = 10;
           if (protocol->serialParity != SERIAL_PARITY_NONE) ++bits;
           charactersPerSecond = protocol->serialBaud / bits;
         }
@@ -2654,6 +2687,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
           if (protocol->probeDisplay(brl)) {
             logCellCount(brl);
 
+            makeOutputTable(protocol->dotsTable[0]);
             if (!clearCellRange(brl, 0, cellCount)) goto failed;
             if (!updateCells(brl)) goto failed;
 
@@ -2710,37 +2744,17 @@ brl_reset (BrailleDisplay *brl) {
 
 static int
 brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
-  int start = 0;
-  int count = brl->textColumns;
-
-  while (count > 0) {
-    if (brl->buffer[count-1] != internalCells[count-1]) break;
-    count -= 1;
-  }
-
-  while (start < count) {
-    if (brl->buffer[start] != internalCells[start]) break;
-    start += 1;
-  }
-  count -= start;
-
-  memcpy(&internalCells[start], &brl->buffer[start], count);
-  if (!updateCellRange(brl, start, count)) return 0;
+  if (!putCells(brl, brl->buffer, 0, brl->textColumns)) return 0;
   return updateCells(brl);
 }
 
 static int
 brl_writeStatus (BrailleDisplay *brl, const unsigned char *status) {
-  if (memcmp(&internalCells[brl->textColumns], status, brl->statusColumns) != 0) {
-    memcpy(&internalCells[brl->textColumns], status, brl->statusColumns);
-    if (!updateCellRange(brl, brl->textColumns, brl->statusColumns)) return 0;
-  }
-
-  return 1;
+  return putCells(brl, status, brl->textColumns, brl->statusColumns);
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
   protocol->updateKeys(brl);
   return (errno == EAGAIN)? EOF: BRL_CMD_RESTARTBRL;
 }
