@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -37,8 +37,8 @@
 
 #include "io_serial.h"
 static SerialDevice *serialDevice = NULL;
-static const int serialBaud = 38400;
-static int charactersPerSecond;
+static const unsigned int serialBaud = 38400;
+static unsigned int charactersPerSecond;
 
 typedef union {
   unsigned char bytes[3];
@@ -86,9 +86,6 @@ static unsigned int persistentRoutingOperation;
 static unsigned int temporaryRoutingOperation;
 static unsigned int currentRoutingOperation;
 
-static TranslationTable outputTable;
-static TranslationTable inputTable;
-
 static int
 readBytes (unsigned char *buffer, int count, int wait) {
   const int timeout = 100;
@@ -200,11 +197,7 @@ refreshCells (BrailleDisplay *brl) {
   unsigned char *byte = buffer;
 
   *byte++ = BNO_WRITE;
-
-  {
-    int i;
-    for (i=0; i<cellCount; ++i) *byte++ = outputTable[cellBuffer[i]];
-  }
+  byte = translateOutputCells(byte, cellBuffer, cellCount);
 
   return writePacket(brl, buffer, byte-buffer);
 }
@@ -237,7 +230,7 @@ getCharacter (BrailleDisplay *brl) {
       default:
         break;
       case BNI_CHARACTER:
-        return convertDotsToCharacter(textTable, inputTable[getByte()]);
+        return convertDotsToCharacter(textTable, translateInputCell(getByte()));
       case BNI_SPACE:
         switch (getByte()) {
           default:
@@ -271,8 +264,9 @@ static int
 getVirtualTerminal (void) {
   int vt = -1;
 #ifdef HAVE_LINUX_VT_H
-  int consoleDescriptor = getConsole();
-  if (consoleDescriptor != -1) {
+  FILE *console = getConsole();
+  if (console) {
+    int consoleDescriptor = fileno(console);
     struct vt_stat state;
     if (ioctl(consoleDescriptor, VT_GETSTATE, &state) != -1) {
       vt = state.v_active;
@@ -285,17 +279,18 @@ getVirtualTerminal (void) {
 static void
 setVirtualTerminal (int vt) {
 #ifdef HAVE_LINUX_VT_H
-  int consoleDescriptor = getConsole();
-  if (consoleDescriptor != -1) {
-    LogPrint(LOG_DEBUG, "switching to virtual terminal %d", vt);
+  FILE *console = getConsole();
+  if (console) {
+    int consoleDescriptor = fileno(console);
+    logMessage(LOG_DEBUG, "switching to virtual terminal %d", vt);
     if (ioctl(consoleDescriptor, VT_ACTIVATE, vt) != -1) {
       if (ioctl(consoleDescriptor, VT_WAITACTIVE, vt) != -1) {
-        LogPrint(LOG_INFO, "switched to virtual terminal %d", vt);
+        logMessage(LOG_INFO, "switched to virtual terminal %d", vt);
       } else {
-        LogError("virtual console wait");
+        logSystemError("virtual console wait");
       }
     } else {
-      LogError("virtual console activate");
+      logSystemError("virtual console activate");
     }
   }
 #endif /* HAVE_LINUX_VT_H */
@@ -305,13 +300,14 @@ static void
 openVisualDisplay (void) {
 #ifdef HAVE_LINUX_VT_H
   if (displayDescriptor == -1) {
-    int consoleDescriptor = getConsole();
-    if (consoleDescriptor != -1) {
+    FILE *console = getConsole();
+    if (console) {
+      int consoleDescriptor = fileno(console);
       if (ioctl(consoleDescriptor, VT_OPENQRY, &displayTerminal) != -1) {
         char path[0X20];
         snprintf(path, sizeof(path), "/dev/tty%d", displayTerminal);
         if ((displayDescriptor = open(path, O_WRONLY)) != -1) {
-          LogPrint(LOG_INFO, "visual display is %s", path);
+          logMessage(LOG_INFO, "visual display is %s", path);
         }
       }
     }
@@ -334,17 +330,22 @@ closeVisualDisplay (int vt) {
   }
 }
 
-static void
+static int
 writeVisualDisplay (unsigned char c) {
   if (displayDescriptor != -1) {
-    write(displayDescriptor, &c, 1);
+    if (write(displayDescriptor, &c, 1) == -1) {
+      logSystemError("write");
+      return 0;
+    }
   }
+
+  return 1;
 }
 
 static int
-visualDisplay (BrailleDisplay *brl, unsigned char byte, BRL_DriverCommandContext context) {
+visualDisplay (BrailleDisplay *brl, unsigned char byte, KeyTableCommandContext context) {
   int vt = getVirtualTerminal();
-  const unsigned char end[] = {0X1B, 0};
+  const unsigned char end[] = {ESC, 0};
   unsigned int state = 0;
   openVisualDisplay();
   writeVisualDisplay(BNI_DISPLAY);
@@ -373,12 +374,6 @@ visualDisplay (BrailleDisplay *brl, unsigned char byte, BRL_DriverCommandContext
 
 static int
 brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
-  {
-     static const DotsTable dots = {0X01, 0X02, 0X04, 0X08, 0X10, 0X20, 0X40, 0X80};
-     makeOutputTable(dots, outputTable);
-     reverseTranslationTable(outputTable, inputTable);
-  }
-
   if (!isSerialDevice(&device)) {
      unsupportedDevice(device);
      return 0;
@@ -386,7 +381,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
 
   if ((serialDevice = serialOpenDevice(device))) {
     if (serialRestartDevice(serialDevice, serialBaud)) {
-      unsigned char request[] = {BNO_DESCRIBE};
+      static const unsigned char request[] = {BNO_DESCRIBE};
       charactersPerSecond = serialBaud / 10;
       if (writePacket(brl, request, sizeof(request)) != -1) {
         while (serialAwaitInput(serialDevice, 100)) {
@@ -405,6 +400,10 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
               }
               dataCells = brl->textColumns * brl->textRows;
               cellCount = statusCells + dataCells;
+
+              makeOutputTable(dotsTable_ISO11548_1);
+              makeInputTable();
+
               if ((cellBuffer = malloc(cellCount))) {
                 memset(cellBuffer, 0, cellCount);
                 statusArea = cellBuffer;
@@ -416,7 +415,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
                 temporaryRoutingOperation = persistentRoutingOperation;
                 return 1;
               } else {
-                LogError("cell buffer allocation");
+                logSystemError("cell buffer allocation");
               }
             } else {
               logUnexpectedPacket(response.bytes, size);
@@ -459,8 +458,7 @@ brl_reset (BrailleDisplay *brl) {
 
 static int
 brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
-  if (memcmp(dataArea, brl->buffer, dataCells) != 0) {
-    memcpy(dataArea, brl->buffer, dataCells);
+  if (cellsHaveChanged(dataArea, brl->buffer, dataCells, NULL, NULL, NULL)) {
     refreshCells(brl);
   }
   return 1;
@@ -494,7 +492,7 @@ getDecimalInteger (BrailleDisplay *brl, unsigned int *integer, unsigned int widt
       case WC_C('7'):
       case WC_C('8'):
       case WC_C('9'):
-        memcpy(buffer, buffer+1, width-1);
+        memmove(buffer, buffer+1, width-1);
         buffer[width-1] = character;
         break;
     }
@@ -580,7 +578,7 @@ getFunctionKey (BrailleDisplay *brl) {
 }
 
 static int
-interpretNavigation (BrailleDisplay *brl, unsigned char dots, BRL_DriverCommandContext context) {
+interpretNavigation (BrailleDisplay *brl, unsigned char dots, KeyTableCommandContext context) {
   switch (dots) {
     default:
       break;
@@ -647,7 +645,7 @@ interpretNavigation (BrailleDisplay *brl, unsigned char dots, BRL_DriverCommandC
 }
 
 static int
-interpretCharacter (BrailleDisplay *brl, unsigned char dots, BRL_DriverCommandContext context) {
+interpretCharacter (BrailleDisplay *brl, unsigned char dots, KeyTableCommandContext context) {
   int mask = 0X00;
   switch (currentKeyboardMode) {
     case KBM_NAVIGATE:
@@ -664,12 +662,13 @@ interpretCharacter (BrailleDisplay *brl, unsigned char dots, BRL_DriverCommandCo
       mask |= 0X80;
       break;
   }
-  return BRL_BLK_PASSDOTS + (inputTable[dots] | mask);
+  return BRL_BLK_PASSDOTS + (translateInputCell(dots) | mask);
 }
 
 static int
-interpretSpaceChord (BrailleDisplay *brl, unsigned char dots, BRL_DriverCommandContext context) {
-  if (context == BRL_CTX_CHORDS) return BRL_BLK_PASSDOTS | inputTable[dots] | BRL_DOTC;
+interpretSpaceChord (BrailleDisplay *brl, unsigned char dots, KeyTableCommandContext context) {
+  if (context == KTB_CTX_CHORDS)
+    return BRL_BLK_PASSDOTS | translateInputCell(dots) | BRL_DOTC;
   switch (dots) {
     default:
     /* These are overridden by the Braille Note itself. */
@@ -720,16 +719,16 @@ interpretSpaceChord (BrailleDisplay *brl, unsigned char dots, BRL_DriverCommandC
       return BRL_BLK_PASSCHAR + character;
     }
     case BNC_LPAREN:
-      temporaryRoutingOperation = BRL_BLK_CUTBEGIN;
+      temporaryRoutingOperation = BRL_BLK_CLIP_NEW;
       return BRL_CMD_NOOP;
     case BNC_LBRACE:
-      temporaryRoutingOperation = BRL_BLK_CUTAPPEND;
+      temporaryRoutingOperation = BRL_BLK_CLIP_ADD;
       return BRL_CMD_NOOP;
     case BNC_RPAREN:
-      temporaryRoutingOperation = BRL_BLK_CUTRECT;
+      temporaryRoutingOperation = BRL_BLK_COPY_RECT;
       return BRL_CMD_NOOP;
     case BNC_RBRACE:
-      temporaryRoutingOperation = BRL_BLK_CUTLINE;
+      temporaryRoutingOperation = BRL_BLK_COPY_LINE;
       return BRL_CMD_NOOP;
     case BNC_BAR:
       return BRL_CMD_CSRJMP_VERT;
@@ -782,7 +781,7 @@ interpretSpaceChord (BrailleDisplay *brl, unsigned char dots, BRL_DriverCommandC
 }
 
 static int
-interpretBackspaceChord (BrailleDisplay *brl, unsigned char dots, BRL_DriverCommandContext context) {
+interpretBackspaceChord (BrailleDisplay *brl, unsigned char dots, KeyTableCommandContext context) {
   switch (dots & 0X3F) {
     default:
       break;
@@ -823,7 +822,7 @@ interpretBackspaceChord (BrailleDisplay *brl, unsigned char dots, BRL_DriverComm
 }
 
 static int
-interpretEnterChord (BrailleDisplay *brl, unsigned char dots, BRL_DriverCommandContext context) {
+interpretEnterChord (BrailleDisplay *brl, unsigned char dots, KeyTableCommandContext context) {
   switch (dots) {
     default:
     /* These are overridden by the Braille Note itself. */
@@ -860,7 +859,7 @@ interpretEnterChord (BrailleDisplay *brl, unsigned char dots, BRL_DriverCommandC
 }
 
 static int
-interpretThumbKeys (BrailleDisplay *brl, unsigned char keys, BRL_DriverCommandContext context) {
+interpretThumbKeys (BrailleDisplay *brl, unsigned char keys, KeyTableCommandContext context) {
   switch (keys) {
     default:
       break;
@@ -889,17 +888,17 @@ interpretThumbKeys (BrailleDisplay *brl, unsigned char keys, BRL_DriverCommandCo
 }
 
 static int
-interpretRoutingKey (BrailleDisplay *brl, unsigned char key, BRL_DriverCommandContext context) {
+interpretRoutingKey (BrailleDisplay *brl, unsigned char key, KeyTableCommandContext context) {
   return currentRoutingOperation + key;
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
   ResponsePacket packet;
   int size;
 
   while ((size = getPacket(&packet))) {
-    int (*handler)(BrailleDisplay *, unsigned char, BRL_DriverCommandContext);
+    int (*handler)(BrailleDisplay *, unsigned char, KeyTableCommandContext);
     unsigned char data;
 
     switch (packet.data.code) {
