@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -23,14 +23,14 @@
 #include <errno.h>
 
 #include "log.h"
+#include "ascii.h"
 
 #include "brl_driver.h"
 #include "io_serial.h"
 
 static SerialDevice *serialDevice = NULL;
-static int charactersPerSecond;
+static unsigned int charactersPerSecond;
 static unsigned char *outputBuffer = NULL;
-static TranslationTable outputTable;
 
 static int
 readBytes (unsigned char *buffer, int size, size_t *length) {
@@ -44,13 +44,13 @@ readBytes (unsigned char *buffer, int size, size_t *length) {
     }
     byte = buffer[*length - 1];
 
-    if ((*length == 1) && (byte == 0X06)) {
+    if ((*length == 1) && (byte == ACK)) {
       *length = 0;
       continue;
     }
 
-    if (byte == '\r') {
-      LogBytes(LOG_DEBUG, "Read", buffer, *length);
+    if (byte == CR) {
+      logBytes(LOG_DEBUG, "Read", buffer, *length);
       return 1;
     }
   }
@@ -60,7 +60,7 @@ readBytes (unsigned char *buffer, int size, size_t *length) {
 
 static int
 writeBytes (BrailleDisplay *brl, const unsigned char *bytes, int count) {
-  LogBytes(LOG_DEBUG, "Write", bytes, count);
+  logBytes(LOG_DEBUG, "Write", bytes, count);
   if (serialWriteData(serialDevice, bytes, count) == -1) return 0;
   brl->writeDelay += (count * 1000 / charactersPerSecond) + 1;
   return 1;
@@ -68,27 +68,20 @@ writeBytes (BrailleDisplay *brl, const unsigned char *bytes, int count) {
 
 static int
 writeAcknowledgement (BrailleDisplay *brl) {
-  static const unsigned char acknowledgement[] = {0X06};
+  static const unsigned char acknowledgement[] = {ACK};
   return writeBytes(brl, acknowledgement, sizeof(acknowledgement));
 }
 
 static int
 writeCells (BrailleDisplay *brl) {
   static const unsigned char header[] = {'D'};
-  static const unsigned char trailer[] = {'\r'};
+  static const unsigned char trailer[] = {CR};
   unsigned char buffer[sizeof(header) + brl->textColumns + sizeof(trailer)];
   unsigned char *byte = buffer;
 
-  memcpy(byte, header, sizeof(header));
-  byte += sizeof(header);
-
-  {
-    int i;
-    for (i=0; i<brl->textColumns; *byte++=outputTable[outputBuffer[i++]]);
-  }
-
-  memcpy(byte, trailer, sizeof(trailer));
-  byte += sizeof(trailer);
+  byte = mempcpy(byte, header, sizeof(header));
+  byte = translateOutputCells(byte, outputBuffer, brl->textColumns);
+  byte = mempcpy(byte, trailer, sizeof(trailer));
 
   return writeBytes(brl, buffer, byte-buffer);
 }
@@ -131,7 +124,7 @@ interpretNumber (int *number, const unsigned char **bytes, int *count) {
 
 static int
 identifyDisplay (BrailleDisplay *brl) {
-  static const unsigned char identify[] = {'I', '\r'};
+  static const unsigned char identify[] = {'I', CR};
 
   if (writeBytes(brl, identify, sizeof(identify))) {
     if (serialAwaitInput(serialDevice, 1000)) {
@@ -151,7 +144,7 @@ identifyDisplay (BrailleDisplay *brl) {
             skipCharacter(' ', &bytes, &count);
             if (interpretNumber(&cells, &bytes, &count)) {
               if (!count) {
-                LogPrint(LOG_INFO, "Detected: %.*s", (int)length, identity);
+                logMessage(LOG_INFO, "Detected: %.*s", (int)length, identity);
 
                 brl->textColumns = cells;
                 brl->textRows = 1;
@@ -178,21 +171,23 @@ setTable (BrailleDisplay *brl, int table) {
 
 static int
 brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
-  {
-    static const DotsTable dots = {0X01, 0X02, 0X04, 0X10, 0X20, 0X40, 0X08, 0X80};
-    makeOutputTable(dots, outputTable);
-  }
-  
   if (!isSerialDevice(&device)) {
     unsupportedDevice(device);
     return 0;
   }
 
   if ((serialDevice = serialOpenDevice(device))) {
-    static const int baud = 9600;
+    static const unsigned int baud = 9600;
     charactersPerSecond = baud / 10;
     if (serialRestartDevice(serialDevice, baud)) {
       if (identifyDisplay(brl)) {
+        {
+          static const DotsTable dots = {
+            0X01, 0X02, 0X04, 0X10, 0X20, 0X40, 0X08, 0X80
+          };
+          makeOutputTable(dots);
+        }
+  
         if ((outputBuffer = malloc(brl->textColumns))) {
           if (setTable(brl, 0)) {
             memset(outputBuffer, 0, brl->textColumns);
@@ -204,7 +199,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
           free(outputBuffer);
           outputBuffer = NULL;
         } else {
-          LogError("Output buffer allocation");
+          logSystemError("Output buffer allocation");
         }
       }
     }
@@ -231,15 +226,14 @@ brl_destruct (BrailleDisplay *brl) {
 
 static int
 brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
-  if (memcmp(brl->buffer, outputBuffer, brl->textColumns) != 0) {
-    memcpy(outputBuffer, brl->buffer, brl->textColumns);
+  if (cellsHaveChanged(outputBuffer, brl->buffer, brl->textColumns, NULL, NULL, NULL)) {
     writeCells(brl);
   }
   return 1;
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
   unsigned char buffer[0X100];
   size_t length;
 
