@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -29,7 +29,7 @@
 
 #include "log.h"
 #include "timing.h"
-#include "misc.h"
+#include "ascii.h"
 
 #define BRL_STATUS_FIELDS sfCursorAndWindowColumn, sfCursorAndWindowRow, sfStateDots
 #define BRL_HAVE_STATUS_CELLS
@@ -75,55 +75,56 @@ END_KEY_TABLE_LIST
 SerialDevice *CB_serialDevice;			/* file descriptor for Braille display */
 int CB_charactersPerSecond;			/* file descriptor for Braille display */
 
-static TranslationTable outputTable;	/* dot mapping table (output) */
-static unsigned char *prevdata;	/* previously received data */
-static unsigned char status[5], oldstatus[5];	/* status cells - always five */
-static unsigned char *rawdata;		/* writebrl() buffer for raw Braille data */
-static short rawlen;			/* length of rawdata buffer */
+static unsigned char *prevdata;
+
+#define MAX_STATUS_CELLS 5
+static unsigned char status[MAX_STATUS_CELLS];
+static unsigned char oldstatus[MAX_STATUS_CELLS];
 
 static int
 brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
-  {
-    static const DotsTable dots = {0X01, 0X02, 0X04, 0X80, 0X40, 0X20, 0X08, 0X10};
-    makeOutputTable(dots, outputTable);
-  }
-
   if (!isSerialDevice(&device)) {
     unsupportedDevice(device);
     return 0;
   }
 
-  prevdata = rawdata = NULL;		/* clear pointers */
+  prevdata = NULL;		/* clear pointers */
 
   if ((CB_serialDevice = serialOpenDevice(device))) {
     if (serialRestartDevice(CB_serialDevice, BAUDRATE)) {
-      static const unsigned char init_seq[] = { 27, '?' };
-      static const unsigned char init_ack[] = { 27, '?' };
-
       CB_charactersPerSecond = BAUDRATE / 10;
 
-      if (serialWriteData(CB_serialDevice, init_seq, sizeof(init_seq)) == sizeof(init_seq)) {
-        short n;
-        unsigned char c;
-        signed char id = -1;
+      if (serialSetFlowControl(CB_serialDevice, SERIAL_FLOW_HARDWARE)) {
+        static const unsigned char init_seq[] = {ESC, '?'};
 
-        hasTimedOut(0);		/* initialise timeout testing */
-        n = 0;
-        do {
-          approximateDelay (20);
-          if (serialReadData(CB_serialDevice, &c, 1, 0, 0) != 1)
-            continue;
-          if (n < sizeof(init_ack) && c != init_ack[n])
-            continue;
-          if (n == sizeof(init_ack)) {
-            id = c;
-            break;
-          }
-          n++;
-        } while (!hasTimedOut(ACK_TIMEOUT) && n <= sizeof(init_ack));
+        if (serialWriteData(CB_serialDevice, init_seq, sizeof(init_seq)) == sizeof(init_seq)) {
+          static const unsigned char init_ack[] = {ESC, '?'};
+          unsigned int n = 0;
+          unsigned char c;
+          signed char id = -1;
 
-        if (id != -1) {
-          if (serialSetFlowControl(CB_serialDevice, SERIAL_FLOW_HARDWARE)) {
+          hasTimedOut(0);		/* initialise timeout testing */
+          do {
+            approximateDelay(20);
+            if (serialReadData(CB_serialDevice, &c, 1, 0, 0) != 1) continue;
+
+            if (n == sizeof(init_ack)) {
+              id = c;
+              break;
+            }
+
+          testCharacter:
+            if (c != init_ack[n]) {
+              if (n == 0) continue;
+
+              n = 0;
+              goto testCharacter;
+            }
+
+            n += 1;
+          } while (!hasTimedOut(ACK_TIMEOUT));
+
+          if (id != -1) {
             typedef struct {
               char identifier;
               char textColumns;
@@ -138,36 +139,40 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
               { .identifier = 9, .textColumns = 80 },
               { .textColumns = 0 }
             };
-
             const ModelEntry *model = modelTable;
 
             while (model->textColumns) {
               if (id == model->identifier) break;
-              model++;
+              model += 1;
             }
               
             if (model->textColumns) {
-              brl->textColumns = model->textColumns;
-              brl->textRows = BRLROWS;
-              brl->statusColumns = 5;
-              brl->statusRows = 1;
+              if ((prevdata = malloc(brl->textColumns))) {
+                brl->textColumns = model->textColumns;
+                brl->textRows = 1;
 
-              {
-                const KeyTableDefinition *ktd = &KEY_TABLE_DEFINITION(all);
+                brl->statusColumns = 5;
+                brl->statusRows = 1;
 
-                brl->keyBindings = ktd->bindings;
-                brl->keyNameTables = ktd->names;
+                {
+                  const KeyTableDefinition *ktd = &KEY_TABLE_DEFINITION(all);
+                  brl->keyBindings = ktd->bindings;
+                  brl->keyNameTables = ktd->names;
+                }
+
+                {
+                  static const DotsTable dots = {
+                    0X01, 0X02, 0X04, 0X80, 0X40, 0X20, 0X08, 0X10
+                  };
+                  makeOutputTable(dots);
+                }
+
+                return 1;
+              } else {
+                logMallocError();
               }
-
-              /* Allocate space for buffers */
-              prevdata = mallocWrapper (brl->textColumns * brl->textRows);
-              /* rawdata has to have room for the pre- and post-data sequences,
-               * the status cells, and escaped 0x1b's: */
-              rawdata = mallocWrapper (20 + brl->textColumns * brl->textRows * 2);
-
-              return 1;
             } else {
-              LogPrint(LOG_ERR, "Detected unknown CombiBraille model with ID %02X.", id);
+              logMessage(LOG_ERR, "detected unknown CombiBraille model with ID %02X", id);
             }
           }
         }
@@ -175,6 +180,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
     }
 
     serialCloseDevice(CB_serialDevice);
+    CB_serialDevice = NULL;
   }
 
   return 0;
@@ -182,91 +188,56 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
 
 static void
 brl_destruct (BrailleDisplay *brl) {
-  unsigned char *pre_data = (unsigned char *)PRE_DATA;	/* bytewise accessible copies */
-  unsigned char *post_data = (unsigned char *)POST_DATA;
-  unsigned char *close_seq = (unsigned char *)CLOSE_SEQ;
+  if (prevdata) {
+    free(prevdata);
+    prevdata = NULL;
+  }
 
-  rawlen = 0;
-  if (pre_data[0])
-    {
-      memcpy (rawdata + rawlen, pre_data + 1, pre_data[0]);
-      rawlen += pre_data[0];
-    }
-  /* Clear the five status cells and the main display: */
-  memset (rawdata + rawlen, 0, 5 + brl->textColumns * brl->textRows);
-  rawlen += 5 + brl->textColumns * brl->textRows;
-  if (post_data[0])
-    {
-      memcpy (rawdata + rawlen, post_data + 1, post_data[0]);
-      rawlen += post_data[0];
-    }
-
-  /* Send closing sequence: */
-  if (close_seq[0])
-    {
-      memcpy (rawdata + rawlen, close_seq + 1, close_seq[0]);
-      rawlen += close_seq[0];
-    }
-  serialWriteData (CB_serialDevice, rawdata, rawlen);
-
-  free (prevdata);
-  free (rawdata);
-
-  serialCloseDevice (CB_serialDevice);
+  if (CB_serialDevice) {
+    serialCloseDevice(CB_serialDevice);
+    CB_serialDevice = NULL;
+  }
 }
 
 static int
 brl_writeStatus (BrailleDisplay *brl, const unsigned char *s) {
-  short i;
-
-  /* Dot mapping: */
-  for (i = 0; i < 5; status[i] = outputTable[s[i]], i++);
+  memcpy(status, s, brl->statusColumns);
   return 1;
 }
 
 static int
 brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
-  short i;			/* loop counter */
-  unsigned char *pre_data = (unsigned char *)PRE_DATA;	/* bytewise accessible copies */
-  unsigned char *post_data = (unsigned char *)POST_DATA;
+  int textChanged = cellsHaveChanged(prevdata, brl->buffer, brl->textColumns, NULL, NULL, NULL);
+  int statusChanged = cellsHaveChanged(oldstatus, status, brl->statusColumns, NULL, NULL, NULL);
 
   /* Only refresh display if the data has changed: */
-  if (memcmp (brl->buffer, prevdata, brl->textColumns * brl->textRows) || \
-      memcmp (status, oldstatus, 5))
-    {
-      /* Save new Braille data: */
-      memcpy (prevdata, brl->buffer, brl->textColumns * brl->textRows);
+  if (textChanged || statusChanged) {
+    static const unsigned char header[] = {ESC, 'B'};
+    unsigned char buffer[sizeof(header) + ((brl->statusColumns + brl->textColumns) * 2)];
+    unsigned char *byte = buffer;
+    int i;			/* loop counter */
 
-      /* Dot mapping from standard to CombiBraille: */
-      for (i = 0; i < brl->textColumns * brl->textRows; brl->buffer[i] = outputTable[brl->buffer[i]], \
-	   i++);
+    byte = mempcpy(byte, header, sizeof(header));
 
-      rawlen = 0;
-      if (pre_data[0])
-	{
-	  memcpy (rawdata + rawlen, pre_data + 1, pre_data[0]);
-	  rawlen += pre_data[0];
-	}
-      for (i = 0; i < 5; i++)
-	{
-	  rawdata[rawlen++] = status[i];
-	  if (status[i] == 0x1b)	/* CombiBraille hack */
-	    rawdata[rawlen++] = 0x1b;
-	}
-      for (i = 0; i < brl->textColumns * brl->textRows; i++)
-	{
-	  rawdata[rawlen++] = brl->buffer[i];
-	  if (brl->buffer[i] == 0x1b)	/* CombiBraille hack */
-	    rawdata[rawlen++] = brl->buffer[i];
-	}
-      if (post_data[0])
-	{
-	  memcpy (rawdata + rawlen, post_data + 1, post_data[0]);
-	  rawlen += post_data[0];
-	}
-      serialWriteData (CB_serialDevice, rawdata, rawlen);
-      brl->writeDelay += (rawlen * 1000 / CB_charactersPerSecond) + 1;
+    for (i=0; i<brl->statusColumns; i+=1) {
+      const unsigned char c = translateOutputCell(status[i]);
+      if (c == ESC) *byte++ = c;
+      *byte++ = c;
     }
+
+    for (i=0; i<brl->textColumns; i+=1) {
+      const unsigned char c = translateOutputCell(brl->buffer[i]);
+      if (c == ESC) *byte++ = c;
+      *byte++ = c;
+    }
+
+    {
+      const size_t size = byte - buffer;
+      serialWriteData(CB_serialDevice, buffer, size);
+      brl->writeDelay += (size * 1000 / CB_charactersPerSecond) + 1;
+    }
+  }
+
   return 1;
 }
 
@@ -319,7 +290,7 @@ putKeys (unsigned char bits, unsigned char keys, unsigned char count) {
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
   int key;
 
   while ((key = getKey()) != EOF) {
