@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -73,11 +73,6 @@ static BRLPARAMS Models[NB_MODEL] ={
 #define BRLROWS		1
 #define MAX_STCELLS	4	/* hiest number of status cells */
 
-/* Eco dot translate table. This Braille Line use Spanish Braille file 
- * text.es.tbl for default
- */
-static TranslationTable outputTable;
-
 /* Global variables */
 static SerialDevice *serialDevice;			/* file descriptor for Braille display */
 static unsigned char *rawdata;		/* translated data to send to Braille */
@@ -139,24 +134,16 @@ static char BRL_KEY[] = "\x10\x02\x88";
 
 static int WriteToBrlDisplay(unsigned char *Data)
 {
-  int size = DIM_BRL_WRITE_PREFIX + DIM_BRL_WRITE_SUFIX + BrailleSize;
-  unsigned char *buffTmp;
+  unsigned int size = DIM_BRL_WRITE_PREFIX + BrailleSize + DIM_BRL_WRITE_SUFIX;
+  unsigned char buffer[size];
+  unsigned char *byte = buffer;
   
-  /* Make temporal buffer */
-  buffTmp = malloc(size);
-
-  /* Copy the prefix, Data and sufix */
-  memcpy(buffTmp, BRL_WRITE_PREFIX, DIM_BRL_WRITE_PREFIX);
-  memcpy(buffTmp + DIM_BRL_WRITE_PREFIX, Data, BrailleSize);
-  memcpy(buffTmp + DIM_BRL_WRITE_PREFIX + BrailleSize, BRL_WRITE_SUFIX, DIM_BRL_WRITE_SUFIX); 
-  
-  /* write directly to Braille Line  */
-  serialWriteData(serialDevice, buffTmp, size);
-  
-  /* Destroy temporal buffer */
-  free(buffTmp);
-  
-return(0);
+  byte = mempcpy(byte, BRL_WRITE_PREFIX, DIM_BRL_WRITE_PREFIX);
+  byte = mempcpy(byte, Data, BrailleSize);
+  byte = mempcpy(byte, BRL_WRITE_SUFIX, DIM_BRL_WRITE_SUFIX);
+ 
+  serialWriteData(serialDevice, buffer, byte-buffer);
+  return 0;
 }
 
 static int brl_construct(BrailleDisplay *brl, char **parameters, const char *device)
@@ -164,11 +151,6 @@ static int brl_construct(BrailleDisplay *brl, char **parameters, const char *dev
   short ModelID = MODEL;
   unsigned char buffer[DIM_BRL_ID + 6];
 
-  {
-    static const DotsTable dots = {0X10, 0X20, 0X40, 0X01, 0X02, 0X04, 0X80, 0X08};
-    makeOutputTable(dots, outputTable);
-  }
-
   if (!isSerialDevice(&device)) {
     unsupportedDevice(device);
     return 0;
@@ -225,13 +207,20 @@ static int brl_construct(BrailleDisplay *brl, char **parameters, const char *dev
       serialReadData(serialDevice, &buffer, DIM_BRL_READY + 6, 100, 100);
       /*}while(strncmp (buffer, BRL_READY, DIM_BRL_READY));*/
       
-      LogPrint(LOG_DEBUG, "buffer is: %s",buffer);
+      logMessage(LOG_DEBUG, "buffer is: %s",buffer);
   
   /* Set model params */
   model = &Models[ModelID];
   brl->textColumns = model->Cols;		/* initialise size of main display */
   brl->textRows = BRLROWS;		/* ever is 1 in this type of braille lines */
   
+  {
+    static const DotsTable dots = {
+      0X10, 0X20, 0X40, 0X01, 0X02, 0X04, 0X80, 0X08
+    };
+    makeOutputTable(dots);
+  }
+
   /* Need to calculate the size; Cols + Status + 1 (space between) */
   BrailleSize = brl->textColumns + model->NbStCells + 1;
 
@@ -269,19 +258,17 @@ static void brl_destruct(BrailleDisplay *brl)
 
 static int brl_writeWindow(BrailleDisplay *brl, const wchar_t *text)
 {
-  int i, j;
-
+  unsigned char *byte = rawdata;
   /* This Braille Line need to display all information, include status */
   
   /* Make status info to rawdata */
-  for(i=0; i < model->NbStCells; i++)
-      rawdata[i] = outputTable[Status[i]];
+  byte = translateOutputCells(byte, Status, model->NbStCells);
 
-  i++;  /* step a phisical space with main cells */
+  /* step a physical space with main cells */
+  *byte++ = 0;
   
   /* Make main info to rawdata */
-  for(j=0; j < brl->textColumns; j++)
-      rawdata[i++] = outputTable[brl->buffer[j]];
+  byte = translateOutputCells(byte, brl->buffer, brl->textColumns);
      
   /* Write to Braille Display */
   WriteToBrlDisplay(rawdata);
@@ -297,7 +284,7 @@ static int brl_writeStatus(BrailleDisplay *brl, const unsigned char *st)
 }
 
 
-static int brl_readCommand(BrailleDisplay *brl, BRL_DriverCommandContext context)
+static int brl_readCommand(BrailleDisplay *brl, KeyTableCommandContext context)
 {
   int res = EOF;
   long bytes = 0;
