@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -33,7 +33,6 @@
 
 #define BRLSTAT ST_AlvaStyle
 #define BRL_HAVE_PACKET_IO
-#define BRL_HAVE_FIRMNESS
 #include "brl_driver.h"
 #include "brldefs-fs.h"
 
@@ -389,48 +388,47 @@ typedef struct {
   unsigned char type;
 } ModelEntry;
 
-static const DotsTable dots12345678 = {0X01, 0X02, 0X04, 0X08, 0X10, 0X20, 0X40, 0X80};
-static const DotsTable dots12374568 = {0X01, 0X02, 0X04, 0X10, 0X20, 0X40, 0X08, 0X80};
+static const DotsTable dotsTable_Focus1 = {0X01, 0X02, 0X04, 0X10, 0X20, 0X40, 0X08, 0X80};
 
 static const ModelEntry modelTable[] = {
   { .identifier = "Focus 40",
-    .dotsTable = &dots12345678,
+    .dotsTable = &dotsTable_ISO11548_1,
     .cellCount = 40,
     .type = MOD_TYPE_Focus
   }
   ,
   { .identifier = "Focus 44",
-    .dotsTable = &dots12374568,
+    .dotsTable = &dotsTable_Focus1,
     .cellCount = 44,
     .type = MOD_TYPE_Focus
   }
   ,
   { .identifier = "Focus 70",
-    .dotsTable = &dots12374568,
+    .dotsTable = &dotsTable_Focus1,
     .cellCount = 70,
     .type = MOD_TYPE_Focus
   }
   ,
   { .identifier = "Focus 80",
-    .dotsTable = &dots12345678,
+    .dotsTable = &dotsTable_ISO11548_1,
     .cellCount = 80,
     .type = MOD_TYPE_Focus
   }
   ,
   { .identifier = "Focus 84",
-    .dotsTable = &dots12374568,
+    .dotsTable = &dotsTable_Focus1,
     .cellCount = 84,
     .type = MOD_TYPE_Focus
   }
   ,
   { .identifier = "pm display 20",
-    .dotsTable = &dots12345678,
+    .dotsTable = &dotsTable_ISO11548_1,
     .cellCount = 20,
     .type = MOD_TYPE_Pacmate
   }
   ,
   { .identifier = "pm display 40",
-    .dotsTable = &dots12345678,
+    .dotsTable = &dotsTable_ISO11548_1,
     .cellCount = 40,
     .type = MOD_TYPE_Pacmate
   }
@@ -441,17 +439,16 @@ static const ModelEntry modelTable[] = {
 static const ModelEntry *model;
 static const KeyTableDefinition *keyTableDefinition;
 
-static TranslationTable outputTable;
 static unsigned char outputBuffer[84];
 
-static int writeFrom;
-static int writeTo;
-static int writingFrom;
-static int writingTo;
+static int writeFirst;
+static int writeLast;
+static int writingFirst;
+static int writingLast;
 
 typedef void (*AcknowledgementHandler) (int ok);
 static AcknowledgementHandler acknowledgementHandler;
-static struct timeval acknowledgementTime;
+static TimeValue acknowledgementTime;
 static int acknowledgementsMissing;
 static unsigned char configFlags;
 static int firmnessSetting;
@@ -575,9 +572,9 @@ logNegativeAcknowledgement (const Packet *packet) {
       break;
   }
 
-  LogPrint(LOG_WARNING, "Negative Acknowledgement: [%02X] %s in [%02X] %s",
-           packet->header.arg1, problem,
-           packet->header.arg2, component);
+  logMessage(LOG_WARNING, "Negative Acknowledgement: [%02X] %s in [%02X] %s",
+             packet->header.arg1, problem,
+             packet->header.arg2, component);
 }
 
 static void
@@ -593,15 +590,15 @@ handleFirmnessAcknowledgement (int ok) {
 static void
 handleWriteAcknowledgement (int ok) {
   if (!ok) {
-    if ((writeFrom == -1) || (writingFrom < writeFrom)) writeFrom = writingFrom;
-    if ((writeTo == -1) || (writingTo > writeTo)) writeTo = writingTo;
+    if ((writeFirst == -1) || (writingFirst < writeFirst)) writeFirst = writingFirst;
+    if ((writeLast == -1) || (writingLast > writeLast)) writeLast = writingLast;
   }
 }
 
 static void
 setAcknowledgementHandler (AcknowledgementHandler handler) {
   acknowledgementHandler = handler;
-  gettimeofday(&acknowledgementTime, NULL);
+  getCurrentTime(&acknowledgementTime);
 }
 
 static int
@@ -620,23 +617,24 @@ writeRequest (BrailleDisplay *brl) {
     return 1;
   }
 
-  if (writeTo != -1) {
-    int count = writeTo + 1 - writeFrom;
+  if (writeLast != -1) {
+    unsigned int count = writeLast + 1 - writeFirst;
+    unsigned char buffer[count];
     int truncate = count > outputPayloadLimit;
 
     if (truncate) count = outputPayloadLimit;
-    if (writePacket(brl, PKT_WRITE, count, writeFrom, 0,
-                    &outputBuffer[writeFrom]) == -1) return 0;
+    translateOutputCells(buffer, &outputBuffer[writeFirst], count);
+    if (writePacket(brl, PKT_WRITE, count, writeFirst, 0, buffer) == -1) return 0;
 
     setAcknowledgementHandler(handleWriteAcknowledgement);
-    writingFrom = writeFrom;
+    writingFirst = writeFirst;
 
     if (truncate) {
-      writingTo = (writeFrom += count) - 1;
+      writingLast = (writeFirst += count) - 1;
     } else {
-      writingTo = writeTo;
-      writeFrom = -1;
-      writeTo = -1;
+      writingLast = writeLast;
+      writeFirst = -1;
+      writeLast = -1;
     }
 
     return 1;
@@ -652,16 +650,15 @@ updateCells (
   unsigned char count,
   unsigned char offset
 ) {
-  int index;
-  for (index=0; index<count; ++index) {
-    unsigned char cell = outputTable[cells[index]];
-    unsigned char position = offset + index;
-    unsigned char *byte = &outputBuffer[position];
-    if (cell != *byte) {
-      if ((writeFrom == -1) || (position < writeFrom)) writeFrom = position;
-      if (position > writeTo) writeTo = position;
-      *byte = cell;
-    }
+  unsigned int from;
+  unsigned int to;
+
+  if (cellsHaveChanged(&outputBuffer[offset], cells, count, &from, &to, NULL)) {
+    int first = from + offset;
+    int last = to + offset - 1;
+
+    if ((writeFirst == -1) || (first < writeFirst)) writeFirst = first;
+    if (last > writeLast) writeLast = last;
   }
 }
 
@@ -686,7 +683,7 @@ readPacket (BrailleDisplay *brl, Packet *packet) {
         int index;
 
         for (index=0; index<size; index+=1) checksum -= inputBuffer.bytes[index];
-        if (checksum) LogPrint(LOG_WARNING, "Input packet checksum error.");
+        if (checksum) logMessage(LOG_WARNING, "Input packet checksum error.");
       }
 
       memcpy(packet, &inputBuffer, size);
@@ -701,7 +698,7 @@ readPacket (BrailleDisplay *brl, Packet *packet) {
 
       if (count < 1) {
         if (count == -1) {
-          LogError("read");
+          logSystemError("read");
         } else if ((count == 0) && (inputCount > 0)) {
           if (io->awaitInput(1000)) goto retry;
           if (errno != EAGAIN) count = -1;
@@ -750,7 +747,7 @@ getPacket (BrailleDisplay *brl, Packet *packet) {
           logNegativeAcknowledgement(packet);
 
           if (!acknowledgementHandler) {
-            LogPrint(LOG_WARNING, "Unexpected NAK.");
+            logMessage(LOG_WARNING, "Unexpected NAK.");
             continue;
           }
 
@@ -765,8 +762,8 @@ getPacket (BrailleDisplay *brl, Packet *packet) {
                 outputPayloadLimit--;
 
               if (outputPayloadLimit != originalLimit)
-                LogPrint(LOG_WARNING, "Maximum payload length reduced from %d to %d.",
-                         originalLimit, outputPayloadLimit);
+                logMessage(LOG_WARNING, "Maximum payload length reduced from %d to %d.",
+                           originalLimit, outputPayloadLimit);
               break;
             }
           }
@@ -777,7 +774,7 @@ getPacket (BrailleDisplay *brl, Packet *packet) {
 
         case PKT_ACK:
           if (!acknowledgementHandler) {
-            LogPrint(LOG_WARNING, "Unexpected ACK.");
+            logMessage(LOG_WARNING, "Unexpected ACK.");
             continue;
           }
 
@@ -794,11 +791,11 @@ getPacket (BrailleDisplay *brl, Packet *packet) {
     } else if ((count == 0) && acknowledgementHandler &&
                (millisecondsSince(&acknowledgementTime) > 500)) {
       if (++acknowledgementsMissing < 5) {
-        LogPrint(LOG_WARNING, "Missing ACK; assuming NAK.");
+        logMessage(LOG_WARNING, "Missing ACK; assuming NAK.");
         goto handleNegativeAcknowledgement;
       }
 
-      LogPrint(LOG_WARNING, "Too many missing ACKs.");
+      logMessage(LOG_WARNING, "Too many missing ACKs.");
       count = -1;
     }
 
@@ -806,6 +803,12 @@ getPacket (BrailleDisplay *brl, Packet *packet) {
   }
 }
 
+static int
+setFirmness (BrailleDisplay *brl, BrailleFirmness setting) {
+  firmnessSetting = setting * 0XFF / BRL_FIRMNESS_MAXIMUM;
+  return writeRequest(brl);
+}
+
 static int
 brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
   if (isSerialDevice(&device)) {
@@ -844,9 +847,9 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
 
         switch (response.header.type) {
           case PKT_INFO:
-            LogPrint(LOG_DEBUG, "Manufacturer: %s", response.payload.info.manufacturer);
-            LogPrint(LOG_DEBUG, "Model: %s", response.payload.info.model);
-            LogPrint(LOG_DEBUG, "Firmware: %s", response.payload.info.firmware);
+            logMessage(LOG_DEBUG, "Manufacturer: %s", response.payload.info.manufacturer);
+            logMessage(LOG_DEBUG, "Model: %s", response.payload.info.model);
+            logMessage(LOG_DEBUG, "Firmware: %s", response.payload.info.firmware);
 
             model = modelTable;
             while (model->identifier) {
@@ -857,12 +860,12 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
             if (!model->identifier) {
               static ModelEntry generic;
               model = &generic;
-              LogPrint(LOG_WARNING, "Detected unknown model: %s", response.payload.info.model);
+              logMessage(LOG_WARNING, "Detected unknown model: %s", response.payload.info.model);
 
               memset(&generic, 0, sizeof(generic));
               generic.identifier = "Generic";
               generic.cellCount = 20;
-              generic.dotsTable = &dots12345678;
+              generic.dotsTable = &dotsTable_ISO11548_1;
               generic.type = MOD_TYPE_Pacmate;
 
               {
@@ -871,7 +874,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
                   const DotsTable *dotsTable;
                 } ExceptionEntry;
                 static const ExceptionEntry exceptionTable[] = {
-                  {"Focus", &dots12374568},
+                  {"Focus", &dotsTable_Focus1},
                   {NULL   , NULL         }
                 };
                 const ExceptionEntry *exception = exceptionTable;
@@ -901,10 +904,10 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
 
             if (model) {
               keyTableDefinition = modelTypeTable[model->type].keyTableDefinition;
-              makeOutputTable(model->dotsTable[0], outputTable);
+              makeOutputTable(model->dotsTable[0]);
               memset(outputBuffer, 0, model->cellCount);
-              writeFrom = 0;
-              writeTo = model->cellCount - 1;
+              writeFirst = 0;
+              writeLast = model->cellCount - 1;
 
               acknowledgementHandler = NULL;
               acknowledgementsMissing = 0;
@@ -927,10 +930,10 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
 
               oldKeys = 0;
 
-              LogPrint(LOG_INFO, "Detected %s: cells=%d, firmware=%s",
-                       model->identifier,
-                       model->cellCount,
-                       response.payload.info.firmware);
+              logMessage(LOG_INFO, "Detected %s: cells=%d, firmware=%s",
+                         model->identifier,
+                         model->cellCount,
+                         response.payload.info.firmware);
             }
             break;
 
@@ -953,6 +956,8 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
           brl->keyBindings = keyTableDefinition->bindings;
           brl->keyNameTables = keyTableDefinition->names;
 
+          brl->setFirmness = setFirmness;
+
           if (!writeRequest(brl)) break;
           return 1;
         }
@@ -1011,7 +1016,7 @@ updateKeys (uint64_t newKeys, unsigned char keyBase, unsigned char keyCount) {
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
   Packet packet;
   int count;
 
@@ -1085,11 +1090,11 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
       }
     }
 
-    LogPrint(LOG_WARNING, "unsupported packet: %02X %02X %02X %02X",
-             packet.header.type,
-             packet.header.arg1,
-             packet.header.arg2,
-             packet.header.arg3);
+    logMessage(LOG_WARNING, "unsupported packet: %02X %02X %02X %02X",
+               packet.header.type,
+               packet.header.arg1,
+               packet.header.arg2,
+               packet.header.arg3);
   }
 
   return EOF;
@@ -1102,7 +1107,7 @@ brl_readPacket (BrailleDisplay *brl, void *buffer, size_t length) {
   if (count > 0) {
     if (count > sizeof(packet.header)) count--;
     if (length < count) {
-      LogPrint(LOG_WARNING, "Input packet buffer too small: %d < %d", (int)length, count);
+      logMessage(LOG_WARNING, "Input packet buffer too small: %d < %d", (int)length, count);
       count = length;
     }
     memcpy(buffer, &packet, count);
@@ -1122,13 +1127,13 @@ brl_writePacket (BrailleDisplay *brl, const void *packet, size_t length) {
     }
     if (length >= size) {
       if (length > size)
-        LogPrint(LOG_WARNING, "Output packet buffer larger than necessary: %d > %d",
-                 (int)length, size);
+        logMessage(LOG_WARNING, "Output packet buffer larger than necessary: %d > %d",
+                   (int)length, size);
       return writePacket(brl, bytes[0], bytes[1], bytes[2], bytes[3],
                          (hasPayload? &bytes[4]: NULL));
     }
   }
-  LogPrint(LOG_WARNING, "Output packet buffer too small: %d < %d", (int)length, size);
+  logMessage(LOG_WARNING, "Output packet buffer too small: %d < %d", (int)length, size);
   return 0;
 }
 
@@ -1136,9 +1141,3 @@ static int
 brl_reset (BrailleDisplay *brl) {
   return 0;
 }
-
-static void
-brl_firmness (BrailleDisplay *brl, BrailleFirmness setting) {
-  firmnessSetting = setting * 0XFF / BRL_FIRMNESS_MAXIMUM;
-  writeRequest(brl);
-}
