@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -20,16 +20,23 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <time.h>
 #include <errno.h>
 
+#include "bitfield.h"
 #include "log.h"
+#include "parse.h"
 #include "timing.h"
-#include "misc.h"
+#include "ascii.h"
+
+typedef enum {
+  PARM_SETTIME
+} DriverParameter;
+#define BRLPARMS "settime"
 
 #define BRLSTAT ST_AlvaStyle
 #define BRL_HAVE_STATUS_CELLS
 #define BRL_HAVE_PACKET_IO
-#define BRL_HAVE_SENSITIVITY
 #include "brl_driver.h"
 #include "touch.h"
 #include "brldefs-ht.h"
@@ -137,6 +144,13 @@ BEGIN_KEY_NAME_TABLES(bs80)
   KEY_NAME_TABLE(brailleStar),
 END_KEY_NAME_TABLES
 
+BEGIN_KEY_NAME_TABLES(ab40)
+  KEY_NAME_TABLE(routing),
+  KEY_NAME_TABLE(dots),
+  KEY_NAME_TABLE(rockers),
+  KEY_NAME_TABLE(brailleStar),
+END_KEY_NAME_TABLES
+
 BEGIN_KEY_NAME_TABLE(brailleWave)
   KEY_NAME_ENTRY(HT_KEY_Up, "Left"),
   KEY_NAME_ENTRY(HT_KEY_Down, "Right"),
@@ -188,6 +202,7 @@ DEFINE_KEY_TABLE(me64)
 DEFINE_KEY_TABLE(me88)
 DEFINE_KEY_TABLE(bs40)
 DEFINE_KEY_TABLE(bs80)
+DEFINE_KEY_TABLE(ab40)
 DEFINE_KEY_TABLE(wave)
 DEFINE_KEY_TABLE(easy)
 DEFINE_KEY_TABLE(bkwm)
@@ -198,6 +213,7 @@ BEGIN_KEY_TABLE_LIST
   &KEY_TABLE_DEFINITION(me88),
   &KEY_TABLE_DEFINITION(bs40),
   &KEY_TABLE_DEFINITION(bs80),
+  &KEY_TABLE_DEFINITION(ab40),
   &KEY_TABLE_DEFINITION(wave),
   &KEY_TABLE_DEFINITION(easy),
   &KEY_TABLE_DEFINITION(bkwm),
@@ -206,13 +222,20 @@ END_KEY_TABLE_LIST
 static const unsigned char BookwormSessionEnd[] = {0X05, 0X07};	/* bookworm trailer to display braille */
 
 typedef int (ByteInterpreter) (unsigned char byte);
-static ByteInterpreter interpretKeyByte;
-static ByteInterpreter interpretBookwormByte;
+static ByteInterpreter interpretByte_key;
+static ByteInterpreter interpretByte_Bookworm;
 
 typedef int (CellWriter) (BrailleDisplay *brl);
-static CellWriter writeStatusAndTextCells;
-static CellWriter writeBookwormCells;
-static CellWriter writeEvolutionCells;
+static CellWriter writeCells_statusAndText;
+static CellWriter writeCells_Bookworm;
+static CellWriter writeCells_Evolution;
+
+typedef int (FirmnessSetter) (BrailleDisplay *brl, BrailleFirmness setting);
+static FirmnessSetter setFirmness;
+
+typedef int (SensitivitySetter) (BrailleDisplay *brl, BrailleSensitivity setting);
+static SensitivitySetter setSensitivity_Evolution;
+static SensitivitySetter setSensitivity_ActiveBraille;
 
 typedef struct {
   const char *name;
@@ -220,6 +243,8 @@ typedef struct {
 
   ByteInterpreter *interpretByte;
   CellWriter *writeCells;
+  FirmnessSetter *setFirmness;
+  SensitivitySetter *setSensitivity;
 
   const unsigned char *sessionEndAddress;
 
@@ -239,8 +264,8 @@ static const ModelEntry modelTable[] = {
     .textCells = 20,
     .statusCells = 4,
     .keyTableDefinition = &KEY_TABLE_DEFINITION(mdlr),
-    .interpretByte = interpretKeyByte,
-    .writeCells = writeStatusAndTextCells
+    .interpretByte = interpretByte_key,
+    .writeCells = writeCells_statusAndText
   }
   ,
   { .identifier = HT_MODEL_Modular40,
@@ -248,8 +273,8 @@ static const ModelEntry modelTable[] = {
     .textCells = 40,
     .statusCells = 4,
     .keyTableDefinition = &KEY_TABLE_DEFINITION(mdlr),
-    .interpretByte = interpretKeyByte,
-    .writeCells = writeStatusAndTextCells
+    .interpretByte = interpretByte_key,
+    .writeCells = writeCells_statusAndText
   }
   ,
   { .identifier = HT_MODEL_Modular80,
@@ -257,8 +282,8 @@ static const ModelEntry modelTable[] = {
     .textCells = 80,
     .statusCells = 4,
     .keyTableDefinition = &KEY_TABLE_DEFINITION(mdlr),
-    .interpretByte = interpretKeyByte,
-    .writeCells = writeStatusAndTextCells
+    .interpretByte = interpretByte_key,
+    .writeCells = writeCells_statusAndText
   }
   ,
   { .identifier = HT_MODEL_ModularEvolution64,
@@ -266,8 +291,9 @@ static const ModelEntry modelTable[] = {
     .textCells = 64,
     .statusCells = 0,
     .keyTableDefinition = &KEY_TABLE_DEFINITION(me64),
-    .interpretByte = interpretKeyByte,
-    .writeCells = writeEvolutionCells,
+    .interpretByte = interpretByte_key,
+    .writeCells = writeCells_Evolution,
+    .setSensitivity = setSensitivity_Evolution,
     .hasATC = 1
   }
   ,
@@ -276,18 +302,19 @@ static const ModelEntry modelTable[] = {
     .textCells = 88,
     .statusCells = 0,
     .keyTableDefinition = &KEY_TABLE_DEFINITION(me88),
-    .interpretByte = interpretKeyByte,
-    .writeCells = writeEvolutionCells,
+    .interpretByte = interpretByte_key,
+    .writeCells = writeCells_Evolution,
+    .setSensitivity = setSensitivity_Evolution,
     .hasATC = 1
   }
   ,
   { .identifier = HT_MODEL_BrailleWave,
-    .name = "Braille Wave 40",
+    .name = "Braille Wave",
     .textCells = 40,
     .statusCells = 0,
     .keyTableDefinition = &KEY_TABLE_DEFINITION(wave),
-    .interpretByte = interpretKeyByte,
-    .writeCells = writeStatusAndTextCells
+    .interpretByte = interpretByte_key,
+    .writeCells = writeCells_statusAndText
   }
   ,
   { .identifier = HT_MODEL_Bookworm,
@@ -295,18 +322,18 @@ static const ModelEntry modelTable[] = {
     .textCells = 8,
     .statusCells = 0,
     .keyTableDefinition = &KEY_TABLE_DEFINITION(bkwm),
-    .interpretByte = interpretBookwormByte,
-    .writeCells = writeBookwormCells,
+    .interpretByte = interpretByte_Bookworm,
+    .writeCells = writeCells_Bookworm,
     HT_BYTE_SEQUENCE(sessionEnd, BookwormSessionEnd)
   }
   ,
   { .identifier = HT_MODEL_Braillino,
-    .name = "Braillino 20",
+    .name = "Braillino",
     .textCells = 20,
     .statusCells = 0,
     .keyTableDefinition = &KEY_TABLE_DEFINITION(bs40),
-    .interpretByte = interpretKeyByte,
-    .writeCells = writeStatusAndTextCells
+    .interpretByte = interpretByte_key,
+    .writeCells = writeCells_statusAndText
   }
   ,
   { .identifier = HT_MODEL_BrailleStar40,
@@ -314,8 +341,8 @@ static const ModelEntry modelTable[] = {
     .textCells = 40,
     .statusCells = 0,
     .keyTableDefinition = &KEY_TABLE_DEFINITION(bs40),
-    .interpretByte = interpretKeyByte,
-    .writeCells = writeStatusAndTextCells
+    .interpretByte = interpretByte_key,
+    .writeCells = writeCells_statusAndText
   }
   ,
   { .identifier = HT_MODEL_BrailleStar80,
@@ -323,8 +350,8 @@ static const ModelEntry modelTable[] = {
     .textCells = 80,
     .statusCells = 0,
     .keyTableDefinition = &KEY_TABLE_DEFINITION(bs80),
-    .interpretByte = interpretKeyByte,
-    .writeCells = writeStatusAndTextCells
+    .interpretByte = interpretByte_key,
+    .writeCells = writeCells_statusAndText
   }
   ,
   { .identifier = HT_MODEL_EasyBraille,
@@ -332,8 +359,20 @@ static const ModelEntry modelTable[] = {
     .textCells = 40,
     .statusCells = 0,
     .keyTableDefinition = &KEY_TABLE_DEFINITION(easy),
-    .interpretByte = interpretKeyByte,
-    .writeCells = writeStatusAndTextCells
+    .interpretByte = interpretByte_key,
+    .writeCells = writeCells_statusAndText
+  }
+  ,
+  { .identifier = HT_MODEL_ActiveBraille,
+    .name = "Active Braille",
+    .textCells = 40,
+    .statusCells = 0,
+    .keyTableDefinition = &KEY_TABLE_DEFINITION(ab40),
+    .interpretByte = interpretByte_key,
+    .writeCells = writeCells_Evolution,
+    .setFirmness = setFirmness,
+    .setSensitivity = setSensitivity_ActiveBraille,
+    .hasATC = 1
   }
   ,
   { /* end of table */
@@ -345,8 +384,6 @@ static const ModelEntry modelTable[] = {
 #define BRLROWS		1
 #define MAX_STCELLS	4	/* highest number of status cells */
 
-static TranslationTable outputTable;
-
 /* Global variables */
 static unsigned char *rawData = NULL;		/* translated data to send to Braille */
 static unsigned char *prevData = NULL;	/* previously sent raw data */
@@ -354,10 +391,6 @@ static unsigned char rawStatus[MAX_STCELLS];		/* to hold status info */
 static unsigned char prevStatus[MAX_STCELLS];	/* to hold previous status */
 static const ModelEntry *model;		/* points to terminal model config struct */
 
-static unsigned char *at2Buffer;
-static int at2Size;
-static int at2Count;
-
 typedef struct {
   int (*openPort) (char **parameters, const char *device);
   void (*closePort) ();
@@ -367,8 +400,8 @@ typedef struct {
 } InputOutputOperations;
 
 static const InputOutputOperations *io;
-static const int baud = 19200;
-static int charactersPerSecond;
+static const unsigned int baud = 19200;
+static unsigned int charactersPerSecond;
 
 /* Serial IO */
 #include "io_serial.h"
@@ -442,12 +475,12 @@ typedef enum {
   HT_HID_CMD_FlushBuffers = 0X01, /* flush input and output buffers */
 } HtHidCommand;
 
-static uint32_t hidReportSize_OutData;
-static uint32_t hidReportSize_InData;
-static uint32_t hidReportSize_InCommand;
-static uint32_t hidReportSize_OutVersion;
-static uint32_t hidReportSize_OutBaud;
-static uint32_t hidReportSize_InBaud;
+static size_t hidReportSize_OutData;
+static size_t hidReportSize_InData;
+static size_t hidReportSize_InCommand;
+static size_t hidReportSize_OutVersion;
+static size_t hidReportSize_OutBaud;
+static size_t hidReportSize_InBaud;
 
 static uint16_t hidFirmwareVersion;
 static unsigned char *hidInputReport = NULL;
@@ -460,8 +493,8 @@ getHidReport (unsigned char number, unsigned char *buffer, int size) {
   int result = usbHidGetReport(usb->device, usb->definition.interface,
                                number, buffer, size, HT_HID_REPORT_TIMEOUT);
   if (result > 0 && buffer[0] != number) {
-    LogPrint(LOG_WARNING, "unexpected HID report number: expected %02X, received %02X",
-             number, buffer[0]);
+    logMessage(LOG_WARNING, "unexpected HID report number: expected %02X, received %02X",
+               number, buffer[0]);
     errno = EIO;
     result = -1;
   }
@@ -475,54 +508,22 @@ setHidReport (const unsigned char *report, int size) {
                          report[0], report, size, HT_HID_REPORT_TIMEOUT);
 }
 
-static void
-getHidReportSize (const unsigned char *items, uint16_t size, unsigned char identifier, uint32_t *value) {
-  UsbHidReportDescription description;
-
-  *value = 0;
-
-  if (usbHidFillReportDescription(items, size, identifier, &description)) {
-    if (description.defined & USB_HID_ITEM_BIT(UsbHidItemType_ReportCount)) {
-      if (description.defined & USB_HID_ITEM_BIT(UsbHidItemType_ReportSize)) {
-        uint32_t size = ((description.reportCount * description.reportSize) + 7) / 8;
-        LogPrint(LOG_DEBUG, "HID Report Size: %02X = %"PRIu32, identifier, size);
-        *value = 1 + size;
-      } else {
-        LogPrint(LOG_WARNING, "HID report size not defined: %02X", identifier);
-      }
-    } else {
-      LogPrint(LOG_WARNING, "HID report count not defined: %02X", identifier);
-    }
-  } else {
-    LogPrint(LOG_WARNING, "HID report not found: %02X", identifier);
-  }
-}
+typedef struct {
+  HT_HidReportNumber number;
+  size_t *size;
+} ReportEntry;
 
 static void
-getHidReportSizes (void) {
+getHidReportSizes (const ReportEntry *table) {
   unsigned char *items;
-  uint16_t size = usbHidGetItems(usb->device, usb->definition.interface, 0,
-                                 &items, HT_HID_REPORT_TIMEOUT);
+  ssize_t length = usbHidGetItems(usb->device, usb->definition.interface, 0,
+                                  &items, HT_HID_REPORT_TIMEOUT);
 
   if (items) {
-    typedef struct {
-      HT_HidReportNumber number;
-      uint32_t *size;
-    } ReportEntry;
-
-    static const ReportEntry reportTable[] = {
-      {.number=HT_HID_RPT_OutData, .size=&hidReportSize_OutData},
-      {.number=HT_HID_RPT_InData, .size=&hidReportSize_InData},
-      {.number=HT_HID_RPT_InCommand, .size=&hidReportSize_InCommand},
-      {.number=HT_HID_RPT_OutVersion, .size=&hidReportSize_OutVersion},
-      {.number=HT_HID_RPT_OutBaud, .size=&hidReportSize_OutBaud},
-      {.number=HT_HID_RPT_InBaud, .size=&hidReportSize_InBaud},
-      {.number=0}
-    };
-    const ReportEntry *report = reportTable;
+    const ReportEntry *report = table;
 
     while (report->number) {
-      getHidReportSize(items, size, report->number, report->size);
+      usbHidGetReportSize(items, length, report->number, report->size);
       report += 1;
     }
 
@@ -531,13 +532,13 @@ getHidReportSizes (void) {
 }
 
 static void
-getHidInputBuffer (void) {
+allocateHidInputBuffer (void) {
   if (hidReportSize_OutData) {
     if ((hidInputReport = malloc(hidReportSize_OutData))) {
       hidInputLength = 0;
       hidInputOffset = 0;
     } else {
-      LogPrint(LOG_WARNING, "HID input buffer not allocated: %s", strerror(errno));
+      logMessage(LOG_ERR, "HID input buffer not allocated: %s", strerror(errno));
     }
   }
 }
@@ -552,7 +553,7 @@ getHidFirmwareVersion (void) {
 
     if (result > 0) {
       hidFirmwareVersion = (report[1] << 8) | report[2];
-      LogPrint(LOG_INFO, "Firmware Version: %u.%u", report[1], report[2]);
+      logMessage(LOG_INFO, "Firmware Version: %u.%u", report[1], report[2]);
     }
   }
 }
@@ -569,13 +570,218 @@ executeHidFirmwareCommand (HtHidCommand command) {
   }
 }
 
+typedef struct {
+  void (*initialize) (void);
+  int (*awaitInput) (int milliseconds);
+  int (*readBytes) (unsigned char *buffer, int length, int wait);
+  int (*writeBytes) (const unsigned char *buffer, int length);
+} UsbOperations;
+
+static const UsbOperations *usbOps;
+
+static void
+initializeUsb1 (void) {
+}
+
+static int
+awaitUsbInput1 (int milliseconds) {
+  return usbAwaitInput(usb->device, usb->definition.inputEndpoint, milliseconds);
+}
+
+static int
+readUsbBytes1 (unsigned char *buffer, int length, int wait) {
+  int timeout = 100;
+
+  return usbReapInput(usb->device, usb->definition.inputEndpoint, buffer, length,
+                      (wait? timeout: 0), timeout);
+}
+
+static int
+writeUsbBytes1 (const unsigned char *buffer, int length) {
+  return usbWriteEndpoint(usb->device, usb->definition.outputEndpoint,
+                          buffer, length, 1000);
+}
+
+static const UsbOperations usbOperations1 = {
+  .initialize = initializeUsb1,
+  .awaitInput = awaitUsbInput1,
+  .readBytes = readUsbBytes1,
+  .writeBytes = writeUsbBytes1
+};
+
+static void
+initializeUsb2 (void) {
+  static const ReportEntry reportTable[] = {
+    {.number=HT_HID_RPT_OutData, .size=&hidReportSize_OutData},
+    {.number=HT_HID_RPT_InData, .size=&hidReportSize_InData},
+    {.number=HT_HID_RPT_InCommand, .size=&hidReportSize_InCommand},
+    {.number=HT_HID_RPT_OutVersion, .size=&hidReportSize_OutVersion},
+    {.number=HT_HID_RPT_OutBaud, .size=&hidReportSize_OutBaud},
+    {.number=HT_HID_RPT_InBaud, .size=&hidReportSize_InBaud},
+    {.number=0}
+  };
+  getHidReportSizes(reportTable);
+  allocateHidInputBuffer();
+  getHidFirmwareVersion();
+  executeHidFirmwareCommand(HT_HID_CMD_FlushBuffers);
+}
+
+static int
+awaitUsbInput2 (int milliseconds) {
+  if (hidReportSize_OutData) {
+    TimeValue startTime;
+
+    if (hidInputOffset < hidInputLength) return 1;
+    getCurrentTime(&startTime);
+
+    while (1) {
+      int result = getHidReport(HT_HID_RPT_OutData, hidInputReport,
+                                hidReportSize_OutData);
+
+      if (result == -1) return 0;
+      hidInputOffset = 0;
+      if (hidInputLength > 0) return 1;
+
+      if (millisecondsSince(&startTime) >= milliseconds) break;
+      approximateDelay(10);
+    }
+  }
+
+  errno = EAGAIN;
+  return 0;
+}
+
+static int
+readUsbBytes2 (unsigned char *buffer, int length, int wait) {
+  int count = 0;
+
+  while (count < length) {
+    if (!io->awaitInput(wait? 100: 0)) {
+      count = -1;
+      break;
+    }
+
+    {
+      int amount = MIN(length-count, hidInputLength-hidInputOffset);
+
+      memcpy(&buffer[count], &hidInputBuffer[hidInputOffset], amount);
+      hidInputOffset += amount;
+      count += amount;
+    }
+  }
+
+  return count;
+}
+
+static int
+writeUsbBytes2 (const unsigned char *buffer, int length) {
+  int index = 0;
+
+  if (hidReportSize_InData) {
+    while (length) {
+      unsigned char report[hidReportSize_InData];
+      unsigned char count = MIN(length, (sizeof(report) - 2));
+      int result;
+
+      report[0] = HT_HID_RPT_InData;
+      report[1] = count;
+      memcpy(report+2, &buffer[index], count);
+      memset(&report[count+2], 0, sizeof(report)-count-2);
+
+      result = setHidReport(report, sizeof(report));
+      if (result == -1) return -1;
+
+      index += count;
+      length -= count;
+    }
+  }
+
+  return index;
+}
+
+static const UsbOperations usbOperations2 = {
+  .initialize = initializeUsb2,
+  .awaitInput = awaitUsbInput2,
+  .readBytes = readUsbBytes2,
+  .writeBytes = writeUsbBytes2
+};
+
+static void
+initializeUsb3 (void) {
+  static const ReportEntry reportTable[] = {
+    {.number=HT_HID_RPT_OutData, .size=&hidReportSize_OutData},
+    {.number=HT_HID_RPT_InData, .size=&hidReportSize_InData},
+    {.number=0}
+  };
+  getHidReportSizes(reportTable);
+  allocateHidInputBuffer();
+}
+
+static int
+awaitUsbInput3 (int milliseconds) {
+  if (hidReportSize_OutData) {
+    TimeValue startTime;
+
+    if (hidInputOffset < hidInputLength) return 1;
+    getCurrentTime(&startTime);
+
+    while (1) {
+      int result = usbReapInput(usb->device, usb->definition.inputEndpoint,
+                                hidInputReport, hidReportSize_OutData,
+                                0, 100);
+
+      if (result == -1) return 0;
+      hidInputOffset = 0;
+      if (hidInputLength > 0) return 1;
+
+      if (millisecondsSince(&startTime) >= milliseconds) break;
+      approximateDelay(10);
+    }
+  }
+
+  errno = EAGAIN;
+  return 0;
+}
+
+static int
+writeUsbBytes3 (const unsigned char *buffer, int length) {
+  int index = 0;
+
+  if (hidReportSize_InData) {
+    while (length) {
+      unsigned char report[hidReportSize_InData];
+      unsigned char count = MIN(length, (sizeof(report) - 2));
+      int result;
+
+      report[0] = HT_HID_RPT_InData;
+      report[1] = count;
+      memcpy(report+2, &buffer[index], count);
+      memset(&report[count+2], 0, sizeof(report)-count-2);
+
+      result = usbWriteEndpoint(usb->device, usb->definition.outputEndpoint,
+                                report, sizeof(report), 1000);
+      if (result == -1) return -1;
+
+      index += count;
+      length -= count;
+    }
+  }
+
+  return index;
+}
+
+static const UsbOperations usbOperations3 = {
+  .initialize = initializeUsb3,
+  .awaitInput = awaitUsbInput3,
+  .readBytes = readUsbBytes2,
+  .writeBytes = writeUsbBytes3
+};
+
 static int
 openUsbPort (char **parameters, const char *device) {
   const SerialParameters serial = {
+    SERIAL_DEFAULT_PARAMETERS,
     .baud = baud,
-    .flow = SERIAL_FLOW_NONE,
-    .data = 8,
-    .stop = 1,
     .parity = SERIAL_PARITY_ODD
   };
 
@@ -584,41 +790,49 @@ openUsbPort (char **parameters, const char *device) {
       .vendor=0X0921, .product=0X1200,
       .configuration=1, .interface=0, .alternative=0,
       .inputEndpoint=1, .outputEndpoint=1,
-      .serial = &serial
+      .serial = &serial,
+      .data=&usbOperations1
     }
     ,
     { /* FTDI chip */
       .vendor=0X0403, .product=0X6001,
       .configuration=1, .interface=0, .alternative=0,
       .inputEndpoint=1, .outputEndpoint=2,
-      .serial = &serial
+      .serial = &serial,
+      .data=&usbOperations1
     }
     ,
     { /* Easy Braille (HID) */
       .vendor=HT_USB_VENDOR, .product=HT_MODEL_EasyBraille,
-      .configuration=1, .interface=0, .alternative=0
+      .configuration=1, .interface=0, .alternative=0,
+      .data=&usbOperations2
     }
     ,
     { /* Braille Star 40 (HID) */
       .vendor=HT_USB_VENDOR, .product=HT_MODEL_BrailleStar40,
-      .configuration=1, .interface=0, .alternative=0
+      .configuration=1, .interface=0, .alternative=0,
+      .data=&usbOperations2
     }
     ,
     { /* USB-HID adapter */
       .vendor=HT_USB_VENDOR, .product=HT_MODEL_UsbHidAdapter,
-      .configuration=1, .interface=0, .alternative=0
+      .configuration=1, .interface=0, .alternative=0,
+      .data=&usbOperations2
+    }
+    ,
+    { /* Active Braille */
+      .vendor=HT_USB_VENDOR, .product=HT_MODEL_ActiveBraille,
+      .configuration=1, .interface=0, .alternative=0,
+      .inputEndpoint=1, .outputEndpoint=1,
+      .data=&usbOperations3
     }
     ,
     { .vendor=0 }
   };
 
   if ((usb = usbFindChannel(definitions, (void *)device))) {
-    if (!usb->definition.outputEndpoint) {
-      getHidReportSizes();
-      getHidInputBuffer();
-      getHidFirmwareVersion();
-      executeHidFirmwareCommand(HT_HID_CMD_FlushBuffers);
-    }
+    usbOps = usb->definition.data;
+    usbOps->initialize();
 
     return 1;
   }
@@ -640,57 +854,12 @@ closeUsbPort (void) {
 
 static int
 awaitUsbInput (int milliseconds) {
-  if (!usb->definition.inputEndpoint) {
-    if (hidReportSize_OutData) {
-      struct timeval startTime;
-
-      if (hidInputOffset < hidInputLength) return 1;
-      gettimeofday(&startTime, NULL);
-
-      while (1) {
-        int result = getHidReport(HT_HID_RPT_OutData, hidInputReport, hidReportSize_OutData);
-
-        if (result == -1) return 0;
-        hidInputOffset = 0;
-        if (hidInputLength > 0) return 1;
-
-        if (millisecondsSince(&startTime) >= milliseconds) break;
-        approximateDelay(10);
-      }
-    }
-
-    errno = EAGAIN;
-    return 0;
-  }
-
-  return usbAwaitInput(usb->device, usb->definition.inputEndpoint, milliseconds);
+  return usbOps->awaitInput(milliseconds);
 }
 
 static int
 readUsbBytes (unsigned char *buffer, int length, int wait) {
-  const int timeout = 100;
-  int count;
-
-  if (usb->definition.inputEndpoint) {
-    count = usbReapInput(usb->device, usb->definition.inputEndpoint, buffer, length,
-                         (wait? timeout: 0), timeout);
-  } else {
-    count = 0;
-
-    while (count < length) {
-      if (!io->awaitInput(wait? timeout: 0)) {
-        count = -1;
-        break;
-      }
-
-      {
-        int amount = MIN(length-count, hidInputLength-hidInputOffset);
-        memcpy(&buffer[count], &hidInputBuffer[hidInputOffset], amount);
-        hidInputOffset += amount;
-        count += amount;
-      }
-    }
-  }
+  int count = usbOps->readBytes(buffer, length, wait);
 
   if (count != -1) return count;
   if (errno == EAGAIN) return 0;
@@ -701,33 +870,7 @@ static int
 writeUsbBytes (const unsigned char *buffer, int length, unsigned int *delay) {
   if (delay) *delay += (length * 1000 / charactersPerSecond) + 1;
 
-  if (!usb->definition.outputEndpoint) {
-    int index = 0;
-
-    if (hidReportSize_InData) {
-      while (length) {
-        unsigned char report[hidReportSize_InData];
-        unsigned char count = MIN(length, (sizeof(report) - 2));
-        int result;
-
-        report[0] = HT_HID_RPT_InData;
-        report[1] = count;
-        memcpy(report+2, &buffer[index], count);
-        memset(&report[count+2], 0, sizeof(report)-count-2);
-
-        result = setHidReport(report, sizeof(report));
-        if (result == -1) return -1;
-
-        index += count;
-        length -= count;
-      }
-    }
-
-    return index;
-  }
-
-  return usbWriteEndpoint(usb->device, usb->definition.outputEndpoint,
-                          buffer, length, 1000);
+  return usbOps->writeBytes(buffer, length);
 }
 
 static const InputOutputOperations usbOperations = {
@@ -771,9 +914,9 @@ writeBluetoothBytes (const unsigned char *buffer, int length, unsigned int *dela
   if (delay) *delay += (length * 1000 / charactersPerSecond) + 1;
   if (count != length) {
     if (count == -1) {
-      LogError("HandyTech Bluetooth write");
+      logSystemError("HandyTech Bluetooth write");
     } else {
-      LogPrint(LOG_WARNING, "Trunccated bluetooth write: %d < %d", count, length);
+      logMessage(LOG_WARNING, "Trunccated bluetooth write: %d < %d", count, length);
     }
   }
   return count;
@@ -784,15 +927,13 @@ static const InputOutputOperations bluetoothOperations = {
   awaitBluetoothInput, readBluetoothBytes, writeBluetoothBytes
 };
 
-#define SYN 0X16
-
 typedef enum {
   BDS_OFF,
   BDS_READY,
   BDS_WRITING
 } BrailleDisplayState;
 static BrailleDisplayState currentState = BDS_OFF;
-static struct timeval stateTime;
+static TimeValue stateTime;
 static unsigned int retryCount = 0;
 static unsigned char updateRequired = 0;
 
@@ -832,7 +973,15 @@ brl_readPacket (BrailleDisplay *brl, void *buffer, size_t size) {
     } else {
       switch (packet[0]) {
         case HT_PKT_Extended:
-          if (offset == 2) length += byte;
+          if (offset == 2) {
+            length += byte;
+          } else if (offset == 4) {
+            if ((packet[1] == HT_MODEL_ActiveBraille) && 
+                (packet[2] == 2) &&
+                (packet[3] == HT_EXTPKT_Confirmation) &&
+                (byte == 0X15))
+              length += 1;
+          }
           break;
       }
     }
@@ -886,8 +1035,8 @@ setState (BrailleDisplayState state) {
     retryCount = 0;
     currentState = state;
   }
-  gettimeofday(&stateTime, NULL);
-  // LogPrint(LOG_DEBUG, "State: %d+%d", currentState, retryCount);
+  getCurrentTime(&stateTime);
+  // logMessage(LOG_DEBUG, "State: %d+%d", currentState, retryCount);
 }
 
 static int
@@ -916,7 +1065,7 @@ reallocateBuffer (unsigned char **buffer, size_t size) {
   if (allocated) {
     *buffer = address;
   } else {
-    LogError("buffer allocation");
+    logSystemError("buffer allocation");
   }
   return allocated;
 }
@@ -930,23 +1079,27 @@ identifyModel (BrailleDisplay *brl, unsigned char identifier) {
   );
 
   if (!model->name) {
-    LogPrint(LOG_ERR, "Detected unknown HandyTech model with ID %02X.",
-             identifier);
+    logMessage(LOG_ERR, "Detected unknown HandyTech model with ID %02X.",
+               identifier);
     return 0;
   }
 
-  LogPrint(LOG_INFO, "Detected %s: %d data %s, %d status %s.",
-           model->name,
-           model->textCells, (model->textCells == 1)? "cell": "cells",
-           model->statusCells, (model->statusCells == 1)? "cell": "cells");
+  logMessage(LOG_INFO, "Detected %s: %d data %s, %d status %s.",
+             model->name,
+             model->textCells, (model->textCells == 1)? "cell": "cells",
+             model->statusCells, (model->statusCells == 1)? "cell": "cells");
 
   brl->textColumns = model->textCells;			/* initialise size of display */
   brl->textRows = BRLROWS;
   brl->statusColumns = model->statusCells;
   brl->statusRows = 1;
+
   brl->keyBindings = model->keyTableDefinition->bindings;
   brl->keyNameTables = model->keyTableDefinition->names;
 
+  brl->setFirmness = model->setFirmness;
+  brl->setSensitivity = model->setSensitivity;
+
   if (!reallocateBuffer(&rawData, brl->textColumns*brl->textRows)) return 0;
   if (!reallocateBuffer(&prevData, brl->textColumns*brl->textRows)) return 0;
 
@@ -984,22 +1137,104 @@ setAtcMode (BrailleDisplay *brl, unsigned char value) {
 }
 
 static int
-setAtcSensitivity (BrailleDisplay *brl, unsigned char value) {
-  const unsigned char data[] = {value};
+setFirmness (BrailleDisplay *brl, BrailleFirmness setting) {
+  const unsigned char data[] = {setting * 2 / BRL_FIRMNESS_MAXIMUM};
+  return writeExtendedPacket(brl, HT_EXTPKT_SetFirmness, data, sizeof(data));
+}
+
+static int
+setSensitivity_Evolution (BrailleDisplay *brl, BrailleSensitivity setting) {
+  const unsigned char data[] = {0XFF - (setting * 0XF0 / BRL_SENSITIVITY_MAXIMUM)};
   return writeExtendedPacket(brl, HT_EXTPKT_SetAtcSensitivity, data, sizeof(data));
 }
 
 static int
-brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
-  at2Buffer = NULL;
-  at2Size = 0;
-  at2Count = 0;
+setSensitivity_ActiveBraille (BrailleDisplay *brl, BrailleSensitivity setting) {
+  const unsigned char data[] = {setting * 6 / BRL_SENSITIVITY_MAXIMUM};
+  return writeExtendedPacket(brl, HT_EXTPKT_SetAtcSensitivity2, data, sizeof(data));
+}
+
+typedef int (DateTimeProcessor) (BrailleDisplay *brl, const HT_DateTime *dateTime);
+static DateTimeProcessor *dateTimeProcessor = NULL;
+
+static int
+requestDateTime (BrailleDisplay *brl, DateTimeProcessor *processor) {
+  int result = writeExtendedPacket(brl, HT_EXTPKT_GetRTC, NULL, 0);
+
+  if (result) {
+    dateTimeProcessor = processor;
+  }
+
+  return result;
+}
+
+static int
+logDateTime (BrailleDisplay *brl, const HT_DateTime *dateTime) {
+  logMessage(LOG_INFO,
+             "date and time of %s:"
+             " %04" PRIu16 "-%02" PRIu8 "-%02" PRIu8
+             " %02" PRIu8 ":%02" PRIu8 ":%02" PRIu8,
+             model->name,
+             getBigEndian16(dateTime->year), dateTime->month, dateTime->day,
+             dateTime->hour, dateTime->minute, dateTime->second);
+
+  return 1;
+}
+
+static int
+synchronizeDateTime (BrailleDisplay *brl, const HT_DateTime *dateTime) {
+  long int delta;
+  TimeValue hostTime;
+  getCurrentTime(&hostTime);
 
   {
-    static const DotsTable dots = {0X01, 0X02, 0X04, 0X08, 0X10, 0X20, 0X40, 0X80};
-    makeOutputTable(dots, outputTable);
+    TimeValue deviceTime;
+
+    {
+      TimeComponents components = {
+        .year = getBigEndian16(dateTime->year),
+        .month = dateTime->month - 1,
+        .day = dateTime->day - 1,
+        .hour = dateTime->hour,
+        .minute = dateTime->minute,
+        .second = dateTime->second
+      };
+
+      makeTimeValue(&deviceTime, &components);
+    }
+
+    delta = millisecondsBetween(&hostTime, &deviceTime);
+    if (delta < 0) delta = -delta;
   }
 
+  if (delta > 1000) {
+    TimeComponents components;
+    HT_DateTime payload;
+
+    expandTimeValue(&hostTime, &components);
+    putLittleEndian16(&payload.year, components.year);
+    payload.month = components.month + 1;
+    payload.day = components.day + 1;
+    payload.hour = components.hour;
+    payload.minute = components.minute;
+    payload.second = components.second;
+
+    logMessage(LOG_DEBUG, "Time difference between host and device: %ld.%03ld",
+               (delta / MSECS_PER_SEC), (delta % MSECS_PER_SEC));
+
+    if (writeExtendedPacket(brl, HT_EXTPKT_SetRTC,
+                            (unsigned char *)&payload, sizeof(payload))) {
+      return requestDateTime(brl, logDateTime);
+    }
+  }
+
+  return 1;
+}
+
+static int
+brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
+  unsigned int setTime = 0;
+
   if (isSerialDevice(&device)) {
     io = &serialOperations;
   } else if (isUsbDevice(&device)) {
@@ -1014,6 +1249,11 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
   rawData = prevData = NULL;		/* clear pointers */
   charactersPerSecond = baud / 11;
 
+  if (*parameters[PARM_SETTIME])
+    if (!validateYesNo(&setTime, parameters[PARM_SETTIME]))
+      logMessage(LOG_WARNING, "%s: %s", "invalid set time setting", parameters[PARM_SETTIME]);
+  setTime = !!setTime;
+
   if (io->openPort(parameters, device)) {
     int tries = 0;
 
@@ -1025,14 +1265,23 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
         if (length > 0) {
           if (response.fields.type == HT_PKT_OK) {
             if (identifyModel(brl, response.fields.data.ok.model)) {
+              makeOutputTable(dotsTable_ISO11548_1);
+
               if (model->hasATC) {
                 setAtcMode(brl, 1);
-                setAtcSensitivity(brl, 50);
 
                 touchAnalyzeCells(brl, NULL);
                 brl->touchEnabled = 1;
               }
 
+              if (setTime) {
+                if (model->identifier == HT_MODEL_ActiveBraille) {
+                  requestDateTime(brl, synchronizeDateTime);
+                } else {
+                  logMessage(LOG_INFO, "%s does not support setting the clock", model->name);
+                }
+              }
+
               return 1;
             }
 
@@ -1058,11 +1307,6 @@ brl_destruct (BrailleDisplay *brl) {
   }
   io->closePort();
 
-  if (at2Buffer) {
-    free(at2Buffer);
-    at2Buffer = NULL;
-  }
-
   deallocateBuffers();
 }
 
@@ -1074,18 +1318,19 @@ writeCells (BrailleDisplay *brl) {
 }
 
 static int
-writeStatusAndTextCells (BrailleDisplay *brl) {
+writeCells_statusAndText (BrailleDisplay *brl) {
   unsigned char buffer[1 + model->statusCells + model->textCells];
+  unsigned char *byte = buffer;
 
-  buffer[0] = 0X01;
-  memcpy(buffer+1, rawStatus, model->statusCells);
-  memcpy(buffer+model->statusCells+1, rawData, model->textCells);
+  *byte++ = 0X01;
+  byte = mempcpy(byte, rawStatus, model->statusCells);
+  byte = mempcpy(byte, rawData, model->textCells);
 
-  return brl_writePacket(brl, buffer, sizeof(buffer)) != -1;
+  return brl_writePacket(brl, buffer, byte-buffer) != -1;
 }
 
 static int
-writeBookwormCells (BrailleDisplay *brl) {
+writeCells_Bookworm (BrailleDisplay *brl) {
   unsigned char buffer[1 + model->statusCells + model->textCells + 1];
 
   buffer[0] = 0X01;
@@ -1095,7 +1340,7 @@ writeBookwormCells (BrailleDisplay *brl) {
 }
 
 static int
-writeEvolutionCells (BrailleDisplay *brl) {
+writeCells_Evolution (BrailleDisplay *brl) {
   return writeExtendedPacket(brl, HT_EXTPKT_Braille, rawData, model->textCells);
 }
 
@@ -1115,11 +1360,9 @@ updateCells (BrailleDisplay *brl) {
 
 static int
 brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
-  if (memcmp(brl->buffer, prevData, model->textCells) != 0) {
-    int i;
-    for (i=0; i<model->textCells; ++i) {
-      rawData[i] = outputTable[(prevData[i] = brl->buffer[i])];
-    }
+  const size_t cellCount = model->textCells;
+  if (cellsHaveChanged(prevData, brl->buffer, cellCount, NULL, NULL, NULL)) {
+    translateOutputCells(rawData, prevData, cellCount);
     updateRequired = 1;
   }
   updateCells(brl);
@@ -1128,19 +1371,16 @@ brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
 
 static int
 brl_writeStatus (BrailleDisplay *brl, const unsigned char *st) {
-  if (model->statusCells &&
-      (memcmp(st, prevStatus, model->statusCells) != 0)) {
-    int i;
-    for (i=0; i<model->statusCells; ++i) {
-      rawStatus[i] = outputTable[(prevStatus[i] = st[i])];
-    }
+  const size_t cellCount = model->statusCells;
+  if (cellsHaveChanged(prevStatus, st, cellCount, NULL, NULL, NULL)) {
+    translateOutputCells(rawStatus, prevStatus, cellCount);
     updateRequired = 1;
   }
   return 1;
 }
 
 static int
-interpretKeyByte (unsigned char byte) {
+interpretByte_key (unsigned char byte) {
   int release = (byte & HT_KEY_RELEASE) != 0;
   if (release) byte &= ~HT_KEY_RELEASE;
 
@@ -1162,7 +1402,7 @@ interpretKeyByte (unsigned char byte) {
 }
 
 static int
-interpretBookwormByte (unsigned char byte) {
+interpretByte_Bookworm (unsigned char byte) {
   static const unsigned char keys[] = {
     HT_BWK_Backward,
     HT_BWK_Forward,
@@ -1196,15 +1436,9 @@ interpretBookwormByte (unsigned char byte) {
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
   int noInput = 1;
 
-  if (at2Count) {
-    unsigned char code = at2Buffer[0];
-    memcpy(at2Buffer, at2Buffer+1, --at2Count);
-    return BRL_BLK_PASSAT + code;
-  }
-
   while (1) {
     HT_Packet packet;
     int size = brl_readPacket(brl, &packet, sizeof(packet));
@@ -1300,23 +1534,27 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
                     break;
 
                   case HT_EXTPKT_Scancode: {
-                    if (length) {
-                      unsigned char code = bytes++[0];
-                      if (--length) {
-                        int newCount = at2Count + length;
-                        if (newCount > at2Size) {
-                          int newSize = (newCount | 0XF) + 1;
-                          at2Buffer = reallocWrapper(at2Buffer, newSize);
-                          at2Size = newSize;
-                        }
-                        memcpy(at2Buffer+at2Count, bytes, length);
-                        at2Count = newCount;
-                      }
-                      return BRL_BLK_PASSAT + code;
+                    while (length) {
+                      enqueueCommand(BRL_BLK_PASSAT + bytes++[0]);
+                      length -= 1;
                     }
                     break;
                   }
 
+                  case HT_EXTPKT_GetRTC: {
+                    const HT_DateTime *const payload = (HT_DateTime *)bytes;
+                    DateTimeProcessor *processor = dateTimeProcessor;
+                    dateTimeProcessor = NULL;
+
+                    if (processor) {
+                      if (!processor(brl, payload)) {
+                        break;
+                      }
+                    }
+
+                    continue;
+                  }
+
                   case HT_EXTPKT_AtcInfo: {
                     unsigned int cellCount = model->textCells + model->statusCells;
                     unsigned char pressureValues[cellCount];
@@ -1331,10 +1569,10 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
                         unsigned char byte = bytes[dataIndex];
                         unsigned char nibble;
 
-                        nibble = byte & 0XF0;
+                        nibble = HIGH_NIBBLE(byte);
                         pressureValues[cellIndex++] = nibble | (nibble >> 4);
 
-                        nibble = byte & 0X0F;
+                        nibble = LOW_NIBBLE(byte);
                         pressureValues[cellIndex++] = nibble | (nibble << 4);
                       }
 
@@ -1351,6 +1589,30 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
                     continue;
                   }
 
+                  case HT_EXTPKT_ReadingPosition: {
+                    const size_t cellCount = model->textCells + model->statusCells;
+                    unsigned char pressureValues[cellCount];
+                    const unsigned char *pressure;
+
+                    if (bytes[0] != 0XFF) {
+                      const int cellIndex = bytes[0];
+
+                      memset(pressureValues, 0, cellCount);
+                      pressureValues[cellIndex] = 0XFF;
+
+                      pressure = &pressureValues[0];
+                    } else {
+                      pressure = NULL;
+                    }
+
+                    {
+                      int command = touchAnalyzePressure(brl, pressure);
+                      if (command != EOF) return command;
+                    }
+
+                    continue;
+                  }
+
                   default:
                     break;
                 }
@@ -1370,7 +1632,7 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
     }
 
     logUnexpectedPacket(packet.bytes, size);
-    LogPrint(LOG_WARNING, "state %d", currentState);
+    logMessage(LOG_WARNING, "state %d", currentState);
   }
 
   if (noInput) {
@@ -1393,11 +1655,3 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
 
   return EOF;
 }
-
-static void
-brl_sensitivity (BrailleDisplay *brl, BrailleSensitivity setting) {
-  if (model->hasATC) {
-    unsigned char value = 0XFF - (setting * 0XF0 / BRL_SENSITIVITY_MAXIMUM);
-    setAtcSensitivity(brl, value);
-  }
-}
