@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -41,27 +41,9 @@ typedef enum {
 
 #include "brl_driver.h"
 
-static TranslationTable outputTable;
-static TranslationTable inputTable;
-
 static int
 brl_construct(BrailleDisplay *brl, char **parameters, const char *device)
 {
-  {
-    static const DotsTable dots = {
-      BRAILLE(1, 0, 0, 0, 0, 0, 0, 0),
-      BRAILLE(0, 1, 0, 0, 0, 0, 0, 0),
-      BRAILLE(0, 0, 1, 0, 0, 0, 0, 0),
-      BRAILLE(0, 0, 0, 1, 0, 0, 0, 0),
-      BRAILLE(0, 0, 0, 0, 1, 0, 0, 0),
-      BRAILLE(0, 0, 0, 0, 0, 1, 0, 0),
-      BRAILLE(0, 0, 0, 0, 0, 0, 1, 0),
-      BRAILLE(0, 0, 0, 0, 0, 0, 0, 1)
-    };
-    makeOutputTable(dots, outputTable);
-    reverseTranslationTable(outputTable, inputTable);
-  }
-  
   if(*parameters[PARM_DEVICE])
     braille_config(BRL_DEVICE, parameters[PARM_DEVICE]);
 
@@ -73,41 +55,56 @@ brl_construct(BrailleDisplay *brl, char **parameters, const char *device)
 
   if(braille_init())
     {
-      LogPrint(LOG_INFO, "Libbraille Version: %s", braille_info(BRL_VERSION));
+      logMessage(LOG_INFO, "Libbraille Version: %s", braille_info(BRL_VERSION));
 
 #ifdef BRL_PATH
-      LogPrint(LOG_DEBUG, "Libbraille Installation Directory: %s", braille_info(BRL_PATH));
+      logMessage(LOG_DEBUG, "Libbraille Installation Directory: %s", braille_info(BRL_PATH));
 #endif /* BRL_PATH */
 
 #ifdef BRL_PATHCONF
-      LogPrint(LOG_DEBUG, "Libbraille Configuration Directory: %s", braille_info(BRL_PATHCONF));
+      logMessage(LOG_DEBUG, "Libbraille Configuration Directory: %s", braille_info(BRL_PATHCONF));
 #endif /* BRL_PATHCONF */
 
 #ifdef BRL_PATHTBL
-      LogPrint(LOG_DEBUG, "Libbraille Tables Directory: %s", braille_info(BRL_PATHTBL));
+      logMessage(LOG_DEBUG, "Libbraille Tables Directory: %s", braille_info(BRL_PATHTBL));
 #endif /* BRL_PATHTBL */
 
 #ifdef BRL_PATHDRV
-      LogPrint(LOG_DEBUG, "Libbraille Drivers Directory: %s", braille_info(BRL_PATHDRV));
+      logMessage(LOG_DEBUG, "Libbraille Drivers Directory: %s", braille_info(BRL_PATHDRV));
 #endif /* BRL_PATHDRV */
 
-      LogPrint(LOG_INFO, "Libbraille Table: %s", braille_info(BRL_TABLE));
-      LogPrint(LOG_INFO, "Libbraille Driver: %s", braille_info(BRL_DRIVER));
-      LogPrint(LOG_INFO, "Libbraille Device: %s", braille_info(BRL_DEVICE));
+      logMessage(LOG_INFO, "Libbraille Table: %s", braille_info(BRL_TABLE));
+      logMessage(LOG_INFO, "Libbraille Driver: %s", braille_info(BRL_DRIVER));
+      logMessage(LOG_INFO, "Libbraille Device: %s", braille_info(BRL_DEVICE));
 
-      LogPrint(LOG_INFO, "Display Type: %s", braille_info(BRL_TERMINAL));
-      LogPrint(LOG_INFO, "Display Size: %d", braille_size());
+      logMessage(LOG_INFO, "Display Type: %s", braille_info(BRL_TERMINAL));
+      logMessage(LOG_INFO, "Display Size: %d", braille_size());
 
       brl->textColumns = braille_size();  /* initialise size of display */
       brl->textRows = 1;
 
+      {
+        static const DotsTable dots = {
+          BRAILLE(1, 0, 0, 0, 0, 0, 0, 0),
+          BRAILLE(0, 1, 0, 0, 0, 0, 0, 0),
+          BRAILLE(0, 0, 1, 0, 0, 0, 0, 0),
+          BRAILLE(0, 0, 0, 1, 0, 0, 0, 0),
+          BRAILLE(0, 0, 0, 0, 1, 0, 0, 0),
+          BRAILLE(0, 0, 0, 0, 0, 1, 0, 0),
+          BRAILLE(0, 0, 0, 0, 0, 0, 1, 0),
+          BRAILLE(0, 0, 0, 0, 0, 0, 0, 1)
+        };
+        makeOutputTable(dots);
+        makeInputTable();
+      }
+  
       braille_timeout(100);
 
       return 1;
     }
   else
     {
-      LogPrint(LOG_DEBUG, "Libbraille initialization erorr: %s", braille_geterror());
+      logMessage(LOG_DEBUG, "Libbraille initialization erorr: %s", braille_geterror());
     }
   
   return 0;
@@ -136,7 +133,7 @@ brl_writeWindow(BrailleDisplay *brl, const wchar_t *text)
 
       if(brl->cursor >= 0)
         {
-          braille_filter(outputTable[cursorDots()], brl->cursor);
+          braille_filter(translateOutputCell(cursorDots()), brl->cursor);
         }
 
       braille_render();
@@ -146,7 +143,7 @@ brl_writeWindow(BrailleDisplay *brl, const wchar_t *text)
 }
 
 static int
-brl_readCommand(BrailleDisplay *brl, BRL_DriverCommandContext context)
+brl_readCommand(BrailleDisplay *brl, KeyTableCommandContext context)
 {
   int res = EOF;
   signed char status;
@@ -155,7 +152,7 @@ brl_readCommand(BrailleDisplay *brl, BRL_DriverCommandContext context)
   status = braille_read(&key);
   if(status == -1)
     {
-      LogPrint(LOG_ERR, "error in braille_read: %s", braille_geterror());
+      logMessage(LOG_ERR, "error in braille_read: %s", braille_geterror());
       res = BRL_CMD_RESTARTBRL;
     }
   else if(status)
@@ -214,7 +211,7 @@ brl_readCommand(BrailleDisplay *brl, BRL_DriverCommandContext context)
 	    }
 	  break;
 	case BRL_KEY:
-	  res = BRL_BLK_PASSDOTS | inputTable[key.braille];
+	  res = BRL_BLK_PASSDOTS | translateInputCell(key.braille);
 	  break;
 	default:
           break;
