@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -61,6 +61,7 @@
 #include <string.h>
 
 #include "log.h"
+#include "ascii.h"
 
 #define BRL_STATUS_FIELDS sfWindowCoordinates
 #define BRL_HAVE_STATUS_CELLS
@@ -76,9 +77,6 @@
 #define EXPECTEDNRSTATCELLS 2
 
 /* protocol packets */
-#define SOH 0x01
-#define STX 0x02
-#define ETX 0x03
 #define CKSUM_MASK 0xAA55
 
 /* Packet is SOH STX <code> <len> ETX <data...> <CKSUM_LOW-CKSUM_HIGH> */
@@ -142,9 +140,6 @@ put_cksum(unsigned char *buf)
 #define QUERYREPLY 37
 /* len 6, see bellow */
 
-/* This defines the mapping between brltty and MDV's dots pattern coding. */
-static TranslationTable outputTable;
-
 /* Key codes */
 #define NRFKEYS 10 /* values 1-10 */
 /* Shift and Long modifiers for all F-keys, (except LONG-F10 and
@@ -216,15 +211,15 @@ receive_rest(unsigned char *packet)
   if(myread(packet+1, PACKET_HDR_LEN-1) != PACKET_HDR_LEN-1) return 0;
   /* Check for STX and ETX */
   if(packet[1] != packet_hdr[1] || packet[4] != packet_hdr[4]){
-    LogPrint(LOG_DEBUG,"Invalid packet: STX %02x, ETX %02x",
-	     packet[1],packet[4]);
+    logMessage(LOG_DEBUG,"Invalid packet: STX %02x, ETX %02x",
+	       packet[1],packet[4]);
     return 0;
   }
   len = packet[OFF_LEN];
   if(len > MAXPACKETLEN) return 0;
   if(myread(packet+PACKET_HDR_LEN, len+NRCKSUMBYTES)
      != len+NRCKSUMBYTES){
-    LogPrint(LOG_DEBUG,"receive_rest(): short read count");
+    logMessage(LOG_DEBUG,"receive_rest(): short read count");
     return 0;
   }
 #if 0
@@ -236,15 +231,15 @@ receive_rest(unsigned char *packet)
     sprintf(hexbuf, "%02x ", packet[i+PACKET_HDR_LEN]);
     strcat(msgbuf, hexbuf);
   }
-  LogPrint(LOG_DEBUG,"Received packet: code %u, body %s",
-	   packet[OFF_CODE],msgbuf);
+  logMessage(LOG_DEBUG,"Received packet: code %u, body %s",
+	     packet[OFF_CODE],msgbuf);
 }
 #endif /* 0 */
   /* Verify checksum */
   cksum = calc_cksum(packet);
   if(packet[PACKET_HDR_LEN+len] != (unsigned char)(cksum & 0xFF)
      || packet[PACKET_HDR_LEN+len+1] != (unsigned char)(cksum >> 8)){
-    LogPrint(LOG_DEBUG,"Checksum invalid");
+    logMessage(LOG_DEBUG,"Checksum invalid");
     return 0;
   }
   return 1;
@@ -293,11 +288,6 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
 {
   int hasrouting, dotspercell, version1, version2;
 
-  {
-    static const DotsTable dots = {0X08, 0X04, 0X02, 0X80, 0X40, 0X20, 0X01, 0X10};
-    makeOutputTable(dots, outputTable);
-  }
-
   if (!isSerialDevice(&device)) {
     unsupportedDevice(device);
     return 0;
@@ -330,16 +320,16 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
   if(serialWriteData(serialDevice, sendpacket, PACKET_HDR_LEN+NRCKSUMBYTES)
      != PACKET_HDR_LEN+NRCKSUMBYTES)
     goto failure;
-  serialDrainOutput(serialDevice);
+  serialAwaitOutput(serialDevice);
   while(1){
     if(!expect_receive_packet(recvpacket))
       goto failure;
     if(memcmp(recvpacket, query_reply_packet_hdr, PACKET_HDR_LEN) == 0)
       break;
     if(recvpacket[OFF_CODE] == ACK)
-      LogPrint(LOG_DEBUG,"Skipping probable ACK packet");
+      logMessage(LOG_DEBUG,"Skipping probable ACK packet");
     else
-      LogPrint(LOG_DEBUG,"Skipping invalid response to query");
+      logMessage(LOG_DEBUG,"Skipping invalid response to query");
   }
 
   brl_cols = recvpacket[OFF_NRCONTENTCELLS];
@@ -349,24 +339,24 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
   version1 = recvpacket[OFF_PRIMARYVERSION];
   version2 = recvpacket[OFF_SECONDARYVERSION];
 
-  LogPrint(LOG_INFO,"Display replyed: %d cells, %d status cells, "
-	   "%d dots per cell, has routing keys flag %d, version %d.%d",
-	   brl_cols, nrstatcells, dotspercell, hasrouting, version1,version2);
+  logMessage(LOG_INFO,"Display replied: %d cells, %d status cells, "
+	     "%d dots per cell, has routing keys flag %d, version %d.%d",
+	     brl_cols, nrstatcells, dotspercell, hasrouting, version1,version2);
 
   if(brl_cols > MAXNRCONTENTCELLS || brl_cols < 1){
-    LogPrint(LOG_ERR, "Invalid number of cells: %d", brl_cols);
+    logMessage(LOG_ERR, "Invalid number of cells: %d", brl_cols);
     goto failure;
   }
   if(nrstatcells != EXPECTEDNRSTATCELLS)
-    LogPrint(LOG_NOTICE, "Unexpected number of status cells: %d", nrstatcells);
+    logMessage(LOG_NOTICE, "Unexpected number of status cells: %d", nrstatcells);
   if(nrstatcells < 0){
-    LogPrint(LOG_ERR, "Invalid number of status cells: %d", nrstatcells);
+    logMessage(LOG_ERR, "Invalid number of status cells: %d", nrstatcells);
     goto failure;
   }
   if(brl_cols + nrstatcells > MAXPACKETLEN){
     /* This is to make sure we don't overflow the sendpacket when sending
        braille to be displayed. */
-    LogPrint(LOG_ERR, "Invalid total number of cells");
+    logMessage(LOG_ERR, "Invalid total number of cells");
     goto failure;
   }
 
@@ -375,6 +365,13 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device)
   brl->statusColumns = nrstatcells;
   brl->statusRows = 1;
 
+  {
+    static const DotsTable dots = {
+      0X08, 0X04, 0X02, 0X80, 0X40, 0X20, 0X01, 0X10
+    };
+    makeOutputTable(dots);
+  }
+
   /* Allocate space for buffers */
   /* For portability we want to avoid declarations such as char array[v]
      where v is not a compilation constant. BRLTTY must work as reliably as
@@ -438,36 +435,32 @@ brl_writeStatus (BrailleDisplay *brl, const unsigned char *s)
 static int 
 brl_writeWindow (BrailleDisplay *brl, const wchar_t *text)
 {
-  int i;
+  unsigned int textChanged = cellsHaveChanged(prevdata, brl->buffer, brl_cols, NULL, NULL, NULL);
+  unsigned int statusChanged = cellsHaveChanged(prevstatbuf, statbuf, nrstatcells, NULL, NULL, NULL);
   unsigned char *p;
 
-  if(memcmp(prevdata, brl->buffer, brl_cols) == 0
-     && memcmp(prevstatbuf, statbuf, nrstatcells) == 0)
+  if(!(textChanged || statusChanged))
     return 1;
-  memcpy(prevdata, brl->buffer, brl_cols);
-  memcpy(prevstatbuf, statbuf, nrstatcells);
 
   sendpacket[OFF_CODE] = FULLREFRESH;
   sendpacket[OFF_LEN] = nrstatcells+brl_cols;
   p = sendpacket+PACKET_HDR_LEN;
-  for(i=0; i<nrstatcells; i++)
-    *(p++) = outputTable[statbuf[i]];
-  for(i=0; i < brl_cols; i++)
-     *(p++) = outputTable[brl->buffer[i]];
+  p = translateOutputCells(p, statbuf, nrstatcells);
+  p = translateOutputCells(p, brl->buffer, brl_cols);
   put_cksum(sendpacket);
 
   serialWriteData(serialDevice, sendpacket, PACKET_HDR_LEN+nrstatcells+brl_cols+NRCKSUMBYTES);
-  serialDrainOutput(serialDevice);
+  serialAwaitOutput(serialDevice);
 
   if(expect_receive_packet(recvpacket)){
     if(memcmp(recvpacket, ackpacket, ACKPACKETLEN) == 0)
       return 1;
     else{
       packet_to_process = 1;
-      LogPrint(LOG_DEBUG, "After sending update, received code %d packet",
-	       recvpacket[OFF_CODE]);
+      logMessage(LOG_DEBUG, "After sending update, received code %d packet",
+	         recvpacket[OFF_CODE]);
     }
-  }else LogPrint(LOG_DEBUG, "No ACK after update");
+  }else logMessage(LOG_DEBUG, "No ACK after update");
   /* We'll do resends like this for now */
   memset(prevdata, 0xFF, brl_cols);
   memset(prevstatbuf, 0, nrstatcells);
@@ -477,7 +470,7 @@ brl_writeWindow (BrailleDisplay *brl, const wchar_t *text)
 
 
 static int 
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context)
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context)
 {
   static char
     ignore_next_release = 0; /* when a command is triggered by combining
@@ -509,7 +502,7 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context)
   /* Note that we discard ACK packets */
   /* All these packets should have a length of 1 */
   if(recvpacket[OFF_LEN] != 1){
-    LogPrint(LOG_NOTICE,"Received key code 0x%x with length %d",
+    logMessage(LOG_NOTICE,"Received key code 0x%x with length %d",
 	       code, recvpacket[OFF_LEN]);
     return EOF;
   }
@@ -521,14 +514,14 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context)
       /* SHIFT_PRESS and SHIFT_RELEASE are exceptions to the following */
       int key = keycode & KEY_MASK;
       int modifier = keycode & MODIFIER_MASK;
-      LogPrint(LOG_DEBUG,"Received key code 0x%x with modifier 0x%x",
-	       key, modifier);
+      logMessage(LOG_DEBUG,"Received key code 0x%x with modifier 0x%x",
+	         key, modifier);
       if(nr_routing_cur_pressed > 0){
 	ignore_next_release = 1;
 	if(howmanykeys == 1 && modifier == 0){
 	  switch(key){
-	    case RG: cmd = BRL_BLK_CUTBEGIN + which_routing_keys[0]; break;
-	    case LF: cmd = BRL_BLK_CUTRECT + which_routing_keys[0]; break;
+	    case RG: cmd = BRL_BLK_CLIP_NEW + which_routing_keys[0]; break;
+	    case LF: cmd = BRL_BLK_COPY_RECT + which_routing_keys[0]; break;
 	  };
 	}
 	if(cmd == EOF){
@@ -593,9 +586,9 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context)
     case REPORTROUTINGKEYPRESS:
     case REPORTROUTINGKEYRELEASE: {
       int whichkey = recvpacket[PACKET_HDR_LEN];
-      LogPrint(LOG_DEBUG,"Received routing key %s for key %d",
-	       ((code == REPORTROUTINGKEYPRESS) ? "press" : "release"),
-	       whichkey);
+      logMessage(LOG_DEBUG,"Received routing key %s for key %d",
+	         ((code == REPORTROUTINGKEYPRESS) ? "press" : "release"),
+	         whichkey);
       if(whichkey < 1 || whichkey > brl_cols+nrstatcells) return EOF;
       if(whichkey <= nrstatcells){
 	/* special handling for routing keys over status cells: currently
@@ -634,10 +627,10 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context)
 	  cmd = BRL_BLK_ROUTE + which_routing_keys[0];
 	else if (howmanykeys == 3 && which_routing_keys[1] == brl_cols-2
 		 && which_routing_keys[2] == brl_cols-1)
-	  cmd = BRL_BLK_CUTBEGIN + which_routing_keys[0];
+	  cmd = BRL_BLK_CLIP_NEW + which_routing_keys[0];
 	else if (howmanykeys == 3 && which_routing_keys[0] == 0
 		 && which_routing_keys[1] == 1)
-	  cmd = BRL_BLK_CUTRECT + which_routing_keys[2];
+	  cmd = BRL_BLK_COPY_RECT + which_routing_keys[2];
 	else if ((howmanykeys == 4 && which_routing_keys[0] == 0
 		  && which_routing_keys[1] == 1
 		  && which_routing_keys[2] == brl_cols-2
@@ -650,8 +643,8 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context)
 	  cmd = BRL_CMD_HELP;
 	else if(howmanykeys == 3
 		&& which_routing_keys[0]+2 == which_routing_keys[1]){
-	  cmd = BRL_BLK_CUTBEGIN + which_routing_keys[0];
-	  pending_cmd = BRL_BLK_CUTRECT + which_routing_keys[2];
+	  cmd = BRL_BLK_CLIP_NEW + which_routing_keys[0];
+	  pending_cmd = BRL_BLK_COPY_RECT + which_routing_keys[2];
 	}
 	/* Reset to no keys pressed */
 	memset(routing_were_pressed, 0, brl_cols);
