@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -35,6 +35,7 @@
 
 #include "log.h"
 #include "timing.h"
+#include "ascii.h"
 #include "message.h"
 
 #define BRL_STATUS_FIELDS sfCursorAndWindowColumn, sfCursorAndWindowRow, sfStateDots
@@ -43,11 +44,8 @@
 
 #include "io_serial.h"
 static SerialDevice *serialDevice = NULL;
-static const int serialBaud = 9600;
-static int serialCharactersPerSecond;
-
-#define CR  0X0D
-#define ESC 0X1B
+static const unsigned int serialBaud = 9600;
+static unsigned int serialCharactersPerSecond;
 
 #define KEY_F1     0x01
 #define KEY_F2     0x02
@@ -59,7 +57,6 @@ static int serialCharactersPerSecond;
 
 #define POST_COMMAND_DELAY 30
 
-static TranslationTable outputTable;
 static unsigned char textCells[20];
 static unsigned char statusCells[2];
 static int refreshNeeded;
@@ -69,7 +66,7 @@ writeData (BrailleDisplay *brl, const unsigned char *bytes, int count) {
   ssize_t result = serialWriteData(serialDevice, bytes, count);
 
   if (result == -1) {
-    LogError("write");
+    logSystemError("write");
     return 0;
   }
 
@@ -85,24 +82,18 @@ writeCells  (BrailleDisplay *brl) {
 
   unsigned char buffer[sizeof(beginSequence) + sizeof(statusCells) + sizeof(textCells) + sizeof(endSequence)];
   unsigned char *byte = buffer;
-  int i;
 
-  memcpy(byte, beginSequence, sizeof(beginSequence));
-  byte += sizeof(beginSequence);
-
-  for (i=0; i<sizeof(statusCells); ++i) *byte++ = outputTable[statusCells[i]];
-  for (i=0; i<sizeof(textCells); ++i) *byte++ = outputTable[textCells[i]];
-
-  memcpy(byte, endSequence, sizeof(endSequence));
-  byte += sizeof(endSequence);
+  byte = mempcpy(byte, beginSequence, sizeof(beginSequence));
+  byte = translateOutputCells(byte, statusCells, sizeof(statusCells));
+  byte = translateOutputCells(byte, textCells, sizeof(textCells));
+  byte = mempcpy(byte, endSequence, sizeof(endSequence));
 
   return writeData(brl, buffer, byte-buffer);
 }
 
 static void
 updateCells (unsigned char *target, const unsigned char *source, size_t count) {
-  if (memcmp(target, source, count) != 0) {
-    memcpy(target, source, count);
+  if (cellsHaveChanged(target, source, count, NULL, NULL, NULL)) {
     refreshNeeded = 1;
   }
 }
@@ -195,11 +186,11 @@ struct InputModeStruct {
 static const InputMode inputMode_char_f1 = {
   BIND_BLOCK(F1, SETLEFT),
   BIND_BLOCK(F2, DESCCHAR),
-  BIND_BLOCK(Left, CUTAPPEND),
-  BIND_BLOCK(Up, CUTBEGIN),
+  BIND_BLOCK(Left, CLIP_ADD),
+  BIND_BLOCK(Up, CLIP_NEW),
   BIND_BLOCK(Center, ROUTE),
-  BIND_BLOCK(Down, CUTRECT),
-  BIND_BLOCK(Right, CUTLINE),
+  BIND_BLOCK(Down, COPY_RECT),
+  BIND_BLOCK(Right, COPY_LINE),
 
   .temporary = 1,
   .name = "Char-F1"
@@ -327,7 +318,7 @@ static const InputMode inputMode_basic = {
 };
 
 static const InputMode *inputMode;
-static struct timeval inputTime;
+static TimeValue inputTime;
 
 static void
 setInputMode (const InputMode *mode) {
@@ -338,7 +329,7 @@ setInputMode (const InputMode *mode) {
   }
 
   inputMode = mode;
-  gettimeofday(&inputTime, NULL);
+  getCurrentTime(&inputTime);
 }
 
 static void
@@ -348,11 +339,6 @@ resetInputMode (void) {
 
 static int
 brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
-  {
-    static const DotsTable dots = {0X01, 0X02, 0X04, 0X80, 0X40, 0X20, 0X08, 0X10};
-    makeOutputTable(dots, outputTable);
-  }
-
   if (!isSerialDevice(&device)) {
     unsupportedDevice(device);
     return 0;
@@ -369,6 +355,13 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
       serialSetBaud(serialDevice, 38400);
       */
 
+      {
+        static const DotsTable dots = {
+          0X01, 0X02, 0X04, 0X80, 0X40, 0X20, 0X08, 0X10
+        };
+        makeOutputTable(dots);
+      }
+
       clearCells(textCells,  sizeof(textCells));
       clearCells(statusCells,  sizeof(statusCells));
       resetInputMode();
@@ -418,7 +411,7 @@ brl_writeStatus (BrailleDisplay *brl, const unsigned char *s) {
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
   unsigned char byte;
   const InputMode *mode;
   const InputBinding *binding;
@@ -435,7 +428,7 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
     }
 
     if (result == -1) {
-      LogError("read");
+      logSystemError("read");
       return BRL_CMD_RESTARTBRL;
     }
   }
@@ -453,14 +446,14 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
     case KEY_CENTER: binding = &mode->keyCenter; break;
 
     default:
-      LogPrint(LOG_WARNING, "unhandled key: %s -> %02X", mode->name, byte);
+      logMessage(LOG_WARNING, "unhandled key: %s -> %02X", mode->name, byte);
       beep(brl);
       return EOF;
   }
 
   switch (binding->type) {
     case IBT_unbound:
-      LogPrint(LOG_WARNING, "unbound key: %s -> %02X", mode->name, byte);
+      logMessage(LOG_WARNING, "unbound key: %s -> %02X", mode->name, byte);
       beep(brl);
       break;
 
@@ -479,7 +472,7 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
     }
 
     default:
-      LogPrint(LOG_WARNING, "unhandled input binding type: %02X", binding->type);
+      logMessage(LOG_WARNING, "unhandled input binding type: %02X", binding->type);
       break;
   }
 
