@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -59,7 +59,7 @@
 
 #include "log.h"
 #include "timing.h"
-#include "misc.h"
+#include "ascii.h"
 
 #define BRL_STATUS_FIELDS sfCursorAndWindowColumn, sfCursorAndWindowRow, sfStateDots
 #define BRL_HAVE_STATUS_CELLS
@@ -68,10 +68,6 @@
 #include "tables.h"		/* for keybindings */
 #include "io_serial.h"
 
-#define ESC '\033'
-#define CR '\015'
-
-static TranslationTable outputTable;	/* dot mapping table (output) */
 SerialDevice *MB_serialDevice;			/* file descriptor for Braille display */
 static int brlcols;		/* length of braille display (auto-detected) */
 static unsigned char *prevdata;	/* previously received data */
@@ -100,11 +96,6 @@ static int brl_construct (BrailleDisplay *brl, char **parameters, const char *de
 	unsigned char *init_ack = (unsigned char *)"\002\033V";	/* string to expect as acknowledgement: [ESC][V]... */
 	unsigned char c;
 
-	{
-		static const DotsTable dots = {0X01, 0X02, 0X04, 0X80, 0X40, 0X20, 0X08, 0X10};
-		makeOutputTable(dots, outputTable);
-	}
-
 	if (!isSerialDevice(&device)) {
 		unsupportedDevice(device);
 		return 0;
@@ -120,6 +111,7 @@ static int brl_construct (BrailleDisplay *brl, char **parameters, const char *de
 	/* Now open the Braille display device for random access */
 	if (!(MB_serialDevice = serialOpenDevice(device))) goto failure;
 	if (!serialRestartDevice(MB_serialDevice, BAUDRATE)) goto failure;
+	if (!serialSetFlowControl(MB_serialDevice, SERIAL_FLOW_HARDWARE)) goto failure;
 
 	/* MultiBraille initialisation procedure:
 	 * [ESC][V][Braillelength][Software Version][CR]
@@ -144,7 +136,7 @@ static int brl_construct (BrailleDisplay *brl, char **parameters, const char *de
 			/* reading version-info */
 			/* firmware version == [Software Version] / 10.0 */
 			serialReadData (MB_serialDevice, &c, 1, 0, 0);
-			LogPrint (LOG_INFO, "MultiBraille: Version: %2.1f", c/10.0);
+			logMessage (LOG_INFO, "MultiBraille: Version: %2.1f", c/10.0);
 
 			/* read trailing [CR] */
 			serialReadData (MB_serialDevice, &c, 1, 0, 0);
@@ -153,30 +145,39 @@ static int brl_construct (BrailleDisplay *brl, char **parameters, const char *de
 	}
 	while (!hasTimedOut (ACK_TIMEOUT) && n <= init_ack[0]);
 
-	if (!success) goto failure;
-	if (!serialSetFlowControl(MB_serialDevice, SERIAL_FLOW_HARDWARE)) goto failure;
+	if (success && (brlcols != 25)) {
+          if ((prevdata = malloc(brl->textColumns * brl->textRows))) {
+            if ((rawdata = malloc(20 + (brl->textColumns * brl->textRows * 2)))) {
+              brl->textColumns = brlcols;
+              brl->textRows = BRLROWS;
 
-	if (brlcols == 25) goto failure;						/* MultiBraille Vertical uses a different protocol --> not supported */
-	brl->textColumns = brlcols;
-	brl->textRows = BRLROWS;
-	brl->statusColumns = 5;
-	brl->statusRows = 1;
+              brl->statusColumns = 5;
+              brl->statusRows = 1;
 
-	/* Allocate space for buffers */
-	prevdata = mallocWrapper (brl->textColumns * brl->textRows);
-	/* rawdata has to have room for the pre- and post-data sequences,
-	 * the status cells, and escaped 0x1b's: */
-	rawdata = mallocWrapper (20 + brl->textColumns * brl->textRows * 2);
+              {
+                static const DotsTable dots = {
+                  0X01, 0X02, 0X04, 0X80, 0X40, 0X20, 0X08, 0X10
+                };
 
-	return 1;
+                makeOutputTable(dots);
+              }
 
-failure:
-	if (prevdata)
-		free (prevdata);
-	if (rawdata)
-		free (rawdata);
-	if (MB_serialDevice)
-		serialCloseDevice (MB_serialDevice);
+              return 1;
+            } else {
+              logMallocError();
+            }
+
+            free(prevdata);
+          } else {
+            logMallocError();
+          }
+        }
+
+      failure:
+	if (MB_serialDevice) {
+           serialCloseDevice(MB_serialDevice);
+           MB_serialDevice = NULL;
+        }
 	return 0;
 }
 
@@ -215,26 +216,23 @@ static void brl_destruct (BrailleDisplay *brl) {
 
 
 static int brl_writeStatus (BrailleDisplay *brl, const unsigned char *s) {
-	short i;
-
 	/* Dot mapping: */
-	for (i = 0; i < 5; status[i] = outputTable[s[i]], i++);
+	translateOutputCells(status, s, 5);
 	return 1;
 }
 
 
 static int brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
+	int textChanged = cellsHaveChanged(prevdata, brl->buffer, brl->textColumns*brl->textRows, NULL, NULL, NULL);
+	int statusChanged = cellsHaveChanged(oldstatus, status, 5, NULL, NULL, NULL);
 	short i;			/* loop counter */
 	unsigned char *pre_data = (unsigned char *)"\002\033Z";	/* bytewise accessible copies */
 	unsigned char *post_data = (unsigned char *)"\001\015";
 
 	/* Only refresh display if the data has changed: */
-	if (memcmp (brl->buffer, prevdata, brl->textColumns * brl->textRows) || memcmp (status, oldstatus, 5)) {
-		/* Save new Braille data: */
-		memcpy (prevdata, brl->buffer, brl->textColumns * brl->textRows);
-
+	if (textChanged || statusChanged) {
 		/* Dot mapping from standard to MultiBraille: */
-		for (i = 0; i < brl->textColumns * brl->textRows; brl->buffer[i] = outputTable[brl->buffer[i]], i++);
+		translateOutputCells(brl->buffer, brl->buffer, brl->textColumns*brl->textRows);
 
     rawlen = 0;
 		if (pre_data[0]) {
@@ -270,7 +268,7 @@ static int brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
 }
 
 
-static int brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
+static int brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
 	static short status = 0;	/* cursor routing keys mode */
 	
 	KeyStroke keystroke;
@@ -288,7 +286,7 @@ static int brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext contex
 		else
 			keystroke.key = cmd_S_trans[keystroke.key];
 		status = 0;
-                if ((keystroke.key == BRL_BLK_CUTLINE) || (keystroke.key == BRL_BLK_CUTRECT))
+                if ((keystroke.key == BRL_BLK_COPY_LINE) || (keystroke.key == BRL_BLK_COPY_RECT))
                   keystroke.key += brlcols - 1;
 		return keystroke.key;
 	} else { /* directly process 'R' events */
@@ -308,10 +306,10 @@ static int brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext contex
 				return keystroke.key + BRL_BLK_ROUTE - MB_CR_EXTRAKEYS;
 			case 1:			/* begin block */
 				status = 0;
-				return keystroke.key + BRL_BLK_CUTBEGIN - MB_CR_EXTRAKEYS;
+				return keystroke.key + BRL_BLK_CLIP_NEW - MB_CR_EXTRAKEYS;
 			case 2:			/* end block */
 				status = 0;
-				return keystroke.key + BRL_BLK_CUTRECT - MB_CR_EXTRAKEYS;
+				return keystroke.key + BRL_BLK_COPY_RECT - MB_CR_EXTRAKEYS;
 		}
 		status = 0;
 	}
@@ -340,14 +338,14 @@ static struct KeyStroke getbrlkey (void) {
 				serialReadData (MB_serialDevice, &c, 1, 0, 0);		/* read keynumber */
 				keystroke.key = c;
 				serialReadData (MB_serialDevice, &c, 1, 0, 0);		/* read trailing [CR] */
-  				/* LogPrint(LOG_NOTICE, "MultiBraille.o: Receiving: Key=%d, Block=%c", keystroke.key, keystroke.block); */
+  				/* logMessage(LOG_NOTICE, "MultiBraille.o: Receiving: Key=%d, Block=%c", keystroke.key, keystroke.block); */
 				return keystroke;
 			default:			/* not supported command --> ignore */
 				c_temp = c;
 				keystroke.block = EOF;	/* invalid / not supported keystroke */
 				serialReadData (MB_serialDevice, &c, 1, 0, 0);		/* read keynumber */
   				keystroke.key = 0;
-  				/* LogPrint(LOG_NOTICE, "MultiBraille.o: Ignored: Key=%d, Block=%c", keystroke.key, c_temp); */
+  				logMessage(LOG_NOTICE, "MultiBraille.o: Ignored: Key=%d, Block=%c", keystroke.key, c_temp);
 				return keystroke;
 		}
 	}
