@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -50,7 +50,8 @@ static iconv_t conversionDescriptor = NULL;
 #define getch() my_getch()
 #endif /* HAVE_PKG_CURSES */
 
-#include "misc.h"
+#include "log.h"
+#include "parse.h"
 #include "charset.h"
 
 #ifdef USE_CURSES
@@ -109,7 +110,7 @@ my_getch (void) {
 
 static int
 brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
-  int ttyBaud = 9600;
+  unsigned int ttyBaud = 9600;
   char *ttyType = "vt100";
   int windowLines = 1;
   int windowColumns = 40;
@@ -124,7 +125,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
   }
 
   {
-    int baud = ttyBaud;
+    unsigned int baud = ttyBaud;
     if (serialValidateBaud(&baud, "TTY baud", parameters[PARM_BAUD], NULL))
       ttyBaud = baud;
   }
@@ -141,7 +142,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
     if (validateInteger(&lines, parameters[PARM_LINES], &minimum, &maximum)) {
       windowLines = lines;
     } else {
-      LogPrint(LOG_WARNING, "%s: %s", "invalid line count", parameters[PARM_LINES]);
+      logMessage(LOG_WARNING, "%s: %s", "invalid line count", parameters[PARM_LINES]);
     }
   }
 
@@ -152,7 +153,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
     if (validateInteger(&columns, parameters[PARM_COLUMNS], &minimum, &maximum)) {
       windowColumns = columns;
     } else {
-      LogPrint(LOG_WARNING, "%s: %s", "invalid column count", parameters[PARM_COLUMNS]);
+      logMessage(LOG_WARNING, "%s: %s", "invalid column count", parameters[PARM_COLUMNS]);
     }
   }
 
@@ -187,12 +188,12 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
             brl->textColumns = windowColumns;
             brl->textRows = windowLines; 
 
-            LogPrint(LOG_INFO, "TTY: type=%s baud=%d size=%dx%d",
-                     ttyType, ttyBaud, windowColumns, windowLines);
+            logMessage(LOG_INFO, "TTY: type=%s baud=%u size=%dx%d",
+                       ttyType, ttyBaud, windowColumns, windowLines);
             return 1;
 #ifdef USE_CURSES
           } else {
-            LogError("newterm");
+            logSystemError("newterm");
           }
 
           ttyStream = NULL;
@@ -207,7 +208,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
 #ifdef HAVE_ICONV_H
     iconv_close(conversionDescriptor);
   } else {
-    LogError("iconv_open");
+    logSystemError("iconv_open");
   }
   conversionDescriptor = NULL;
 #endif /* HAVE_ICONV_H */
@@ -267,10 +268,11 @@ brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
   static int previousCursor = -1;
   char *previousLocale;
 
-  if ((memcmp(previousContent, brl->buffer, brl->textColumns*brl->textRows) == 0) &&
-      (brl->cursor == previousCursor))
+  if (!cellsHaveChanged(previousContent, brl->buffer, brl->textColumns*brl->textRows, NULL, NULL, NULL) &&
+      (brl->cursor == previousCursor)) {
     return 1;
-  memcpy(previousContent, brl->buffer, brl->textColumns*brl->textRows);
+  }
+
   previousCursor = brl->cursor;
 
   if (classificationLocale) {
@@ -315,13 +317,13 @@ brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
 }
 
 int
-brl_keyToCommand (BrailleDisplay *brl, BRL_DriverCommandContext context, int key) {
+brl_keyToCommand (BrailleDisplay *brl, KeyTableCommandContext context, int key) {
 #define KEY(key,cmd) case (key): return (cmd)
   switch (key) {
     KEY(EOF, EOF);
     default:
       if (key <= 0XFF) return BRL_BLK_PASSCHAR|key;
-      LogPrint(LOG_WARNING, "Unknown key: %d", key);
+      logMessage(LOG_WARNING, "Unknown key: %d", key);
       return BRL_CMD_NOOP;
 
 #ifdef USE_CURSES
@@ -330,12 +332,17 @@ brl_keyToCommand (BrailleDisplay *brl, BRL_DriverCommandContext context, int key
     KEY(KEY_UP, BRL_CMD_LNUP);
     KEY(KEY_DOWN, BRL_CMD_LNDN);
 
-    KEY(KEY_PPAGE, BRL_CMD_TOP);
-    KEY(KEY_NPAGE, BRL_CMD_BOT);
     KEY(KEY_HOME, BRL_CMD_TOP_LEFT);
+    KEY(KEY_A1, BRL_CMD_TOP_LEFT);
     KEY(KEY_END, BRL_CMD_BOT_LEFT);
+    KEY(KEY_C1, BRL_CMD_BOT_LEFT);
     KEY(KEY_IC, BRL_CMD_HOME);
+    KEY(KEY_B2, BRL_CMD_HOME);
     KEY(KEY_DC, BRL_CMD_CSRTRK);
+    KEY(KEY_PPAGE, BRL_CMD_TOP);
+    KEY(KEY_A3, BRL_CMD_TOP);
+    KEY(KEY_NPAGE, BRL_CMD_BOT);
+    KEY(KEY_C3, BRL_CMD_BOT);
 
     KEY(KEY_F(1), BRL_CMD_HELP);
     KEY(KEY_F(2), BRL_CMD_LEARN);
@@ -371,13 +378,13 @@ brl_readKey (BrailleDisplay *brl) {
   if (key == ERR) return EOF;
 #endif /* USE_CURSES */
 
-  LogPrint(LOG_DEBUG, "key %d", key);
+  logMessage(LOG_DEBUG, "key %d", key);
   return key;
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
   int command = brl_keyToCommand(brl, context, brl_readKey(brl));
-  if (command != EOF) LogPrint(LOG_DEBUG, "cmd %04X", command);
+  if (command != EOF) logMessage(LOG_DEBUG, "cmd %04X", command);
   return command;
 }
