@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -34,12 +34,6 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
-
-#ifdef HAVE_SYS_SELECT_H
-#include <sys/select.h>
-#else /* HAVE_SYS_SELECT_H */
-#include <sys/time.h>
-#endif /* HAVE_SYS_SELECT_H */
 #endif /* __MINGW32__ */
 
 #if !defined(AF_LOCAL) && defined(AF_UNIX)
@@ -56,16 +50,16 @@
 
 #ifdef __MINGW32__
 #define close(fd) CloseHandle((HANDLE)(fd))
-#define LogSocketError(msg) LogWindowsSocketError(msg)
+#define LogSocketError(msg) logWindowsSocketError(msg)
 #else /* __MINGW32__ */
-#define LogSocketError(msg) LogError(msg)
+#define LogSocketError(msg) logSystemError(msg)
 #endif /* __MINGW32__ */
 
 #include "log.h"
+#include "io_misc.h"
 #include "device.h"
 #include "parse.h"
 #include "timing.h"
-#include "misc.h"
 #include "charset.h"
 #include "cmd.h"
 
@@ -106,7 +100,7 @@ static int statusColumns;
 static int statusRows;
 static int statusCount;
 static unsigned char *statusCells = NULL;
-static unsigned char genericCells[BRL_genericStatusCellCount];
+static unsigned char genericCells[GSC_COUNT];
 
 typedef struct {
 #ifdef AF_LOCAL
@@ -128,29 +122,10 @@ static const OperationsEntry *operations;
 
 static int
 readSocket (int descriptor, void *buffer, int size) {
-  fd_set readMask;
-  struct timeval timeout;
-
-  FD_ZERO(&readMask);
-  FD_SET(fileDescriptor, &readMask);
-
-  memset(&timeout, 0, sizeof(timeout));
-
-  switch (select(fileDescriptor+1, &readMask, NULL, NULL, &timeout)) {
-    case -1:
-      LogSocketError("select");
-      break;
-
-    case 0:
-      errno = EAGAIN;
-      break;
-  
-    default: {
-      int count = recv(descriptor, buffer, size, 0);
-      if (count != -1) return count;
-      LogSocketError("recv");
-      break;
-    }
+  if (awaitInput(descriptor, 0)) {
+    int count = recv(descriptor, buffer, size, 0);
+    if (count != -1) return count;
+    LogSocketError("recv");
   }
 
   return -1;
@@ -162,11 +137,15 @@ static const OperationsEntry socketOperationsEntry = {
 
 static char *
 formatSocketAddress (const struct sockaddr *address) {
+  char *string;
+
   switch (address->sa_family) {
 #ifdef AF_LOCAL
     case AF_LOCAL: {
       const struct sockaddr_un *localAddress = (const struct sockaddr_un *)address;
-      return strdupWrapper(localAddress->sun_path);
+
+      string = strdup(localAddress->sun_path);
+      break;
     }
 #endif /* AF_LOCAL */
 
@@ -175,13 +154,19 @@ formatSocketAddress (const struct sockaddr *address) {
       const char *host = inet_ntoa(inetAddress->sin_addr);
       unsigned short port = ntohs(inetAddress->sin_port);
       char buffer[strlen(host) + 7];
+
       snprintf(buffer, sizeof(buffer), "%s:%u", host, port);
-      return strdupWrapper(buffer);
+      string = strdup(buffer);
+      break;
     }
 
     default:
-      return strdupWrapper("");
+      string = strdup("");
+      break;
   }
+
+  if (!string) logMallocError();
+  return string;
 }
 
 static int
@@ -203,8 +188,11 @@ acceptSocketConnection (
 
           {
             char *address = formatSocketAddress(localAddress);
-            LogPrint(LOG_NOTICE, "listening on: %s", address);
-            free(address);
+
+            if (address) {
+              logMessage(LOG_NOTICE, "listening on: %s", address);
+              free(address);
+            }
           }
 
           while (1) {
@@ -225,7 +213,7 @@ acceptSocketConnection (
                 break;
 
               case 0:
-                LogPrint(LOG_DEBUG, "no connection yet, still waiting (%d).", attempts);
+                logMessage(LOG_DEBUG, "no connection yet, still waiting (%d).", attempts);
                 continue;
 
               default: {
@@ -233,8 +221,11 @@ acceptSocketConnection (
 
                 if ((serverSocket = accept(queueSocket, remoteAddress, remoteSize)) != -1) {
                   char *address = formatSocketAddress(remoteAddress);
-                  LogPrint(LOG_NOTICE, "client is: %s", address);
-                  free(address);
+
+                  if (address) {
+                    logMessage(LOG_NOTICE, "client is: %s", address);
+                    free(address);
+                  }
                 } else {
                   LogSocketError("accept");
                 }
@@ -270,22 +261,28 @@ requestConnection (
 
   {
     char *address = formatSocketAddress(remoteAddress);
-    LogPrint(LOG_DEBUG, "connecting to: %s", address);
-    free(address);
+
+    if (address) {
+      logMessage(LOG_DEBUG, "connecting to: %s", address);
+      free(address);
+    }
   }
 
   if ((clientSocket = getSocket()) != -1) {
     if (connect(clientSocket, remoteAddress, remoteSize) != -1) {
       {
         char *address = formatSocketAddress(remoteAddress);
-        LogPrint(LOG_NOTICE, "connected to: %s", address);
-        free(address);
+
+        if (address) {
+          logMessage(LOG_NOTICE, "connected to: %s", address);
+          free(address);
+        }
       }
 
       operations = &socketOperationsEntry;
       return clientSocket;
     } else {
-      LogPrint(LOG_WARNING, "connect error: %s", strerror(errno));
+      logMessage(LOG_WARNING, "connect error: %s", strerror(errno));
     }
 
     close(clientSocket);
@@ -320,7 +317,7 @@ setLocalAddress (const char *string, struct sockaddr_un *address) {
     strncpy(address->sun_path, string, sizeof(address->sun_path)-1);
   } else {
     ok = 0;
-    LogPrint(LOG_WARNING, "Local socket path too long: %s", string);
+    logMessage(LOG_WARNING, "Local socket path too long: %s", string);
   }
 
   return ok;
@@ -335,7 +332,7 @@ static void
 unbindLocalAddress (const struct sockaddr *address) {
   const struct sockaddr_un *localAddress = (const struct sockaddr_un *)address;
   if (unlink(localAddress->sun_path) == -1) {
-    LogError("unlink");
+    logSystemError("unlink");
   }
 }
 
@@ -363,7 +360,7 @@ readNamedPipe (int descriptor, void *buffer, int size) {
     DWORD available;
 
     if (!PeekNamedPipe((HANDLE)descriptor, NULL, 0, NULL, &available, NULL)) {
-      LogWindowsError("PeekNamedPipe");
+      logWindowsSystemError("PeekNamedPipe");
       return 0;
     }
 
@@ -382,10 +379,10 @@ readNamedPipe (int descriptor, void *buffer, int size) {
 
     if (!ReadFile((HANDLE)descriptor, buffer, size, &received, &overl)) {
       if (GetLastError() != ERROR_IO_PENDING) {
-        LogWindowsError("ReadPipe");
+        logWindowsSystemError("ReadPipe");
         received = 0;
       } else if (!GetOverlappedResult((HANDLE)descriptor, &overl, &received, TRUE)) {
-        LogWindowsError("GetOverlappedResult");
+        logWindowsSystemError("GetOverlappedResult");
         received = 0;
       }
     }
@@ -410,7 +407,7 @@ acceptNamedPipeConnection (const char *path) {
                                 PIPE_ACCESS_DUPLEX | FILE_FLAG_OVERLAPPED,
                                 PIPE_TYPE_BYTE | PIPE_READMODE_BYTE,
                                 1, 0, 0, 0, NULL)) == INVALID_HANDLE_VALUE) {
-    LogWindowsError("CreateNamedPipe");
+    logWindowsSystemError("CreateNamedPipe");
     return -1;
   }
 
@@ -421,9 +418,9 @@ acceptNamedPipeConnection (const char *path) {
         while ((res = WaitForSingleObject(overl.hEvent, 10000)) != WAIT_OBJECT_0) {
           if (res == WAIT_TIMEOUT) {
             ++attempts;
-            LogPrint(LOG_DEBUG, "no connection yet, still waiting (%d).", attempts);
+            logMessage(LOG_DEBUG, "no connection yet, still waiting (%d).", attempts);
           } else {
-            LogWindowsError("ConnectNamedPipe");
+            logWindowsSystemError("ConnectNamedPipe");
             CloseHandle(h);
             h = (HANDLE) -1;
             break;
@@ -434,7 +431,7 @@ acceptNamedPipeConnection (const char *path) {
         break;
 
       default:
-        LogWindowsError("ConnectNamedPipe");
+        logWindowsSystemError("ConnectNamedPipe");
         CloseHandle(h);
         h = (HANDLE) -1;
         break;
@@ -454,7 +451,7 @@ requestNamedPipeConnection (const char *path) {
                       GENERIC_READ|GENERIC_WRITE,
                       FILE_SHARE_READ|FILE_SHARE_WRITE,
                       NULL, OPEN_EXISTING, 0, NULL)) == INVALID_HANDLE_VALUE) {
-    LogWindowsError("Connect to named pipe");
+    logWindowsSystemError("Connect to named pipe");
     return -1;
   }
 
@@ -466,53 +463,62 @@ requestNamedPipeConnection (const char *path) {
 static int
 setInetAddress (const char *string, struct sockaddr_in *address) {
   int ok = 1;
-  char *hostName = strdupWrapper(string);
-  char *portNumber = strchr(hostName, ':');
+  char *hostName = strdup(string);
 
-  if (portNumber) {
-    *portNumber++ = 0;
-  } else {
-    portNumber = "";
-  }
+  if (hostName) {
+    char *portNumber = strchr(hostName, ':');
 
-  memset(address, 0, sizeof(*address));
-  address->sin_family = AF_INET;
-
-  if (*hostName) {
-    const struct hostent *host = gethostbyname(hostName);
-    if (host && (host->h_addrtype == AF_INET) && (host->h_length == sizeof(address->sin_addr))) {
-      memcpy(&address->sin_addr, host->h_addr, sizeof(address->sin_addr));
+    if (portNumber) {
+      *portNumber++ = 0;
     } else {
-      ok = 0;
-      LogPrint(LOG_WARNING, "Unknown host name: %s", hostName);
+      portNumber = "";
     }
-  } else {
-    address->sin_addr.s_addr = INADDR_ANY;
-  }
 
-  if (*portNumber) {
-    int port;
-    if (isInteger(&port, portNumber)) {
-      if ((port > 0) && (port <= 0XFFFF)) {
-        address->sin_port = htons(port);
+    memset(address, 0, sizeof(*address));
+    address->sin_family = AF_INET;
+
+    if (*hostName) {
+      const struct hostent *host = gethostbyname(hostName);
+      if (host && (host->h_addrtype == AF_INET) && (host->h_length == sizeof(address->sin_addr))) {
+        memcpy(&address->sin_addr, host->h_addr, sizeof(address->sin_addr));
       } else {
         ok = 0;
-        LogPrint(LOG_WARNING, "Invalid port number: %s", portNumber);
+        logMessage(LOG_WARNING, "Unknown host name: %s", hostName);
       }
     } else {
-      const struct servent *service = getservbyname(portNumber, "tcp");
-      if (service) {
-        address->sin_port = service->s_port;
-      } else {
-        ok = 0;
-        LogPrint(LOG_WARNING, "Unknown service: %s", portNumber);
-      }
+      address->sin_addr.s_addr = INADDR_ANY;
     }
+
+    if (*portNumber) {
+      int port;
+
+      if (isInteger(&port, portNumber)) {
+        if ((port > 0) && (port <= 0XFFFF)) {
+          address->sin_port = htons(port);
+        } else {
+          ok = 0;
+          logMessage(LOG_WARNING, "Invalid port number: %s", portNumber);
+        }
+      } else {
+        const struct servent *service = getservbyname(portNumber, "tcp");
+
+        if (service) {
+          address->sin_port = service->s_port;
+        } else {
+          ok = 0;
+          logMessage(LOG_WARNING, "Unknown service: %s", portNumber);
+        }
+      }
+    } else {
+      address->sin_port = htons(VR_DEFAULT_PORT);
+    }
+
+    free(hostName);
   } else {
-    address->sin_port = htons(VR_DEFAULT_PORT);
+    ok = 0;
+    logMallocError();
   }
 
-  free(hostName);
   return ok;
 }
 
@@ -545,9 +551,15 @@ requestInetConnection (const struct sockaddr_in *remoteAddress) {
 
 static char *
 makeString (const char *characters, int count) {
-  char *string = mallocWrapper(count+1);
-  memcpy(string, characters, count);
-  string[count] = 0;
+  char *string = malloc(count+1);
+
+  if (string) {
+    memcpy(string, characters, count);
+    string[count] = 0;
+  } else {
+    logMallocError();
+  }
+
   return string;
 }
 
@@ -576,14 +588,17 @@ readCommandLine (void) {
   if (fillInputBuffer()) {
     if (inputStart < inputLength) {
       const char *newline = memchr(&inputBuffer[inputStart], '\n', inputLength-inputStart);
+
       if (newline) {
         char *string;
         int stringLength = newline - inputBuffer;
         inputCarriageReturn = 0;
+
         if ((newline != inputBuffer) && (*(newline-1) == '\r')) {
           inputCarriageReturn = 1;
-          --stringLength;
+          stringLength -= 1;
         }
+
         string = makeString(inputBuffer, stringLength);
         inputLength -= ++newline - inputBuffer;
         memmove(inputBuffer, newline, inputLength);
@@ -594,6 +609,7 @@ readCommandLine (void) {
       }
     } else if (inputEnd) {
       char *string;
+
       if (inputLength) {
         string = makeString(inputBuffer, inputLength);
         inputLength = 0;
@@ -601,9 +617,11 @@ readCommandLine (void) {
       } else {
         string = copyString("quit");
       }
+
       return string;
     }
   }
+
   return NULL;
 }
 
@@ -779,11 +797,16 @@ sortCommandsByName (void) {
   sortCommands(compareCommandNames);
 }
 
-static void
+static int
 allocateCommandDescriptors (void) {
   if (!commandDescriptors) {
     commandCount = getCommandCount();
-    commandDescriptors = mallocWrapper(commandCount * commandSize);
+    commandDescriptors = malloc(commandCount * commandSize);
+
+    if (!commandDescriptors) {
+      logMallocError();
+      return 0;
+    }
 
     {
       CommandDescriptor *descriptor = commandDescriptors;
@@ -815,6 +838,8 @@ allocateCommandDescriptors (void) {
 
     sortCommandsByName();
   }
+
+  return 1;
 }
 
 static void
@@ -861,26 +886,26 @@ dimensionsChanged (BrailleDisplay *brl) {
               if ((word = nextWord())) {
                 if (isInteger(&rows2, word) && (rows2 > 0)) {
                 } else {
-                  LogPrint(LOG_WARNING, "invalid status row count: %s", word);
+                  logMessage(LOG_WARNING, "invalid status row count: %s", word);
                   ok = 0;
                 }
               }
             } else {
-              LogPrint(LOG_WARNING, "invalid status column count: %s", word);
+              logMessage(LOG_WARNING, "invalid status column count: %s", word);
               ok = 0;
             }
           }
         } else {
-          LogPrint(LOG_WARNING, "invalid text row count: %s", word);
+          logMessage(LOG_WARNING, "invalid text row count: %s", word);
           ok = 0;
         }
       }
     } else {
-      LogPrint(LOG_WARNING, "invalid text column count: %s", word);
+      logMessage(LOG_WARNING, "invalid text column count: %s", word);
       ok = 0;
     }
   } else {
-    LogPrint(LOG_WARNING, "missing text column count");
+    logMessage(LOG_WARNING, "missing text column count");
   }
 
   if (ok) {
@@ -912,7 +937,7 @@ dimensionsChanged (BrailleDisplay *brl) {
           if (statusCells) free(statusCells);
           statusCells = status;
           memset(statusCells, 0, count2);
-          memset(genericCells, 0, BRL_genericStatusCellCount);
+          memset(genericCells, 0, GSC_COUNT);
 
           brl->textColumns = brailleColumns;
           brl->textRows = brailleRows;
@@ -933,7 +958,7 @@ dimensionsChanged (BrailleDisplay *brl) {
 
 static int
 brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
-  allocateCommandDescriptors();
+  if (!allocateCommandDescriptors()) return 0;
 
   inputLength = 0;
   inputStart = 0;
@@ -987,7 +1012,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
   } else {
     static WSADATA wsadata;
     if (WSAStartup(MAKEWORD(1, 1), &wsadata)) {
-      LogWindowsError("socket library start");
+      logWindowsSystemError("socket library start");
       goto failed;
     }
   }
@@ -1007,7 +1032,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
       if (line) free(line);
       if ((line = readCommandLine())) {
         const char *word;
-        LogPrint(LOG_DEBUG, "command received: %s", line);
+        logMessage(LOG_DEBUG, "command received: %s", line);
 
         if ((word = strtok(line, inputDelimiters))) {
           if (testWord(word, "cells")) {
@@ -1018,7 +1043,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
           } else if (testWord(word, "quit")) {
             break;
           } else {
-            LogPrint(LOG_WARNING, "unexpected command: %s", word);
+            logMessage(LOG_WARNING, "unexpected command: %s", word);
           }
         }
       } else {
@@ -1090,13 +1115,11 @@ brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
     }
   }
 
-  if (memcmp(brl->buffer, brailleCells, brailleCount) != 0) {
+  if (cellsHaveChanged(brailleCells, brl->buffer, brailleCount, NULL, NULL, NULL)) {
     writeString("Braille \"");
     writeDots(brl->buffer, brailleCount);
     writeString("\"");
     writeLine();
-
-    memcpy(brailleCells, brl->buffer, brailleCount);
   }
 
   return 1;
@@ -1104,52 +1127,52 @@ brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
 
 static int
 brl_writeStatus (BrailleDisplay *brl, const unsigned char *status) {
-  int generic = status[BRL_firstStatusCell] == BRL_STATUS_CELLS_GENERIC;
+  int generic = status[GSC_FIRST] == GSC_MARKER;
   unsigned char *cells;
   int count;
 
   if (generic) {
     cells = genericCells;
-    count = BRL_genericStatusCellCount;
+    count = GSC_COUNT;
   } else {
     cells = statusCells;
     count = statusCount;
   }
 
-  if (memcmp(status, cells, count) != 0) {
+  if (cellsHaveChanged(cells, status, count, NULL, NULL, NULL)) {
     if (generic) {
-      int all = cells[BRL_firstStatusCell] != BRL_STATUS_CELLS_GENERIC;
+      int all = cells[GSC_FIRST] != GSC_MARKER;
       int i;
 
       for (i=1; i<count; ++i) {
         unsigned char value = status[i];
         if (all || (value != cells[i])) {
           static const char *const names[] = {
-            [BRL_firstStatusCell] = NULL,
-            [BRL_GSC_BRLCOL] = "BRLCOL",
-            [BRL_GSC_BRLROW] = "BRLROW",
-            [BRL_GSC_CSRCOL] = "CSRCOL",
-            [BRL_GSC_CSRROW] = "CSRROW",
-            [BRL_GSC_SCRNUM] = "SCRNUM",
-            [BRL_GSC_FREEZE] = "FREEZE",
-            [BRL_GSC_DISPMD] = "DISPMD",
-            [BRL_GSC_SIXDOTS] = "SIXDOTS",
-            [BRL_GSC_SLIDEWIN] = "SLIDEWIN",
-            [BRL_GSC_SKPIDLNS] = "SKPIDLNS",
-            [BRL_GSC_SKPBLNKWINS] = "SKPBLNKWINS",
-            [BRL_GSC_CSRVIS] = "CSRVIS",
-            [BRL_GSC_CSRHIDE] = "CSRHIDE",
-            [BRL_GSC_CSRTRK] = "CSRTRK",
-            [BRL_GSC_CSRSIZE] = "CSRSIZE",
-            [BRL_GSC_CSRBLINK] = "CSRBLINK",
-            [BRL_GSC_ATTRVIS] = "ATTRVIS",
-            [BRL_GSC_ATTRBLINK] = "ATTRBLINK",
-            [BRL_GSC_CAPBLINK] = "CAPBLINK",
-            [BRL_GSC_TUNES] = "TUNES",
-            [BRL_GSC_HELP] = "HELP",
-            [BRL_GSC_INFO] = "INFO",
-            [BRL_GSC_AUTOREPEAT] = "AUTOREPEAT",
-            [BRL_GSC_AUTOSPEAK] = "AUTOSPEAK"
+            [GSC_FIRST] = NULL,
+            [gscWindowColumn] = "BRLCOL",
+            [gscWindowRow] = "BRLROW",
+            [gscCursorColumn] = "CSRCOL",
+            [gscCursorRow] = "CSRROW",
+            [gscScreenNumber] = "SCRNUM",
+            [gscFrozenScreen] = "FREEZE",
+            [gscDisplayMode] = "DISPMD",
+            [gscTextStyle] = "SIXDOTS",
+            [gscSlidingWindow] = "SLIDEWIN",
+            [gscSkipIdenticalLines] = "SKPIDLNS",
+            [gscSkipBlankWindows] = "SKPBLNKWINS",
+            [gscShowCursor] = "CSRVIS",
+            [gscHideCursor] = "CSRHIDE",
+            [gscTrackCursor] = "CSRTRK",
+            [gscCursorStyle] = "CSRSIZE",
+            [gscBlinkingCursor] = "CSRBLINK",
+            [gscShowAttributes] = "ATTRVIS",
+            [gscBlinkingAttributes] = "ATTRBLINK",
+            [gscBlinkingCapitals] = "CAPBLINK",
+            [gscAlertTunes] = "TUNES",
+            [gscHelpScreen] = "HELP",
+            [gscInfoMode] = "INFO",
+            [gscAutorepeat] = "AUTOREPEAT",
+            [gscAutospeak] = "AUTOSPEAK"
           };
           const int nameCount = ARRAY_COUNT(names);
 
@@ -1170,21 +1193,19 @@ brl_writeStatus (BrailleDisplay *brl, const unsigned char *status) {
       writeString("\"");
       writeLine();
     }
-
-    memcpy(cells, status, count);
   }
 
   return 1;
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
+brl_readCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
   int command = EOF;
   char *line = readCommandLine();
 
   if (line) {
     const char *word;
-    LogPrint(LOG_DEBUG, "Command received: %s", line);
+    logMessage(LOG_DEBUG, "Command received: %s", line);
 
     if ((word = strtok(line, inputDelimiters))) {
       if (testWord(word, "cells")) {
@@ -1227,20 +1248,20 @@ brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
                   command += number;
                   continue;
                 } else {
-                  LogPrint(LOG_WARNING, "Number out of range.");
+                  logMessage(LOG_WARNING, "Number out of range.");
                 }
               }
             }
 
-            LogPrint(LOG_WARNING, "unknown option: %s", word);
+            logMessage(LOG_WARNING, "unknown option: %s", word);
           }
 
           if (needsNumber && !numberSpecified) {
-            LogPrint(LOG_WARNING, "Number not specified.");
+            logMessage(LOG_WARNING, "Number not specified.");
             command = EOF;
           }
         } else {
-          LogPrint(LOG_WARNING, "unknown command: %s", word);
+          logMessage(LOG_WARNING, "unknown command: %s", word);
         }
       }
     }
