@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -24,8 +24,12 @@
 #include <fcntl.h>
 #include <pthread.h>
 #include <limits.h>
-#include <langinfo.h>
 #include <locale.h>
+
+#ifdef HAVE_LANGINFO_H
+#include <langinfo.h>
+#endif /* HAVE_LANGINFO_H */
+
 #ifdef __MINGW32__
 #include "win_pthread.h"
 #else /* __MINGW32__ */
@@ -41,10 +45,11 @@
 #define SPI2_DBUS_INTERFACE_TREE	SPI2_DBUS_INTERFACE".Tree"
 #define SPI2_DBUS_INTERFACE_TEXT	SPI2_DBUS_INTERFACE".Text"
 #define SPI2_DBUS_INTERFACE_ACCESSIBLE	SPI2_DBUS_INTERFACE".Accessible"
-#define SPI2_DBUS_INTERFACE_PROP	"org.a11y.DBus.Properties"
+#define SPI2_DBUS_INTERFACE_PROP	"org.freedesktop.DBus.Properties"
 
+#ifdef HAVE_X11_KEYSYM_H
 #include <X11/keysym.h>
-#include <X11/Xlib.h>
+#endif /* HAVE_X11_KEYSYM_H */
 
 #include "log.h"
 #include "parse.h"
@@ -238,14 +243,14 @@ processParameters_AtSpi2Screen (char **parameters) {
         if (validateChoice(&choice, type, choices)) {
           int *flag = flags[choice];
           if ((flag == &typeAll) && (index > 0)) {
-            LogPrint(LOG_WARNING, "widget type is mutually exclusive: %s", type);
+            logMessage(LOG_WARNING, "widget type is mutually exclusive: %s", type);
           } else if (*flag || typeAll) {
-            LogPrint(LOG_WARNING, "widget type specified more than once: %s", type);
+            logMessage(LOG_WARNING, "widget type specified more than once: %s", type);
           } else {
             *flag = 1;
           }
         } else {
-          LogPrint(LOG_WARNING, "%s: %s", "invalid widget type", type);
+          logMessage(LOG_WARNING, "%s: %s", "invalid widget type", type);
         }
       }
     }
@@ -287,7 +292,7 @@ static void caretPosition(long caret) {
 }
 
 static void finiTerm(void) {
-  LogPrint(LOG_DEBUG,"end of term %s:%s",curSender,curPath);
+  logMessage(LOG_DEBUG,"end of term %s:%s",curSender,curPath);
   free(curSender);
   curSender = NULL;
   free(curPath);
@@ -309,12 +314,12 @@ static char *getRole(const char *sender, const char *path) {
   dbus_error_init(&error);
   msg = dbus_message_new_method_call(sender, path, SPI2_DBUS_INTERFACE_ACCESSIBLE, "GetRoleName");
   if (dbus_error_is_set(&error)) {
-    LogPrint(LOG_DEBUG, "error while making getrole message: %s %s", error.name, error.message);
+    logMessage(LOG_DEBUG, "error while making getrole message: %s %s", error.name, error.message);
     dbus_error_free(&error);
     return NULL;
   }
   if (!msg) {
-    LogPrint(LOG_DEBUG, "no memory while getting role");
+    logMessage(LOG_DEBUG, "no memory while getting role");
     return NULL;
   }
 
@@ -322,21 +327,21 @@ static char *getRole(const char *sender, const char *path) {
   /* 1s max delay */
   reply = dbus_connection_send_with_reply_and_block(bus, msg, 1000, &error);
   if (dbus_error_is_set(&error)) {
-    LogPrint(LOG_DEBUG, "error while getting role for %s:%s: %s %s", sender, path, error.name, error.message);
+    logMessage(LOG_DEBUG, "error while getting role for %s:%s: %s %s", sender, path, error.name, error.message);
     dbus_error_free(&error);
     goto outMsg;
   }
   if (!reply) {
-    LogPrint(LOG_DEBUG, "timeout while getting role");
+    logMessage(LOG_DEBUG, "timeout while getting role");
     goto outMsg;
   }
   if (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_ERROR) {
-    LogPrint(LOG_DEBUG, "error while getting role");
+    logMessage(LOG_DEBUG, "error while getting role");
     goto out;
   }
   dbus_message_iter_init(reply, &iter);
   if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_STRING) {
-    LogPrint(LOG_DEBUG, "GetRoleName didn't return a string but '%c'", dbus_message_iter_get_arg_type(&iter));
+    logMessage(LOG_DEBUG, "GetRoleName didn't return a string but '%c'", dbus_message_iter_get_arg_type(&iter));
     goto out;
   }
   dbus_message_iter_get_basic(&iter, &text);
@@ -362,12 +367,12 @@ static char *getText(const char *sender, const char *path) {
   dbus_error_init(&error);
   msg = dbus_message_new_method_call(sender, path, SPI2_DBUS_INTERFACE_TEXT, "GetText");
   if (dbus_error_is_set(&error)) {
-    LogPrint(LOG_DEBUG, "error while making gettext message: %s %s", error.name, error.message);
+    logMessage(LOG_DEBUG, "error while making gettext message: %s %s", error.name, error.message);
     dbus_error_free(&error);
     return NULL;
   }
   if (!msg) {
-    LogPrint(LOG_DEBUG, "no memory while getting text");
+    logMessage(LOG_DEBUG, "no memory while getting text");
     return NULL;
   }
   dbus_message_append_args(msg, DBUS_TYPE_INT32, &begin, DBUS_TYPE_INT32, &end, DBUS_TYPE_INVALID);
@@ -376,21 +381,21 @@ static char *getText(const char *sender, const char *path) {
   /* 1s max delay */
   reply = dbus_connection_send_with_reply_and_block(bus, msg, 1000, &error);
   if (dbus_error_is_set(&error)) {
-    LogPrint(LOG_DEBUG, "error while getting text for %s:%s: %s %s", sender, path, error.name, error.message);
+    logMessage(LOG_DEBUG, "error while getting text for %s:%s: %s %s", sender, path, error.name, error.message);
     dbus_error_free(&error);
     goto outMsg;
   }
   if (!reply) {
-    LogPrint(LOG_DEBUG, "timeout while getting text");
+    logMessage(LOG_DEBUG, "timeout while getting text");
     goto outMsg;
   }
   if (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_ERROR) {
-    LogPrint(LOG_DEBUG, "error while getting text");
+    logMessage(LOG_DEBUG, "error while getting text");
     goto out;
   }
   dbus_message_iter_init(reply, &iter);
   if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_STRING) {
-    LogPrint(LOG_DEBUG, "GetText didn't return a string but '%c'", dbus_message_iter_get_arg_type(&iter));
+    logMessage(LOG_DEBUG, "GetText didn't return a string but '%c'", dbus_message_iter_get_arg_type(&iter));
     goto out;
   }
   dbus_message_iter_get_basic(&iter, &text);
@@ -415,12 +420,12 @@ static dbus_int32_t getCaret(const char *sender, const char *path) {
   dbus_error_init(&error);
   msg = dbus_message_new_method_call(sender, path, SPI2_DBUS_INTERFACE_PROP, "Get");
   if (dbus_error_is_set(&error)) {
-    LogPrint(LOG_DEBUG, "error while making caret message: %s %s", error.name, error.message);
+    logMessage(LOG_DEBUG, "error while making caret message: %s %s", error.name, error.message);
     dbus_error_free(&error);
     return -1;
   }
   if (!msg) {
-    LogPrint(LOG_DEBUG, "no memory while making caret message");
+    logMessage(LOG_DEBUG, "no memory while making caret message");
     return -1;
   }
   dbus_message_append_args(msg, DBUS_TYPE_STRING, &interface, DBUS_TYPE_STRING, &property, DBUS_TYPE_INVALID);
@@ -429,30 +434,30 @@ static dbus_int32_t getCaret(const char *sender, const char *path) {
   /* 1s max delay */
   reply = dbus_connection_send_with_reply_and_block(bus, msg, 1000, &error);
   if (dbus_error_is_set(&error)) {
-    LogPrint(LOG_DEBUG, "error while getting caret: %s %s", error.name, error.message);
+    logMessage(LOG_DEBUG, "error while getting caret: %s %s", error.name, error.message);
     dbus_error_free(&error);
     goto outMsg;
   }
   if (!reply) {
-    LogPrint(LOG_DEBUG, "timeout while getting caret");
+    logMessage(LOG_DEBUG, "timeout while getting caret");
     goto outMsg;
   }
   if (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_ERROR) {
-    LogPrint(LOG_DEBUG, "error while getting caret");
+    logMessage(LOG_DEBUG, "error while getting caret");
     goto out;
   }
   dbus_message_iter_init(reply, &iter);
   if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_VARIANT) {
-    LogPrint(LOG_DEBUG, "getText didn't return a variant but '%c'", dbus_message_iter_get_arg_type(&iter));
+    logMessage(LOG_DEBUG, "getText didn't return a variant but '%c'", dbus_message_iter_get_arg_type(&iter));
     goto out;
   }
   dbus_message_iter_recurse(&iter, &iter_variant);
   if (dbus_message_iter_get_arg_type(&iter_variant) != DBUS_TYPE_INT32) {
-    LogPrint(LOG_DEBUG, "getText didn't return an int32 but '%c'", dbus_message_iter_get_arg_type(&iter_variant));
+    logMessage(LOG_DEBUG, "getText didn't return an int32 but '%c'", dbus_message_iter_get_arg_type(&iter_variant));
     goto out;
   }
   dbus_message_iter_get_basic(&iter_variant, &res);
-  LogPrint(LOG_DEBUG,"Got caret %d", res);
+  logMessage(LOG_DEBUG,"Got caret %d", res);
 
 out:
   dbus_message_unref(reply);
@@ -477,7 +482,7 @@ static void restartTerm(const char *sender, const char *path) {
 
   curSender = strdup(sender);
   curPath = strdup(path);
-  LogPrint(LOG_DEBUG,"new term %s:%s with text %s",curSender,curPath, text);
+  logMessage(LOG_DEBUG,"new term %s:%s with text %s",curSender,curPath, text);
 
   if (curRows) {
     for (i=0;i<curNumRows;i++)
@@ -493,7 +498,7 @@ static void restartTerm(const char *sender, const char *path) {
       break;
     c++;
   }
-  LogPrint(LOG_DEBUG,"%ld rows",curNumRows);
+  logMessage(LOG_DEBUG,"%ld rows",curNumRows);
   curRows = malloc(curNumRows * sizeof(*curRows));
   curRowLengths = malloc(curNumRows * sizeof(*curRowLengths));
   i = 0;
@@ -508,9 +513,9 @@ static void restartTerm(const char *sender, const char *path) {
       curNumCols = len;
     else if (len < 0) {
       if (len==-2)
-	LogPrint(LOG_ERR,"unterminated sequence %s",c);
+	logMessage(LOG_ERR,"unterminated sequence %s",c);
       else if (len==-1)
-	LogError("mbrlen");
+	logSystemError("mbrlen");
       curRowLengths[i] = (len = -1) + (d != NULL);
     }
     curRows[i] = malloc((len + (d!=NULL)) * sizeof(*curRows[i]));
@@ -522,7 +527,7 @@ static void restartTerm(const char *sender, const char *path) {
       break;
     i++;
   }
-  LogPrint(LOG_DEBUG,"%ld cols",curNumCols);
+  logMessage(LOG_DEBUG,"%ld cols",curNumCols);
   caretPosition(getCaret(sender, path));
   free(text);
 }
@@ -540,36 +545,38 @@ static void AtSpi2HandleEvent(const char *interface, DBusMessage *message)
 
   dbus_message_iter_init(message, &iter);
 
-  /* skip struct */
-  dbus_message_iter_next(&iter);
+  if (dbus_message_iter_get_arg_type(&iter) == DBUS_TYPE_STRUCT) {
+    /* skip struct */
+    dbus_message_iter_next(&iter);
+  }
 
   if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_STRING) {
-    LogPrint(LOG_DEBUG, "message detail not a string but '%c'", dbus_message_iter_get_arg_type(&iter));
+    logMessage(LOG_DEBUG, "message detail not a string but '%c'", dbus_message_iter_get_arg_type(&iter));
     return;
   }
   dbus_message_iter_get_basic(&iter, &detail);
 
   dbus_message_iter_next(&iter);
   if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_INT32) {
-    LogPrint(LOG_DEBUG, "message detail1 not an int32 but '%c'", dbus_message_iter_get_arg_type(&iter));
+    logMessage(LOG_DEBUG, "message detail1 not an int32 but '%c'", dbus_message_iter_get_arg_type(&iter));
     return;
   }
   dbus_message_iter_get_basic(&iter, &detail1);
 
   dbus_message_iter_next(&iter);
   if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_INT32) {
-    LogPrint(LOG_DEBUG, "message detail2 not an int32 but '%c'", dbus_message_iter_get_arg_type(&iter));
+    logMessage(LOG_DEBUG, "message detail2 not an int32 but '%c'", dbus_message_iter_get_arg_type(&iter));
     return;
   }
   dbus_message_iter_get_basic(&iter, &detail2);
 
   dbus_message_iter_next(&iter);
   if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_VARIANT) {
-    LogPrint(LOG_DEBUG, "message detail2 not a variant but '%c'", dbus_message_iter_get_arg_type(&iter));
+    logMessage(LOG_DEBUG, "message detail2 not a variant but '%c'", dbus_message_iter_get_arg_type(&iter));
     return;
   }
   dbus_message_iter_recurse(&iter, &iter_variant);
-  //LogPrint(LOG_DEBUG, "event %s %s %s %s %s %d %d", interface, member, sender, path, detail, detail1, detail2);
+  //logMessage(LOG_DEBUG, "event %s %s %s %s %s %d %d", interface, member, sender, path, detail, detail1, detail2);
 
   StateChanged_focused =
        !strcmp(interface, "Object")
@@ -581,7 +588,7 @@ static void AtSpi2HandleEvent(const char *interface, DBusMessage *message)
       finiTerm();
   } else if (!strcmp(interface,"Focus") || (StateChanged_focused && detail1)) {
     char *role = getRole(sender, path);
-    LogPrint(LOG_DEBUG, "state changed focused to role %s", role);
+    logMessage(LOG_DEBUG, "state changed focused to role %s", role);
     if (typeAll || (typeText && !strcmp(role, "text")) || (typeTerminal && !strcmp(role, "terminal"))) {
       restartTerm(sender, path);
     } else {
@@ -591,13 +598,13 @@ static void AtSpi2HandleEvent(const char *interface, DBusMessage *message)
     free(role);
   } else if (!strcmp(interface, "Object") && !strcmp(member, "TextCaretMoved")) {
     if (!curSender || strcmp(sender, curSender) || strcmp(path, curPath)) return;
-    LogPrint(LOG_DEBUG, "caret move to %d", detail1);
+    logMessage(LOG_DEBUG, "caret move to %d", detail1);
     caretPosition(detail1);
   } else if (!strcmp(interface, "Object") && !strcmp(member, "TextChanged") && !strcmp(detail, "delete")) {
     long x,y,toDelete = detail2;
     long length = 0, toCopy;
     long downTo; /* line that will provide what will follow x */
-    LogPrint(LOG_DEBUG,"delete %d from %d",detail2,detail1);
+    logMessage(LOG_DEBUG,"delete %d from %d",detail2,detail1);
     if (!curSender || strcmp(sender, curSender) || strcmp(path, curPath)) return;
     findPosition(detail1,&x,&y);
     downTo = y;
@@ -610,7 +617,7 @@ static void AtSpi2HandleEvent(const char *interface, DBusMessage *message)
       else {
 	/* imaginary extra line doesn't provide more length, and shouldn't need to ! */
 	if (x+toDelete > length) {
-	  LogPrint(LOG_ERR,"deleting past end of text !");
+	  logMessage(LOG_ERR,"deleting past end of text !");
 	  /* discarding */
 	  toDelete = length - x;
 	}
@@ -637,15 +644,16 @@ static void AtSpi2HandleEvent(const char *interface, DBusMessage *message)
       /* imaginary extra lines don't need to be deleted */
       downTo=curNumRows-1;
     delRows(y+1,downTo-y);
+    caretPosition(curCaret);
   } else if (!strcmp(interface, "Object") && !strcmp(member, "TextChanged") && !strcmp(detail, "insert")) {
     long len=detail2,semilen,x,y;
     const char *added;
     const char *adding,*c;
-    LogPrint(LOG_DEBUG,"insert %d from %d",detail2,detail1);
+    logMessage(LOG_DEBUG,"insert %d from %d",detail2,detail1);
     if (!curSender || strcmp(sender, curSender) || strcmp(path, curPath)) return;
     findPosition(detail1,&x,&y);
     if (dbus_message_iter_get_arg_type(&iter_variant) != DBUS_TYPE_STRING) {
-      LogPrint(LOG_DEBUG, "ergl, not string but '%c'", dbus_message_iter_get_arg_type(&iter_variant));
+      logMessage(LOG_DEBUG, "ergl, not string but '%c'", dbus_message_iter_get_arg_type(&iter_variant));
       return;
     }
     dbus_message_iter_get_basic(&iter_variant, &added);
@@ -699,8 +707,9 @@ static void AtSpi2HandleEvent(const char *interface, DBusMessage *message)
       if (curRowLengths[y]-(curRows[y][curRowLengths[y]-1]=='\n')>curNumCols)
 	curNumCols=curRowLengths[y]-(curRows[y][curRowLengths[y]-1]=='\n');
     }
+    caretPosition(curCaret);
   } else {
-      //LogPrint(LOG_DEBUG,"interface %s, member %s, detail %s, detail1 %d detail2 %d",interface, member, detail, detail1, detail2);
+      //logMessage(LOG_DEBUG,"interface %s, member %s, detail %s, detail1 %d detail2 %d",interface, member, detail, detail1, detail2);
   }
 }
 
@@ -713,9 +722,9 @@ static DBusHandlerResult AtSpi2Filter(DBusConnection *connection, DBusMessage *m
     if (!strncmp(interface, SPI2_DBUS_INTERFACE_EVENT".", strlen(SPI2_DBUS_INTERFACE_EVENT".")))
       AtSpi2HandleEvent(interface + strlen(SPI2_DBUS_INTERFACE_EVENT"."), message);
     else
-      LogPrint(LOG_DEBUG, "unknown signal %s %s", interface, member);
+      logMessage(LOG_DEBUG, "unknown signal %s %s", interface, member);
   } else
-    LogPrint(LOG_DEBUG, "unknown message %d %s %s", type, interface, member);
+    logMessage(LOG_DEBUG, "unknown message %d %s %s", type, interface, member);
   return DBUS_HANDLER_RESULT_HANDLED;
 }
 
@@ -723,19 +732,19 @@ static int finished;
 
 static void *doAtSpi2ScreenOpen(void *arg) {
   DBusError error;
-  int res = 0;
 
   sem_t *SPI2_init_sem = (sem_t *)arg;
 
   dbus_error_init(&error);
+  /* TODO: try to use the accessibility bus */
   bus = dbus_bus_get(DBUS_BUS_SESSION, &error);
   if (dbus_error_is_set(&error)) {
-    LogPrint(LOG_ERR, "Can't get dbus session bus: %s %s", error.name, error.message);
+    logMessage(LOG_ERR, "Can't get dbus session bus: %s %s", error.name, error.message);
     dbus_error_free(&error);
     goto out;
   }
   if (!bus) {
-    LogPrint(LOG_ERR, "Can't get dbus session bus.");
+    logMessage(LOG_ERR, "Can't get dbus session bus.");
     goto out;
   }
   if (!dbus_connection_add_filter(bus, AtSpi2Filter, NULL, NULL)) {
@@ -745,18 +754,16 @@ static void *doAtSpi2ScreenOpen(void *arg) {
   dbus_error_init(&error); \
   dbus_bus_add_match(bus, str, &error); \
   if (dbus_error_is_set(&error)) { \
-    LogPrint(LOG_ERR, "error while adding watch %s: %s %s", str, error.name, error.message); \
+    logMessage(LOG_ERR, "error while adding watch %s: %s %s", str, error.name, error.message); \
     dbus_error_free(&error); \
     goto out; \
   }
   WATCH("type='method_call',interface='"SPI2_DBUS_INTERFACE_TREE"'");
-  WATCH("type='signal',interface='org.a11y.atspi.Event.Focus'");
-  WATCH("type='signal',interface='org.a11y.atspi.Event.Object'");
-  WATCH("type='signal',interface='org.a11y.atspi.Event.Object',member='TextChanged'");
-  WATCH("type='signal',interface='org.a11y.atspi.Event.Object',member='TextCaretMoved'");
-  WATCH("type='signal',interface='org.a11y.atspi.Event.Object',member='StateChanged'");
-
-  res = 1;
+  WATCH("type='signal',interface='"SPI2_DBUS_INTERFACE_EVENT".Focus'");
+  WATCH("type='signal',interface='"SPI2_DBUS_INTERFACE_EVENT".Object'");
+  WATCH("type='signal',interface='"SPI2_DBUS_INTERFACE_EVENT".Object',member='TextChanged'");
+  WATCH("type='signal',interface='"SPI2_DBUS_INTERFACE_EVENT".Object',member='TextCaretMoved'");
+  WATCH("type='signal',interface='"SPI2_DBUS_INTERFACE_EVENT".Object',member='StateChanged'");
 
   /* TODO: use dbus_watch_get_unix_fd() or dbus_watch_get_socket() instead */
   sem_post(SPI2_init_sem);
@@ -780,17 +787,17 @@ construct_AtSpi2Screen (void) {
   sem_init(&SPI2_init_sem,0,0);
   finished = 0;
   if (pthread_create(&SPI2_main_thread,NULL,doAtSpi2ScreenOpen,(void *)&SPI2_init_sem)) {
-    LogPrint(LOG_ERR,"main SPI2 thread failed to be launched");
+    logMessage(LOG_ERR,"main SPI2 thread failed to be launched");
     return 0;
   }
   do {
     errno = 0;
   } while (sem_wait(&SPI2_init_sem) == -1 && errno == EINTR);
   if (errno) {
-    LogError("SPI2 initialization wait failed");
+    logSystemError("SPI2 initialization wait failed");
     return 0;
   }
-  LogPrint(LOG_DEBUG,"SPI2 initialized");
+  logMessage(LOG_DEBUG,"SPI2 initialized");
   return 1;
 }
 
@@ -798,7 +805,7 @@ static void
 destruct_AtSpi2Screen (void) {
   finished = 1;
   pthread_join(SPI2_main_thread,NULL);
-  LogPrint(LOG_DEBUG,"SPI2 stopped");
+  logMessage(LOG_DEBUG,"SPI2 stopped");
 }
 
 static int
@@ -878,7 +885,7 @@ AtSpi2GenerateKeyboardEvent (dbus_uint32_t keysym, enum key_type_e key_type)
 
   msg = dbus_message_new_method_call(SPI2_DBUS_INTERFACE_REG, SPI2_DBUS_PATH_DEC, SPI2_DBUS_INTERFACE_DEC, "GenerateKeyboardEvent");
   if (!msg) {
-    LogPrint(LOG_DEBUG, "no memory while getting text");
+    logMessage(LOG_DEBUG, "no memory while getting text");
     return 0;
   }
   dbus_message_append_args(msg, DBUS_TYPE_INT32, &keysym, DBUS_TYPE_STRING, &s, DBUS_TYPE_INT32, &key_type, DBUS_TYPE_INVALID);
@@ -887,11 +894,11 @@ AtSpi2GenerateKeyboardEvent (dbus_uint32_t keysym, enum key_type_e key_type)
   /* 1s max delay */
   reply = dbus_connection_send_with_reply_and_block(bus, msg, 1000, &error);
   if (dbus_error_is_set(&error)) {
-    LogPrint(LOG_DEBUG, "error while generating keyboard event: %s %s", error.name, error.message);
+    logMessage(LOG_DEBUG, "error while generating keyboard event: %s %s", error.name, error.message);
     goto outMsg;
   }
   if (!reply) {
-    LogPrint(LOG_DEBUG, "timeout while generating keyboard event");
+    logMessage(LOG_DEBUG, "timeout while generating keyboard event");
     goto outMsg;
   }
   res = 1;
@@ -910,6 +917,7 @@ insertKey_AtSpi2Screen (ScreenKey key) {
 
   if (isSpecialKey(key)) {
     switch (key & SCR_KEY_CHAR_MASK) {
+#ifdef HAVE_X11_KEYSYM_H
       case SCR_KEY_ENTER:         keysym = XK_KP_Enter;  break;
       case SCR_KEY_TAB:           keysym = XK_Tab;       break;
       case SCR_KEY_BACKSPACE:     keysym = XK_BackSpace; break;
@@ -959,7 +967,10 @@ insertKey_AtSpi2Screen (ScreenKey key) {
       case SCR_KEY_FUNCTION + 32: keysym = XK_F33;       break;
       case SCR_KEY_FUNCTION + 33: keysym = XK_F34;       break;
       case SCR_KEY_FUNCTION + 34: keysym = XK_F35;       break;
-      default: LogPrint(LOG_WARNING, "key not insertable: %04X", key); return 0;
+#else /* HAVE_X11_KEYSYM_H */
+#warning insertion of non-character key presses not supported by this build - check that X11 protocol headers have been installed
+#endif /* HAVE_X11_KEYSYM_H */
+      default: logMessage(LOG_WARNING, "key not insertable: %04X", key); return 0;
     }
   } else {
     wchar_t wc;
@@ -980,47 +991,52 @@ insertKey_AtSpi2Screen (ScreenKey key) {
     else
       keysym = 0x1000000 | wc;
   }
-  LogPrint(LOG_DEBUG, "inserting key: %04X -> %s%s%ld",
-           key,
-           (modMeta? "meta ": ""),
-           (modControl? "control ": ""),
-           keysym);
+  logMessage(LOG_DEBUG, "inserting key: %04X -> %s%s%ld",
+             key,
+             (modMeta? "meta ": ""),
+             (modControl? "control ": ""),
+             keysym);
 
   {
     int ok = 0;
 
+#ifdef HAVE_X11_KEYSYM_H
     if (!modMeta || AtSpi2GenerateKeyboardEvent(XK_Meta_L,PRESS)) {
       if (!modControl || AtSpi2GenerateKeyboardEvent(XK_Control_L,PRESS)) {
+#endif /* HAVE_X11_KEYSYM_H */
+
         if (AtSpi2GenerateKeyboardEvent(keysym,SYM)) {
           ok = 1;
         } else {
-          LogPrint(LOG_WARNING, "key insertion failed.");
+          logMessage(LOG_WARNING, "key insertion failed.");
         }
 
+#ifdef HAVE_X11_KEYSYM_H
         if (modControl && !AtSpi2GenerateKeyboardEvent(XK_Control_L,RELEASE)) {
-          LogPrint(LOG_WARNING, "control release failed.");
+          logMessage(LOG_WARNING, "control release failed.");
           ok = 0;
         }
       } else {
-        LogPrint(LOG_WARNING, "control press failed.");
+        logMessage(LOG_WARNING, "control press failed.");
       }
 
       if (modMeta && !AtSpi2GenerateKeyboardEvent(XK_Meta_L,RELEASE)) {
-        LogPrint(LOG_WARNING, "meta release failed.");
+        logMessage(LOG_WARNING, "meta release failed.");
         ok = 0;
       }
     } else {
-      LogPrint(LOG_WARNING, "meta press failed.");
+      logMessage(LOG_WARNING, "meta press failed.");
     }
+#endif /* HAVE_X11_KEYSYM_H */
 
     return ok;
   }
 }
 
 static int
-executeCommand_AtSpi2Screen (int command) {
-  int blk UNUSED = command & BRL_MSK_BLK;
-  int arg UNUSED = command & BRL_MSK_ARG;
+executeCommand_AtSpi2Screen (int *command) {
+  int blk UNUSED = *command & BRL_MSK_BLK;
+  int arg UNUSED = *command & BRL_MSK_ARG;
 
   return 0;
 }
