@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -67,7 +67,7 @@ spk_construct (SpeechSynthesizer *spk, char **parameters)
   if ((spk_buffer = malloc(spk_size))) {
     return 1;
   } else {
-    LogError("malloc");
+    logMallocError();
   }
   return 0;
 }
