@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -73,7 +73,7 @@ static void myerror(SpeechSynthesizer *spk, char *fmt, ...)
   }
   buf[ERRBUFLEN-1] = 0;
   va_end(argp);
-  LogPrint(LOG_ERR, "%s", buf);
+  logMessage(LOG_ERR, "%s", buf);
   spk_destruct(spk);
 }
 static void myperror(SpeechSynthesizer *spk, char *fmt, ...)
@@ -90,7 +90,7 @@ static void myperror(SpeechSynthesizer *spk, char *fmt, ...)
   }
   buf[ERRBUFLEN-1] = 0;
   va_end(argp);
-  LogPrint(LOG_ERR, "%s", buf);
+  logMessage(LOG_ERR, "%s", buf);
   spk_destruct(spk);
 }
 
@@ -112,7 +112,7 @@ static int spk_construct (SpeechSynthesizer *spk, char **parameters)
 
   if (!CreatePipe(&fd1R,&fd1W,&attributes,0)
       ||!CreatePipe(&fd2R,&fd2W,&attributes,0)) {
-    LogWindowsError("CreatePipe");
+    logWindowsSystemError("CreatePipe");
     return 0;
   }
 
@@ -122,15 +122,15 @@ static int spk_construct (SpeechSynthesizer *spk, char **parameters)
   startupinfo.hStdInput = fd2R;
   startupinfo.hStdOutput = fd1W;
   if (!CreateProcess(NULL, extProgPath, NULL, NULL, TRUE, CREATE_NEW_PROCESS_GROUP, NULL, NULL, &startupinfo, &processinfo)) {
-    LogWindowsError("CreateProcess");
+    logWindowsSystemError("CreateProcess");
     return 0;
   }
   if ((helper_fd_in = _open_osfhandle((long)fd1R, O_RDONLY)) < 0) {
-    LogError("open_osfhandle");
+    logSystemError("open_osfhandle");
     return 0;
   }
   if ((helper_fd_out = _open_osfhandle((long)fd2W, O_WRONLY)) < 0) {
-    LogError("open_osfhandle");
+    logSystemError("open_osfhandle");
     return 0;
   }
 #else /* __MINGW32__ */
@@ -179,8 +179,8 @@ static int spk_construct (SpeechSynthesizer *spk, char **parameters)
     myperror(spk, "pipe");
     return 0;
   }
-  LogPrint(LOG_DEBUG, "pipe fds: fd1 %d %d, fd2 %d %d",
-	   fd1[0],fd1[1], fd2[0],fd2[1]);
+  logMessage(LOG_DEBUG, "pipe fds: fd1 %d %d, fd2 %d %d",
+	     fd1[0],fd1[1], fd2[0],fd2[1]);
   switch(fork()) {
   case -1:
     myperror(spk, "fork");
@@ -199,7 +199,7 @@ static int spk_construct (SpeechSynthesizer *spk, char **parameters)
     {
       unsigned long uid = getuid();
       unsigned long gid = getgid();
-      LogPrint(LOG_INFO, "ExternalSpeech program uid is %lu, gid is %lu", uid, gid);
+      logMessage(LOG_INFO, "ExternalSpeech program uid is %lu, gid is %lu", uid, gid);
     }
 
     if(dup2(fd2[0], 0) < 0 /* stdin */
@@ -228,8 +228,8 @@ static int spk_construct (SpeechSynthesizer *spk, char **parameters)
   };
 #endif /* __MINGW32__ */
 
-  LogPrint(LOG_INFO,"Opened pipe to external speech program '%s'",
-	   extProgPath);
+  logMessage(LOG_INFO,"Opened pipe to external speech program '%s'",
+	     extProgPath);
 
   return 1;
 }
@@ -311,10 +311,10 @@ static void spk_doTrack(SpeechSynthesizer *spk)
   while(myread(spk, helper_fd_in, b, 2)) {
     unsigned inx;
     inx = (b[0]<<8 | b[1]);
-    LogPrint(LOG_DEBUG, "spktrk: Received index %u", inx);
+    logMessage(LOG_DEBUG, "spktrk: Received index %u", inx);
     if(inx >= finalIndex) {
       speaking = 0;
-      LogPrint(LOG_DEBUG, "spktrk: Done speaking %d", lastIndex);
+      logMessage(LOG_DEBUG, "spktrk: Done speaking %d", lastIndex);
 	/* do not change last_inx: remain on position of last spoken words,
 	   not after them. */
     }else lastIndex = inx;
@@ -335,18 +335,18 @@ static void spk_mute (SpeechSynthesizer *spk)
 {
   unsigned char c = 1;
   if(helper_fd_out < 0) return;
-  LogPrint(LOG_DEBUG,"mute");
+  logMessage(LOG_DEBUG,"mute");
   speaking = 0;
   mywrite(spk, helper_fd_out, &c,1);
 }
 
-static void spk_rate (SpeechSynthesizer *spk, unsigned char setting)
+static void spk_setRate (SpeechSynthesizer *spk, unsigned char setting)
 {
   float expand = 1.0 / getFloatSpeechRate(setting); 
   unsigned char *p = (unsigned char *)&expand;
   unsigned char l[5];
   if(helper_fd_out < 0) return;
-  LogPrint(LOG_DEBUG,"set rate to %u (time scale %f)", setting, expand);
+  logMessage(LOG_DEBUG,"set rate to %u (time scale %f)", setting, expand);
   l[0] = 3; /* time scale code */
 #ifdef WORDS_BIGENDIAN
   l[1] = p[0]; l[2] = p[1]; l[3] = p[2]; l[4] = p[3];
