@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -64,13 +64,13 @@ spk_construct (SpeechSynthesizer *spk, char **parameters)
     int pitch = 100, minpitch = 50, maxpitch = 200;
     if (*parameters[PARM_pitch])
       if (!validateInteger(&pitch, parameters[PARM_pitch], &minpitch, &maxpitch))
-        LogPrint(LOG_WARNING, "%s: %s", "invalid pitch specification", parameters[PARM_pitch]);
+        logMessage(LOG_WARNING, "%s: %s", "invalid pitch specification", parameters[PARM_pitch]);
     feat_set_int(voice->features, "int_f0_target_mean", pitch);
   }
 
-  LogPrint(LOG_INFO, "Festival Lite Engine: version %s-%s, %s",
-	   FLITE_PROJECT_VERSION, FLITE_PROJECT_STATE,
-	   FLITE_PROJECT_DATE);
+  logMessage(LOG_INFO, "Festival Lite Engine: version %s-%s, %s",
+	     FLITE_PROJECT_VERSION, FLITE_PROJECT_STATE,
+	     FLITE_PROJECT_DATE);
   return 1;
 }
 
@@ -102,7 +102,7 @@ spk_say (SpeechSynthesizer *spk, const unsigned char *buffer, size_t length, siz
 
   if (pipe(fds) != -1) {
     if ((child = fork()) == -1) {
-      LogError("fork");
+      logSystemError("fork");
     } else if (child == 0) {
       _exit(doChild());
     } else
@@ -117,7 +117,7 @@ spk_say (SpeechSynthesizer *spk, const unsigned char *buffer, size_t length, siz
     close(*readfd);
     close(*writefd);
   } else {
-    LogError("pipe");
+    logSystemError("pipe");
   }
 }
 
@@ -135,7 +135,7 @@ spk_mute (SpeechSynthesizer *spk)
 }
 
 static void
-spk_rate (SpeechSynthesizer *spk, unsigned char setting)
+spk_setRate (SpeechSynthesizer *spk, unsigned char setting)
 {
   feat_set_float(voice->features, "duration_stretch", 1.0/getFloatSpeechRate(setting));
 }
