@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -26,6 +26,8 @@
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
+#include <time.h>
+#include <sys/time.h>
 
 #ifdef __MINGW32__
 #include "win_pthread.h"
@@ -110,15 +112,15 @@ openSoundDevice (void) {
         ++format;
       }
       if (format->internal == PCM_FMT_UNKNOWN) {
-        LogPrint(LOG_WARNING, "No supported sound format.");
+        logMessage(LOG_WARNING, "No supported sound format.");
         closePcmDevice(pcm);
         pcm = NULL;
         return 0;
       }
       speechParameters.nBits = format->external;
     }
-    LogPrint(LOG_DEBUG, "Mikropuhe audio configuration: channels=%d rate=%d bits=%d",
-             speechParameters.nChannels, speechParameters.nSampleFreq, speechParameters.nBits);
+    logMessage(LOG_DEBUG, "Mikropuhe audio configuration: channels=%d rate=%d bits=%d",
+               speechParameters.nChannels, speechParameters.nSampleFreq, speechParameters.nBits);
   }
   return 1;
 }
@@ -193,14 +195,14 @@ logSynthesisError (int code, const char *action) {
       explanation = "sound device";
       break;
   }
-  LogPrint(LOG_ERR, "Mikropuhe %s error: %s", action, explanation);
+  logMessage(LOG_ERR, "Mikropuhe %s error: %s", action, explanation);
 }
 
 static int
 writeSound (const void *bytes, unsigned int count, void *data, void *reserved) {
   if (synthesisThreadStarted) {
     if (writePcmData(pcm, bytes, count)) return 0;
-    LogError("Mikropuhe write");
+    logSystemError("Mikropuhe write");
     return MPINT_ERR_GENERAL;
   }
   return 1;
@@ -265,7 +267,7 @@ awaitSpeechSegment (void) {
         continue;
 
       default:
-        LogError("pthread_cond_timedwait");
+        logSystemError("pthread_cond_timedwait");
         return 0;
     }
   }
@@ -302,20 +304,20 @@ startSynthesisThread (void) {
         if (!error) {
           return 1;
         } else {
-          LogPrint(LOG_ERR, "Cannot create speech thread: %s", strerror(error));
+          logMessage(LOG_ERR, "Cannot create speech thread: %s", strerror(error));
         }
       } else {
-        LogPrint(LOG_ERR, "Cannot initialize speech thread attributes: %s", strerror(error));
+        logMessage(LOG_ERR, "Cannot initialize speech thread attributes: %s", strerror(error));
       }
 
       pthread_cond_destroy(&speechConditional);
     } else {
-      LogPrint(LOG_ERR, "Cannot initialize speech conditional: %s", strerror(error));
+      logMessage(LOG_ERR, "Cannot initialize speech conditional: %s", strerror(error));
     }
 
     pthread_mutex_destroy(&speechMutex);
   } else {
-    LogPrint(LOG_ERR, "Cannot initialize speech mutex: %s", strerror(error));
+    logMessage(LOG_ERR, "Cannot initialize speech mutex: %s", strerror(error));
   }
   synthesisThreadStarted = 0;
   return 0;
@@ -376,15 +378,15 @@ loadSynthesisLibrary (void) {
         while (symbol->name) {
           void **address = symbol->address;
           if (findSharedSymbol(speechLibrary, symbol->name, address)) {
-            LogPrint(LOG_DEBUG, "Mikropuhe symbol: %s -> %p",
-                     symbol->name, *address);
+            logMessage(LOG_DEBUG, "Mikropuhe symbol: %s -> %p",
+                       symbol->name, *address);
           } else {
-            LogPrint(LOG_ERR, "Mikropuhe symbol not found: %s", symbol->name);
+            logMessage(LOG_ERR, "Mikropuhe symbol not found: %s", symbol->name);
           }
           ++symbol;
         }
       } else {
-        LogPrint(LOG_ERR, "Mikropuhe library not loaded: %s", path);
+        logMessage(LOG_ERR, "Mikropuhe library not loaded: %s", path);
       }
       free(path);
     }
@@ -425,7 +427,7 @@ spk_construct (SpeechSynthesizer *spk, char **parameters) {
               snprintf(tag, sizeof(tag), "<pitch absmiddle=\"%d\"/>", setting);
               enqueueTag(tag);
             } else {
-              LogPrint(LOG_WARNING, "%s: %s", "invalid pitch specification", pitch);
+              logMessage(LOG_WARNING, "%s: %s", "invalid pitch specification", pitch);
             }
           }
         }
@@ -436,7 +438,7 @@ spk_construct (SpeechSynthesizer *spk, char **parameters) {
       }
     }
   } else {
-    LogPrint(LOG_ERR, "Cannot allocate speech queue.");
+    logMessage(LOG_ERR, "Cannot allocate speech queue.");
   }
 
   spk_destruct(spk);
@@ -485,18 +487,18 @@ spk_mute (SpeechSynthesizer *spk) {
 }
 
 static void
-spk_rate (SpeechSynthesizer *spk, unsigned char setting) {
-  char tag[0X40];
-  snprintf(tag, sizeof(tag), "<rate absspeed=\"%d\"/>",
-           getIntegerSpeechRate(setting, 10)-10);
-  enqueueTag(tag);
-}
-
-static void
-spk_volume (SpeechSynthesizer *spk, unsigned char setting) {
+spk_setVolume (SpeechSynthesizer *spk, unsigned char setting) {
   char tag[0X40];
   unsigned int percentage = getIntegerSpeechVolume(setting, 100);
   snprintf(tag, sizeof(tag), "<volume level=\"%d\"/>",
            MAX(percentage, 1));
   enqueueTag(tag);
 }
+
+static void
+spk_setRate (SpeechSynthesizer *spk, unsigned char setting) {
+  char tag[0X40];
+  snprintf(tag, sizeof(tag), "<rate absspeed=\"%d\"/>",
+           getIntegerSpeechRate(setting, 10)-10);
+  enqueueTag(tag);
+}
