@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -94,13 +94,13 @@ setVoice (const void *data) {
 }
 
 static void
-setRate (const void *data) {
-  spd_set_voice_rate(connectionHandle, relativeRate);
+setVolume (const void *data) {
+  spd_set_volume(connectionHandle, relativeVolume);
 }
 
 static void
-setVolume (const void *data) {
-  spd_set_volume(connectionHandle, relativeVolume);
+setRate (const void *data) {
+  spd_set_voice_rate(connectionHandle, relativeRate);
 }
 
 static void
@@ -122,7 +122,7 @@ static int
 openConnection (void) {
   if (!connectionHandle) {
     if (!(connectionHandle = spd_open("brltty", "main", NULL, SPD_MODE_THREADED))) {
-      LogPrint(LOG_ERR, "speech dispatcher open failure");
+      logMessage(LOG_ERR, "speech dispatcher open failure");
       return 0;
     }
 
@@ -131,8 +131,8 @@ openConnection (void) {
         setModule,
         setLanguage,
         setVoice,
-        setRate,
         setVolume,
+        setRate,
         setPitch,
         setPunctuation,
         NULL
@@ -159,7 +159,7 @@ spk_construct (SpeechSynthesizer *spk, char **parameters) {
       snprintf(number, sizeof(number), "%d", port);
       setenv("SPEECHD_PORT", number, 1);
     } else {
-      LogPrint(LOG_WARNING, "%s: %s", "invalid port number", parameters[PARM_PORT]);
+      logMessage(LOG_WARNING, "%s: %s", "invalid port number", parameters[PARM_PORT]);
     }
   }
 
@@ -192,7 +192,7 @@ spk_construct (SpeechSynthesizer *spk, char **parameters) {
     if (validateChoice(&choice, parameters[PARM_VOICE], choices)) {
       voiceType = voices[choice];
     } else {
-      LogPrint(LOG_WARNING, "%s: %s", "invalid voice type", parameters[PARM_VOICE]);
+      logMessage(LOG_WARNING, "%s: %s", "invalid voice type", parameters[PARM_VOICE]);
     }
   }
 
@@ -249,31 +249,31 @@ spk_mute (SpeechSynthesizer *spk) {
 }
 
 static void
-spk_rate (SpeechSynthesizer *spk, unsigned char setting) {
-  relativeRate = getIntegerSpeechRate(setting, 100) - 100;
-  speechdAction(setRate, NULL);
-  LogPrint(LOG_DEBUG, "set rate: %u -> %d", setting, relativeRate);
-}
-
-static void
-spk_volume (SpeechSynthesizer *spk, unsigned char setting) {
+spk_setVolume (SpeechSynthesizer *spk, unsigned char setting) {
   relativeVolume = getIntegerSpeechVolume(setting, 100) - 100;
   speechdAction(setVolume, NULL);
-  LogPrint(LOG_DEBUG, "set volume: %u -> %d", setting, relativeVolume);
+  logMessage(LOG_DEBUG, "set volume: %u -> %d", setting, relativeVolume);
 }
 
 static void
-spk_pitch (SpeechSynthesizer *spk, unsigned char setting) {
+spk_setRate (SpeechSynthesizer *spk, unsigned char setting) {
+  relativeRate = getIntegerSpeechRate(setting, 100) - 100;
+  speechdAction(setRate, NULL);
+  logMessage(LOG_DEBUG, "set rate: %u -> %d", setting, relativeRate);
+}
+
+static void
+spk_setPitch (SpeechSynthesizer *spk, unsigned char setting) {
   relativePitch = getIntegerSpeechPitch(setting, 100) - 100;
   speechdAction(setPitch, NULL);
-  LogPrint(LOG_DEBUG, "set pitch: %u -> %d", setting, relativePitch);
+  logMessage(LOG_DEBUG, "set pitch: %u -> %d", setting, relativePitch);
 }
 
 static void
-spk_punctuation (SpeechSynthesizer *spk, SpeechPunctuation setting) {
+spk_setPunctuation (SpeechSynthesizer *spk, SpeechPunctuation setting) {
   punctuationVerbosity = (setting <= SPK_PUNCTUATION_NONE)? SPD_PUNCT_NONE: 
                          (setting >= SPK_PUNCTUATION_ALL)? SPD_PUNCT_ALL: 
                          SPD_PUNCT_SOME;
   speechdAction(setPunctuation, NULL);
-  LogPrint(LOG_DEBUG, "set punctuation: %u -> %d", setting, punctuationVerbosity);
+  logMessage(LOG_DEBUG, "set punctuation: %u -> %d", setting, punctuationVerbosity);
 }
