@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -98,7 +98,7 @@ static void showDisplaySize(void)
   fprintf(stderr,"Getting display size: ");
   if (brlapi_getDisplaySize(&x, &y)<0) {
     brlapi_perror("failed");
-    exit(1);
+    exit(PROG_EXIT_FATAL);
   }
   fprintf(stderr, "%dX%d\n", x, y);
 }
@@ -109,7 +109,7 @@ static void showDriverName(void)
   fprintf(stderr, "Getting driver name: ");
   if (brlapi_getDriverName(name, sizeof(name))<0) {
     brlapi_perror("failed");
-    exit(1);
+    exit(PROG_EXIT_FATAL);
   }
   fprintf(stderr, "%s\n", name);
 }
@@ -124,15 +124,15 @@ static void showDots(void)
   brlapi_keyCode_t k;
   if (brlapi_getDisplaySize(&x, &y)<0) {
     brlapi_perror("failed");
-    exit(1);
+    exit(PROG_EXIT_FATAL);
   }
   if (brlapi_enterTtyMode(-1, NULL)<0) {
     brlapi_perror("enterTtyMode");
-    exit(1);
+    exit(PROG_EXIT_FATAL);
   }
   if (x*y<DOTS_TOTALLEN) {
     fprintf(stderr,"can't show dots with a braille display with less than %d cells\n",(int)DOTS_TOTALLEN);
-    exit(1);
+    exit(PROG_EXIT_SEMANTIC);
   }
   {
     char text[x*y];
@@ -156,7 +156,7 @@ static void showDots(void)
     wa.orMask = or;
     if (brlapi_write(&wa)<0) {
       brlapi_perror("brlapi_write");
-      exit(1);
+      exit(PROG_EXIT_FATAL);
     }
   }
   brlapi_readKey(1, &k);
@@ -177,13 +177,14 @@ static void enterLearnMode(void)
 
   if (brlapi_writeText(BRLAPI_CURSOR_OFF, "command learn mode")<0) {
     brlapi_perror("brlapi_writeText");
-    exit(1);
+    exit(PROG_EXIT_FATAL);
   }
 
   while ((res = brlapi_readKey(1, &code)) != -1) {
     fprintf(stderr, "got key %016"BRLAPI_PRIxKEYCODE"\n",code);
     cmd = cmdBrlapiToBrltty(code);
-    describeCommand(cmd, buf, sizeof(buf), 1);
+    describeCommand(cmd, buf, sizeof(buf),
+                    CDO_IncludeName | CDO_IncludeOperand);
     brlapi_writeText(BRLAPI_CURSOR_OFF, buf);
     fprintf(stderr, "%s\n", buf);
     if (cmd==BRL_CMD_LEARN) return;
@@ -214,7 +215,7 @@ static void showKeyCodes(void)
 
   if (brlapi_writeText(BRLAPI_CURSOR_OFF, "show key codes")<0) {
     brlapi_perror("brlapi_writeText");
-    exit(1);
+    exit(PROG_EXIT_FATAL);
   }
 
   while ((res = brlapi_readKey(1, &cmd)) != -1) {
@@ -235,7 +236,7 @@ static void suspendDriver(void)
   fprintf(stderr, "Getting driver name: ");
   if (brlapi_getDriverName(name, sizeof(name))<0) {
     brlapi_perror("failed");
-    exit(1);
+    exit(PROG_EXIT_FATAL);
   }
   fprintf(stderr, "%s\n", name);
   fprintf(stderr, "Suspending\n");
@@ -258,9 +259,9 @@ static void suspendDriver(void)
   }
 }
 
-int main(int argc, char *argv[])
-{
-  int status = 0;
+int
+main (int argc, char *argv[]) {
+  ProgramExitStatus exitStatus = PROG_EXIT_SUCCESS;
   brlapi_fileDescriptor fd;
   settings.host = NULL; settings.auth = NULL;
 
@@ -269,7 +270,7 @@ int main(int argc, char *argv[])
       OPTION_TABLE(programOptions),
       .applicationName = "apitest"
     };
-    processOptions(&descriptor, &argc, &argv);
+    PROCESS_OPTIONS(descriptor, argc, argv);
   }
 
   fprintf(stderr, "Connecting to BrlAPI... ");
@@ -306,7 +307,7 @@ int main(int argc, char *argv[])
   } else {
     fprintf(stderr, "failed to connect to %s using auth %s",settings.host, settings.auth);
     brlapi_perror("");
-    status = 1;
+    exitStatus = PROG_EXIT_FATAL;
   }
-  return status;
+  return exitStatus;
 }
