@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -20,25 +20,30 @@
 
 #include <string.h>
 #include <errno.h>
-#include <sys/time.h>
 
 #ifdef __MSDOS__
 #include "sys_msdos.h"
 #endif /* __MSDOS__ */
 
+#undef ASYNC_CAN_MONITOR_IO
 #if defined(__MINGW32__)
+#define ASYNC_CAN_MONITOR_IO
 
 typedef HANDLE MonitorEntry;
 
 #elif defined(HAVE_SYS_POLL_H)
+#define ASYNC_CAN_MONITOR_IO
 
 #include <sys/poll.h>
 typedef struct pollfd MonitorEntry;
 
-#else /* monitor definitions */
+#elif defined(HAVE_SELECT)
+#define ASYNC_CAN_MONITOR_IO
 
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
+#else /* HAVE_SYS_SELECT_H */
+#include <sys/time.h>
 #endif /* HAVE_SYS_SELECT_H */
 
 typedef struct {
@@ -106,7 +111,7 @@ struct FunctionEntryStruct {
   OVERLAPPED ol;
 #elif defined(HAVE_SYS_POLL_H)
   short pollEvents;
-#else /* monitor definitions */
+#elif defined(HAVE_SELECT)
   SelectDescriptor *selectDescriptor;
 #endif /* monitor definitions */
 };
@@ -128,7 +133,7 @@ awaitOperation (MonitorEntry *monitors, int count, int timeout) {
     if ((result >= WAIT_OBJECT_0) && (result < (WAIT_OBJECT_0 + count))) return 1;
 
     if (result == WAIT_FAILED) {
-      LogWindowsError("WaitForMultipleObjects");
+      logWindowsSystemError("WaitForMultipleObjects");
     }
   } else {
     approximateDelay(timeout);
@@ -148,7 +153,7 @@ testMonitor (const MonitorEntry *monitor) {
   if (result == WAIT_OBJECT_0) return 1;
 
   if (result == WAIT_FAILED) {
-    LogWindowsError("WaitForSingleObject");
+    logWindowsSystemError("WaitForSingleObject");
   }
 
   return 0;
@@ -254,7 +259,9 @@ finishWindowsTransferOperation (OperationEntry *operation) {
   DWORD success = GetOverlappedResult(function->fileDescriptor, &function->ol, &count, FALSE);
   setWindowsTransferResult(operation, success, count);
 }
+
 #else /* __MINGW32__ */
+
 #ifdef HAVE_SYS_POLL_H
 static void
 prepareMonitors (void) {
@@ -266,7 +273,7 @@ awaitOperation (MonitorEntry *monitors, int count, int timeout) {
   if (result > 0) return 1;
 
   if (result == -1) {
-    if (errno != EINTR) LogError("poll");
+    if (errno != EINTR) logSystemError("poll");
   }
 
   return 0;
@@ -293,7 +300,9 @@ static void
 beginUnixOutputFunction (FunctionEntry *function) {
   function->pollEvents = POLLOUT;
 }
-#else /* HAVE_SYS_POLL_H */
+
+#elif defined(HAVE_SELECT)
+
 static void
 prepareSelectDescriptor (SelectDescriptor *descriptor) {
   FD_ZERO(&descriptor->set);
@@ -323,7 +332,7 @@ doSelect (int setSize, fd_set *readSet, fd_set *writeSet, int timeout) {
     if (result > 0) return 1;
 
     if (result == -1) {
-      if (errno != EINTR) LogError("select");
+      if (errno != EINTR) logSystemError("select");
     }
 
     return 0;
@@ -381,8 +390,10 @@ static void
 beginUnixOutputFunction (FunctionEntry *function) {
   function->selectDescriptor = &selectDescriptor_write;
 }
-#endif /* HAVE_SYS_POLL_H */
 
+#endif /* Unix I/O monitoring capabilities */
+
+#ifdef ASYNC_CAN_MONITOR_IO
 static void
 setUnixTransferResult (OperationEntry *operation, ssize_t count) {
   TransferExtension *extension = operation->extension;
@@ -417,8 +428,76 @@ finishUnixWrite (OperationEntry *operation) {
                      extension->size - extension->length);
   setUnixTransferResult(operation, result);
 }
+#endif /* ASYNC_CAN_MONITOR_IO */
 #endif /* __MINGW32__ */
 
+#ifdef ASYNC_CAN_MONITOR_IO
+static void
+deallocateFunctionEntry (void *item, void *data) {
+  FunctionEntry *function = item;
+  if (function->operations) deallocateQueue(function->operations);
+  if (function->methods->endFunction) function->methods->endFunction(function);
+  free(function);
+}
+
+static Queue *
+getFunctionQueue (int create) {
+  static Queue *functions = NULL;
+
+  if (!functions) {
+    if (create) {
+      if ((functions = newQueue(deallocateFunctionEntry, NULL))) {
+      }
+    }
+  }
+
+  return functions;
+}
+
+static OperationEntry *
+getFirstOperation (const FunctionEntry *function) {
+  return getElementItem(getQueueHead(function->operations));
+}
+
+static void
+startOperation (OperationEntry *operation) {
+  if (operation->function->methods->startOperation) operation->function->methods->startOperation(operation);
+}
+
+static void
+finishOperation (OperationEntry *operation) {
+  if (operation->function->methods->finishOperation) operation->function->methods->finishOperation(operation);
+}
+
+typedef struct {
+  MonitorEntry *monitor;
+} AddMonitorData;
+
+static int
+addMonitor (void *item, void *data) {
+  const FunctionEntry *function = item;
+  AddMonitorData *add = data;
+  OperationEntry *operation = getFirstOperation(function);
+  if (operation->finished) return 1;
+
+  initializeMonitor(add->monitor++, function, operation);
+  return 0;
+}
+
+typedef struct {
+  const MonitorEntry *monitor;
+} FindMonitorData;
+
+static int
+findMonitor (void *item, void *data) {
+/*FunctionEntry *function = item;*/
+  FindMonitorData *find = data;
+  if (testMonitor(find->monitor)) return 1;
+
+  find->monitor += 1;
+  return 0;
+}
+
 static int
 invokeInputCallback (OperationEntry *operation) {
   TransferExtension *extension = operation->extension;
@@ -471,14 +550,6 @@ invokeOutputCallback (OperationEntry *operation) {
   return 0;
 }
 
-static void
-deallocateFunctionEntry (void *item, void *data) {
-  FunctionEntry *function = item;
-  if (function->operations) deallocateQueue(function->operations);
-  if (function->methods->endFunction) function->methods->endFunction(function);
-  free(function);
-}
-
 static int
 testFunctionEntry (const void *item, const void *data) {
   const FunctionEntry *function = item;
@@ -494,20 +565,6 @@ deallocateOperationEntry (void *item, void *data) {
   free(operation);
 }
 
-static Queue *
-getFunctionQueue (int create) {
-  static Queue *functions = NULL;
-
-  if (!functions) {
-    if (create) {
-      if ((functions = newQueue(deallocateFunctionEntry, NULL))) {
-      }
-    }
-  }
-
-  return functions;
-}
-
 static Element *
 getFunctionElement (FileDescriptor fileDescriptor, const FunctionMethods *methods, int create) {
   Queue *functions = getFunctionQueue(create);
@@ -542,6 +599,8 @@ getFunctionElement (FileDescriptor fileDescriptor, const FunctionMethods *method
         }
 
         free(function);
+      } else {
+        logMallocError();
       }
     }
   }
@@ -549,16 +608,6 @@ getFunctionElement (FileDescriptor fileDescriptor, const FunctionMethods *method
   return NULL;
 }
 
-static void
-startOperation (OperationEntry *operation) {
-  if (operation->function->methods->startOperation) operation->function->methods->startOperation(operation);
-}
-
-static void
-finishOperation (OperationEntry *operation) {
-  if (operation->function->methods->finishOperation) operation->function->methods->finishOperation(operation);
-}
-
 static int
 createOperation (
   FileDescriptor fileDescriptor,
@@ -590,6 +639,8 @@ createOperation (
     }
 
     free(operation);
+  } else {
+    logMallocError();
   }
 
   return 0;
@@ -614,6 +665,8 @@ createTransferOperation (
     if (createOperation(fileDescriptor, methods, extension, data)) return 1;
 
     free(extension);
+  } else {
+    logMallocError();
   }
 
   return 0;
@@ -645,11 +698,7 @@ createOutputOperation (
   direction.output.callback = callback;
   return createTransferOperation(fileDescriptor, methods, &direction, size, buffer, data);
 }
-
-static OperationEntry *
-getFirstOperation (const FunctionEntry *function) {
-  return getElementItem(getQueueHead(function->operations));
-}
+#endif /* ASYNC_CAN_MONITOR_IO */
 
 int
 asyncRead (
@@ -657,6 +706,7 @@ asyncRead (
   size_t size,
   AsyncInputCallback callback, void *data
 ) {
+#ifdef ASYNC_CAN_MONITOR_IO
   static const FunctionMethods methods = {
 #ifdef __MINGW32__
     .beginFunction = beginWindowsFunction,
@@ -671,6 +721,11 @@ asyncRead (
   };
 
   return createInputOperation(fileDescriptor, &methods, callback, size, data);
+#else /* ASYNC_CAN_MONITOR_IO */
+  errno = ENOSYS;
+  logSystemError("asyncRead");
+  return 0;
+#endif /* ASYNC_CAN_MONITOR_IO */
 }
 
 int
@@ -679,6 +734,7 @@ asyncWrite (
   const void *buffer, size_t size,
   AsyncOutputCallback callback, void *data
 ) {
+#ifdef ASYNC_CAN_MONITOR_IO
   static const FunctionMethods methods = {
 #ifdef __MINGW32__
     .beginFunction = beginWindowsFunction,
@@ -693,10 +749,15 @@ asyncWrite (
   };
 
   return createOutputOperation(fileDescriptor, &methods, callback, size, buffer, data);
+#else /* ASYNC_CAN_MONITOR_IO */
+  errno = ENOSYS;
+  logSystemError("asyncWrite");
+  return 0;
+#endif /* ASYNC_CAN_MONITOR_IO */
 }
 
 typedef struct {
-  struct timeval time;
+  TimeValue time;
   AsyncAlarmCallback callback;
   void *data;
 } AlarmEntry;
@@ -711,9 +772,7 @@ static int
 compareAlarmEntries (const void *item1, const void *item2, void *data) {
   const AlarmEntry *alarm1 = item1;
   const AlarmEntry *alarm2 = item2;
-  if (alarm2->time.tv_sec < alarm1->time.tv_sec) return 0;
-  if (alarm2->time.tv_sec > alarm1->time.tv_sec) return 1;
-  return alarm2->time.tv_usec > alarm1->time.tv_usec;
+  return compareTimeValues(&alarm1->time, &alarm2->time) < 0;
 }
 
 static Queue *
@@ -730,26 +789,9 @@ getAlarmQueue (int create) {
   return alarms;
 }
 
-static void
-normalizeTime (struct timeval *time) {
-  time->tv_sec += time->tv_usec / 1000000;
-  time->tv_usec = time->tv_usec % 1000000;
-}
-
-static void
-adjustTime (struct timeval *time, int amount) {
-  int quotient = amount / 1000;
-  int remainder = amount % 1000;
-
-  if (remainder < 0) remainder += 1000, --quotient;
-  time->tv_sec += quotient;
-  time->tv_usec += remainder * 1000;
-  normalizeTime(time);
-}
-
 int
 asyncAbsoluteAlarm (
-  const struct timeval *time,
+  const TimeValue *time,
   AsyncAlarmCallback callback,
   void *data
 ) {
@@ -766,6 +808,8 @@ asyncAbsoluteAlarm (
       if (enqueueItem(alarms, alarm)) return 1;
 
       free(alarm);
+    } else {
+      logMallocError();
     }
   }
 
@@ -778,68 +822,35 @@ asyncRelativeAlarm (
   AsyncAlarmCallback callback,
   void *data
 ) {
-  struct timeval time;
-  gettimeofday(&time, NULL);
-  adjustTime(&time, interval);
+  TimeValue time;
+  getCurrentTime(&time);
+  adjustTimeValue(&time, interval);
   return asyncAbsoluteAlarm(&time, callback, data);
 }
 
-typedef struct {
-  MonitorEntry *monitor;
-} AddMonitorData;
-
-static int
-addMonitor (void *item, void *data) {
-  const FunctionEntry *function = item;
-  AddMonitorData *add = data;
-  OperationEntry *operation = getFirstOperation(function);
-  if (operation->finished) return 1;
-
-  initializeMonitor(add->monitor++, function, operation);
-  return 0;
-}
-
-typedef struct {
-  const MonitorEntry *monitor;
-} FindMonitorData;
-
-static int
-findMonitor (void *item, void *data) {
-/*FunctionEntry *function = item;*/
-  FindMonitorData *find = data;
-  if (testMonitor(find->monitor)) return 1;
-
-  find->monitor++;
-  return 0;
-}
-
 void
 asyncWait (int duration) {
-  long int elapsed = -1;
-  struct timeval start;
+  long int elapsed = 0;
+  TimeValue start;
+  getCurrentTime(&start);
 
   do {
     long int timeout = duration;
-    Queue *functions = getFunctionQueue(0);
-    int monitorCount = functions? getQueueSize(functions): 0;
-    MonitorEntry *monitorArray = NULL;
-    Element *functionElement = NULL;
-
-    if (elapsed < 0) {
-      elapsed = 0;
-      gettimeofday(&start, NULL);
-    }
 
     {
       Queue *alarms = getAlarmQueue(0);
+
       if (alarms) {
         Element *element = getQueueHead(alarms);
+
         if (element) {
           AlarmEntry *alarm = getElementItem(element);
           long int milliseconds = millisecondsBetween(&start, &alarm->time);
+
           if (milliseconds <= elapsed) {
             AsyncAlarmCallback callback = alarm->callback;
             void *data = alarm->data;
+
             deleteElement(element);
             callback(data);
             continue;
@@ -850,51 +861,69 @@ asyncWait (int duration) {
       }
     }
 
-    prepareMonitors();
-    if (monitorCount) {
-      if ((monitorArray = malloc(ARRAY_SIZE(monitorArray, monitorCount)))) {
-        AddMonitorData add;
-        add.monitor = monitorArray;
-        functionElement = processQueue(functions, addMonitor, &add);
+#ifdef ASYNC_CAN_MONITOR_IO
+    {
+      Queue *functions = getFunctionQueue(0);
+      Element *functionElement = NULL;
 
-        if (!(monitorCount = add.monitor - monitorArray)) {
-          free(monitorArray);
-          monitorArray = NULL;
+      int monitorCount = functions? getQueueSize(functions): 0;
+      MonitorEntry *monitorArray = NULL;
+
+      prepareMonitors();
+
+      if (monitorCount) {
+        if ((monitorArray = malloc(ARRAY_SIZE(monitorArray, monitorCount)))) {
+          AddMonitorData add = {
+            .monitor = monitorArray
+          };
+
+          functionElement = processQueue(functions, addMonitor, &add);
+
+          if (!(monitorCount = add.monitor - monitorArray)) {
+            free(monitorArray);
+            monitorArray = NULL;
+          }
+        } else {
+          logMallocError();
+          monitorCount = 0;
         }
-      } else {
-        monitorCount = 0;
       }
+
+      if (!functionElement) {
+        if (awaitOperation(monitorArray, monitorCount, timeout-elapsed)) {
+          FindMonitorData find = {
+            .monitor = monitorArray
+          };
+
+          functionElement = processQueue(functions, findMonitor, &find);
+        }
+      }
+
+      if (functionElement) {
+        FunctionEntry *function = getElementItem(functionElement);
+        Element *operationElement = getQueueHead(function->operations);
+        OperationEntry *operation = getElementItem(operationElement);
+
+        if (!operation->finished) finishOperation(operation);
+        if (function->methods->invokeCallback(operation)) {
+          operation->error = 0;
+        } else {
+          deleteElement(operationElement);
+        }
+
+        if ((operationElement = getQueueHead(function->operations))) {
+          operation = getElementItem(operationElement);
+          if (!operation->finished) startOperation(operation);
+          requeueElement(functionElement);
+        } else {
+          deleteElement(functionElement);
+        }
+      }
+
+      if (monitorArray) free(monitorArray);
     }
-
-    if (!functionElement) {
-      if (awaitOperation(monitorArray, monitorCount, timeout-elapsed)) {
-        FindMonitorData find;
-        find.monitor = monitorArray;
-        functionElement = processQueue(functions, findMonitor, &find);
-      }
-    }
-
-    if (functionElement) {
-      FunctionEntry *function = getElementItem(functionElement);
-      Element *operationElement = getQueueHead(function->operations);
-      OperationEntry *operation = getElementItem(operationElement);
-
-      if (!operation->finished) finishOperation(operation);
-      if (function->methods->invokeCallback(operation)) {
-        operation->error = 0;
-      } else {
-        deleteElement(operationElement);
-      }
-
-      if ((operationElement = getQueueHead(function->operations))) {
-        operation = getElementItem(operationElement);
-        if (!operation->finished) startOperation(operation);
-        requeueElement(functionElement);
-      } else {
-        deleteElement(functionElement);
-      }
-    }
-
-    if (monitorArray) free(monitorArray);
+#else /* ASYNC_CAN_MONITOR_IO */
+    approximateDelay(timeout-elapsed);
+#endif /* ASYNC_CAN_MONITOR_IO */
   } while ((elapsed = millisecondsSince(&start)) < duration);
 }
