@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -31,9 +31,12 @@ extern AttributesTable *compileAttributesTable (const char *name);
 extern void destroyAttributesTable (AttributesTable *table);
 
 extern char *ensureAttributesTableExtension (const char *path);
+extern char *makeAttributesTablePath (const char *directory, const char *name);
 
 extern unsigned char convertAttributesToDots (AttributesTable *table, unsigned char attributes);
 
+extern int replaceAttributesTable (const char *directory, const char *name);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
