@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -167,24 +167,26 @@ processAttributesTableLine (DataFile *file, void *data) {
 AttributesTable *
 compileAttributesTable (const char *name) {
   AttributesTable *table = NULL;
-  AttributesTableData atd;
 
-  memset(&atd, 0, sizeof(atd));
+  if (setGlobalTableVariables(ATTRIBUTES_TABLE_EXTENSION, ATTRIBUTES_SUBTABLE_EXTENSION)) {
+    AttributesTableData atd;
+    memset(&atd, 0, sizeof(atd));
 
-  if ((atd.area = newDataArea())) {
-    if (allocateDataItem(atd.area, NULL, sizeof(AttributesTableHeader), __alignof__(AttributesTableHeader))) {
-      if (processDataFile(name, processAttributesTableLine, &atd)) {
-        if (makeAttributesToDots(&atd)) {
-          if ((table = malloc(sizeof(*table)))) {
-            table->header.fields = getAttributesTableHeader(&atd);
-            table->size = getDataSize(atd.area);
-            resetDataArea(atd.area);
+    if ((atd.area = newDataArea())) {
+      if (allocateDataItem(atd.area, NULL, sizeof(AttributesTableHeader), __alignof__(AttributesTableHeader))) {
+        if (processDataFile(name, processAttributesTableLine, &atd)) {
+          if (makeAttributesToDots(&atd)) {
+            if ((table = malloc(sizeof(*table)))) {
+              table->header.fields = getAttributesTableHeader(&atd);
+              table->size = getDataSize(atd.area);
+              resetDataArea(atd.area);
+            }
           }
         }
       }
-    }
 
-    destroyDataArea(atd.area);
+      destroyDataArea(atd.area);
+    }
   }
 
   return table;
@@ -200,5 +202,10 @@ destroyAttributesTable (AttributesTable *table) {
 
 char *
 ensureAttributesTableExtension (const char *path) {
-  return ensureExtension(path, ATTRIBUTES_TABLE_EXTENSION);
+  return ensureFileExtension(path, ATTRIBUTES_TABLE_EXTENSION);
+}
+
+char *
+makeAttributesTablePath (const char *directory, const char *name) {
+  return makeFilePath(directory, name, ATTRIBUTES_TABLE_EXTENSION);
 }
