@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -29,7 +29,7 @@ extern AuthDescriptor *authBeginServer (const char *parameter);
 extern void authEnd (AuthDescriptor *auth);
 extern int authPerform (AuthDescriptor *auth, FileDescriptor fd);
 
-extern void formatAddress (char *buffer, int bufferSize, const void *address, int addressSize);
+extern void formatAddress (char *buffer, size_t bufferSize, const void *address, int addressSize);
 
 #ifdef __cplusplus
 }
