@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -33,42 +33,41 @@ beeperConstruct (int errorLevel) {
 
   if ((device = malloc(sizeof(*device)))) {
     if (canBeep()) {
-      LogPrint(LOG_DEBUG, "beeper enabled");
+      logMessage(LOG_DEBUG, "beeper enabled");
       return device;
     }
 
     free(device);
   } else {
-    LogError("malloc");
+    logMallocError();
   }
 
-  LogPrint(LOG_DEBUG, "beeper not available");
+  logMessage(LOG_DEBUG, "beeper not available");
   return NULL;
 }
 
 static int
-beeperPlay (NoteDevice *device, int note, int duration) {
-  LogPrint(LOG_DEBUG, "tone: msec=%d note=%d",
-           duration, note);
+beeperPlay (NoteDevice *device, unsigned char note, unsigned int duration) {
+  logMessage(LOG_DEBUG, "tone: msec=%d note=%d", duration, note);
 
   if (!note) {
     accurateDelay(duration);
     return 1;
   }
 
-  if (asynchronousBeep((int)noteFrequencies[note], duration*4)) {
+  if (asynchronousBeep(getIntegerNoteFrequency(note), duration*4)) {
     accurateDelay(duration);
     stopBeep();
     return 1;
   }
 
-  if (startBeep((int)noteFrequencies[note])) {
+  if (startBeep(getIntegerNoteFrequency(note))) {
     accurateDelay(duration);
     stopBeep();
     return 1;
   }
 
-  return synchronousBeep((int)noteFrequencies[note], duration);
+  return synchronousBeep(getIntegerNoteFrequency(note), duration);
 }
 
 static int
@@ -80,7 +79,7 @@ static void
 beeperDestruct (NoteDevice *device) {
   endBeep();
   free(device);
-  LogPrint(LOG_DEBUG, "beeper disabled");
+  logMessage(LOG_DEBUG, "beeper disabled");
 }
 
 const NoteMethods beeperMethods = {
