@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -171,13 +171,13 @@ bthOpenConnection (const char *address, uint8_t channel, int force) {
         bthRememberConnectError(connection->address, errno);
       }
     } else {
-      LogPrint(LOG_ERR, "invalid Bluetooth device address: %s", address);
+      logMessage(LOG_ERR, "invalid Bluetooth device address: %s", address);
       errno = EINVAL;
     }
 
     free(connection);
   } else {
-    LogError("malloc");
+    logMallocError();
   }
 
   return NULL;
@@ -190,9 +190,9 @@ bthCloseConnection (BluetoothConnection *connection) {
 }
 
 int
-isBluetoothDevice (const char **path) {
-  if (isQualifiedDevice(path, "bluetooth:")) return 1;
-  if (isQualifiedDevice(path, "bt:")) return 1;
-  if (isQualifiedDevice(path, "bluez:")) return 1;
+isBluetoothDevice (const char **identifier) {
+  if (isQualifiedDevice(identifier, "bluetooth")) return 1;
+  if (isQualifiedDevice(identifier, "bt")) return 1;
+  if (isQualifiedDevice(identifier, "bluez")) return 1;
   return 0;
 }
