@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -16,8 +16,6 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#warning Bluetooth support not available on this platform
-
 #include "prologue.h"
 
 #include <errno.h>
@@ -29,7 +27,7 @@
 BluetoothConnectionExtension *
 bthConnect (uint64_t bda, uint8_t channel) {
   errno = ENOSYS;
-  LogError("Bluetooth connect");
+  logSystemError("Bluetooth connect");
   return NULL;
 }
 
@@ -40,7 +38,7 @@ bthDisconnect (BluetoothConnectionExtension *bcx) {
 int
 bthAwaitInput (BluetoothConnection *connection, int milliseconds) {
   errno = ENOSYS;
-  LogError("Bluetooth wait");
+  logSystemError("Bluetooth wait");
   return 0;
 }
 
@@ -50,13 +48,13 @@ bthReadData (
   int initialTimeout, int subsequentTimeout
 ) {
   errno = ENOSYS;
-  LogError("Bluetooth read");
+  logSystemError("Bluetooth read");
   return -1;
 }
 
 ssize_t
 bthWriteData (BluetoothConnection *connection, const void *buffer, size_t size) {
   errno = ENOSYS;
-  LogError("Bluetooth write");
+  logSystemError("Bluetooth write");
   return -1;
 }
