@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -43,7 +43,7 @@ bthSetErrno (DWORD error, const char *action, const DWORD *exceptions) {
         goto isException;
   }
 
-  LogWindowsCodeError(error, action);
+  logWindowsError(error, action);
 isException:
   setErrno(error);
 }
@@ -104,7 +104,7 @@ bthConnect (uint64_t bda, uint8_t channel) {
 
       free(bcx);
     } else {
-      LogError("malloc");
+      logMallocError();
     }
 
     WSACleanup();
@@ -160,7 +160,7 @@ bthReadData (
   size_t count = size;
 
   if (count) {
-    unsigned char *to = buffer;
+    char *to = buffer;
 
     while (1) {
       int result = recv(bcx->socket, to, count, 0);
@@ -197,7 +197,7 @@ bthReadData (
 ssize_t
 bthWriteData (BluetoothConnection *connection, const void *buffer, size_t size) {
   BluetoothConnectionExtension *bcx = connection->extension;
-  const unsigned char *from = buffer;
+  const char *from = buffer;
   size_t count = size;
 
   while (count) {
