@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -30,6 +30,7 @@
 #include "charset.h"
 #include "unicode.h"
 #include "drivers.h"
+#include "io_generic.h"
 #include "brl.h"
 #include "ttb.h"
 #include "ktb.h"
@@ -47,21 +48,22 @@
 #include "brl_driver.h"
 
 static int
-brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
+brl_construct (BrailleDisplay *brl UNUSED, char **parameters UNUSED, const char *device UNUSED) {
+  brl->keyBindings = NULL;
   return 1;
 }
 
 static void
-brl_destruct (BrailleDisplay *brl) {
+brl_destruct (BrailleDisplay *brl UNUSED) {
 }
 
 static int
-brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
+brl_readCommand (BrailleDisplay *brl UNUSED, KeyTableCommandContext context UNUSED) {
   return EOF;
 }
 
 static int
-brl_writeWindow (BrailleDisplay *brl, const wchar_t *characters) {
+brl_writeWindow (BrailleDisplay *brl UNUSED, const wchar_t *characters UNUSED) {
   return 1;
 }
 
@@ -120,6 +122,8 @@ initializeBrailleDisplay (BrailleDisplay *brl) {
   brl->touchEnabled = 0;
   brl->highlightWindow = 0;
   brl->data = NULL;
+  brl->setFirmness = NULL;
+  brl->setSensitivity = NULL;
 }
 
 unsigned int
@@ -232,10 +236,10 @@ clearStatusCells (BrailleDisplay *brl) {
 static void
 brailleBufferResized (BrailleDisplay *brl, int infoLevel) {
   memset(brl->buffer, 0, brl->textColumns*brl->textRows);
-  LogPrint(infoLevel, "Braille Display Dimensions: %d %s, %d %s",
-           brl->textRows, (brl->textRows == 1)? "row": "rows",
-           brl->textColumns, (brl->textColumns == 1)? "column": "columns");
-  if (brl->bufferResized) brl->bufferResized(infoLevel, brl->textRows, brl->textColumns);
+  logMessage(infoLevel, "Braille Display Dimensions: %d %s, %d %s",
+             brl->textRows, (brl->textRows == 1)? "row": "rows",
+             brl->textColumns, (brl->textColumns == 1)? "column": "columns");
+  if (brl->bufferResized) brl->bufferResized(brl->textRows, brl->textColumns);
 }
 
 static int
@@ -252,7 +256,7 @@ resizeBrailleBuffer (BrailleDisplay *brl, int infoLevel) {
         void *newAddress = malloc(newSize);
 
         if (!newAddress) {
-          LogError("malloc");
+          logMallocError();
           return 0;
         }
 
@@ -405,11 +409,100 @@ dequeueKeyEvent (unsigned char *set, unsigned char *key, int *press) {
 }
 
 int
-readBrailleCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
-  int command = dequeueCommand();
+enqueueKey (unsigned char set, unsigned char key) {
+  if (enqueueKeyEvent(set, key, 1))
+    if (enqueueKeyEvent(set, key, 0))
+      return 1;
 
-  if (command == EOF) {
-    command = braille->readCommand(brl, context);
+  return 0;
+}
+
+int
+enqueueKeys (uint32_t bits, unsigned char set, unsigned char key) {
+  unsigned char stack[0X20];
+  unsigned char count = 0;
+
+  while (bits) {
+    if (bits & 0X1) {
+      if (!enqueueKeyEvent(set, key, 1)) return 0;
+      stack[count++] = key;
+    }
+
+    bits >>= 1;
+    key += 1;
+  }
+
+  while (count)
+    if (!enqueueKeyEvent(set, stack[--count], 0))
+      return 0;
+
+  return 1;
+}
+
+int
+enqueueUpdatedKeys (uint32_t new, uint32_t *old, unsigned char set, unsigned char key) {
+  uint32_t bit = 0X1;
+  unsigned char stack[0X20];
+  unsigned char count = 0;
+
+  while (*old != new) {
+    if ((new & bit) && !(*old & bit)) {
+      stack[count++] = key;
+      *old |= bit;
+    } else if (!(new & bit) && (*old & bit)) {
+      if (!enqueueKeyEvent(set, key, 0)) return 0;
+      *old &= ~bit;
+    }
+
+    key += 1;
+    bit <<= 1;
+  }
+
+  while (count)
+    if (!enqueueKeyEvent(set, stack[--count], 1))
+      return 0;
+
+  return 1;
+}
+
+int
+enqueueXtScanCode (
+  unsigned char key, unsigned char escape,
+  unsigned char set00, unsigned char setE0, unsigned char setE1
+) {
+  unsigned char set;
+
+  switch (escape) {
+    case 0X00: set = set00; break;
+    case 0XE0: set = setE0; break;
+    case 0XE1: set = setE1; break;
+
+    default:
+      logMessage(LOG_WARNING, "unsupported XT scan code: %02X %02X", escape, key);
+      return 0;
+  }
+
+  return enqueueKey(set, key);
+}
+
+static KeyTableCommandContext currentCommandContext = KTB_CTX_DEFAULT;
+
+KeyTableCommandContext
+getCurrentCommandContext (void) {
+  return currentCommandContext;
+}
+
+int
+readBrailleCommand (BrailleDisplay *brl, KeyTableCommandContext context) {
+  currentCommandContext = context;
+
+  {
+    int command = dequeueCommand();
+    if (command != EOF) return command;
+  }
+
+  {
+    int command = braille->readCommand(brl, context);
     resizeBrailleBuffer(brl, LOG_INFO);
 
     {
@@ -424,25 +517,95 @@ readBrailleCommand (BrailleDisplay *brl, BRL_DriverCommandContext context) {
       }
     }
 
-    if (command == EOF) command = dequeueCommand();
+    if (command != EOF) enqueueCommand(command);
   }
 
-  return command;
+  {
+    int command = dequeueCommand();
+
+#ifdef ENABLE_API
+    if (command == EOF) command = api_handleCommand(command);
+#endif /* ENABLE_API */
+
+    return command;
+  }
+}
+
+int
+writeBraillePacket (
+  BrailleDisplay *brl, GioEndpoint *endpoint,
+  const void *packet, size_t size
+) {
+  logOutputPacket(packet, size);
+  if (gioWriteData(endpoint, packet, size) == -1) return 0;
+  brl->writeDelay += gioGetMillisecondsToTransfer(endpoint, size);
+  return 1;
+}
+
+size_t
+probeBrailleDisplay (
+  BrailleDisplay *brl, unsigned int retryLimit,
+  GioEndpoint *endpoint, int inputTimeout,
+  BrailleRequestWriter writeRequest,
+  BraillePacketReader readPacket, void *responsePacket, size_t responseSize,
+  BrailleResponseHandler *handleResponse
+) {
+  unsigned int retryCount = 0;
+
+  while (writeRequest(brl)) {
+    drainBrailleOutput(brl, 0);
+
+    while (gioAwaitInput(endpoint, inputTimeout)) {
+      size_t size = readPacket(brl, responsePacket, responseSize);
+      if (!size) break;
+
+      {
+        BrailleResponseResult result = handleResponse(brl, responsePacket, size);
+
+        switch (result) {
+          case BRL_RSP_CONTINUE:
+            break;
+
+          case BRL_RSP_DONE:
+            return 1;
+
+          case BRL_RSP_FAIL:
+            return 0;
+
+          case BRL_RSP_UNEXPECTED:
+            logUnexpectedPacket(responsePacket, size);
+            break;
+
+          default:
+            logMessage(LOG_WARNING, "unimplemented braille response result: %u", result);
+            return 0;
+        }
+      }
+    }
+
+    if (errno != EAGAIN) break;
+    if (retryCount == retryLimit) break;
+    retryCount += 1;
+  }
+
+  return 0;
 }
 
 #ifdef ENABLE_LEARN_MODE
 int
 learnMode (BrailleDisplay *brl, int poll, int timeout) {
   const char *mode = "lrn";
+  TimeValue start;
 
   if (!setStatusText(brl, mode)) return 0;
-  if (!message(mode, "command learn mode", MSG_NODELAY)) return 0;
+  if (!message(mode, gettext("Command Learn Mode"), MSG_NODELAY)) return 0;
 
-  hasTimedOut(0);
+  getCurrentTime(&start);
   do {
-    int command = readBrailleCommand(brl, BRL_CTX_DEFAULT);
+    int command = readBrailleCommand(brl, KTB_CTX_DEFAULT);
+
     if (command != EOF) {
-      LogPrint(LOG_DEBUG, "Learn: command=%06X", command);
+      logMessage(LOG_DEBUG, "Learn: command=%06X", command);
       if (BRL_DELAYED_COMMAND(command)) continue;
 
       {
@@ -453,49 +616,202 @@ learnMode (BrailleDisplay *brl, int poll, int timeout) {
 
       {
         char buffer[0X100];
-        describeCommand(command, buffer, sizeof(buffer), 1);
-        LogPrint(LOG_DEBUG, "Learn: %s", buffer);
-        if (!message(mode, buffer, MSG_NODELAY|MSG_SILENT)) return 0;
+        describeCommand(command, buffer, sizeof(buffer),
+                        CDO_IncludeName | CDO_IncludeOperand);
+        logMessage(LOG_DEBUG, "Learn: %s", buffer);
+        if (!message(mode, buffer, MSG_NODELAY)) return 0;
       }
 
-      hasTimedOut(0);
+      getCurrentTime(&start);
     }
 
     drainBrailleOutput(brl, poll);
-  } while (!hasTimedOut(timeout));
+  } while (millisecondsSince(&start) < timeout);
 
-  return message(mode, "done", 0);
+  return message(mode, gettext("done"), 0);
 }
 #endif /* ENABLE_LEARN_MODE */
 
-void
-reverseTranslationTable (TranslationTable from, TranslationTable to) {
-  int byte;
-  memset(to, 0, sizeof(TranslationTable));
-  for (byte=TRANSLATION_TABLE_SIZE-1; byte>=0; byte--) to[from[byte]] = byte;
+int
+cellsHaveChanged (
+  unsigned char *cells, const unsigned char *new, unsigned int count,
+  unsigned int *from, unsigned int *to, int *force
+) {
+  unsigned int first = 0;
+
+  if (force && *force) {
+    *force = 0;
+  } else if (memcmp(cells, new, count) != 0) {
+    if (to) {
+      while (count) {
+        unsigned int last = count - 1;
+        if (cells[last] != new[last]) break;
+        count = last;
+      }
+    }
+
+    if (from) {
+      while (first < count) {
+        if (cells[first] != new[first]) break;
+        first += 1;
+      }
+    }
+  } else {
+    return 0;
+  }
+
+  if (from) *from = first;
+  if (to) *to = count;
+
+  memcpy(cells+first, new+first, count-first);
+  return 1;
 }
 
+int
+textHasChanged (
+  wchar_t *text, const wchar_t *new, unsigned int count,
+  unsigned int *from, unsigned int *to, int *force
+) {
+  unsigned int first = 0;
+
+  if (force && *force) {
+    *force = 0;
+  } else if (wmemcmp(text, new, count) != 0) {
+    if (to) {
+      while (count) {
+        unsigned int last = count - 1;
+        if (text[last] != new[last]) break;
+        count = last;
+      }
+    }
+
+    if (from) {
+      while (first < count) {
+        if (text[first] != new[first]) break;
+        first += 1;
+      }
+    }
+  } else {
+    return 0;
+  }
+
+  if (from) *from = first;
+  if (to) *to = count;
+
+  wmemcpy(text+first, new+first, count-first);
+  return 1;
+}
+
+int
+cursorHasChanged (int *cursor, int new, int *force) {
+  if (force && *force) {
+    *force = 0;
+  } else if (new == *cursor) {
+    return 0;
+  }
+
+  *cursor = new;
+  return 1;
+}
+
+const DotsTable dotsTable_ISO11548_1 = {
+  BRL_DOT1, BRL_DOT2, BRL_DOT3, BRL_DOT4,
+  BRL_DOT5, BRL_DOT6, BRL_DOT7, BRL_DOT8
+};
+
 void
-makeOutputTable (const DotsTable dots, TranslationTable table) {
+makeTranslationTable (const DotsTable dots, TranslationTable table) {
   int byte;
 
   for (byte=0; byte<TRANSLATION_TABLE_SIZE; byte+=1) {
     unsigned char cell = 0;
     int dot;
 
-    for (dot=0; dot<DOTS_TABLE_SIZE; dot+=1) {
-      static const DotsTable internalDots = {
-        BRL_DOT1, BRL_DOT2, BRL_DOT3, BRL_DOT4,
-        BRL_DOT5, BRL_DOT6, BRL_DOT7, BRL_DOT8
-      };
-
-      if (byte & internalDots[dot]) cell |= dots[dot];
-    }
+    for (dot=0; dot<DOTS_TABLE_SIZE; dot+=1)
+      if (byte & dotsTable_ISO11548_1[dot])
+        cell |= dots[dot];
 
     table[byte] = cell;
   }
 }
 
+void
+reverseTranslationTable (const TranslationTable from, TranslationTable to) {
+  int byte;
+  memset(to, 0, sizeof(TranslationTable));
+  for (byte=TRANSLATION_TABLE_SIZE-1; byte>=0; byte--) to[from[byte]] = byte;
+}
+
+static inline void *
+translateCells (
+  const TranslationTable table,
+  unsigned char *target, const unsigned char *source, size_t count
+) {
+  if (table) {
+    while (count--) *target++ = table[*source++];
+    return target;
+  }
+
+  if (target == source) return target + count;
+  return mempcpy(target, source, count);
+}
+
+static inline unsigned char
+translateCell (const TranslationTable table, unsigned char cell) {
+  return table? table[cell]: cell;
+}
+
+static TranslationTable internalOutputTable;
+static const unsigned char *outputTable;
+
+void
+setOutputTable (const TranslationTable table) {
+  outputTable = table;
+}
+
+void
+makeOutputTable (const DotsTable dots) {
+  if (memcmp(dots, dotsTable_ISO11548_1, DOTS_TABLE_SIZE) == 0) {
+    outputTable = NULL;
+  } else {
+    makeTranslationTable(dots, internalOutputTable);
+    outputTable = internalOutputTable;
+  }
+}
+
+void *
+translateOutputCells (unsigned char *target, const unsigned char *source, size_t count) {
+  return translateCells(outputTable, target, source, count);
+}
+
+unsigned char
+translateOutputCell (unsigned char cell) {
+  return translateCell(outputTable, cell);
+}
+
+static TranslationTable internalInputTable;
+static const unsigned char *inputTable;
+
+void
+makeInputTable (void) {
+  if (outputTable) {
+    reverseTranslationTable(outputTable, internalInputTable);
+    inputTable = internalInputTable;
+  } else {
+    inputTable = NULL;
+  }
+}
+
+void *
+translateInputCells (unsigned char *target, const unsigned char *source, size_t count) {
+  return translateCells(inputTable, target, source, count);
+}
+
+unsigned char
+translateInputCell (unsigned char cell) {
+  return translateCell(inputTable, cell);
+}
+
 /* Functions which support vertical and horizontal status cells. */
 
 unsigned char
@@ -580,14 +896,16 @@ portraitFlag (int number, int on) {
   return dots;
 }
 
-void
+int
 setBrailleFirmness (BrailleDisplay *brl, BrailleFirmness setting) {
-  LogPrint(LOG_DEBUG, "setting braille firmness: %d", setting);
-  braille->firmness(brl, setting);
+  if (!brl->setFirmness) return 0;
+  logMessage(LOG_DEBUG, "setting braille firmness: %d", setting);
+  return brl->setFirmness(brl, setting);
 }
 
-void
+int
 setBrailleSensitivity (BrailleDisplay *brl, BrailleSensitivity setting) {
-  LogPrint(LOG_DEBUG, "setting braille sensitivity: %d", setting);
-  braille->sensitivity(brl, setting);
+  if (!brl->setSensitivity) return 0;
+  logMessage(LOG_DEBUG, "setting braille sensitivity: %d", setting);
+  return brl->setSensitivity(brl, setting);
 }
