@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -19,38 +19,22 @@
 #ifndef BRLTTY_INCLUDED_BRL
 #define BRLTTY_INCLUDED_BRL
 
+#include "prologue.h"
+
+#include "driver.h"
+#include "io_generic.h"
+#include "brldefs.h"
+#include "ktbdefs.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
-#include "prologue.h"
-
-#include "driver.h"
-#include "brldefs.h"
-#include "ktbdefs.h"
-
-typedef enum {
-  sfEnd,
-  sfWindowCoordinates,
-  sfWindowColumn,
-  sfWindowRow,
-  sfCursorCoordinates,
-  sfCursorColumn,
-  sfCursorRow,
-  sfCursorAndWindowColumn,
-  sfCursorAndWindowRow,
-  sfScreenNumber,
-  sfStateDots,
-  sfStateLetter,
-  sfTime,
-  sfAlphabeticWindowCoordinates,
-  sfAlphabeticCursorCoordinates,
-  sfGeneric
-} StatusField;
-
 typedef struct BrailleDataStruct BrailleData;
 
-typedef struct {
+typedef struct BrailleDisplayStruct BrailleDisplay;
+
+struct BrailleDisplayStruct {
   unsigned int textColumns, textRows;
   unsigned int statusColumns, statusRows;
 
@@ -63,11 +47,14 @@ typedef struct {
   unsigned isCoreBuffer:1;
   unsigned resizeRequired:1;
   unsigned int writeDelay;
-  void (*bufferResized) (int infoLevel, int rows, int columns);
+  void (*bufferResized) (unsigned int rows, unsigned int columns);
   unsigned touchEnabled:1;
   unsigned highlightWindow:1;
   BrailleData *data;
-} BrailleDisplay;
+
+  int (*setFirmness) (BrailleDisplay *brl, BrailleFirmness setting);
+  int (*setSensitivity) (BrailleDisplay *brl, BrailleSensitivity setting);
+};
 
 extern void initializeBrailleDisplay (BrailleDisplay *brl);
 extern unsigned int drainBrailleOutput (BrailleDisplay *brl, int minimumDelay);
@@ -92,25 +79,53 @@ extern int setStatusText (BrailleDisplay *brl, const char *text);
 
 extern int enqueueCommand (int command);
 extern int enqueueKeyEvent (unsigned char set, unsigned char key, int press);
-extern int readBrailleCommand (BrailleDisplay *, BRL_DriverCommandContext);
+
+extern int enqueueKey (unsigned char set, unsigned char key);
+extern int enqueueKeys (uint32_t bits, unsigned char set, unsigned char key);
+extern int enqueueUpdatedKeys (uint32_t new, uint32_t *old, unsigned char set, unsigned char key);
+
+extern int enqueueXtScanCode (
+  unsigned char code, unsigned char escape,
+  unsigned char set00, unsigned char setE0, unsigned char setE1
+);
+
+extern int readBrailleCommand (BrailleDisplay *, KeyTableCommandContext);
+extern KeyTableCommandContext getCurrentCommandContext (void);
+
+extern int writeBraillePacket (
+  BrailleDisplay *brl, GioEndpoint *endpoint,
+  const void *packet, size_t size
+);
+
+typedef int BrailleRequestWriter (BrailleDisplay *brl);
+
+typedef size_t BraillePacketReader (
+  BrailleDisplay *brl,
+  void *packet, size_t size
+);
 
 typedef enum {
-  BRL_FIRMNESS_MINIMUM,
-  BRL_FIRMNESS_LOW,
-  BRL_FIRMNESS_MEDIUM,
-  BRL_FIRMNESS_HIGH,
-  BRL_FIRMNESS_MAXIMUM
-} BrailleFirmness;
-extern void setBrailleFirmness (BrailleDisplay *brl, BrailleFirmness setting);
+  BRL_RSP_CONTINUE,
+  BRL_RSP_DONE,
+  BRL_RSP_FAIL,
+  BRL_RSP_UNEXPECTED
+} BrailleResponseResult;
 
-typedef enum {
-  BRL_SENSITIVITY_MINIMUM,
-  BRL_SENSITIVITY_LOW,
-  BRL_SENSITIVITY_MEDIUM,
-  BRL_SENSITIVITY_HIGH,
-  BRL_SENSITIVITY_MAXIMUM
-} BrailleSensitivity;
-extern void setBrailleSensitivity (BrailleDisplay *brl, BrailleSensitivity setting);
+typedef BrailleResponseResult BrailleResponseHandler (
+  BrailleDisplay *brl,
+  const void *packet, size_t size
+);
+
+extern size_t probeBrailleDisplay (
+  BrailleDisplay *brl, unsigned int retryLimit,
+  GioEndpoint *endpoint, int inputTimeout,
+  BrailleRequestWriter writeRequest,
+  BraillePacketReader readPacket, void *responsePacket, size_t responseSize,
+  BrailleResponseHandler *handleResponse
+);
+
+extern int setBrailleFirmness (BrailleDisplay *brl, BrailleFirmness setting);
+extern int setBrailleSensitivity (BrailleDisplay *brl, BrailleSensitivity setting);
 
 /* Routines provided by each braille driver.
  * These are loaded dynamically at run-time into this structure
@@ -118,32 +133,23 @@ extern void setBrailleSensitivity (BrailleDisplay *brl, BrailleSensitivity setti
  */
 typedef struct {
   DRIVER_DEFINITION_DECLARATION;
+
   const char *const *parameters;
   const unsigned char *statusFields;
 
-  /* Routines provided by the braille driver library: */
   int (*construct) (BrailleDisplay *brl, char **parameters, const char *device);
   void (*destruct) (BrailleDisplay *brl);
-  int (*readCommand) (BrailleDisplay *brl, BRL_DriverCommandContext context);
-  int (*writeWindow) (BrailleDisplay *brl, const wchar_t *characters);
 
-  /* These require BRL_HAVE_STATUS_CELLS. */
+  int (*readCommand) (BrailleDisplay *brl, KeyTableCommandContext context);
+  int (*writeWindow) (BrailleDisplay *brl, const wchar_t *characters);
   int (*writeStatus) (BrailleDisplay *brl, const unsigned char *cells);
 
-  /* These require BRL_HAVE_PACKET_IO. */
   ssize_t (*readPacket) (BrailleDisplay *brl, void *buffer, size_t size);
   ssize_t (*writePacket) (BrailleDisplay *brl, const void *packet, size_t size);
   int (*reset) (BrailleDisplay *brl);
   
-  /* These require BRL_HAVE_KEY_CODES. */
   int (*readKey) (BrailleDisplay *brl);
-  int (*keyToCommand) (BrailleDisplay *brl, BRL_DriverCommandContext context, int key);
-
-  /* These require BRL_HAVE_FIRMNESS. */
-  void (*firmness) (BrailleDisplay *brl, BrailleFirmness setting);
-
-  /* These require BRL_HAVE_SENSITIVITY. */
-  void (*sensitivity) (BrailleDisplay *brl, BrailleSensitivity setting);
+  int (*keyToCommand) (BrailleDisplay *brl, KeyTableCommandContext context, int key);
 } BrailleDriver;
 
 extern int haveBrailleDriver (const char *code);
@@ -154,13 +160,36 @@ extern void identifyBrailleDrivers (int full);
 extern const BrailleDriver *braille;
 extern const BrailleDriver noBraille;
 
+extern int cellsHaveChanged (
+  unsigned char *cells, const unsigned char *new, unsigned int count,
+  unsigned int *from, unsigned int *to, int *force
+);
+
+extern int textHasChanged (
+  wchar_t *text, const wchar_t *new, unsigned int count,
+  unsigned int *from, unsigned int *to, int *force
+);
+
+extern int cursorHasChanged (int *cursor, int new, int *force);
+
 #define TRANSLATION_TABLE_SIZE 0X100
 typedef unsigned char TranslationTable[TRANSLATION_TABLE_SIZE];
-extern void reverseTranslationTable (TranslationTable from, TranslationTable to);
 
 #define DOTS_TABLE_SIZE 8
 typedef unsigned char DotsTable[DOTS_TABLE_SIZE];
-extern void makeOutputTable (const DotsTable dots, TranslationTable table);
+extern const DotsTable dotsTable_ISO11548_1;
+
+extern void makeTranslationTable (const DotsTable dots, TranslationTable table);
+extern void reverseTranslationTable (const TranslationTable from, TranslationTable to);
+
+extern void setOutputTable (const TranslationTable table);
+extern void makeOutputTable (const DotsTable dots);
+extern void *translateOutputCells (unsigned char *target, const unsigned char *source, size_t count);
+extern unsigned char translateOutputCell (unsigned char cell);
+
+extern void makeInputTable (void);
+extern void *translateInputCells (unsigned char *target, const unsigned char *source, size_t count);
+extern unsigned char translateInputCell (unsigned char cell);
 
 /* Formatting of status cells. */
 extern unsigned char lowerDigit (unsigned char upper);
