@@ -1,7 +1,7 @@
 /*
  * libbrlapi - A library providing access to braille terminals for applications.
  *
- * Copyright (C) 2002-2010 by
+ * Copyright (C) 2002-2012 by
  *   Samuel Thibault <Samuel.Thibault@ens-lyon.org>
  *   Sébastien Hinderer <Sebastien.Hinderer@ens-lyon.org>
  *
@@ -31,7 +31,9 @@
 #include <locale.h>
 
 #ifndef __MINGW32__
+#ifdef HAVE_LANGINFO_H
 #include <langinfo.h>
+#endif /* HAVE_LANGINFO_H */
 #endif /* __MINGW32__ */
 
 #ifdef __MINGW32__
@@ -1093,52 +1095,57 @@ int BRLAPI_STDCALL brlapi_setFocus(int tty)
   return brlapi__setFocus(&defaultHandle, tty);
 }
 
+static size_t getCharset(
+  void *buffer
 #ifdef WINDOWS
-static size_t getCharset(unsigned char *buffer, int wide)
-#else /* WINDOWS */
-static size_t getCharset(unsigned char *buffer)
+, int wide
 #endif /* WINDOWS */
-{
-  unsigned char *p = buffer;
-  char *locale;
-  unsigned int len;
-  locale = setlocale(LC_CTYPE,NULL);
+) {
+  char *p = buffer;
+  const char *start = p;
+  const char *locale = setlocale(LC_CTYPE, NULL);
+
 #ifdef WINDOWS
+  static const char WIN_WCHAR_T[] =
 #ifdef WORDS_BIGENDIAN
-#define WIN_WCHAR_T "UCS-2BE"
+    "UCS-2BE"
 #else /* WORDS_BIGENDIAN */
-#define WIN_WCHAR_T "UCS-2LE"
+    "UCS-2LE"
 #endif /* WORDS_BIGENDIAN */
-#ifdef __CYGWIN32__
-  if (wide) {
-#else /* __CYGWIN32__ */
-  if (CHECKPROC("ntdll.dll", wcslen) && wide) {
+    ;
+
+  if (wide
+#ifndef __CYGWIN32__
+      && CHECKPROC("ntdll.dll", wcslen)
 #endif /* __CYGWIN32__ */
-    *p++ = strlen(WIN_WCHAR_T);
-    strcpy((char*) p, WIN_WCHAR_T);
-    p += strlen(WIN_WCHAR_T);
+  ) {
+    size_t length = strlen(WIN_WCHAR_T);
+    *p++ = length;
+    p = mempcpy(p, WIN_WCHAR_T, length);
   } else
 #endif /* WINDOWS */
-  if (locale && strcmp(locale,"C")) {
+
+  if (locale && strcmp(locale, "C")) {
     /* not default locale, tell charset to server */
 #ifdef WINDOWS
-    UINT CP;
-    if ((CP = GetACP() || (CP = GetOEMCP()))) {
-      len = sprintf(p+3, "%u", CP);
-      *p++ = 2 + len;
-      *p++ = 'C';
-      *p++ = 'P';
-      p += len;
+    UINT CP = GetACP();
+    if (!CP) CP = GetOEMCP();
+
+    if (CP) {
+      size_t length = sprintf(p+1, "CP%u", CP);
+      *p++ = length;
+      p += length;
     }
-#else /* WINDOWS */
-    char *lang = nl_langinfo(CODESET);
-    len = strlen(lang);
-    *p++ = len;
-    memcpy(p, lang, len);
-    p += len;
-#endif /* WINDOWS */
+#elif defined(CODESET)
+    char *language = nl_langinfo(CODESET);
+    size_t length = strlen(language);
+
+    *p++ = length;
+    p = mempcpy(p, language, length);
+#endif /* get non-wide character set */
   }
-  return p-buffer;
+
+  return p-start;
 }
 
 /* Function : brlapi_writeText */
