@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -41,26 +41,17 @@ int updateInterval = DEFAULT_UPDATE_INTERVAL;
 static BrailleDisplay brl;
 
 static char *opt_brailleDevice;
-static char *opt_libraryDirectory;
+static char *opt_driversDirectory;
 static char *opt_writableDirectory;
 static char *opt_dataDirectory;
 
 BEGIN_OPTION_TABLE(programOptions)
   { .letter = 'D',
-    .word = "data-directory",
+    .word = "drivers-directory",
     .flags = OPT_Hidden,
     .argument = "directory",
-    .setting.string = &opt_dataDirectory,
-    .defaultSetting = DATA_DIRECTORY,
-    .description = "Path to directory for driver help and configuration files."
-  },
-
-  { .letter = 'L',
-    .word = "library-directory",
-    .flags = OPT_Hidden,
-    .argument = "directory",
-    .setting.string = &opt_libraryDirectory,
-    .defaultSetting = LIBRARY_DIRECTORY,
+    .setting.string = &opt_driversDirectory,
+    .defaultSetting = DRIVERS_DIRECTORY,
     .description = "Path to directory for loading drivers."
   },
 
@@ -83,34 +74,36 @@ BEGIN_OPTION_TABLE(programOptions)
 END_OPTION_TABLE
 
 int
-message (const char *mode, const char *string, short flags) {
-  size_t length = strlen(string);
+message (const char *mode, const char *text, short flags) {
   size_t size = brl.textColumns * brl.textRows;
-  char buffer[size];
+  wchar_t buffer[size];
+
+  size_t length = getTextLength(text);
+  wchar_t characters[length + 1];
+  const wchar_t *character = characters;
 
   clearStatusCells(&brl);
+  convertTextToWchars(characters, text, ARRAY_COUNT(characters));
 
-  memset(buffer, ' ', size);
   while (length) {
     int count = (length <= size)? length: (size - 1);
-    int index;
 
-    for (index=0; index<count; buffer[index++]=*string++);
-    if (length -= count) {
+    wmemcpy(buffer, character, count);
+    character += count;
+    length -= count;
+
+    if (length) {
       buffer[(count = size) - 1] = WC_C('-');
     }
 
     {
-      wchar_t characters[count];
-      wchar_t text[brl.textColumns * brl.textRows];
-      convertCharsToWchars(buffer, characters, count);
-      fillTextRegion(text, brl.buffer, 0, brl.textColumns, brl.textColumns, brl.textRows, characters, count);
-      if (!braille->writeWindow(&brl, text)) return 0;
+      wmemset(&buffer[count], WC_C(' '), (size - count));
+      if (!braille->writeWindow(&brl, buffer)) return 0;
     }
 
     if (length) {
       int timer = 0;
-      while (braille->readCommand(&brl, BRL_CTX_WAITING) == EOF) {
+      while (braille->readCommand(&brl, KTB_CTX_WAITING) == EOF) {
         if (timer > 4000) break;
         approximateDelay(updateInterval);
         timer += updateInterval;
@@ -123,7 +116,7 @@ message (const char *mode, const char *string, short flags) {
 
 int
 main (int argc, char *argv[]) {
-  int status;
+  ProgramExitStatus exitStatus;
   const char *driver = NULL;
   void *object;
 
@@ -133,12 +126,12 @@ main (int argc, char *argv[]) {
       .applicationName = "brltest",
       .argumentsSummary = "[driver [parameter=value ...]]"
     };
-    processOptions(&descriptor, &argc, &argv);
+    PROCESS_OPTIONS(descriptor, argc, argv);
   }
 
   {
     char **const paths[] = {
-      &opt_libraryDirectory,
+      &opt_driversDirectory,
       &opt_writableDirectory,
       &opt_dataDirectory,
       NULL
@@ -154,7 +147,7 @@ main (int argc, char *argv[]) {
   }
   if (!opt_brailleDevice) opt_brailleDevice = BRAILLE_DEVICE;
 
-  if ((braille = loadBrailleDriver(driver, &object, opt_libraryDirectory))) {
+  if ((braille = loadBrailleDriver(driver, &object, opt_driversDirectory))) {
     const char *const *parameterNames = braille->parameters;
     char **parameterSettings;
     if (!parameterNames) {
@@ -168,8 +161,8 @@ main (int argc, char *argv[]) {
       while (*name) ++name;
       count = name - parameterNames;
       if (!(parameterSettings = malloc((count + 1) * sizeof(*parameterSettings)))) {
-        LogPrint(LOG_ERR, "insufficient memory.");
-        exit(9);
+        logMallocError();
+        return PROG_EXIT_FATAL;
       }
       setting = parameterSettings;
       while (count--) *setting++ = "";
@@ -180,9 +173,9 @@ main (int argc, char *argv[]) {
       int ok = 0;
       char *delimiter = strchr(assignment, '=');
       if (!delimiter) {
-        LogPrint(LOG_ERR, "missing braille driver parameter value: %s", assignment);
+        logMessage(LOG_ERR, "missing braille driver parameter value: %s", assignment);
       } else if (delimiter == assignment) {
-        LogPrint(LOG_ERR, "missing braille driver parameter name: %s", assignment);
+        logMessage(LOG_ERR, "missing braille driver parameter name: %s", assignment);
       } else {
         size_t nameLength = delimiter - assignment;
         const char *const *name = parameterNames;
@@ -194,9 +187,9 @@ main (int argc, char *argv[]) {
           }
           ++name;
         }
-        if (!ok) LogPrint(LOG_ERR, "invalid braille driver parameter: %s", assignment);
+        if (!ok) logMessage(LOG_ERR, "invalid braille driver parameter: %s", assignment);
       }
-      if (!ok) exit(2);
+      if (!ok) return PROG_EXIT_SYNTAX;
       --argc;
     }
 
@@ -210,25 +203,25 @@ main (int argc, char *argv[]) {
         message("braille test", 0);
 #endif /* ENABLE_LEARN_MODE */
         braille->destruct(&brl);		/* finish with the display */
-        status = 0;
+        exitStatus = PROG_EXIT_SUCCESS;
       } else {
-        LogPrint(LOG_ERR, "can't allocate braille buffer.");
-        status = 6;
+        logMessage(LOG_ERR, "can't allocate braille buffer.");
+        exitStatus = PROG_EXIT_FATAL;
       }
     } else {
-      LogPrint(LOG_ERR, "can't initialize braille driver.");
-      status = 5;
+      logMessage(LOG_ERR, "can't initialize braille driver.");
+      exitStatus = PROG_EXIT_FATAL;
     }
   } else {
-    LogPrint(LOG_ERR, "can't load braille driver.");
-    status = 3;
+    logMessage(LOG_ERR, "can't load braille driver.");
+    exitStatus = PROG_EXIT_FATAL;
   }
-  return status;
+  return exitStatus;
 }
 
 /* dummy functions to allow drivers to link... */
 unsigned char
-cursorDots (void) {
+getCursorDots (void) {
   return (BRL_DOT7 | BRL_DOT8);
 }
 
