@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -21,109 +21,42 @@
 
 #include "prologue.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+#include "program.h"
 #include "cmd.h"
 #include "brl.h"
 #include "spk.h"
 #include "ctb.h"
 #include "ktb.h"
+#include "scr.h"
+#include "ses.h"
+#include "menu.h"
+#include "prefs.h"
 
-typedef enum {
-  sbwAll,
-  sbwEndOfLine,
-  sbwRestOfLine
-} SkipBlankWindowsMode;
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
 
-typedef enum {
-  sayImmediate,
-  sayEnqueue
-} SayMode;
+extern ScreenDescription scr;
+#define SCR_COORDINATE_OK(coordinate,limit) (((coordinate) >= 0) && ((coordinate) < (limit)))
+#define SCR_COLUMN_OK(column) SCR_COORDINATE_OK((column), scr.cols)
+#define SCR_ROW_OK(row) SCR_COORDINATE_OK((row), scr.rows)
+#define SCR_COORDINATES_OK(column,row) (SCR_COLUMN_OK((column)) && SCR_ROW_OK((row)))
+#define SCR_CURSOR_OK() SCR_COORDINATES_OK(scr.posx, scr.posy)
+#define SCR_COLUMN_NUMBER(column) (SCR_COLUMN_OK((column))? (column)+1: 0)
+#define SCR_ROW_NUMBER(row) (SCR_ROW_OK((row))? (row)+1: 0)
 
-typedef enum {
-  spNone,
-  spLeft,
-  spRight
-} StatusPosition;
-
-typedef enum {
-  ssNone,
-  ssSpace,
-  ssBlock,
-  ssStatusSide,
-  ssTextSide
-} StatusSeparator;
-
-/*
- * Structure definition for preferences (settings which are saveable).
- * PREFS_MAGIC_NUMBER has to be bumped whenever the definition of
- * Preferences is modified otherwise this structure could be
- * filled with incompatible data from disk.
- */
-#define PREFS_MAGIC_NUMBER 0x4005
-
-typedef struct {
-  unsigned char magic[2];
-  unsigned char showCursor;
-  unsigned char version;
-  unsigned char showAttributes;
-  unsigned char brailleSensitivity;
-  unsigned char blinkingCursor;
-  unsigned char autorepeat;
-  unsigned char blinkingCapitals;
-  unsigned char autorepeatDelay;
-  unsigned char blinkingAttributes;
-  unsigned char autorepeatInterval;
-  unsigned char cursorStyle;
-  unsigned char sayLineMode;
-  unsigned char cursorVisibleTime;
-  unsigned char autospeak;
-  unsigned char cursorInvisibleTime;
-  unsigned char pcmVolume;
-  unsigned char capitalsVisibleTime;
-  unsigned char midiVolume;
-  unsigned char capitalsInvisibleTime;
-  unsigned char fmVolume;
-  unsigned char attributesVisibleTime;
-  unsigned char highlightWindow;
-  unsigned char attributesInvisibleTime;
-  unsigned char windowFollowsPointer;
-  unsigned char textStyle;
-  unsigned char autorepeatPanning;
-  unsigned char slidingWindow;
-  unsigned char eagerSlidingWindow;
-  unsigned char alertTunes;
-  unsigned char tuneDevice;
-  unsigned char skipIdenticalLines;
-  unsigned char alertMessages;
-  unsigned char blankWindowsSkipMode;
-  unsigned char alertDots;
-  unsigned char skipBlankWindows;
-  unsigned char midiInstrument;
-  unsigned char spare1;
-  unsigned char windowOverlap;
-  unsigned char speechRate;
-  unsigned char speechVolume;
-  unsigned char brailleFirmness;
-  unsigned char speechPunctuation;
-  unsigned char speechPitch;
-  unsigned char statusPosition;
-  unsigned char statusCount;
-  unsigned char statusSeparator;
-  unsigned char statusFields[10];
-} PACKED Preferences;
-
-extern Preferences prefs;		/* current preferences settings */
-#define PREFERENCES_TIME(time) ((time) * 10)
-
-extern unsigned int getStatusFieldsLength (const unsigned char *fields);
+extern SessionEntry *ses;
+extern unsigned int updateIntervals;
+extern unsigned char infoMode;
 
 extern int writeBrailleCharacters (const char *mode, const wchar_t *characters, size_t length);
-extern int writeBrailleBytes (const char *mode, const char *bytes, size_t length);
-extern int writeBrailleString (const char *mode, const char *string);
-extern int showBrailleString (const char *mode, const char *string, unsigned int duration);
+extern int writeBrailleText (const char *mode, const char *text);
+extern int showBrailleText (const char *mode, const char *text, int minimumDelay);
+
+extern char *opt_tablesDirectory;
+extern char *opt_textTable;
+extern char *opt_attributesTable;
+extern char *opt_contractionTable;
 
 extern int opt_releaseDevice;
 extern char *opt_pcmDevice;
@@ -131,15 +64,19 @@ extern char *opt_midiDevice;
 
 extern int updateInterval;
 extern int messageDelay;
-extern ContractionTable *contractionTable;
 
-extern void testProgramTermination (void);
-extern void startup (int argc, char *argv[]);
+extern ContractionTable *contractionTable;
+extern int loadContractionTable (const char *name);
+
+extern KeyTable *keyboardKeyTable;
+
+extern ProgramExitStatus brlttyStart (int argc, char *argv[]);
+
+extern void setPreferences (const Preferences *newPreferences);
 extern int loadPreferences (void);
 extern int savePreferences (void);
-extern int updatePreferences (void);
 
-extern unsigned char cursorDots (void);
+extern unsigned char getCursorDots (void);
 
 extern BrailleDisplay brl;			/* braille driver reference */
 extern unsigned int textStart;
@@ -166,7 +103,7 @@ extern int constructSpeechDriver (void);
 extern void destructSpeechDriver (void);
 #endif /* ENABLE_SPEECH_SUPPORT */
 
-extern int readCommand (BRL_DriverCommandContext context);
+extern int readCommand (KeyTableCommandContext context);
 extern void resetAutorepeat (void);
 extern void handleAutorepeat (int *command, RepeatState *state);
 
@@ -181,7 +118,7 @@ extern void api_suspend (BrailleDisplay *brl);
 extern int api_resume (BrailleDisplay *brl);
 extern int api_claimDriver (BrailleDisplay *brl);
 extern void api_releaseDriver (BrailleDisplay *brl);
-extern void api_flush (BrailleDisplay *brl, BRL_DriverCommandContext caller);
+extern int api_flush (BrailleDisplay *brl);
 extern int api_handleCommand (int command);
 extern int api_handleKeyEvent (unsigned char set, unsigned char key, int press);
 extern const char *const api_parameters[];
