@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -20,7 +20,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <search.h>
 
 #include "log.h"
 #include "timing.h"
@@ -35,6 +34,55 @@ const CommandEntry commandTable[] = {
   {.name=NULL, .code=EOF, .description=NULL}
 };
 
+const CommandModifierEntry commandModifierTable_braille[] = {
+  {.name="dot1" , .bit=BRL_DOT1},
+  {.name="dot2" , .bit=BRL_DOT2},
+  {.name="dot3" , .bit=BRL_DOT3},
+  {.name="dot4" , .bit=BRL_DOT4},
+  {.name="dot5" , .bit=BRL_DOT5},
+  {.name="dot6" , .bit=BRL_DOT6},
+  {.name="dot7" , .bit=BRL_DOT7},
+  {.name="dot8" , .bit=BRL_DOT8},
+  {.name="space", .bit=BRL_DOTC},
+  {.name=NULL   , .bit=0       }
+};
+
+const CommandModifierEntry commandModifierTable_character[] = {
+  {.name="upper", .bit=BRL_FLG_CHAR_UPPER},
+  {.name=NULL   , .bit=0                 }
+};
+
+const CommandModifierEntry commandModifierTable_input[] = {
+  {.name="shift"  , .bit=BRL_FLG_CHAR_SHIFT  },
+  {.name="control", .bit=BRL_FLG_CHAR_CONTROL},
+  {.name="meta"   , .bit=BRL_FLG_CHAR_META   },
+  {.name=NULL     , .bit=0                   }
+};
+
+const CommandModifierEntry commandModifierTable_keyboard[] = {
+  {.name="release", .bit=BRL_FLG_KBD_RELEASE},
+  {.name="emul0"  , .bit=BRL_FLG_KBD_EMUL0  },
+  {.name="emul1"  , .bit=BRL_FLG_KBD_EMUL1  },
+  {.name=NULL     , .bit=0                  }
+};
+
+const CommandModifierEntry commandModifierTable_line[] = {
+  {.name="scaled", .bit=BRL_FLG_LINE_SCALED},
+  {.name="toleft", .bit=BRL_FLG_LINE_TOLEFT},
+  {.name=NULL    , .bit=0                  }
+};
+
+const CommandModifierEntry commandModifierTable_motion[] = {
+  {.name="route", .bit=BRL_FLG_MOTION_ROUTE},
+  {.name=NULL   , .bit=0                   }
+};
+
+const CommandModifierEntry commandModifierTable_toggle[] = {
+  {.name="on" , .bit=BRL_FLG_TOGGLE_ON },
+  {.name="off", .bit=BRL_FLG_TOGGLE_OFF},
+  {.name=NULL , .bit=0                 }
+};
+
 static int
 compareCommandCodes (const void *element1, const void *element2) {
   const CommandEntry *const *cmd1 = element1;
@@ -61,7 +109,7 @@ getCommandEntry (int code) {
       const CommandEntry **entries = malloc(ARRAY_SIZE(entries, commandCount));
 
       if (!entries) {
-        LogError("malloc");
+        logMallocError();
         return NULL;
       }
 
@@ -115,114 +163,262 @@ getCommandEntry (int code) {
   return NULL;
 }
 
-void
-describeCommand (int command, char *buffer, size_t size, int details) {
-  int blk = command & BRL_MSK_BLK;
+static size_t
+formatCommandModifiers (char *buffer, size_t size, int command, const CommandModifierEntry *modifiers) {
+  const CommandModifierEntry *modifier = modifiers;
+  size_t length;
+  STR_BEGIN(buffer, size);
+
+  while (modifier->name) {
+    if (command & modifier->bit) {
+      STR_PRINTF(" + %s", modifier->name);
+    }
+
+    modifier += 1;
+  }
+
+  length = STR_LENGTH;
+  STR_END;
+  return length;
+}
+
+size_t
+describeCommand (int command, char *buffer, size_t size, CommandDescriptionOption options) {
+  size_t length;
+  STR_BEGIN(buffer, size);
+
   unsigned int arg = BRL_ARG_GET(command);
   unsigned int arg1 = BRL_CODE_GET(ARG, command);
   unsigned int arg2 = BRL_CODE_GET(EXT, command);
   const CommandEntry *cmd = getCommandEntry(command);
 
   if (!cmd) {
-    snprintf(buffer, size, "unknown: %06X", command);
+    STR_PRINTF("%s: %06X", gettext("unknown command"), command);
   } else {
-    int length;
-
-    if (details) {
-      snprintf(buffer, size, "%s: %n", cmd->name, &length);
-      buffer += length, size -= length;
+    if (options & CDO_IncludeName) {
+      STR_PRINTF("%s: ", cmd->name);
     }
 
-    snprintf(buffer, size, "%s%n", cmd->description, &length);
     if (cmd->isToggle && (command & BRL_FLG_TOGGLE_MASK)) {
-      const char *oldVerb = "toggle";
-      size_t oldLength = strlen(oldVerb);
+      const char *text = gettext(cmd->description);
+      size_t length = strlen(text);
+      char buffer[length + 1];
+      char *delimiter;
 
-      if (strncmp(buffer, oldVerb, oldLength) == 0) {
-        const char *newVerb = "set";
-        size_t newLength = strlen(newVerb);
+      strcpy(buffer, text);
+      delimiter = strchr(buffer, '/');
 
-        memmove(buffer+newLength, buffer+oldLength, length-oldLength+1);
-        memcpy(buffer, newVerb, newLength);
+      if (delimiter) {
+        char *source;
+        char *target;
 
         if (command & BRL_FLG_TOGGLE_ON) {
-          char *end = strrchr(buffer, '/');
-          if (end) *end = 0;
+          target = delimiter;
+          if (!(source = strchr(target, ' '))) source = target + strlen(target);
+        } else if (command & BRL_FLG_TOGGLE_OFF) {
+          {
+            char oldDelimiter = *delimiter;
+            *delimiter = 0;
+            target = strrchr(buffer, ' ');
+            *delimiter = oldDelimiter;
+          }
+
+          target = target? (target + 1): buffer;
+          source = delimiter + 1;
         } else {
-          char *target = strrchr(buffer, ' ');
+          goto toggleReady;
+        }
 
-          if (target) {
-            const char *source = strchr(++target, '/');
+        memmove(target, source, (strlen(source) + 1));
+      }
 
-            if (source) memmove(target, source+1, strlen(source));
+    toggleReady:
+      STR_PRINTF("%s", buffer);
+    } else {
+      STR_PRINTF("%s", gettext(cmd->description));
+    }
+
+    if (options & CDO_IncludeOperand) {
+      if (cmd->isCharacter) {
+        STR_PRINTF(" [U+%04" PRIX16 "]", arg);
+      }
+
+      if (cmd->isBraille) {
+        int none = 1;
+        STR_PRINTF(" [");
+
+        {
+          static const int dots[] = {
+            BRL_DOTC,
+            BRL_DOT1, BRL_DOT2, BRL_DOT3, BRL_DOT4,
+            BRL_DOT5, BRL_DOT6, BRL_DOT7, BRL_DOT8,
+            0
+          };
+          const int *dot = dots;
+
+          while (*dot) {
+            if (command & *dot) {
+              none = 0;
+
+              if (dot == dots) {
+                STR_PRINTF("C");
+              } else {
+                unsigned int number = dot - dots;
+                STR_PRINTF("%u", number);
+              }
+            }
+
+            dot += 1;
           }
         }
 
-        length = strlen(buffer);
+        if (none) STR_PRINTF("%s", gettext("space"));
+        STR_PRINTF("]");
+      }
+
+      if (cmd->isKeyboard) {
+        STR_PRINTF(" [\\X%02" PRIX8 "]", arg1);
+      }
+
+      if (cmd->isColumn && !cmd->isRouting && (
+           (arg == BRL_MSK_ARG) /* key event processing */
+         ||
+           ((options & CDO_DefaultOperand) && !arg) /* key table listing */
+         )) {
+        STR_PRINTF(" %s", gettext("at cursor"));
+      } else if (cmd->isColumn || cmd->isRow || cmd->isOffset) {
+        STR_PRINTF(" #%u", arg - (cmd->code & BRL_MSK_ARG) + 1);
+      } else if (cmd->isRange) {
+        STR_PRINTF(" #%u-%u", arg1, arg2);
+      }
+
+      if (cmd->isInput) {
+        size_t length = formatCommandModifiers(STR_NEXT, STR_LEFT, command, commandModifierTable_input);
+        STR_ADJUST(length);
+      }
+
+      if (cmd->isCharacter) {
+        size_t length = formatCommandModifiers(STR_NEXT, STR_LEFT, command, commandModifierTable_character);
+        STR_ADJUST(length);
+      }
+
+      if (cmd->isKeyboard) {
+        size_t length = formatCommandModifiers(STR_NEXT, STR_LEFT, command, commandModifierTable_keyboard);
+        STR_ADJUST(length);
       }
     }
-    buffer += length, size -= length;
 
     if (cmd->isMotion) {
-      if (command & BRL_FLG_MOTION_ROUTE) {
-        snprintf(buffer, size, ", drag cursor%n", &length);
-        buffer += length, size -= length;
-      }
-
       if (cmd->isRow) {
-        if (command & BRL_FLG_LINE_SCALED) {
-          snprintf(buffer, size, " (scaled)%n", &length);
-          buffer += length, size -= length;
+        if (command & BRL_FLG_LINE_TOLEFT) {
+          STR_PRINTF(", %s", gettext("left margin"));
         }
 
-        if (command & BRL_FLG_LINE_TOLEFT) {
-          snprintf(buffer, size, ", beginning of line%n", &length);
-          buffer += length, size -= length;
+        if (command & BRL_FLG_LINE_SCALED) {
+          STR_PRINTF(", %s", gettext("normalized position"));
         }
       }
-    }
 
-    if (details) {
-      if (cmd->isColumn || cmd->isRow || cmd->isOffset) {
-        snprintf(buffer, size, " #%u",
-                 arg - (cmd->code & BRL_MSK_ARG) + 1);
-      } else if (cmd->isRange) {
-        snprintf(buffer, size, " #%u-%u", arg1, arg2);
-      } else if (blk) {
-        switch (blk) {
-          case BRL_BLK_PASSKEY:
-            break;
-
-          case BRL_BLK_PASSCHAR:
-            snprintf(buffer, size, " [U+%04" PRIX16 "]", arg);
-            break;
-
-          case BRL_BLK_PASSDOTS:
-            if (arg) {
-              static const unsigned char dots[] = {BRL_DOT1, BRL_DOT2, BRL_DOT3, BRL_DOT4, BRL_DOT5, BRL_DOT6, BRL_DOT7, BRL_DOT8};
-              int dot;
-              unsigned int number = 0;
-
-              for (dot=0; dot<sizeof(dots); dot+=1) {
-                if (arg & dots[dot]) {
-                  number = (number * 10) + (dot + 1);
-                }
-              }
-
-              snprintf(buffer, size, " [%s %u]",
-                       ((number < 10)? "dot": "dots"), number);
-            } else {
-              snprintf(buffer, size, " [space]");
-            }
-            break;
-
-          default:
-            snprintf(buffer, size, " 0X%02X", arg);
-            break;
-        }
+      if (command & BRL_FLG_MOTION_ROUTE) {
+        STR_PRINTF(", %s", gettext("drag cursor"));
       }
     }
   }
+
+  length = STR_LENGTH;
+  STR_END;
+  return length;
+}
+
+static size_t
+formatCommand (char *buffer, size_t size, int command) {
+  size_t length;
+  STR_BEGIN(buffer, size);
+
+  STR_PRINTF("%06X (", command);
+
+  {
+    size_t length = describeCommand(command, STR_NEXT, STR_LEFT, 
+                                    CDO_IncludeName | CDO_IncludeOperand);
+    STR_ADJUST(length);
+  }
+
+  STR_PRINTF(")");
+
+  length = STR_LENGTH;
+  STR_END;
+  return length;
+}
+
+typedef struct {
+  int command;
+} LogCommandData;
+
+static size_t
+formatLogCommandData (char *buffer, size_t size, const void *data) {
+  const LogCommandData *cmd = data;
+  size_t length;
+
+  STR_BEGIN(buffer, size);
+  STR_PRINTF("command: ");
+
+  {
+    size_t sublength = formatCommand(STR_NEXT, STR_LEFT, cmd->command);
+    STR_ADJUST(sublength);
+  }
+
+  length = STR_LENGTH;
+  STR_END
+  return length;
+}
+
+void
+logCommand (int command) {
+  const LogCommandData cmd = {
+    .command = command
+  };
+
+  logData(LOG_DEBUG, formatLogCommandData, &cmd);
+}
+
+typedef struct {
+  int oldCommand;
+  int newCommand;
+} LogTransformedCommandData;
+
+static size_t
+formatLogTransformedCommandData (char *buffer, size_t size, const void *data) {
+  const LogTransformedCommandData *cmd = data;
+  size_t length;
+
+  STR_BEGIN(buffer, size);
+  STR_PRINTF("command: ");
+
+  {
+    size_t sublength = formatCommand(STR_NEXT, STR_LEFT, cmd->oldCommand);
+    STR_ADJUST(sublength);
+  }
+
+  STR_PRINTF(" -> ");
+
+  {
+    size_t sublength = formatCommand(STR_NEXT, STR_LEFT, cmd->newCommand);
+    STR_ADJUST(sublength);
+  }
+
+  length = STR_LENGTH;
+  STR_END
+  return length;
+}
+
+void
+logTransformedCommand (int oldCommand, int newCommand) {
+  const LogTransformedCommandData cmd = {
+    .oldCommand = oldCommand,
+    .newCommand = newCommand
+  };
+
+  logData(LOG_DEBUG, formatLogTransformedCommandData, &cmd);
 }
 
 void
@@ -237,8 +433,8 @@ handleRepeatFlags (int *command, RepeatState *state, int panning, int delay, int
   if (state) {
     if (*command == EOF) {
       if (state->timeout) {
-        struct timeval now;
-        gettimeofday(&now, NULL);
+        TimeValue now;
+        getCurrentTime(&now);
         if (millisecondsBetween(&state->time, &now) >= state->timeout) {
           *command = state->command;
           state->time = now;
@@ -281,6 +477,13 @@ handleRepeatFlags (int *command, RepeatState *state, int panning, int delay, int
             case BRL_BLK_PASSKEY + BRL_KEY_CURSOR_DOWN:
             case BRL_BLK_PASSKEY + BRL_KEY_CURSOR_LEFT:
             case BRL_BLK_PASSKEY + BRL_KEY_CURSOR_RIGHT:
+
+            case BRL_CMD_SPEAK_PREV_CHAR:
+            case BRL_CMD_SPEAK_NEXT_CHAR:
+            case BRL_CMD_SPEAK_PREV_WORD:
+            case BRL_CMD_SPEAK_NEXT_WORD:
+            case BRL_CMD_SPEAK_PREV_LINE:
+            case BRL_CMD_SPEAK_NEXT_LINE:
               break;
           }
 
@@ -300,7 +503,7 @@ handleRepeatFlags (int *command, RepeatState *state, int panning, int delay, int
       state->command = *command;
 
       if (flags & BRL_FLG_REPEAT_DELAY) {
-        gettimeofday(&state->time, NULL);
+        getCurrentTime(&state->time);
         state->timeout = delay;
         if (flags & BRL_FLG_REPEAT_INITIAL) {
           state->started = 1;
@@ -308,7 +511,7 @@ handleRepeatFlags (int *command, RepeatState *state, int panning, int delay, int
           *command = BRL_CMD_NOOP;
         }
       } else if (flags & BRL_FLG_REPEAT_INITIAL) {
-        gettimeofday(&state->time, NULL);
+        getCurrentTime(&state->time);
         state->timeout = interval;
         state->started = 1;
       } else {
