@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -19,6 +19,8 @@
 #ifndef BRLTTY_INCLUDED_CTB_INTERNAL
 #define BRLTTY_INCLUDED_CTB_INTERNAL
 
+#include <stdio.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -47,7 +49,7 @@ typedef struct {
   ContractionTableCharacterAttributes attributes;
 } ContractionTableCharacter;
 
-typedef enum { /*Op codes*/
+typedef enum {
   CTO_CapitalSign, /*dot pattern for capital sign*/
   CTO_BeginCapitalSign, /*dot pattern for beginning capital block*/
   CTO_EndCapitalSign, /*dot pattern for ending capital block*/
@@ -99,7 +101,7 @@ typedef struct {
   wchar_t findrep[1]; /*find and replacement strings*/
 } ContractionTableRule;
 
-typedef struct { /*translation table*/
+typedef struct {
   ContractionTableOffset capitalSign; /*capitalization sign*/
   ContractionTableOffset beginCapitalSign; /*begin capitals sign*/
   ContractionTableOffset endCapitalSign; /*end capitals sign*/
@@ -118,18 +120,61 @@ typedef struct {
 } CharacterEntry;
 
 struct ContractionTableStruct {
+  struct {
+    CharacterEntry *array;
+    int size;
+    int count;
+  } characters;
+
+  struct {
+    struct {
+      wchar_t *characters;
+      unsigned int size;
+      unsigned int count;
+      unsigned int consumed;
+    } input;
+
+    struct {
+      unsigned char *cells;
+      unsigned int size;
+      unsigned int count;
+      unsigned int maximum;
+    } output;
+
+    struct {
+      int *array;
+      unsigned int size;
+      unsigned int count;
+    } offsets;
+
+    int cursorOffset;
+    unsigned char expandCurrentWord;
+    unsigned char capitalizationMode;
+  } cache;
+
+  char *command;
+
   union {
-    ContractionTableHeader *fields;
-    const unsigned char *bytes;
-  } header;
+    struct {
+      union {
+        ContractionTableHeader *fields;
+        const unsigned char *bytes;
+      } header;
 
-  size_t size;
+      size_t size;
+    } internal;
 
-  CharacterEntry *characters;
-  int charactersSize;
-  int characterCount;
+    struct {
+      unsigned commandStarted:1;
+      FILE *standardInput;
+      FILE *standardOutput;
+    } external;
+  } data;
 };
 
+extern int startContractionCommand (ContractionTable *table);
+extern void stopContractionCommand (ContractionTable *table);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
