@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -19,6 +19,7 @@
 #include "prologue.h"
 
 #include <string.h>
+#include <errno.h>
 
 #ifdef HAVE_ICU
 #include <unicode/uchar.h>
@@ -26,33 +27,57 @@
 
 #include "ctb.h"
 #include "ctb_internal.h"
+#include "ttb.h"
+#include "prefs.h"
 #include "unicode.h"
+#include "charset.h"
+#include "ascii.h"
 #include "brldots.h"
+#include "log.h"
+#include "file.h"
+#include "parse.h"
 
 static ContractionTable *table;
 static const wchar_t *src, *srcmin, *srcmax, *cursor;
 static BYTE *dest, *destmin, *destmax;
 static int *offsets;
+
 static wchar_t before, after;	/*the characters before and after a string */
 static int currentFindLength;		/*length of current find string */
 static ContractionTableOpcode currentOpcode;
 static ContractionTableOpcode previousOpcode;
 static const ContractionTableRule *currentRule;	/*pointer to current rule in table */
 
-#define assignOffset(value) if (offsets) offsets[src - srcmin] = (value)
-#define setOffset() assignOffset(dest-destmin)
-#define clearOffset() assignOffset(CTB_NO_OFFSET)
+static inline void
+assignOffset (size_t value) {
+  if (offsets) offsets[src - srcmin] = value;
+}
+
+static inline void
+setOffset (void) {
+  assignOffset(dest - destmin);
+}
+
+static inline void
+clearOffset (void) {
+  assignOffset(CTB_NO_OFFSET);
+}
+
+static inline ContractionTableHeader *
+getContractionTableHeader (void) {
+  return table->data.internal.header.fields;
+}
 
 static inline const void *
 getContractionTableItem (ContractionTableOffset offset) {
-  return &table->header.bytes[offset];
+  return &table->data.internal.header.bytes[offset];
 }
 
 static const ContractionTableCharacter *
 getContractionTableCharacter (wchar_t character) {
-  const ContractionTableCharacter *characters = getContractionTableItem(table->header.fields->characters);
+  const ContractionTableCharacter *characters = getContractionTableItem(getContractionTableHeader()->characters);
   int first = 0;
-  int last = table->header.fields->characterCount - 1;
+  int last = getContractionTableHeader()->characterCount - 1;
 
   while (first <= last) {
     int current = (first + last) / 2;
@@ -73,11 +98,11 @@ getContractionTableCharacter (wchar_t character) {
 static CharacterEntry *
 getCharacterEntry (wchar_t character) {
   int first = 0;
-  int last = table->characterCount - 1;
+  int last = table->characters.count - 1;
 
   while (first <= last) {
     int current = (first + last) / 2;
-    CharacterEntry *entry = &table->characters[current];
+    CharacterEntry *entry = &table->characters.array[current];
 
     if (entry->value < character) {
       first = current + 1;
@@ -88,26 +113,30 @@ getCharacterEntry (wchar_t character) {
     }
   }
 
-  if (table->characterCount == table->charactersSize) {
-    int newSize = table->charactersSize;
+  if (table->characters.count == table->characters.size) {
+    int newSize = table->characters.size;
     newSize = newSize? newSize<<1: 0X80;
 
     {
-      CharacterEntry *newCharacters = realloc(table->characters, (newSize * sizeof(*newCharacters)));
-      if (!newCharacters) return NULL;
+      CharacterEntry *newArray = realloc(table->characters.array, (newSize * sizeof(*newArray)));
 
-      table->characters = newCharacters;
-      table->charactersSize = newSize;
+      if (!newArray) {
+        logMallocError();
+        return NULL;
+      }
+
+      table->characters.array = newArray;
+      table->characters.size = newSize;
     }
   }
 
-  memmove(&table->characters[first+1],
-          &table->characters[first],
-          (table->characterCount - first) * sizeof(*table->characters));
-  table->characterCount += 1;
+  memmove(&table->characters.array[first+1],
+          &table->characters.array[first],
+          (table->characters.count - first) * sizeof(*table->characters.array));
+  table->characters.count += 1;
 
   {
-    CharacterEntry *entry = &table->characters[first];
+    CharacterEntry *entry = &table->characters.array[first];
     memset(entry, 0, sizeof(*entry));
     entry->value = entry->uppercase = entry->lowercase = character;
 
@@ -131,7 +160,7 @@ getCharacterEntry (wchar_t character) {
       entry->attributes |= CTC_Punctuation;
     }
 
-    {
+    if (!table->command) {
       const ContractionTableCharacter *ctc = getContractionTableCharacter(character);
       if (ctc) entry->attributes |= ctc->attributes;
     }
@@ -219,7 +248,7 @@ selectRule (int length) {
     wchar_t characters[2];
     characters[0] = toLowerCase(src[0]);
     characters[1] = toLowerCase(src[1]);
-    ruleOffset = table->header.fields->rules[CTH(characters)];
+    ruleOffset = getContractionTableHeader()->rules[CTH(characters)];
     maximumLength = 0;
   }
 
@@ -234,33 +263,44 @@ selectRule (int length) {
       setAfter(currentFindLength);
 
       if (!maximumLength) {
-        typedef enum {CS_Any, CS_Lower, CS_UpperSingle, CS_UpperMultiple} CapitalizationState;
-#define STATE(c) (testCharacter((c), CTC_UpperCase)? CS_UpperSingle: testCharacter((c), CTC_LowerCase)? CS_Lower: CS_Any)
-        CapitalizationState current = STATE(before);
-        int i;
         maximumLength = currentFindLength;
 
-        for (i=0; i<currentFindLength; ++i) {
-          wchar_t character = src[i];
-          CapitalizationState next = STATE(character);
+        if (prefs.capitalizationMode != CTB_CAP_NONE) {
+          typedef enum {CS_Any, CS_Lower, CS_UpperSingle, CS_UpperMultiple} CapitalizationState;
+#define STATE(c) (testCharacter((c), CTC_UpperCase)? CS_UpperSingle: testCharacter((c), CTC_LowerCase)? CS_Lower: CS_Any)
 
-          if ((i > 0) &&
-              (((current == CS_Lower) && (next == CS_UpperSingle)) ||
-               ((current == CS_UpperMultiple) && (next == CS_Lower)))) {
-            maximumLength = i;
-            break;
-          }
+          CapitalizationState current = STATE(before);
+          int i;
 
-          if ((current > CS_Lower) && (next == CS_UpperSingle)) {
-            current = CS_UpperMultiple;
-          } else if (next != CS_Any) {
-            current = next;
-          } else if (current == CS_Any) {
-            current = CS_Lower;
+          for (i=0; i<currentFindLength; i+=1) {
+            wchar_t character = src[i];
+            CapitalizationState next = STATE(character);
+
+            if (i > 0) {
+              if (((current == CS_Lower) && (next == CS_UpperSingle)) ||
+                  ((current == CS_UpperMultiple) && (next == CS_Lower))) {
+                maximumLength = i;
+                break;
+              }
+
+              if ((prefs.capitalizationMode != CTB_CAP_SIGN) &&
+                  (next == CS_UpperSingle)) {
+                maximumLength = i;
+                break;
+              }
+            }
+
+            if ((prefs.capitalizationMode == CTB_CAP_SIGN) && (current > CS_Lower) && (next == CS_UpperSingle)) {
+              current = CS_UpperMultiple;
+            } else if (next != CS_Any) {
+              current = next;
+            } else if (current == CS_Any) {
+              current = CS_Lower;
+            }
           }
-        }
 
 #undef STATE
+        }
       }
 
       if ((currentFindLength <= maximumLength) &&
@@ -402,8 +442,17 @@ putCell (BYTE byte) {
 }
 
 static int
-putReplace (const ContractionTableRule *rule) {
-  return putCells((BYTE *)&rule->findrep[rule->findlen], rule->replen);
+putReplace (const ContractionTableRule *rule, wchar_t character) {
+  const BYTE *cells = (BYTE *)&rule->findrep[rule->findlen];
+  int count = rule->replen;
+
+  if ((prefs.capitalizationMode == CTB_CAP_DOT7) &&
+      testCharacter(character, CTC_UpperCase)) {
+    if (!putCell(*cells++ | BRL_DOT7)) return 0;
+    if (!(count -= 1)) return 1;
+  }
+
+  return putCells(cells, count);
 }
 
 static const ContractionTableRule *
@@ -419,23 +468,46 @@ getAlwaysRule (wchar_t character) {
   return NULL;
 }
 
+typedef struct {
+  const ContractionTableRule *rule;
+} SetCharacterRuleData;
+
+static int
+setCharacterRule (wchar_t character, void *data) {
+  const ContractionTableRule *rule = getAlwaysRule(character);
+
+  if (rule) {
+    SetCharacterRuleData *scr = data;
+    scr->rule = rule;
+    return 1;
+  }
+
+  return 0;
+}
+
 static int
 putCharacter (wchar_t character) {
   {
-    const ContractionTableRule *rule = getAlwaysRule(character);
-    if (rule) return putReplace(rule);
+    SetCharacterRuleData scr = {
+      .rule = NULL
+    };
+
+    if (handleBestCharacter(character, setCharacterRule, &scr)) {
+      return putReplace(scr.rule, character);
+    }
   }
 
   {
 #ifdef HAVE_WCHAR_H 
-    const wchar_t substitute = UNICODE_REPLACEMENT_CHARACTER;
+    const wchar_t replacementCharacter = UNICODE_REPLACEMENT_CHARACTER;
+    if (getCharacterWidth(character) == 0) return 1;
 #else /* HAVE_WCHAR_H */
-    const wchar_t substitute = 0X1A; /* ASCII SUB (CTRL-Z) */
+    const wchar_t replacementCharacter = SUB;
 #endif /* HAVE_WCHAR_H */
 
-    if (character != substitute) {
-      const ContractionTableRule *rule = getAlwaysRule(substitute);
-      if (rule) return putReplace(rule);
+    if (replacementCharacter != character) {
+      const ContractionTableRule *rule = getAlwaysRule(replacementCharacter);
+      if (rule) return putReplace(rule, character);
     }
   }
 
@@ -859,13 +931,8 @@ findLineBreakOpportunities (unsigned char *opportunities, const wchar_t *charact
 #endif /* HAVE_ICU */
 }
 
-int
-contractText (
-  ContractionTable *contractionTable,
-  const wchar_t *inputBuffer, int *inputLength,
-  BYTE *outputBuffer, int *outputLength,
-  int *offsetsMap, const int cursorOffset
-) {
+static int
+contractTextInternally (void) {
   const wchar_t *srcword = NULL;
   BYTE *destword = NULL;
 
@@ -874,15 +941,9 @@ contractText (
 
   BYTE *destlast = NULL;
   const wchar_t *literal = NULL;
-  unsigned char lineBreakOpportunities[*inputLength];
+  unsigned char lineBreakOpportunities[srcmax - srcmin];
 
-  table = contractionTable;
-  srcmax = (srcmin = src = inputBuffer) + *inputLength;
-  destmax = (destmin = dest = outputBuffer) + *outputLength;
-  offsets = offsetsMap;
-  cursor = (cursorOffset == CTB_NO_CURSOR)? NULL: &src[cursorOffset];
-
-  findLineBreakOpportunities(lineBreakOpportunities, inputBuffer, *inputLength);
+  findLineBreakOpportunities(lineBreakOpportunities, srcmin, ARRAY_COUNT(lineBreakOpportunities));
   previousOpcode = CTO_None;
 
   while (src < srcmax) {
@@ -900,7 +961,7 @@ contractText (
     if ((!literal && selectRule(srcmax-src)) || selectRule(1)) {
       if (!literal &&
           ((currentOpcode == CTO_Literal) ||
-           ((cursor >= src) && (cursor < (src + currentFindLength))))) {
+           (prefs.expandCurrentWord && (cursor >= src) && (cursor < (src + currentFindLength))))) {
         literal = src + currentFindLength;
 
         if (!testCharacter(*src, CTC_Space)) {
@@ -916,10 +977,10 @@ contractText (
         continue;
       }
 
-      if (table->header.fields->numberSign && (previousOpcode != CTO_MidNum) &&
+      if (getContractionTableHeader()->numberSign && (previousOpcode != CTO_MidNum) &&
           !testCharacter(before, CTC_Digit) && testCharacter(*src, CTC_Digit)) {
-        if (!putSequence(table->header.fields->numberSign)) break;
-      } else if (table->header.fields->englishLetterSign && testCharacter(*src, CTC_Letter)) {
+        if (!putSequence(getContractionTableHeader()->numberSign)) break;
+      } else if (getContractionTableHeader()->englishLetterSign && testCharacter(*src, CTC_Letter)) {
         if ((currentOpcode == CTO_Contraction) ||
             ((currentOpcode != CTO_EndNum) && testCharacter(before, CTC_Digit)) ||
             (testCharacter(*src, CTC_Letter) &&
@@ -929,23 +990,25 @@ contractText (
              (((src + 1) == srcmax) ||
               testCharacter(src[1], CTC_Space) ||
               (testCharacter(src[1], CTC_Punctuation) && (src[1] != '.') && (src[1] != '\''))))) {
-          if (!putSequence(table->header.fields->englishLetterSign)) break;
+          if (!putSequence(getContractionTableHeader()->englishLetterSign)) break;
         }
       }
 
-      if (testCharacter(*src, CTC_UpperCase)) {
-        if (!testCharacter(before, CTC_UpperCase)) {
-          if (table->header.fields->beginCapitalSign &&
-              (src + 1 < srcmax) && testCharacter(src[1], CTC_UpperCase)) {
-            if (!putSequence(table->header.fields->beginCapitalSign)) break;
-          } else if (table->header.fields->capitalSign) {
-            if (!putSequence(table->header.fields->capitalSign)) break;
+      if (prefs.capitalizationMode == CTB_CAP_SIGN) {
+        if (testCharacter(*src, CTC_UpperCase)) {
+          if (!testCharacter(before, CTC_UpperCase)) {
+            if (getContractionTableHeader()->beginCapitalSign &&
+                (src + 1 < srcmax) && testCharacter(src[1], CTC_UpperCase)) {
+              if (!putSequence(getContractionTableHeader()->beginCapitalSign)) break;
+            } else if (getContractionTableHeader()->capitalSign) {
+              if (!putSequence(getContractionTableHeader()->capitalSign)) break;
+            }
+          }
+        } else if (testCharacter(*src, CTC_LowerCase)) {
+          if (getContractionTableHeader()->endCapitalSign && (src - 2 >= srcmin) &&
+              testCharacter(src[-1], CTC_UpperCase) && testCharacter(src[-2], CTC_UpperCase)) {
+            if (!putSequence(getContractionTableHeader()->endCapitalSign)) break;
           }
-        }
-      } else if (testCharacter(*src, CTC_LowerCase)) {
-        if (table->header.fields->endCapitalSign && (src - 2 >= srcmin) &&
-            testCharacter(src[-1], CTC_UpperCase) && testCharacter(src[-2], CTC_UpperCase)) {
-          if (!putSequence(table->header.fields->endCapitalSign)) break;
         }
       }
 
@@ -975,7 +1038,7 @@ contractText (
       if (currentRule->replen &&
           !((currentOpcode == CTO_Always) && (currentFindLength == 1))) {
         const wchar_t *srcnxt = src + currentFindLength;
-        if (!putReplace(currentRule)) goto done;
+        if (!putReplace(currentRule, *src)) goto done;
         while (++src != srcnxt) clearOffset();
       } else {
         const wchar_t *srclim = src + currentFindLength;
@@ -1069,29 +1132,502 @@ done:
     }
   }
 
-  if (src < srcmax) {
-    const wchar_t *srcorig = src;
-    int done = 1;
+  return 1;
+}
 
-    setOffset();
-    while (1) {
-      if (done && !testCharacter(*src, CTC_Space)) {
-        done = 0;
+static int
+putExternalRequests (void) {
+  typedef enum {
+    REQ_TEXT,
+    REQ_NUMBER
+  } ExternalRequestType;
 
-        if ((cursor < srcorig) || (cursor >= src)) {
-          setOffset();
-          srcorig = src;
+  typedef struct {
+    const char *name;
+    unsigned char type;
+
+    union {
+      struct {
+        const wchar_t *start;
+        size_t count;
+      } text;
+
+      unsigned int number;
+    } value;
+  } ExternalRequestEntry;
+
+  const ExternalRequestEntry externalRequestTable[] = {
+    { .name = "cursor-position",
+      .type = REQ_NUMBER,
+      .value.number = cursor? cursor-srcmin+1: 0
+    },
+
+    { .name = "expand-current-word",
+      .type = REQ_NUMBER,
+      .value.number = prefs.expandCurrentWord
+    },
+
+    { .name = "capitalization-mode",
+      .type = REQ_NUMBER,
+      .value.number = prefs.capitalizationMode
+    },
+
+    { .name = "maximum-length",
+      .type = REQ_NUMBER,
+      .value.number = destmax - destmin
+    },
+
+    { .name = "text",
+      .type = REQ_TEXT,
+      .value.text = {
+        .start = srcmin,
+        .count = srcmax - srcmin
+      }
+    },
+
+    { .name = NULL }
+  };
+
+  FILE *stream = table->data.external.standardInput;
+  const ExternalRequestEntry *req = externalRequestTable;
+
+  while (req->name) {
+    if (fputs(req->name, stream) == EOF) goto outputError;
+    if (fputc('=', stream) == EOF) goto outputError;
+
+    switch (req->type) {
+      case REQ_TEXT: {
+        const wchar_t *character = req->value.text.start;
+        const wchar_t *end = character + req->value.text.count;
+
+        while (character < end) {
+          Utf8Buffer utf8;
+          size_t utfs = convertWcharToUtf8(*character++, utf8);
+
+          if (!utfs) return 0;
+          if (fputs(utf8, stream) == EOF) goto outputError;
+        }
+
+        break;
+      }
+
+      case REQ_NUMBER:
+        if (fprintf(stream, "%u", req->value.number) == EOF) goto outputError;
+        break;
+
+      default:
+        logMessage(LOG_WARNING, "unimplemented external contraction request property type: %s: %u (%s)", table->command, req->type, req->name);
+        return 0;
+    }
+
+    if (fputc('\n', stream) == EOF) goto outputError;
+    req += 1;
+  }
+
+  if (fflush(stream) == EOF) goto outputError;
+  return 1;
+
+outputError:
+  logMessage(LOG_WARNING, "external contraction output error: %s: %s", table->command, strerror(errno));
+  return 0;
+}
+
+static const unsigned char brfTable[0X40] = {
+  /* 0X20   */ 0,
+  /* 0X21 ! */ BRL_DOT2 | BRL_DOT3 | BRL_DOT4 | BRL_DOT6,
+  /* 0X22 " */ BRL_DOT5,
+  /* 0X23 # */ BRL_DOT3 | BRL_DOT4 | BRL_DOT5 | BRL_DOT6,
+  /* 0X24 $ */ BRL_DOT1 | BRL_DOT2 | BRL_DOT4 | BRL_DOT6,
+  /* 0X25 % */ BRL_DOT1 | BRL_DOT4 | BRL_DOT6,
+  /* 0X26 & */ BRL_DOT1 | BRL_DOT2 | BRL_DOT3 | BRL_DOT4 | BRL_DOT6,
+  /* 0X27 ' */ BRL_DOT3,
+  /* 0X28 ( */ BRL_DOT1 | BRL_DOT2 | BRL_DOT3 | BRL_DOT5 | BRL_DOT6,
+  /* 0X29 ) */ BRL_DOT2 | BRL_DOT3 | BRL_DOT4 | BRL_DOT5 | BRL_DOT6,
+  /* 0X2A * */ BRL_DOT1 | BRL_DOT6,
+  /* 0X2B + */ BRL_DOT3 | BRL_DOT4 | BRL_DOT6,
+  /* 0X2C , */ BRL_DOT6,
+  /* 0X2D - */ BRL_DOT3 | BRL_DOT6,
+  /* 0X2E . */ BRL_DOT4 | BRL_DOT6,
+  /* 0X2F / */ BRL_DOT3 | BRL_DOT4,
+  /* 0X30 0 */ BRL_DOT3 | BRL_DOT5 | BRL_DOT6,
+  /* 0X31 1 */ BRL_DOT2,
+  /* 0X32 2 */ BRL_DOT2 | BRL_DOT3,
+  /* 0X33 3 */ BRL_DOT2 | BRL_DOT5,
+  /* 0X34 4 */ BRL_DOT2 | BRL_DOT5 | BRL_DOT6,
+  /* 0X35 5 */ BRL_DOT2 | BRL_DOT6,
+  /* 0X36 6 */ BRL_DOT2 | BRL_DOT3 | BRL_DOT5,
+  /* 0X37 7 */ BRL_DOT2 | BRL_DOT3 | BRL_DOT5 | BRL_DOT6,
+  /* 0X38 8 */ BRL_DOT2 | BRL_DOT3 | BRL_DOT6,
+  /* 0X39 9 */ BRL_DOT3 | BRL_DOT5,
+  /* 0X3A : */ BRL_DOT1 | BRL_DOT5 | BRL_DOT6,
+  /* 0X3B ; */ BRL_DOT5 | BRL_DOT6,
+  /* 0X3C < */ BRL_DOT1 | BRL_DOT2 | BRL_DOT6,
+  /* 0X3D = */ BRL_DOT1 | BRL_DOT2 | BRL_DOT3 | BRL_DOT4 | BRL_DOT5 | BRL_DOT6,
+  /* 0X3E > */ BRL_DOT3 | BRL_DOT4 | BRL_DOT5,
+  /* 0X3F ? */ BRL_DOT1 | BRL_DOT4 | BRL_DOT5 | BRL_DOT6,
+  /* 0X40 @ */ BRL_DOT4,
+  /* 0X41 A */ BRL_DOT1,
+  /* 0X42 B */ BRL_DOT1 | BRL_DOT2,
+  /* 0X43 C */ BRL_DOT1 | BRL_DOT4,
+  /* 0X44 D */ BRL_DOT1 | BRL_DOT4 | BRL_DOT5,
+  /* 0X45 E */ BRL_DOT1 | BRL_DOT5,
+  /* 0X46 F */ BRL_DOT1 | BRL_DOT2 | BRL_DOT4,
+  /* 0X47 G */ BRL_DOT1 | BRL_DOT2 | BRL_DOT4 | BRL_DOT5,
+  /* 0X48 H */ BRL_DOT1 | BRL_DOT2 | BRL_DOT5,
+  /* 0X49 I */ BRL_DOT2 | BRL_DOT4,
+  /* 0X4A J */ BRL_DOT2 | BRL_DOT4 | BRL_DOT5,
+  /* 0X4B K */  BRL_DOT1 | BRL_DOT3,
+  /* 0X4C L */ BRL_DOT1 | BRL_DOT2 | BRL_DOT3,
+  /* 0X4D M */ BRL_DOT1 | BRL_DOT3 | BRL_DOT4,
+  /* 0X4E N */ BRL_DOT1 | BRL_DOT3 | BRL_DOT4 | BRL_DOT5,
+  /* 0X4F O */ BRL_DOT1 | BRL_DOT3 | BRL_DOT5,
+  /* 0X50 P */ BRL_DOT1 | BRL_DOT2 | BRL_DOT3 | BRL_DOT4,
+  /* 0X51 Q */ BRL_DOT1 | BRL_DOT2 | BRL_DOT3 | BRL_DOT4 | BRL_DOT5,
+  /* 0X52 R */ BRL_DOT1 | BRL_DOT2 | BRL_DOT3 | BRL_DOT5,
+  /* 0X53 S */ BRL_DOT2 | BRL_DOT3 | BRL_DOT4,
+  /* 0X54 T */ BRL_DOT2 | BRL_DOT3 | BRL_DOT4 | BRL_DOT5,
+  /* 0X55 U */ BRL_DOT1 | BRL_DOT3 | BRL_DOT6,
+  /* 0X56 V */ BRL_DOT1 | BRL_DOT2 | BRL_DOT3 | BRL_DOT6,
+  /* 0X57 W */ BRL_DOT2 | BRL_DOT4 | BRL_DOT5 | BRL_DOT6,
+  /* 0X58 X */ BRL_DOT1 | BRL_DOT3 | BRL_DOT4 | BRL_DOT6,
+  /* 0X59 Y */ BRL_DOT1 | BRL_DOT3 | BRL_DOT4 | BRL_DOT5 | BRL_DOT6,
+  /* 0X5A Z */ BRL_DOT1 | BRL_DOT3 | BRL_DOT5 | BRL_DOT6,
+  /* 0X5B [ */ BRL_DOT2 | BRL_DOT4 | BRL_DOT6,
+  /* 0X5C \ */ BRL_DOT1 | BRL_DOT2 | BRL_DOT5 | BRL_DOT6,
+  /* 0X5D ] */ BRL_DOT1 | BRL_DOT2 | BRL_DOT4 | BRL_DOT5 | BRL_DOT6,
+  /* 0X5E ^ */ BRL_DOT4 | BRL_DOT5,
+  /* 0X5F _ */ BRL_DOT4 | BRL_DOT5 | BRL_DOT6
+};
+
+static int
+handleExternalResponse_brf (const char *value) {
+  int useDot7 = prefs.capitalizationMode == CTB_CAP_DOT7;
+
+  while (*value && (dest < destmax)) {
+    unsigned char brf = *value++ & 0XFF;
+    unsigned char dots = 0;
+    unsigned char superimpose = 0;
+
+    if ((brf >= 0X60) && (brf <= 0X7F)) {
+      brf -= 0X20;
+    } else if ((brf >= 0X41) && (brf <= 0X5A)) {
+      if (useDot7) superimpose |= BRL_DOT7;
+    }
+
+    if ((brf >= 0X20) && (brf <= 0X5F)) dots = brfTable[brf - 0X20] | superimpose;
+    *dest++ = dots;
+  }
+
+  return 1;
+}
+
+static int
+handleExternalResponse_consumedLength (const char *value) {
+  int length;
+
+  if (!isInteger(&length, value)) return 0;
+  if (length < 1) return 0;
+  if (length > (srcmax - srcmin)) return 0;
+
+  src = srcmin + length;
+  return 1;
+}
+
+static int
+handleExternalResponse_outputOffsets (const char *value) {
+  if (offsets) {
+    int previous = CTB_NO_OFFSET;
+    unsigned int count = srcmax - srcmin;
+    unsigned int index = 0;
+
+    while (*value && (index < count)) {
+      int offset;
+
+      {
+        char *delimiter = strchr(value, ',');
+
+        if (delimiter) {
+          int ok;
+
+          {
+            char oldDelimiter = *delimiter;
+            *delimiter = 0;
+            ok = isInteger(&offset, value);
+            *delimiter = oldDelimiter;
+          }
+
+          if (!ok) return 0;
+          value = delimiter + 1;
+        } else if (isInteger(&offset, value)) {
+          value += strlen(value);
+        } else {
+          return 0;
         }
       }
 
-      if (++src == srcmax) break;
-      clearOffset();
+      if (offset < ((index == 0)? 0: previous)) return 0;
+      if (offset >= (destmax - destmin)) return 0;
+
+      offsets[index++] = (offset == previous)? CTB_NO_OFFSET: offset;
+      previous = offset;
+    }
+  }
+
+  return 1;
+}
+
+typedef struct {
+  const char *name;
+  int (*handler) (const char *value);
+  unsigned stop:1;
+} ExternalResponseEntry;
+
+static const ExternalResponseEntry externalResponseTable[] = {
+  { .name = "brf",
+    .stop = 1,
+    .handler = handleExternalResponse_brf
+  },
+
+  { .name = "consumed-length",
+    .handler = handleExternalResponse_consumedLength
+  },
+
+  { .name = "output-offsets",
+    .handler = handleExternalResponse_outputOffsets
+  },
+
+  { .name = NULL }
+};
+
+static int
+getExternalResponses (void) {
+  static char *buffer = NULL;
+  static size_t size = 0;
+
+  FILE *stream = table->data.external.standardOutput;
+
+  while (readLine(stream, &buffer, &size)) {
+    int ok = 0;
+    int stop = 0;
+    char *delimiter = strchr(buffer, '=');
+
+    if (delimiter) {
+      const char *value = delimiter + 1;
+      const ExternalResponseEntry *rsp = externalResponseTable;
+
+      char oldDelimiter = *delimiter;
+      *delimiter = 0;
+
+      while (rsp->name) {
+        if (strcmp(buffer, rsp->name) == 0) {
+          if (rsp->handler(value)) ok = 1;
+          if (rsp->stop) stop = 1;
+          break;
+        }
+
+        rsp += 1;
+      }
+
+      *delimiter = oldDelimiter;
     }
 
-    if (!done) src = srcorig;
+    if (!ok) logMessage(LOG_WARNING, "unexpected external contraction response: %s: %s", table->command, buffer);
+    if (stop) return 1;
+  }
+
+  logMessage(LOG_WARNING, "incomplete external contraction response: %s", table->command);
+  return 0;
+}
+
+static int
+contractTextExternally (void) {
+  setOffset();
+  while (++src < srcmax) clearOffset();
+
+  if (startContractionCommand(table)) {
+    if (putExternalRequests()) {
+      if (getExternalResponses()) {
+        return 1;
+      }
+    }
+  }
+
+  stopContractionCommand(table);
+  return 0;
+}
+
+static inline unsigned int
+makeCachedInputCount (void) {
+  return srcmax - srcmin;
+}
+
+static inline unsigned int
+makeCachedOutputMaximum (void) {
+  return destmax - destmin;
+}
+
+static inline int
+makeCachedCursorOffset (void) {
+  return cursor? (cursor - srcmin): CTB_NO_CURSOR;
+}
+
+static int
+checkCache (void) {
+  if (!table->cache.input.characters) return 0;
+  if (!table->cache.output.cells) return 0;
+  if (offsets && !table->cache.offsets.count) return 0;
+  if (table->cache.output.maximum != makeCachedOutputMaximum()) return 0;
+  if (table->cache.cursorOffset != makeCachedCursorOffset()) return 0;
+  if (table->cache.expandCurrentWord != prefs.expandCurrentWord) return 0;
+  if (table->cache.capitalizationMode != prefs.capitalizationMode) return 0;
+
+  {
+    unsigned int count = makeCachedInputCount();
+    if (table->cache.input.count != count) return 0;
+    if (wmemcmp(srcmin, table->cache.input.characters, count) != 0) return 0;
+  }
+
+  return 1;
+}
+
+static void
+updateCache (void) {
+  {
+    unsigned int count = makeCachedInputCount();
+
+    if (count > table->cache.input.size) {
+      unsigned int newSize = count | 0X7F;
+      wchar_t *newCharacters = malloc(ARRAY_SIZE(newCharacters, newSize));
+
+      if (!newCharacters) {
+        logMallocError();
+        table->cache.input.count = 0;
+        goto inputDone;
+      }
+
+      if (table->cache.input.characters) free(table->cache.input.characters);
+      table->cache.input.characters = newCharacters;
+      table->cache.input.size = newSize;
+    }
+
+    wmemcpy(table->cache.input.characters, srcmin, count);
+    table->cache.input.count = count;
+    table->cache.input.consumed = src - srcmin;
+  }
+inputDone:
+
+  {
+    unsigned int count = dest - destmin;
+
+    if (count > table->cache.output.size) {
+      unsigned int newSize = count | 0X7F;
+      unsigned char *newCells = malloc(ARRAY_SIZE(newCells, newSize));
+
+      if (!newCells) {
+        logMallocError();
+        table->cache.output.count = 0;
+        goto outputDone;
+      }
+
+      if (table->cache.output.cells) free(table->cache.output.cells);
+      table->cache.output.cells = newCells;
+      table->cache.output.size = newSize;
+    }
+
+    memcpy(table->cache.output.cells, destmin, count);
+    table->cache.output.count = count;
+    table->cache.output.maximum = makeCachedOutputMaximum();
+  }
+outputDone:
+
+  if (offsets) {
+    unsigned int count = makeCachedInputCount();
+
+    if (count > table->cache.offsets.size) {
+      unsigned int newSize = count | 0X7F;
+      int *newArray = malloc(ARRAY_SIZE(newArray, newSize));
+
+      if (!newArray) {
+        logMallocError();
+        table->cache.offsets.count = 0;
+        goto offsetsDone;
+      }
+
+      if (table->cache.offsets.array) free(table->cache.offsets.array);
+      table->cache.offsets.array = newArray;
+      table->cache.offsets.size = newSize;
+    }
+
+    memcpy(table->cache.offsets.array, offsets, ARRAY_SIZE(offsets, count));
+    table->cache.offsets.count = count;
+  } else {
+    table->cache.offsets.count = 0;
+  }
+offsetsDone:
+
+  table->cache.cursorOffset = makeCachedCursorOffset();
+  table->cache.expandCurrentWord = prefs.expandCurrentWord;
+  table->cache.capitalizationMode = prefs.capitalizationMode;
+}
+
+void
+contractText (
+  ContractionTable *contractionTable,
+  const wchar_t *inputBuffer, int *inputLength,
+  BYTE *outputBuffer, int *outputLength,
+  int *offsetsMap, const int cursorOffset
+) {
+  table = contractionTable;
+  srcmax = (srcmin = src = inputBuffer) + *inputLength;
+  destmax = (destmin = dest = outputBuffer) + *outputLength;
+  offsets = offsetsMap;
+  cursor = (cursorOffset == CTB_NO_CURSOR)? NULL: &src[cursorOffset];
+
+  if (checkCache()) {
+    src = srcmin + table->cache.input.consumed;
+    if (offsets)
+      memcpy(offsets, table->cache.offsets.array,
+             ARRAY_SIZE(offsets, table->cache.offsets.count));
+
+    dest = destmin + table->cache.output.count;
+    memcpy(destmin, table->cache.output.cells,
+           ARRAY_SIZE(destmin, table->cache.output.count));
+  } else {
+    if (!(table->command? contractTextExternally(): contractTextInternally())) {
+      src = srcmin;
+      dest = destmin;
+
+      while ((src < srcmax) && (dest < destmax)) {
+        setOffset();
+        *dest++ = convertCharacterToDots(textTable, *src++);
+      }
+    }
+
+    if (src < srcmax) {
+      const wchar_t *srcorig = src;
+      int done = 1;
+
+      setOffset();
+      while (1) {
+        if (done && !testCharacter(*src, CTC_Space)) {
+          done = 0;
+
+          if (!cursor || (cursor < srcorig) || (cursor >= src)) {
+            setOffset();
+            srcorig = src;
+          }
+        }
+
+        if (++src == srcmax) break;
+        clearOffset();
+      }
+
+      if (!done) src = srcorig;
+    }
+
+    updateCache();
   }
 
   *inputLength = src - srcmin;
   *outputLength = dest - destmin;
-  return 1;
 }
