@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -32,22 +32,26 @@
 #include "queue.h"
 #include "datafile.h"
 #include "charset.h"
+#include "unicode.h"
 #include "brldots.h"
 
 struct DataFileStruct {
   const char *name;
-  int line;		/*line number in table */
+  int line;
+
   DataProcessor *processor;
   void *data;
 
-  DataFile *includer;
   Queue *variables;
 
   const wchar_t *start;
   const wchar_t *end;
 };
 
-const wchar_t brlDotNumbers[BRL_DOT_COUNT] = WS_C("12345678");
+const wchar_t brlDotNumbers[BRL_DOT_COUNT] = {
+  WC_C('1'), WC_C('2'), WC_C('3'), WC_C('4'),
+  WC_C('5'), WC_C('6'), WC_C('7'), WC_C('8')
+};
 const unsigned char brlDotBits[BRL_DOT_COUNT] = {
   BRL_DOT1, BRL_DOT2, BRL_DOT3, BRL_DOT4,
   BRL_DOT5, BRL_DOT6, BRL_DOT7, BRL_DOT8
@@ -88,11 +92,11 @@ reportDataError (DataFile *file, char *format, ...) {
     va_end(args);
   }
 
-  LogPrint(LOG_WARNING, "%s", message);
+  logMessage(LOG_WARNING, "%s", message);
 }
 
 int
-isKeyword (const wchar_t *keyword, const wchar_t *characters, int length) {
+isKeyword (const wchar_t *keyword, const wchar_t *characters, size_t length) {
   if (length != wcslen(keyword)) return 0;
 
   while (length > 0) {
@@ -163,7 +167,7 @@ typedef struct {
 } DataVariable;
 
 static void
-deallocateDataVariable (void *item, void *data) {
+deallocateDataVariable (void *item, void *data UNUSED) {
   DataVariable *variable = item;
 
   if (variable->name.characters) free((void *)variable->name.characters);
@@ -171,6 +175,13 @@ deallocateDataVariable (void *item, void *data) {
   free(variable);
 }
 
+static Queue *
+newDataVariableQueue (Queue *previous) {
+  Queue *queue = newQueue(deallocateDataVariable, NULL);
+  if (queue) setQueueData(queue, previous);
+  return queue;
+}
+
 static int
 testDataVariableName (const void *item, const void *data) {
   const DataVariable *variable = item;
@@ -182,67 +193,129 @@ testDataVariableName (const void *item, const void *data) {
   return 0;
 }
 
+static DataVariable *
+getDataVariable (Queue *variables, const DataOperand *name, int create) {
+  DataVariable *variable = findItem(variables, testDataVariableName, name);
+  if (variable) return variable;
+
+  if (create) {
+    if ((variable = malloc(sizeof(*variable)))) {
+      wchar_t *nameCharacters;
+
+      memset(variable, 0, sizeof(*variable));
+
+      if ((nameCharacters = malloc(ARRAY_SIZE(nameCharacters, name->length)))) {
+        variable->name.characters = wmemcpy(nameCharacters, name->characters, name->length);
+        variable->name.length = name->length;
+
+        variable->value.characters = NULL;
+        variable->value.length = 0;
+
+        if (enqueueItem(variables, variable)) return variable;
+
+        free(nameCharacters);
+      } else {
+        logMallocError();
+      }
+
+      free(variable);
+    } else {
+      logMallocError();
+    }
+  }
+
+  return NULL;
+}
+
 static const DataVariable *
 getReadableDataVariable (DataFile *file, const DataOperand *name) {
-  while (file) {
-    if (file->variables) {
-      DataVariable *variable = findItem(file->variables, testDataVariableName, name);
-      if (variable) return variable;
-    }
+  Queue *variables = file->variables;
 
-    file = file->includer;
-  }
+  do {
+    DataVariable *variable = getDataVariable(variables, name, 0);
+    if (variable) return variable;
+  } while ((variables = getQueueData(variables)));
 
   return NULL;
 }
 
 static DataVariable *
 getWritableDataVariable (DataFile *file, const DataOperand *name) {
-  if (!file->variables)
-    if (!(file->variables = newQueue(deallocateDataVariable, NULL)))
-      return NULL;
-
-  if (file->variables) {
-    DataVariable *variable = findItem(file->variables, testDataVariableName, name);
-    if (variable) return variable;
-
-    if ((variable = malloc(sizeof(*variable)))) {
-      wchar_t *nameCharacters;
-
-      if ((nameCharacters = malloc(ARRAY_SIZE(nameCharacters, name->length)))) {
-        wmemcpy(nameCharacters, name->characters, name->length);
-        variable->name.characters = nameCharacters;
-        variable->name.length = name->length;
-        variable->value.characters = NULL;
-        variable->value.length = 0;
-
-        if (enqueueItem(file->variables, variable)) return variable;
-
-        free(nameCharacters);
-      }
-
-      free(variable);
-    }
-  }
-
-  return NULL;
+  return getDataVariable(file->variables, name, 1);
 }
 
 static int
-setDataVariable (DataVariable *variable, const DataOperand *value) {
-  wchar_t *valueCharacters;
+setDataVariable (DataVariable *variable, const wchar_t *characters, int length) {
+  wchar_t *value;
 
-  if (!value->length) {
-    valueCharacters = NULL;
-  } else if (!(valueCharacters = malloc(ARRAY_SIZE(valueCharacters, value->length)))) {
+  if (!length) {
+    value = NULL;
+  } else if (!(value = malloc(ARRAY_SIZE(value, length)))) {
+    logMallocError();
     return 0;
   } else {
-    wmemcpy(valueCharacters, value->characters, value->length);
+    wmemcpy(value, characters, length);
   }
 
   if (variable->value.characters) free((void *)variable->value.characters);
-  variable->value.characters = valueCharacters;
-  variable->value.length = value->length;
+  variable->value.characters = value;
+  variable->value.length = length;
+  return 1;
+}
+
+static Queue *
+getGlobalDataVariables () {
+  static Queue *variables = NULL;
+
+  if (!variables) variables = newDataVariableQueue(NULL);
+  return variables;
+}
+
+int
+setGlobalDataVariable (const char *name, const char *value) {
+  size_t nameLength = getUtf8Length(name);
+  wchar_t nameBuffer[nameLength + 1];
+
+  size_t valueLength = getUtf8Length(value);
+  wchar_t valueBuffer[valueLength + 1];
+
+  {
+    const char *utf8 = name;
+    wchar_t *wc = nameBuffer;
+    convertUtf8ToWchars(&utf8, &wc, ARRAY_COUNT(nameBuffer));
+  }
+
+  {
+    const char *utf8 = value;
+    wchar_t *wc = valueBuffer;
+    convertUtf8ToWchars(&utf8, &wc, ARRAY_COUNT(valueBuffer));
+  }
+
+  {
+    Queue *variables = getGlobalDataVariables();
+
+    if (variables) {
+      const DataOperand nameArgument = {
+        .characters = nameBuffer,
+        .length = nameLength
+      };
+      DataVariable *variable = getDataVariable(variables, &nameArgument, 1);
+
+      if (variable) {
+        if (setDataVariable(variable, valueBuffer, valueLength)) {
+          return 1;
+        }
+      }
+    }
+  }
+
+  return 0;
+}
+
+int
+setGlobalTableVariables (const char *tableExtension, const char *subtableExtension) {
+  if (!setGlobalDataVariable("tableExtension", tableExtension)) return 0;
+  if (!setGlobalDataVariable("subtableExtension", subtableExtension)) return 0;
   return 1;
 }
 
@@ -307,7 +380,7 @@ getDataText (DataFile *file, DataOperand *text, const char *description) {
   return 1;
 }
 
-static int
+int
 parseDataString (DataFile *file, DataString *string, const wchar_t *characters, int length, int noUnicode) {
   int index = 0;
 
@@ -322,6 +395,7 @@ parseDataString (DataFile *file, DataString *string, const wchar_t *characters,
 
     if (character == WC_C('\\')) {
       int start = index;
+      const char *problem = strtext("invalid escape sequence");
       int ok = 0;
 
       if (++index < length) {
@@ -367,6 +441,7 @@ parseDataString (DataFile *file, DataString *string, const wchar_t *characters,
             break;
 
           {
+            uint32_t result;
             int count;
             int (*isDigit) (wchar_t character, int *value, int *shift);
 
@@ -395,7 +470,7 @@ parseDataString (DataFile *file, DataString *string, const wchar_t *characters,
             goto doNumber;
 
           doNumber:
-            character = 0;
+            result = 0;
 
             while (++index < length) {
               {
@@ -403,11 +478,17 @@ parseDataString (DataFile *file, DataString *string, const wchar_t *characters,
                 int shift;
 
                 if (!isDigit(characters[index], &value, &shift)) break;
-                character = (character << shift) | value;
+                result = (result << shift) | value;
               }
 
               if (!--count) {
-                ok = 1;
+                if (result > WCHAR_MAX) {
+                  problem = NULL;
+                } else {
+                  character = result;
+                  ok = 1;
+                }
+
                 break;
               }
             }
@@ -484,8 +565,13 @@ parseDataString (DataFile *file, DataString *string, const wchar_t *characters,
 
       if (!ok) {
         if (index < length) index += 1;
-        reportDataError(file, "invalid escape sequence: %.*" PRIws,
-                        index-start, &characters[start]);
+
+        if (problem) {
+          reportDataError(file, "%s: %.*" PRIws,
+                          gettext(problem),
+                          index-start, &characters[start]);
+        }
+
         return 0;
       }
     }
@@ -542,7 +628,7 @@ getDotOperand (DataFile *file, int *index) {
 }
 
 int
-processAssignOperands (DataFile *file, void *data) {
+processAssignOperands (DataFile *file, void *data UNUSED) {
   DataOperand name;
 
   if (getDataOperand(file, &name, "variable name")) {
@@ -557,7 +643,7 @@ processAssignOperands (DataFile *file, void *data) {
       DataVariable *variable = getWritableDataVariable(file, &name);
 
       if (variable) {
-        if (setDataVariable(variable, &value)) return 1;
+        if (setDataVariable(variable, value.characters, value.length)) return 1;
       }
     }
   }
@@ -583,7 +669,7 @@ includeDataFile (DataFile *file, const wchar_t *name, unsigned int length) {
     snprintf(path, sizeof(path), "%.*s%.*" PRIws,
              prefixLength, prefixAddress, length, name);
     if ((stream = openDataFile(path, "r", 0))) {
-      if (processDataStream(file, stream, path, file->processor, file->data)) ok = 1;
+      if (processDataStream(file->variables, stream, path, file->processor, file->data)) ok = 1;
       fclose(stream);
     }
 
@@ -592,7 +678,7 @@ includeDataFile (DataFile *file, const wchar_t *name, unsigned int length) {
 }
 
 int
-processIncludeOperands (DataFile *file, void *data) {
+processIncludeOperands (DataFile *file, void *data UNUSED) {
   DataString path;
 
   if (getDataString(file, &path, 0, "include file path"))
@@ -647,7 +733,7 @@ processUtf8Line (char *line, void *dataAddress) {
   wchar_t *character = characters;
 
   file->line += 1;
-  convertStringToWchars(&byte, &character, size);
+  convertUtf8ToWchars(&byte, &character, size);
 
   if (*byte) {
     unsigned int offset = byte - line;
@@ -660,28 +746,30 @@ processUtf8Line (char *line, void *dataAddress) {
 
 int
 processDataStream (
-  DataFile *includer,
+  Queue *variables,
   FILE *stream, const char *name,
   DataProcessor processor, void *data
 ) {
+  int ok = 0;
   DataFile file;
 
   file.name = name;
   file.line = 0;
+
   file.processor = processor;
   file.data = data;
 
-  file.includer = includer;
-  file.variables = NULL;
+  if (!variables)
+    if (!(variables = getGlobalDataVariables()))
+      return 0;
 
-  LogPrint(LOG_DEBUG, "including data file: %s", file.name);
-  {
-    int ok = processLines(stream, processUtf8Line, &file);
-
-    if (file.variables) deallocateQueue(file.variables);
-
-    return ok;
+  logMessage(LOG_DEBUG, "including data file: %s", file.name);
+  if ((file.variables = newDataVariableQueue(variables))) {
+    if (processLines(stream, processUtf8Line, &file)) ok = 1;
+    deallocateQueue(file.variables);
   }
+
+  return ok;
 }
 
 int
