@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -18,7 +18,7 @@
 
 #include "prologue.h"
 
-#include "brltty.h"
+#include "prefs.h"
 #include "log.h"
 #include "timing.h"
 #include "notes.h"
@@ -37,7 +37,7 @@ fmConstruct (int errorLevel) {
       if (AL_testCard(errorLevel)) {
         device->channelNumber = 0;
 
-        LogPrint(LOG_DEBUG, "FM enabled");
+        logMessage(LOG_DEBUG, "FM enabled");
         return device;
       }
 
@@ -46,20 +46,20 @@ fmConstruct (int errorLevel) {
 
     free(device);
   } else {
-    LogError("malloc");
+    logMallocError();
   }
 
-  LogPrint(LOG_DEBUG, "FM not available");
+  logMessage(LOG_DEBUG, "FM not available");
   return NULL;
 }
 
 static int
-fmPlay (NoteDevice *device, int note, int duration) {
-  LogPrint(LOG_DEBUG, "tone: msec=%d note=%d",
-           duration, note);
+fmPlay (NoteDevice *device, unsigned char note, unsigned int duration) {
+  logMessage(LOG_DEBUG, "tone: msec=%d note=%d",
+             duration, note);
 
   if (note) {
-    AL_playTone(device->channelNumber, (int)noteFrequencies[note], duration, prefs.fmVolume);
+    AL_playTone(device->channelNumber, getIntegerNoteFrequency(note), duration, prefs.fmVolume);
   } else {
     accurateDelay(duration);
   }
@@ -76,7 +76,7 @@ static void
 fmDestruct (NoteDevice *device) {
   free(device);
   AL_disablePorts();
-  LogPrint(LOG_DEBUG, "FM disabled");
+  logMessage(LOG_DEBUG, "FM disabled");
 }
 
 const NoteMethods fmMethods = {
