@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -33,6 +33,7 @@
 
 #include "io_misc.h"
 #include "log.h"
+#include "timing.h"
 
 #ifdef __MSDOS__
 #include "sys_msdos.h"
@@ -40,6 +41,12 @@
 
 static int
 awaitFileDescriptor (int fileDescriptor, int milliseconds, int output) {
+#if defined(GRUB_RUNTIME)
+  approximateDelay(milliseconds);
+  errno = EAGAIN;
+  return 0;
+#else
+
 #ifdef __MSDOS__
   int left = milliseconds * 1000;
 #endif /* __MSDOS__ */
@@ -64,7 +71,7 @@ awaitFileDescriptor (int fileDescriptor, int milliseconds, int output) {
 
         if (result == -1) {
           if (errno == EINTR) continue;
-          LogError("poll");
+          logSystemError("poll");
           return 0;
         }
 
@@ -91,7 +98,7 @@ awaitFileDescriptor (int fileDescriptor, int milliseconds, int output) {
 
         if (result == -1) {
           if (errno == EINTR) continue;
-          LogError("select");
+          logSystemError("select");
           return 0;
         }
 
@@ -108,12 +115,13 @@ awaitFileDescriptor (int fileDescriptor, int milliseconds, int output) {
 #endif /* __MSDOS__ */
 
     if (milliseconds > 0)
-      LogPrint(LOG_DEBUG, "timeout after %d %s",
-               milliseconds, ((milliseconds == 1)? "millisecond": "milliseconds"));
+      logMessage(LOG_DEBUG, "timeout after %d %s",
+                 milliseconds, ((milliseconds == 1)? "millisecond": "milliseconds"));
 
     errno = EAGAIN;
     return 0;
   }
+#endif
 }
 
 int
@@ -153,7 +161,7 @@ readChunk (
       if (errno == EWOULDBLOCK) goto noInput;
 #endif /* EWOULDBLOCK */
 
-      LogError("read");
+      logSystemError("read");
       return 0;
     }
 
@@ -162,7 +170,7 @@ readChunk (
     noInput:
       if ((timeout = *offset? subsequentTimeout: initialTimeout)) {
         if (awaitInput(fileDescriptor, timeout)) continue;
-        LogPrint(LOG_WARNING, "input byte missing at offset %d", (int)*offset);
+        logMessage(LOG_WARNING, "input byte missing at offset %d", (int)*offset);
 #ifdef __MSDOS__
       } else if (!tried) {
         if (awaitInput(fileDescriptor, 0)) continue;
@@ -209,7 +217,7 @@ canWrite:
 #ifdef EWOULDBLOCK
       if (errno == EWOULDBLOCK) goto noOutput;
 #endif /* EWOULDBLOCK */
-      LogError("Write");
+      logSystemError("Write");
       return count;
     }
 
@@ -245,10 +253,10 @@ changeOpenFlags (int fileDescriptor, int flagsToClear, int flagsToSet) {
     if (fcntl(fileDescriptor, F_SETFL, flags) != -1) {
       return 1;
     } else {
-      LogError("F_SETFL");
+      logSystemError("F_SETFL");
     }
   } else {
-    LogError("F_GETFL");
+    logSystemError("F_GETFL");
   }
 #else /* defined(F_GETFL) && defined(F_SETFL) */
   errno = ENOSYS;
