@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -19,53 +19,57 @@
 #ifndef BRLTTY_INCLUDED_IO_SERIAL
 #define BRLTTY_INCLUDED_IO_SERIAL
 
+#include <stdio.h>
+
+#include "serialdefs.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
-#include "prologue.h"
-
-#include <stdio.h>
-
-#include "io_defs.h"
-
 typedef struct SerialDeviceStruct SerialDevice;
 
-extern int isSerialDevice (const char **path);
-extern int serialValidateBaud (int *baud, const char *description, const char *word, const int *choices);
+extern int isSerialDevice (const char **identifier);
+extern int serialValidateBaud (unsigned int *baud, const char *description, const char *word, const unsigned int *choices);
 
 extern SerialDevice *serialOpenDevice (const char *path);
 extern void serialCloseDevice (SerialDevice *serial);
-extern int serialRestartDevice (SerialDevice *serial, int baud);
+extern int serialRestartDevice (SerialDevice *serial, unsigned int baud);
 extern FILE *serialGetStream (SerialDevice *serial);
 
+extern int serialDiscardInput (SerialDevice *serial);
+extern int serialDiscardOutput (SerialDevice *serial);
+extern int serialFlushOutput (SerialDevice *serial);
+
 extern int serialAwaitInput (SerialDevice *serial, int timeout);
+extern int serialAwaitOutput (SerialDevice *serial);
+
 extern ssize_t serialReadData (
   SerialDevice *serial,
   void *buffer, size_t size,
   int initialTimeout, int subsequentTimeout
 );
+
 extern int serialReadChunk (
   SerialDevice *serial,
   void *buffer, size_t *offset, size_t count,
   int initialTimeout, int subsequentTimeout
 );
+
 extern ssize_t serialWriteData (
   SerialDevice *serial,
   const void *data, size_t size
 );
 
-extern int serialSetBaud (SerialDevice *serial, int baud);
-extern int serialSetDataBits (SerialDevice *serial, int bits);
-extern int serialSetStopBits (SerialDevice *serial, int bits);
+extern int serialSetParameters (SerialDevice *serial, const SerialParameters *parameters);
+extern int serialSetBaud (SerialDevice *serial, unsigned int baud);
+extern int serialSetDataBits (SerialDevice *serial, unsigned int bits);
+extern int serialSetStopBits (SerialDevice *serial, SerialStopBits bits);
 extern int serialSetParity (SerialDevice *serial, SerialParity parity);
 extern int serialSetFlowControl (SerialDevice *serial, SerialFlowControl flow);
-extern int serialGetCharacterBits (SerialDevice *serial);
 
-extern int serialDiscardInput (SerialDevice *serial);
-extern int serialDiscardOutput (SerialDevice *serial);
-extern int serialFlushOutput (SerialDevice *serial);
-extern int serialDrainOutput (SerialDevice *serial);
+extern unsigned int serialGetCharacterSize (const SerialParameters *parameters);
+extern unsigned int serialGetCharacterBits (SerialDevice *serial);
 
 extern int serialSetLineRTS (SerialDevice *serial, int up);
 extern int serialSetLineDTR (SerialDevice *serial, int up);
