@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -29,22 +29,22 @@
 #include "ktb.h"
 #include "ktb_internal.h"
 
-const KeyboardFunctionEntry keyboardFunctionTable[KBF_None] = {
-  [KBF_Dot1] = {.name="dot1", .bit=BRL_DOT1},
-  [KBF_Dot2] = {.name="dot2", .bit=BRL_DOT2},
-  [KBF_Dot3] = {.name="dot3", .bit=BRL_DOT3},
-  [KBF_Dot4] = {.name="dot4", .bit=BRL_DOT4},
-  [KBF_Dot5] = {.name="dot5", .bit=BRL_DOT5},
-  [KBF_Dot6] = {.name="dot6", .bit=BRL_DOT6},
-  [KBF_Dot7] = {.name="dot7", .bit=BRL_DOT7},
-  [KBF_Dot8] = {.name="dot8", .bit=BRL_DOT8},
-
-  [KBF_Space] = {.name="space", .bit=0},
-  [KBF_Shift] = {.name="shift", .bit=BRL_FLG_CHAR_SHIFT},
-  [KBF_Uppercase] = {.name="uppercase", .bit=BRL_FLG_CHAR_UPPER},
-  [KBF_Control] = {.name="control", .bit=BRL_FLG_CHAR_CONTROL},
-  [KBF_Meta] = {.name="meta", .bit=BRL_FLG_CHAR_META}
+const KeyboardFunction keyboardFunctionTable[] = {
+  {.name="dot1", .bit=BRL_DOT1},
+  {.name="dot2", .bit=BRL_DOT2},
+  {.name="dot3", .bit=BRL_DOT3},
+  {.name="dot4", .bit=BRL_DOT4},
+  {.name="dot5", .bit=BRL_DOT5},
+  {.name="dot6", .bit=BRL_DOT6},
+  {.name="dot7", .bit=BRL_DOT7},
+  {.name="dot8", .bit=BRL_DOT8},
+  {.name="space", .bit=BRL_DOTC},
+  {.name="shift", .bit=BRL_FLG_CHAR_SHIFT},
+  {.name="uppercase", .bit=BRL_FLG_CHAR_UPPER},
+  {.name="control", .bit=BRL_FLG_CHAR_CONTROL},
+  {.name="meta", .bit=BRL_FLG_CHAR_META}
 };
+unsigned char keyboardFunctionCount = ARRAY_COUNT(keyboardFunctionTable);
 
 typedef struct {
   KeyTable *table;
@@ -123,7 +123,7 @@ insertKeyValue (
     KeyValue *newValues = realloc(*values, ARRAY_SIZE(newValues, newSize));
 
     if (!newValues) {
-      LogError("realloc");
+      logSystemError("realloc");
       return 0;
     }
 
@@ -164,7 +164,7 @@ getKeyContext (KeyTableData *ktd, unsigned char context) {
     KeyContext *newTable = realloc(ktd->table->keyContextTable, ARRAY_SIZE(newTable, newCount));
 
     if (!newTable) {
-      LogError("realloc");
+      logSystemError("realloc");
       return NULL;
     }
     ktd->table->keyContextTable = newTable;
@@ -184,7 +184,9 @@ getKeyContext (KeyTableData *ktd, unsigned char context) {
       ctx->hotkeyCount = 0;
       ctx->sortedHotkeyEntries = NULL;
 
-      ctx->keyMap = NULL;
+      ctx->mappedKeyTable = NULL;
+      ctx->mappedKeyCount = 0;
+      ctx->sortedMappedKeyEntries = NULL;
       ctx->superimposedBits = 0;
     }
   }
@@ -202,7 +204,7 @@ setKeyContextTitle (KeyContext *ctx, const wchar_t *title, size_t length) {
   if (ctx->title) free(ctx->title);
 
   if (!(ctx->title = malloc(ARRAY_SIZE(ctx->title, length+1)))) {
-    LogError("malloc");
+    logMallocError();
     return 0;
   }
 
@@ -219,11 +221,11 @@ setDefaultKeyContextProperties (KeyTableData *ktd) {
   } PropertiesEntry;
 
   static const PropertiesEntry propertiesTable[] = {
-    { .context = BRL_CTX_DEFAULT,
+    { .context = KTB_CTX_DEFAULT,
       .title = WS_C("Default Bindings")
     }
     ,
-    { .context = BRL_CTX_MENU,
+    { .context = KTB_CTX_MENU,
       .title = WS_C("Menu Bindings")
     }
     ,
@@ -284,7 +286,11 @@ static int
 sortKeyValues (const void *element1, const void *element2) {
   const KeyNameEntry *const *kne1 = element1;
   const KeyNameEntry *const *kne2 = element2;
-  return compareKeyValues(&(*kne1)->value, &(*kne2)->value);
+  int result = compareKeyValues(&(*kne1)->value, &(*kne2)->value);
+  if (result != 0) return result;
+  if (*kne1 < *kne2) return -1;
+  if (*kne1 > *kne2) return 1;
+  return 0;
 }
 
 static int
@@ -319,7 +325,7 @@ allocateKeyNameTable (KeyTableData *ktd, KEY_NAME_TABLES_REFERENCE keys) {
   return 0;
 }
 
-static const KeyNameEntry **
+static const KeyNameEntry *const *
 findKeyName (const wchar_t *characters, int length, KeyTableData *ktd) {
   const DataOperand name = {
     .characters = characters,
@@ -331,15 +337,59 @@ findKeyName (const wchar_t *characters, int length, KeyTableData *ktd) {
 
 static int
 parseKeyName (DataFile *file, KeyValue *value, const wchar_t *characters, int length, KeyTableData *ktd) {
-  const KeyNameEntry **kne = findKeyName(characters, length, ktd);
+  const wchar_t *suffix = wmemchr(characters, WC_C('.'), length);
+  int prefixLength;
+  int suffixLength;
 
-  if (kne) {
-    *value = (*kne)->value;
-    return 1;
+  if (suffix) {
+    if (!(prefixLength = suffix - characters)) {
+      reportDataError(file, "missing key set name: %.*" PRIws, length, characters);
+      return 0;
+    }
+
+    if (!(suffixLength = (characters + length) - ++suffix)) {
+      reportDataError(file, "missing key number: %.*" PRIws, length, characters);
+      return 0;
+    }
+  } else {
+    prefixLength = length;
+    suffixLength = 0;
   }
 
-  reportDataError(file, "unknown key: %.*" PRIws, length, characters);
-  return 0;
+  {
+    const KeyNameEntry *const *kne = findKeyName(characters, prefixLength, ktd);
+
+    if (!kne) {
+      reportDataError(file, "unknown key name: %.*" PRIws, prefixLength, characters);
+      return 0;
+    }
+
+    *value = (*kne)->value;
+  }
+
+  if (suffix) {
+    int ok = 0;
+    int number;
+
+    if (isNumber(&number, suffix, suffixLength))
+      if (number > 0)
+        if (--number <= KTB_KEY_MAX)
+          ok = 1;
+
+    if (!ok) {
+      reportDataError(file, "invalid key number: %.*" PRIws, suffixLength, suffix);
+      return 0;
+    }
+
+    if (value->key != KTB_KEY_ANY) {
+      reportDataError(file, "not a key set: %.*" PRIws, prefixLength, characters);
+      return 0;
+    }
+
+    value->key = number;
+  }
+
+  return 1;
 }
 
 static int
@@ -416,6 +466,7 @@ parseKeyCombination (DataFile *file, KeyCombination *combination, const wchar_t
         }
 
         if (!insertModifier(file, combination, position, &value)) return 0;
+        if (value.key == KTB_KEY_ANY) combination->anyKeyCount += 1;
       }
 
       length -= count + 1;
@@ -449,6 +500,7 @@ parseKeyCombination (DataFile *file, KeyCombination *combination, const wchar_t
     } else if (!insertModifier(file, combination, position, &value)) {
       return 0;
     }
+    if (value.key == KTB_KEY_ANY) combination->anyKeyCount += 1;
   }
 
   return 1;
@@ -467,40 +519,40 @@ getKeysOperand (DataFile *file, KeyCombination *combination, KeyTableData *ktd)
 
 static int
 sortKeyboardFunctionNames (const void *element1, const void *element2) {
-  const KeyboardFunctionEntry *const *kbf1 = element1;
-  const KeyboardFunctionEntry *const *kbf2 = element2;
+  const KeyboardFunction *const *kbf1 = element1;
+  const KeyboardFunction *const *kbf2 = element2;
   return strcasecmp((*kbf1)->name, (*kbf2)->name);
 }
 
 static int
 searchKeyboardFunctionName (const void *target, const void *element) {
   const DataOperand *name = target;
-  const KeyboardFunctionEntry *const *kbf = element;
+  const KeyboardFunction *const *kbf = element;
   return compareToName(name->characters, name->length, (*kbf)->name);
 }
 
 static int
-parseKeyboardFunctionName (DataFile *file, unsigned char *function, const wchar_t *characters, int length, KeyTableData *ktd) {
-  static const KeyboardFunctionEntry **sortedKeyboardFunctions = NULL;
+parseKeyboardFunctionName (DataFile *file, const KeyboardFunction **keyboardFunction, const wchar_t *characters, int length, KeyTableData *ktd) {
+  static const KeyboardFunction **sortedKeyboardFunctions = NULL;
 
   if (!sortedKeyboardFunctions) {
-    const KeyboardFunctionEntry **newTable = malloc(ARRAY_SIZE(newTable, KBF_None));
+    const KeyboardFunction **newTable = malloc(ARRAY_SIZE(newTable, keyboardFunctionCount));
 
     if (!newTable) {
-      LogError("malloc");
+      logMallocError();
       return 0;
     }
 
     {
-      const KeyboardFunctionEntry *source = keyboardFunctionTable;
-      const KeyboardFunctionEntry **target = newTable;
-      unsigned int count = KBF_None;
+      const KeyboardFunction *source = keyboardFunctionTable;
+      const KeyboardFunction **target = newTable;
+      unsigned int count = keyboardFunctionCount;
 
       do {
         *target++ = source++;
       } while (--count);
 
-      qsort(newTable, KBF_None, sizeof(*newTable), sortKeyboardFunctionNames);
+      qsort(newTable, keyboardFunctionCount, sizeof(*newTable), sortKeyboardFunctionNames);
     }
 
     sortedKeyboardFunctions = newTable;
@@ -511,10 +563,10 @@ parseKeyboardFunctionName (DataFile *file, unsigned char *function, const wchar_
       .characters = characters,
       .length = length
     };
-    const KeyboardFunctionEntry **kbf = bsearch(&name, sortedKeyboardFunctions, KBF_None, sizeof(*sortedKeyboardFunctions), searchKeyboardFunctionName);
+    const KeyboardFunction *const *kbf = bsearch(&name, sortedKeyboardFunctions, keyboardFunctionCount, sizeof(*sortedKeyboardFunctions), searchKeyboardFunctionName);
 
     if (kbf) {
-      *function = *kbf - keyboardFunctionTable;
+      *keyboardFunction = *kbf;
       return 1;
     }
   }
@@ -524,11 +576,11 @@ parseKeyboardFunctionName (DataFile *file, unsigned char *function, const wchar_
 }
 
 static int
-getKeyboardFunctionOperand (DataFile *file, unsigned char *function, KeyTableData *ktd) {
+getKeyboardFunctionOperand (DataFile *file, const KeyboardFunction **keyboardFunction, KeyTableData *ktd) {
   DataOperand name;
 
   if (getDataOperand(file, &name, "keyboard function name")) {
-    if (parseKeyboardFunctionName(file, function, name.characters, name.length, ktd)) return 1;
+    if (parseKeyboardFunctionName(file, keyboardFunction, name.characters, name.length, ktd)) return 1;
   }
 
   return 0;
@@ -574,13 +626,31 @@ allocateCommandTable (KeyTableData *ktd) {
   return 0;
 }
 
+static int
+applyCommandModifier (int *command, const CommandModifierEntry *modifiers, const DataOperand *name) {
+  const CommandModifierEntry *modifier = modifiers;
+
+  while (modifier->name) {
+    if (!(*command & modifier->bit)) {
+      if (compareToName(name->characters, name->length, modifier->name) == 0) {
+        *command |= modifier->bit;
+        return 1;
+      }
+    }
+
+    modifier += 1;
+  }
+
+  return 0;
+}
+
 static const CommandEntry *
 parseCommandOperand (DataFile *file, int *value, const wchar_t *characters, int length, KeyTableData *ktd) {
-  int toggleDone = 0;
   int offsetDone = 0;
+  int unicodeDone = 0;
 
   const wchar_t *end = wmemchr(characters, WC_C('+'), length);
-  const CommandEntry **command;
+  const CommandEntry *const *command;
 
   {
     const DataOperand name = {
@@ -614,43 +684,44 @@ parseCommandOperand (DataFile *file, int *value, const wchar_t *characters, int
     modifier.characters = characters;
     modifier.length = end? end-characters: length;
 
-    if ((*command)->isToggle && !toggleDone) {
-      if (isKeyword(WS_C("on"), modifier.characters, modifier.length)) {
-        *value |= BRL_FLG_TOGGLE_ON;
-      }  else if (isKeyword(WS_C("off"), modifier.characters, modifier.length)) {
-        *value |= BRL_FLG_TOGGLE_OFF;
-      } else {
-        goto notToggle;
-      }
-
-      toggleDone = 1;
-      continue;
+    if ((*command)->isToggle && !(*value & BRL_FLG_TOGGLE_MASK)) {
+      if (applyCommandModifier(value, commandModifierTable_toggle, &modifier)) continue;
     }
-  notToggle:
 
     if ((*command)->isMotion) {
-      if (isKeyword(WS_C("route"), modifier.characters, modifier.length) &&
-          !(*value & BRL_FLG_MOTION_ROUTE)) {
-        *value |= BRL_FLG_MOTION_ROUTE;
-        continue;
-      }
+      if (applyCommandModifier(value, commandModifierTable_motion, &modifier)) continue;
 
       if ((*command)->isRow) {
-        if (isKeyword(WS_C("scaled"), modifier.characters, modifier.length) &&
-            !(*value & BRL_FLG_LINE_SCALED)) {
-          *value |= BRL_FLG_LINE_SCALED;
-          continue;
-        }
+        if (applyCommandModifier(value, commandModifierTable_line, &modifier)) continue;
+      }
+    }
 
-        if (isKeyword(WS_C("toleft"), modifier.characters, modifier.length) &&
-            !(*value & BRL_FLG_LINE_TOLEFT)) {
-          *value |= BRL_FLG_LINE_TOLEFT;
+    if ((*command)->isInput) {
+      if (applyCommandModifier(value, commandModifierTable_input, &modifier)) continue;
+    }
+
+    if ((*command)->isCharacter) {
+      if (applyCommandModifier(value, commandModifierTable_character, &modifier)) continue;
+
+      if (!unicodeDone) {
+        if (modifier.length == 1) {
+          *value |= BRL_ARG_SET(modifier.characters[0]);
+          unicodeDone = 1;
           continue;
         }
       }
     }
 
-    if ((*command)->isOffset && !offsetDone) {
+    if ((*command)->isBraille) {
+      if (applyCommandModifier(value, commandModifierTable_braille, &modifier)) continue;
+      if (applyCommandModifier(value, commandModifierTable_character, &modifier)) continue;
+    }
+
+    if ((*command)->isKeyboard) {
+      if (applyCommandModifier(value, commandModifierTable_keyboard, &modifier)) continue;
+    }
+
+    if (((*command)->isOffset || (*command)->isColumn) && !offsetDone) {
       int maximum = BRL_MSK_ARG - ((*command)->code & BRL_MSK_ARG);
       int offset;
 
@@ -690,7 +761,7 @@ addKeyBinding (KeyContext *ctx, const KeyBinding *binding) {
     KeyBinding *newTable = realloc(ctx->keyBindingTable, ARRAY_SIZE(newTable, newSize));
 
     if (!newTable) {
-      LogError("realloc");
+      logSystemError("realloc");
       return 0;
     }
 
@@ -723,6 +794,8 @@ processBindOperands (DataFile *file, void *data) {
       if (cmd->isColumn) binding.flags |= KBF_COLUMN;
       if (cmd->isRow) binding.flags |= KBF_ROW;
       if (cmd->isRange) binding.flags |= KBF_RANGE;
+      if (cmd->isRouting) binding.flags |= KBF_ROUTE;
+      if (cmd->isKeyboard) binding.flags |= KBF_KEYBOARD;
       if (!addKeyBinding(ctx, &binding)) return 0;
     }
   }
@@ -739,20 +812,20 @@ processContextOperands (DataFile *file, void *data) {
     int ok = 0;
 
     if (isKeyword(WS_C("default"), context.characters, context.length)) {
-      ktd->context = BRL_CTX_DEFAULT;
+      ktd->context = KTB_CTX_DEFAULT;
       ok = 1;
     } else if (isKeyword(WS_C("menu"), context.characters, context.length)) {
-      ktd->context = BRL_CTX_MENU;
+      ktd->context = KTB_CTX_MENU;
       ok = 1;
     } else {
       int number;
 
       if (!isNumber(&number, context.characters, context.length)) {
         reportDataError(file, "unknown context name: %.*" PRIws, context.length, context.characters);
-      } else if ((number < 0) || (number > (BRL_MSK_ARG - BRL_CTX_DEFAULT))) {
+      } else if ((number < 0) || (number > (BRL_MSK_ARG - KTB_CTX_DEFAULT))) {
         reportDataError(file, "invalid context number: %.*" PRIws, context.length, context.characters);
       } else {
-        ktd->context = BRL_CTX_DEFAULT + number;
+        ktd->context = KTB_CTX_DEFAULT + number;
         ok = 1;
       }
     }
@@ -807,6 +880,7 @@ processHotkeyOperands (DataFile *file, void *data) {
 
   if (!ctx) return 0;
   memset(&hotkey, 0, sizeof(hotkey));
+  if (hideBindings(ktd)) hotkey.flags |= HKF_HIDDEN;
 
   if (getKeyOperand(file, &hotkey.keyValue, ktd)) {
     if (getCommandOperand(file, &hotkey.pressCommand, ktd)) {
@@ -815,7 +889,7 @@ processHotkeyOperands (DataFile *file, void *data) {
         HotkeyEntry *newTable = realloc(ctx->hotkeyTable, ARRAY_SIZE(newTable, newCount));
 
         if (!newTable) {
-          LogError("realloc");
+          logSystemError("realloc");
           return 0;
         }
 
@@ -863,31 +937,28 @@ static int
 processMapOperands (DataFile *file, void *data) {
   KeyTableData *ktd = data;
   KeyContext *ctx = getCurrentKeyContext(ktd);
+  MappedKeyEntry map;
+
   if (!ctx) return 0;
+  memset(&map, 0, sizeof(map));
 
-  {
-    KeyValue value;
+  if (getKeyOperand(file, &map.keyValue, ktd)) {
+    if (map.keyValue.key != KTB_KEY_ANY) {
+      if (getKeyboardFunctionOperand(file, &map.keyboardFunction, ktd)) {
+        unsigned int newCount = ctx->mappedKeyCount + 1;
+        MappedKeyEntry *newTable = realloc(ctx->mappedKeyTable, ARRAY_SIZE(newTable, newCount));
 
-    if (getKeyOperand(file, &value, ktd)) {
-      if (!value.set) {
-        unsigned char function;
-
-        if (getKeyboardFunctionOperand(file, &function, ktd)) {
-          if (!ctx->keyMap) {
-            if (!(ctx->keyMap = malloc(ARRAY_SIZE(ctx->keyMap, MAX_KEYS_PER_SET)))) {
-              LogError("malloc");
-              return 0;
-            }
-
-            memset(ctx->keyMap, KBF_None, MAX_KEYS_PER_SET);
-          }
-
-          ctx->keyMap[value.key] = function;
-          return 1;
+        if (!newTable) {
+          logSystemError("realloc");
+          return 0;
         }
-      } else {
-        reportDataError(file, "unmappable key");
+
+        ctx->mappedKeyTable = newTable;
+        ctx->mappedKeyTable[ctx->mappedKeyCount++] = map;
+        return 1;
       }
+    } else {
+      reportDataError(file, "cannot map key set");
     }
   }
 
@@ -897,32 +968,36 @@ processMapOperands (DataFile *file, void *data) {
 static int
 processNoteOperands (DataFile *file, void *data) {
   KeyTableData *ktd = data;
-  DataOperand note;
+  DataOperand operand;
 
-  if (getDataText(file, &note, "note text")) {
+  if (getDataText(file, &operand, "note text")) {
     if (!hideBindings(ktd)) {
-      unsigned int newCount = ktd->table->noteCount + 1;
-      wchar_t **newTable = realloc(ktd->table->noteTable, ARRAY_SIZE(newTable, newCount));
+      DataString string;
 
-      if (!newTable) {
-        LogError("realloc");
-        return 0;
-      }
+      if (parseDataString(file, &string, operand.characters, operand.length, 0)) {
+        unsigned int newCount = ktd->table->noteCount + 1;
+        wchar_t **newTable = realloc(ktd->table->noteTable, ARRAY_SIZE(newTable, newCount));
 
-      ktd->table->noteTable = newTable;
-
-      {
-        wchar_t *noteString = malloc(ARRAY_SIZE(*ktd->table->noteTable, note.length+1));
-
-        if (!noteString) {
-          LogError("malloc");
+        if (!newTable) {
+          logSystemError("realloc");
           return 0;
         }
 
-        wmemcpy(noteString, note.characters, note.length);
-        noteString[note.length] = 0;
-        ktd->table->noteTable[ktd->table->noteCount++] = noteString;
-        return 1;
+        ktd->table->noteTable = newTable;
+
+        {
+          wchar_t *noteString = malloc(ARRAY_SIZE(*ktd->table->noteTable, string.length+1));
+
+          if (!noteString) {
+            logMallocError();
+            return 0;
+          }
+
+          wmemcpy(noteString, string.characters, string.length);
+          noteString[string.length] = 0;
+          ktd->table->noteTable[ktd->table->noteCount++] = noteString;
+          return 1;
+        }
       }
     }
   }
@@ -937,16 +1012,9 @@ processSuperimposeOperands (DataFile *file, void *data) {
   if (!ctx) return 0;
 
   {
-    unsigned char function;
-
-    if (getKeyboardFunctionOperand(file, &function, ktd)) {
-      const KeyboardFunctionEntry *kbf = &keyboardFunctionTable[function];
-
-      if (!kbf->bit) {
-        reportDataError(file, "unsuperimposable keyboard function");
-        return 0;
-      }
+    const KeyboardFunction *kbf;
 
+    if (getKeyboardFunctionOperand(file, &kbf, ktd)) {
       ctx->superimposedBits |= kbf->bit;
       return 1;
     }
@@ -964,7 +1032,7 @@ processTitleOperands (DataFile *file, void *data) {
     if (ktd->table->title) {
       reportDataError(file, "table title specified more than once");
     } else if (!(ktd->table->title = malloc(ARRAY_SIZE(ktd->table->title, title.length+1)))) {
-      LogError("malloc");
+      logMallocError();
       return 0;
     } else {
       wmemcpy(ktd->table->title, title.characters, title.length);
@@ -998,7 +1066,7 @@ processKeyTableLine (DataFile *file, void *data) {
 
 void
 resetKeyTable (KeyTable *table) {
-  table->currentContext = table->persistentContext = BRL_CTX_DEFAULT;
+  table->currentContext = table->persistentContext = KTB_CTX_DEFAULT;
   table->pressedCount = 0;
   table->command = EOF;
   table->immediate = 0;
@@ -1062,7 +1130,7 @@ addBindingIndex (KeyContext *ctx, const KeyValue *keys, unsigned char count, uns
     unsigned int *newTable = realloc(ibd->indexTable, ARRAY_SIZE(newTable, newSize));
 
     if (!newTable) {
-      LogError("realloc");
+      logSystemError("realloc");
       return 0;
     }
 
@@ -1170,7 +1238,7 @@ prepareKeyBindings (KeyContext *ctx) {
       KeyBinding *newTable = realloc(ctx->keyBindingTable, ARRAY_SIZE(newTable, ctx->keyBindingCount));
 
       if (!newTable) {
-        LogError("realloc");
+        logSystemError("realloc");
         return 0;
       }
 
@@ -1185,7 +1253,7 @@ prepareKeyBindings (KeyContext *ctx) {
 
   if (ctx->keyBindingCount) {
     if (!(ctx->sortedKeyBindings = malloc(ARRAY_SIZE(ctx->sortedKeyBindings, ctx->keyBindingCount)))) {
-      LogError("malloc");
+      logMallocError();
       return 0;
     }
 
@@ -1218,12 +1286,12 @@ static int
 prepareHotkeyEntries (KeyContext *ctx) {
   if (ctx->hotkeyCount) {
     if (!(ctx->sortedHotkeyEntries = malloc(ARRAY_SIZE(ctx->sortedHotkeyEntries, ctx->hotkeyCount)))) {
-      LogError("malloc");
+      logMallocError();
       return 0;
     }
 
     {
-      HotkeyEntry *source = ctx->hotkeyTable;
+      const HotkeyEntry *source = ctx->hotkeyTable;
       const HotkeyEntry **target = ctx->sortedHotkeyEntries;
       unsigned int count = ctx->hotkeyCount;
 
@@ -1239,6 +1307,39 @@ prepareHotkeyEntries (KeyContext *ctx) {
   return 1;
 }
 
+static int
+sortMappedKeyEntries (const void *element1, const void *element2) {
+  const MappedKeyEntry *const *map1 = element1;
+  const MappedKeyEntry *const *map2 = element2;
+
+  return compareKeyValues(&(*map1)->keyValue, &(*map2)->keyValue);
+}
+
+static int
+prepareMappedKeyEntries (KeyContext *ctx) {
+  if (ctx->mappedKeyCount) {
+    if (!(ctx->sortedMappedKeyEntries = malloc(ARRAY_SIZE(ctx->sortedMappedKeyEntries, ctx->mappedKeyCount)))) {
+      logMallocError();
+      return 0;
+    }
+
+    {
+      const MappedKeyEntry *source = ctx->mappedKeyTable;
+      const MappedKeyEntry **target = ctx->sortedMappedKeyEntries;
+      unsigned int count = ctx->mappedKeyCount;
+
+      while (count) {
+        *target++ = source++;
+        count -= 1;
+      }
+    }
+
+    qsort(ctx->sortedMappedKeyEntries, ctx->mappedKeyCount, sizeof(*ctx->sortedMappedKeyEntries), sortMappedKeyEntries);
+  }
+
+  return 1;
+}
+
 int
 finishKeyTable (KeyTableData *ktd) {
   {
@@ -1249,6 +1350,7 @@ finishKeyTable (KeyTableData *ktd) {
 
       if (!prepareKeyBindings(ctx)) return 0;
       if (!prepareHotkeyEntries(ctx)) return 0;
+      if (!prepareMappedKeyEntries(ctx)) return 0;
     }
   }
 
@@ -1261,43 +1363,46 @@ finishKeyTable (KeyTableData *ktd) {
 KeyTable *
 compileKeyTable (const char *name, KEY_NAME_TABLES_REFERENCE keys) {
   KeyTable *table = NULL;
-  KeyTableData ktd;
 
-  memset(&ktd, 0, sizeof(ktd));
-  ktd.context = BRL_CTX_DEFAULT;
+  if (setGlobalTableVariables(KEY_TABLE_EXTENSION, KEY_SUBTABLE_EXTENSION)) {
+    KeyTableData ktd;
+    memset(&ktd, 0, sizeof(ktd));
 
-  if ((ktd.table = malloc(sizeof(*ktd.table)))) {
-    ktd.table->title = NULL;
+    ktd.context = KTB_CTX_DEFAULT;
 
-    ktd.table->noteTable = NULL;
-    ktd.table->noteCount = 0;
+    if ((ktd.table = malloc(sizeof(*ktd.table)))) {
+      ktd.table->title = NULL;
 
-    ktd.table->keyNameTable = NULL;
-    ktd.table->keyNameCount = 0;
+      ktd.table->noteTable = NULL;
+      ktd.table->noteCount = 0;
 
-    ktd.table->keyContextTable = NULL;
-    ktd.table->keyContextCount = 0;
+      ktd.table->keyNameTable = NULL;
+      ktd.table->keyNameCount = 0;
 
-    ktd.table->pressedKeys = NULL;
-    ktd.table->pressedSize = 0;
-    ktd.table->pressedCount = 0;
+      ktd.table->keyContextTable = NULL;
+      ktd.table->keyContextCount = 0;
 
-    if (allocateKeyNameTable(&ktd, keys)) {
-      if (allocateCommandTable(&ktd)) {
-        if (processDataFile(name, processKeyTableLine, &ktd)) {
-          if (finishKeyTable(&ktd)) {
-            table = ktd.table;
-            ktd.table = NULL;
+      ktd.table->pressedKeys = NULL;
+      ktd.table->pressedSize = 0;
+      ktd.table->pressedCount = 0;
+
+      if (allocateKeyNameTable(&ktd, keys)) {
+        if (allocateCommandTable(&ktd)) {
+          if (processDataFile(name, processKeyTableLine, &ktd)) {
+            if (finishKeyTable(&ktd)) {
+              table = ktd.table;
+              ktd.table = NULL;
+            }
           }
+
+          if (ktd.commandTable) free(ktd.commandTable);
         }
-
-        if (ktd.commandTable) free(ktd.commandTable);
       }
-    }
 
-    if (ktd.table) destroyKeyTable(ktd.table);
-  } else {
-    LogError("malloc");
+      if (ktd.table) destroyKeyTable(ktd.table);
+    } else {
+      logMallocError();
+    }
   }
 
   return table;
@@ -1318,7 +1423,8 @@ destroyKeyTable (KeyTable *table) {
     if (ctx->hotkeyTable) free(ctx->hotkeyTable);
     if (ctx->sortedHotkeyEntries) free(ctx->sortedHotkeyEntries);
 
-    if (ctx->keyMap) free(ctx->keyMap);
+    if (ctx->mappedKeyTable) free(ctx->mappedKeyTable);
+    if (ctx->sortedMappedKeyEntries) free(ctx->sortedMappedKeyEntries);
   }
 
   if (table->keyContextTable) free(table->keyContextTable);
@@ -1330,5 +1436,10 @@ destroyKeyTable (KeyTable *table) {
 
 char *
 ensureKeyTableExtension (const char *path) {
-  return ensureExtension(path, KEY_TABLE_EXTENSION);
+  return ensureFileExtension(path, KEY_TABLE_EXTENSION);
+}
+
+char *
+makeKeyTablePath (const char *directory, const char *name) {
+  return makeFilePath(directory, name, KEY_TABLE_EXTENSION);
 }
