@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -27,6 +27,13 @@
 #include "ktb_inspect.h"
 #include "brl.h"
 
+static int
+sortModifierKeys (const void *element1, const void *element2) {
+  const KeyValue *modifier1 = element1;
+  const KeyValue *modifier2 = element2;
+  return compareKeyValues(modifier1, modifier2);
+}
+
 static int
 searchKeyBinding (const void *target, const void *element) {
   const KeyBinding *reference = target;
@@ -47,29 +54,39 @@ findKeyBinding (KeyTable *table, unsigned char context, const KeyValue *immediat
       target.combination.immediateKey = *immediate;
       target.combination.flags |= KCF_IMMEDIATE_KEY;
     }
-
     target.combination.modifierCount = table->pressedCount;
-    copyKeyValues(target.combination.modifierKeys, table->pressedKeys, target.combination.modifierCount);
 
-    {
-      int index;
+    while (1) {
+      unsigned int all = (1 << table->pressedCount) - 1;
+      unsigned int bits;
 
-      for (index=0; index<target.combination.modifierCount; index+=1) {
-        KeyValue *modifier = &target.combination.modifierKeys[index];
-        if (modifier->set) modifier->key = KTB_KEY_ANY;
+      for (bits=0; bits<=all; bits+=1) {
+        {
+          unsigned int index;
+          unsigned int bit;
+
+          for (index=0, bit=1; index<table->pressedCount; index+=1, bit<<=1) {
+            KeyValue *modifier = &target.combination.modifierKeys[index];
+
+            *modifier = table->pressedKeys[index];
+            if (bits & bit) modifier->key = KTB_KEY_ANY;
+          }
+        }
+        qsort(target.combination.modifierKeys, table->pressedCount, sizeof(*target.combination.modifierKeys), sortModifierKeys);
+
+        {
+          const KeyBinding *const *binding = bsearch(&target, ctx->sortedKeyBindings, ctx->keyBindingCount, sizeof(*ctx->sortedKeyBindings), searchKeyBinding);
+
+          if (binding) {
+            if ((*binding)->command != EOF) return *binding;
+            *isIncomplete = 1;
+          }
+        }
       }
-    }
 
-    if (target.combination.flags & KCF_IMMEDIATE_KEY)
-      if (target.combination.immediateKey.set)
-        target.combination.immediateKey.key = KTB_KEY_ANY;
-
-    {
-      const KeyBinding **binding = bsearch(&target, ctx->sortedKeyBindings, ctx->keyBindingCount, sizeof(*ctx->sortedKeyBindings), searchKeyBinding);
-      if (binding) {
-        if ((*binding)->command != EOF) return *binding;
-        *isIncomplete = 1;
-      }
+      if (!(target.combination.flags & KCF_IMMEDIATE_KEY)) break;
+      if (target.combination.immediateKey.key == KTB_KEY_ANY) break;
+      target.combination.immediateKey.key = KTB_KEY_ANY;
     }
   }
 
@@ -93,7 +110,7 @@ findHotkeyEntry (KeyTable *table, unsigned char context, const KeyValue *keyValu
     };
 
     {
-      const HotkeyEntry **hotkey = bsearch(&target, ctx->sortedHotkeyEntries, ctx->hotkeyCount, sizeof(*ctx->sortedHotkeyEntries), searchHotkeyEntry);
+      const HotkeyEntry *const *hotkey = bsearch(&target, ctx->sortedHotkeyEntries, ctx->hotkeyCount, sizeof(*ctx->sortedHotkeyEntries), searchHotkeyEntry);
       if (hotkey) return *hotkey;
     }
   }
@@ -101,49 +118,61 @@ findHotkeyEntry (KeyTable *table, unsigned char context, const KeyValue *keyValu
   return NULL;
 }
 
+static int
+searchMappedKeyEntry (const void *target, const void *element) {
+  const MappedKeyEntry *reference = target;
+  const MappedKeyEntry *const *map = element;
+  return compareKeyValues(&reference->keyValue, &(*map)->keyValue);
+}
+
+static const MappedKeyEntry *
+findMappedKeyEntry (const KeyContext *ctx, const KeyValue *keyValue) {
+  if (ctx->sortedMappedKeyEntries) {
+    MappedKeyEntry target = {
+      .keyValue = *keyValue
+    };
+
+    {
+      const MappedKeyEntry *const *map = bsearch(&target, ctx->sortedMappedKeyEntries, ctx->mappedKeyCount, sizeof(*ctx->sortedMappedKeyEntries), searchMappedKeyEntry);
+      if (map) return *map;
+    }
+  }
+
+  return NULL;
+}
+
 static int
 makeKeyboardCommand (KeyTable *table, unsigned char context) {
-  int chordsRequested = context == BRL_CTX_CHORDS;
+  int chordsRequested = context == KTB_CTX_CHORDS;
   const KeyContext *ctx;
 
   if (chordsRequested) context = table->persistentContext;
-  if (!(ctx = getKeyContext(table, context))) return EOF;
 
-  if (ctx->keyMap) {
+  if ((ctx = getKeyContext(table, context))) {
     int keyboardCommand = BRL_BLK_PASSDOTS;
-    int dotPressed = 0;
-    int spacePressed = 0;
 
     {
       unsigned int pressedIndex;
 
       for (pressedIndex=0; pressedIndex<table->pressedCount; pressedIndex+=1) {
         const KeyValue *keyValue = &table->pressedKeys[pressedIndex];
-        KeyboardFunction function = ctx->keyMap[keyValue->key];
+        const MappedKeyEntry *map = findMappedKeyEntry(ctx, keyValue);
 
-        if (keyValue->set) return EOF;
-        if (function == KBF_None) return EOF;
-
-        {
-          const KeyboardFunctionEntry *kbf = &keyboardFunctionTable[function];
-
-          keyboardCommand |= kbf->bit;
-
-          if (!kbf->bit) {
-            spacePressed = 1;
-          } else if (kbf->bit & BRL_MSK_ARG) {
-            dotPressed = 1;
-          }
-        }
+        if (!map) return EOF;
+        keyboardCommand |= map->keyboardFunction->bit;
       }
-
-      if (dotPressed) keyboardCommand |= ctx->superimposedBits;
     }
 
-    if (chordsRequested && spacePressed) {
-      keyboardCommand |= BRL_DOTC;
-    } else if (dotPressed == spacePressed) {
-      return EOF;
+    {
+      int dotPressed = !!(keyboardCommand & (BRL_DOT1 | BRL_DOT2 | BRL_DOT3 | BRL_DOT4 | BRL_DOT5 | BRL_DOT6 | BRL_DOT7 | BRL_DOT8));
+      int spacePressed = !!(keyboardCommand & BRL_DOTC);
+
+      if (dotPressed) keyboardCommand |= ctx->superimposedBits;
+
+      if (!chordsRequested) {
+        if (dotPressed == spacePressed) return EOF;
+        keyboardCommand &= ~BRL_DOTC;
+      }
     }
 
     return keyboardCommand;
@@ -152,33 +181,6 @@ makeKeyboardCommand (KeyTable *table, unsigned char context) {
   return EOF;
 }
 
-static int
-processCommand (KeyTable *table, int command) {
-  int blk = command & BRL_MSK_BLK;
-  int arg = command & BRL_MSK_ARG;
-
-  switch (blk) {
-    case BRL_BLK_CONTEXT:
-      if (!BRL_DELAYED_COMMAND(command)) {
-        unsigned char context = BRL_CTX_DEFAULT + arg;
-        const KeyContext *ctx = getKeyContext(table, context);
-
-        if (ctx) {
-          table->currentContext = context;
-          if (!isTemporaryKeyContext(table, ctx)) table->persistentContext = context;
-        }
-      }
-
-      command = BRL_CMD_NOOP;
-      break;
-
-    default:
-      break;
-  }
-
-  return enqueueCommand(command);
-}
-
 static int
 findPressedKey (KeyTable *table, const KeyValue *value, unsigned int *position) {
   return findKeyValue(table->pressedKeys, table->pressedCount, value, position);
@@ -199,6 +201,38 @@ deleteExplicitKeyValue (KeyValue *values, unsigned int *count, const KeyValue *v
   if (value->key != KTB_KEY_ANY) deleteKeyValue(values, count, value);
 }
 
+static int
+processCommand (KeyTable *table, int command) {
+  int blk = command & BRL_MSK_BLK;
+  int arg = command & BRL_MSK_ARG;
+
+  switch (blk) {
+    case BRL_BLK_CONTEXT:
+      if (!BRL_DELAYED_COMMAND(command)) {
+        unsigned char context = KTB_CTX_DEFAULT + arg;
+        const KeyContext *ctx = getKeyContext(table, context);
+
+        if (ctx) {
+          command = BRL_CMD_NOOP;
+          table->currentContext = context;
+
+          if (isTemporaryKeyContext(table, ctx)) {
+            command |= BRL_FLG_TOGGLE_ON;
+          } else {
+            table->persistentContext = context;
+            command |= BRL_FLG_TOGGLE_OFF;
+          }
+        }
+      }
+      break;
+
+    default:
+      break;
+  }
+
+  return enqueueCommand(command);
+}
+
 static int
 sortKeyOffsets (const void *element1, const void *element2) {
   const KeyValue *value1 = element1;
@@ -215,21 +249,31 @@ processKeyEvent (KeyTable *table, unsigned char context, unsigned char set, unsi
     .set = set,
     .key = key
   };
-  unsigned int keyPosition;
 
   KeyTableState state = KTS_UNBOUND;
   int command = EOF;
-  int immediate = 1;
   const HotkeyEntry *hotkey;
 
-  if (context == BRL_CTX_DEFAULT) context = table->currentContext;
+  if (context == KTB_CTX_DEFAULT) context = table->currentContext;
   if (press) table->currentContext = table->persistentContext;
 
-  if ((hotkey = findHotkeyEntry(table, context, &keyValue))) {
+  if (!(hotkey = findHotkeyEntry(table, context, &keyValue))) {
+    const KeyValue anyKey = {
+      .set = keyValue.set,
+      .key = KTB_KEY_ANY
+    };
+
+    hotkey = findHotkeyEntry(table, context, &anyKey);
+  }
+
+  if (hotkey) {
     int cmd = press? hotkey->pressCommand: hotkey->releaseCommand;
     if (cmd != BRL_CMD_NOOP) processCommand(table, (command = cmd));
     state = KTS_HOTKEY;
   } else {
+    int immediate = 1;
+    unsigned int keyPosition;
+
     if (findPressedKey(table, &keyValue, &keyPosition)) removePressedKey(table, keyPosition);
 
     if (press) {
@@ -244,18 +288,18 @@ processKeyEvent (KeyTable *table, unsigned char context, unsigned char set, unsi
         immediate = 0;
       } else if ((command = makeKeyboardCommand(table, context)) != EOF) {
         immediate = 0;
-      } else if (context == BRL_CTX_DEFAULT) {
+      } else if (context == KTB_CTX_DEFAULT) {
         command = EOF;
       } else if (!inserted) {
         command = EOF;
       } else {
         removePressedKey(table, keyPosition);
-        binding = findKeyBinding(table, BRL_CTX_DEFAULT, &keyValue, &isIncomplete);
+        binding = findKeyBinding(table, KTB_CTX_DEFAULT, &keyValue, &isIncomplete);
         inserted = insertPressedKey(table, &keyValue, keyPosition);
 
         if (binding) {
           command = binding->command;
-        } else if ((binding = findKeyBinding(table, BRL_CTX_DEFAULT, NULL, &isIncomplete))) {
+        } else if ((binding = findKeyBinding(table, KTB_CTX_DEFAULT, NULL, &isIncomplete))) {
           command = binding->command;
           immediate = 0;
         } else {
@@ -273,7 +317,7 @@ processKeyEvent (KeyTable *table, unsigned char context, unsigned char set, unsi
       } else {
         if (command != table->command) {
           if (binding) {
-            if (binding->flags & (KBF_OFFSET | KBF_COLUMN | KBF_ROW | KBF_RANGE)) {
+            if (binding->flags & (KBF_OFFSET | KBF_COLUMN | KBF_ROW | KBF_RANGE | KBF_KEYBOARD)) {
               unsigned int keyCount = table->pressedCount;
               KeyValue keyValues[keyCount];
               copyKeyValues(keyValues, table->pressedKeys, keyCount);
@@ -298,17 +342,22 @@ processKeyEvent (KeyTable *table, unsigned char context, unsigned char set, unsi
 
                 command += keyValues[0].key;
               } else if (binding->flags & KBF_COLUMN) {
-                command |= BRL_MSK_ARG;
+                if (!(binding->flags & KBF_ROUTE)) command |= BRL_MSK_ARG;
               }
             }
           }
 
-          table->command = command;
-
-          if ((table->immediate = immediate)) {
-            command |= BRL_FLG_REPEAT_INITIAL | BRL_FLG_REPEAT_DELAY;
+          if (context == KTB_CTX_WAITING) {
+            table->command = EOF;
+            table->immediate = 0;
           } else {
-            command |= BRL_FLG_REPEAT_DELAY;
+            table->command = command;
+
+            if ((table->immediate = immediate)) {
+              command |= BRL_FLG_REPEAT_INITIAL | BRL_FLG_REPEAT_DELAY;
+            } else {
+              command |= BRL_FLG_REPEAT_DELAY;
+            }
           }
 
           processCommand(table, command);
@@ -319,6 +368,8 @@ processKeyEvent (KeyTable *table, unsigned char context, unsigned char set, unsi
         state = KTS_COMMAND;
       }
     } else {
+      if (context == KTB_CTX_WAITING) table->command = EOF;
+
       if (table->command != EOF) {
         if (table->immediate) {
           command = BRL_CMD_NOOP;
@@ -332,29 +383,26 @@ processKeyEvent (KeyTable *table, unsigned char context, unsigned char set, unsi
     }
   }
 
-  if (table->logKeyEvents) {
+  if (table->logKeyEvents && *table->logKeyEvents) {
     char buffer[0X40];
-    size_t size = sizeof(buffer);
-    int offset = 0;
-    int length;
 
-    snprintf(&buffer[offset], size, "Key %s: Ctx:%u Set:%u Key:%u%n",
-             press? "Press": "Release",
-             context, set, key, &length);
-    offset += length, size -= length;
+    STR_BEGIN(buffer, sizeof(buffer));
+    STR_PRINTF("Key %s: Ctx:%u Set:%u Key:%u",
+               press? "Press": "Release",
+               context, set, key);
 
     if (command != EOF) {
-      snprintf(&buffer[offset], size, " Cmd:%06X%n", command, &length);
-      offset += length, size -= length;
+      STR_PRINTF(" Cmd:%06X", command);
     }
 
-    LogPrint(LOG_DEBUG, "%s", buffer);
+    STR_END
+    logMessage(categoryLogLevel, "%s", buffer);
   }
 
   return state;
 }
 
 void
-logKeyEvents (KeyTable *table) {
-  table->logKeyEvents = 1;
+setKeyEventLoggingFlag (KeyTable *table, const unsigned char *flag) {
+  table->logKeyEvents = flag;
 }
