@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -22,7 +22,6 @@
 
 #include "parse.h"
 #include "log.h"
-#include "misc.h"
 
 char **
 splitString (const char *string, char delimiter, int *count) {
@@ -36,15 +35,22 @@ splitString (const char *string, char delimiter, int *count) {
       if (*start) {
         while (1) {
           const char *end = strchr(start, delimiter);
-          int length = end? end-start: strlen(start);
+          size_t length = end? (size_t)(end-start): strlen(start);
 
           if (array) {
-            char *element = mallocWrapper(length+1);
+            char *element = malloc(length+1);
+
+            if (!(array[index] = element)) {
+              logMallocError();
+              deallocateStrings(array);
+              array = NULL;
+              goto done;
+            }
+
             memcpy(element, start, length);
             element[length] = 0;
-            array[index] = element;
           }
-          index++;
+          index += 1;
 
           if (!end) break;
           start = end + 1;
@@ -57,12 +63,15 @@ splitString (const char *string, char delimiter, int *count) {
         break;
       }
 
-      array = mallocWrapper((index + 1) * sizeof(*array));
+      if (!(array = malloc((index + 1) * sizeof(*array)))) {
+        logMallocError();
+        break;
+      }
     }
-  } else if (count) {
-    *count = 0;
   }
 
+done:
+  if (!array && count) *count = 0;
   return array;
 }
 
@@ -118,18 +127,46 @@ isInteger (int *value, const char *string) {
 }
 
 int
-isFloat (float *value, const char *string) {
+isUnsignedInteger (unsigned int *value, const char *string) {
   if (*string) {
     char *end;
-    double d = strtod(string, &end);
+    unsigned long l = strtoul(string, &end, 0);
     if (!*end) {
-      *value = d;
+      *value = l;
       return 1;
     }
   }
   return 0;
 }
 
+int
+isLogLevel (int *level, const char *string) {
+  if (!*string) return 1;
+
+  {
+    int length = strlen(string);
+    int index;
+
+    for (index=0; index<logLevelCount; index+=1) {
+      if (strncasecmp(string, logLevelNames[index], length) == 0) {
+        *level = index;
+        return 1;
+      }
+    }
+  }
+
+  {
+    int value;
+
+    if (isInteger(&value, string) && (value >= 0) && (value < logLevelCount)) {
+      *level = value;
+      return 1;
+    }
+  }
+
+  return 0;
+}
+
 int
 validateInteger (int *value, const char *string, const int *minimum, const int *maximum) {
   if (*string) {
@@ -142,18 +179,6 @@ validateInteger (int *value, const char *string, const int *minimum, const int *
   return 1;
 }
 
-int
-validateFloat (float *value, const char *string, const float *minimum, const float *maximum) {
-  if (*string) {
-    float f;
-    if (!isFloat(&f, string)) return 0;
-    if (minimum && (f < *minimum)) return 0;
-    if (maximum && (f > *maximum)) return 0;
-    *value = f;
-  }
-  return 1;
-}
-
 int
 validateChoice (unsigned int *value, const char *string, const char *const *choices) {
   int length = strlen(string);
@@ -190,103 +215,171 @@ validateYesNo (unsigned int *value, const char *string) {
   return validateFlag(value, string, "yes", "no");
 }
 
-static void
+#ifndef NO_FLOAT
+int
+isFloat (float *value, const char *string) {
+  if (*string) {
+    char *end;
+    double d = strtod(string, &end);
+    if (!*end) {
+      *value = d;
+      return 1;
+    }
+  }
+  return 0;
+}
+
+int
+validateFloat (float *value, const char *string, const float *minimum, const float *maximum) {
+  if (*string) {
+    float f;
+    if (!isFloat(&f, string)) return 0;
+    if (minimum && (f < *minimum)) return 0;
+    if (maximum && (f > *maximum)) return 0;
+    *value = f;
+  }
+  return 1;
+}
+#endif /* NO_FLOAT */
+
+static int
 parseParameters (
   char **values,
   const char *const *names,
   const char *qualifier,
   const char *parameters
 ) {
+  int ok = 0;
+
   if (parameters && *parameters) {
-    char *copy = strdupWrapper(parameters);
-    char *name = copy;
+    char *copiedParameters = strdup(parameters);
 
-    while (1) {
-      char *end = strchr(name, ',');
-      int done = end == NULL;
-      if (!done) *end = 0;
+    if (copiedParameters) {
+      char *name = copiedParameters;
 
-      if (*name) {
-        char *value = strchr(name, '=');
-        if (!value) {
-          LogPrint(LOG_ERR, "%s: %s", gettext("missing parameter value"), name);
-        } else if (value == name) {
-        noName:
-          LogPrint(LOG_ERR, "%s: %s", gettext("missing parameter name"), name);
-        } else {
-          int nameLength = value++ - name;
-          int eligible = 1;
+      ok = 1;
+      while (1) {
+        char *end = strchr(name, ',');
+        int done = !end;
+        if (!done) *end = 0;
 
-          if (qualifier) {
-            char *colon = memchr(name, ':', nameLength);
-            if (colon) {
-              int qualifierLength = colon - name;
-              int nameAdjustment = qualifierLength + 1;
-              eligible = 0;
-              if (!qualifierLength) {
-                LogPrint(LOG_ERR, "%s: %s", gettext("missing parameter qualifier"), name);
-              } else if (!(nameLength -= nameAdjustment)) {
-                goto noName;
-              } else if ((qualifierLength == strlen(qualifier)) &&
-                         (memcmp(name, qualifier, qualifierLength) == 0)) {
-                name += nameAdjustment;
-                eligible = 1;
+        if (*name) {
+          char *value = strchr(name, '=');
+
+          if (!value) {
+            logMessage(LOG_ERR, "%s: %s", gettext("missing parameter value"), name);
+          } else if (value == name) {
+          noName:
+            logMessage(LOG_ERR, "%s: %s", gettext("missing parameter name"), name);
+          } else {
+            size_t nameLength = value++ - name;
+            int eligible = 1;
+
+            if (qualifier) {
+              char *colon = memchr(name, ':', nameLength);
+
+              if (colon) {
+                size_t qualifierLength = (size_t)(colon - name);
+                size_t nameAdjustment = qualifierLength + 1;
+
+                eligible = 0;
+                if (!qualifierLength) {
+                  logMessage(LOG_ERR, "%s: %s", gettext("missing parameter qualifier"), name);
+                } else if (!(nameLength -= nameAdjustment)) {
+                  goto noName;
+                } else if ((qualifierLength == strlen(qualifier)) &&
+                           (memcmp(name, qualifier, qualifierLength) == 0)) {
+                  name += nameAdjustment;
+                  eligible = 1;
+                }
               }
             }
-          }
 
-          if (eligible) {
-            unsigned int index = 0;
-            while (names[index]) {
-              if (strncasecmp(name, names[index], nameLength) == 0) {
-                free(values[index]);
-                values[index] = strdupWrapper(value);
-                break;
+            if (eligible) {
+              unsigned int index = 0;
+
+              while (names[index]) {
+                if (strncasecmp(name, names[index], nameLength) == 0) {
+                  char *copiedValue = strdup(value);
+
+                  if (copiedValue) {
+                    free(values[index]);
+                    values[index] = copiedValue;
+                  } else {
+                    logMallocError();
+                    ok = 0;
+                  }
+
+                  break;
+                }
+
+                index += 1;
               }
-              ++index;
-            }
 
-            if (!names[index]) {
-              LogPrint(LOG_ERR, "%s: %s", gettext("unsupported parameter"), name);
+              if (!ok) break;
+              if (!names[index]) logMessage(LOG_ERR, "%s: %s", gettext("unsupported parameter"), name);
             }
           }
         }
+
+        if (done) break;
+        name = end + 1;
       }
 
-      if (done) break;
-      name = end + 1;
+      free(copiedParameters);
+    } else {
+      logMallocError();
     }
-
-    free(copy);
+  } else {
+    ok = 1;
   }
+
+  return ok;
 }
 
 char **
 getParameters (const char *const *names, const char *qualifier, const char *parameters) {
-  char **values;
-
   if (!names) {
     static const char *const noNames[] = {NULL};
     names = noNames;
   }
 
   {
+    char **values;
     unsigned int count = 0;
-    while (names[count]) ++count;
-    values = mallocWrapper((count + 1) * sizeof(*values));
-    values[count] = NULL;
-    while (count--) values[count] = strdupWrapper("");
+    while (names[count]) count += 1;
+
+    if ((values = malloc((count + 1) * sizeof(*values)))) {
+      unsigned int index = 0;
+
+      while (index < count) {
+        if (!(values[index] = strdup(""))) {
+          logMallocError();
+          break;
+        }
+
+        index += 1;
+      }
+
+      if (index == count) {
+        values[index] = NULL;
+        if (parseParameters(values, names, qualifier, parameters)) return values;
+      }
+
+      deallocateStrings(values);
+    } else {
+      logMallocError();
+    }
   }
 
-  parseParameters(values, names, qualifier, parameters);
-  return values;
+  return NULL;
 }
 
 void
-logParameters (const char *const *names, char **values, char *description) {
+logParameters (const char *const *names, char **values, const char *description) {
   if (names && values) {
     while (*names) {
-      LogPrint(LOG_INFO, "%s: %s=%s", description, *names, *values);
+      logMessage(LOG_INFO, "%s: %s=%s", description, *names, *values);
       ++names;
       ++values;
     }
