@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -22,6 +22,7 @@
 #include <errno.h>
 
 #include "brltty.h"
+#include "prefs.h"
 #include "log.h"
 #include "system.h"
 #include "notes.h"
@@ -49,11 +50,11 @@ pcmConstruct (int errorLevel) {
       device->blockUsed = 0;
 
       if ((device->blockAddress = malloc(device->blockSize))) {
-        LogPrint(LOG_DEBUG, "PCM enabled: blk=%d rate=%d chan=%d fmt=%d",
-                 device->blockSize, device->sampleRate, device->channelCount, device->amplitudeFormat);
+        logMessage(LOG_DEBUG, "PCM enabled: blk=%d rate=%d chan=%d fmt=%d",
+                   device->blockSize, device->sampleRate, device->channelCount, device->amplitudeFormat);
         return device;
       } else {
-        LogError("malloc");
+        logMallocError();
       }
 
       closePcmDevice(device->pcm);
@@ -61,10 +62,10 @@ pcmConstruct (int errorLevel) {
 
     free(device);
   } else {
-    LogError("malloc");
+    logMallocError();
   }
 
-  LogPrint(LOG_DEBUG, "PCM not available");
+  logMessage(LOG_DEBUG, "PCM not available");
   return NULL;
 }
 
@@ -180,7 +181,7 @@ writeSample (NoteDevice *device, int amplitude) {
 }
 
 static int
-pcmPlay (NoteDevice *device, int note, int duration) {
+pcmPlay (NoteDevice *device, unsigned char note, unsigned int duration) {
   long int sampleCount = device->sampleRate * duration / 1000;
 
   if (note) {
@@ -189,33 +190,52 @@ pcmPlay (NoteDevice *device, int note, int duration) {
      * expensive math functions like sin(). Considerations like
      * these are especially important on PDAs without any FPU.
      */ 
-    float maxAmplitude = 32767.0 * prefs.pcmVolume / 100;
-    float waveLength = device->sampleRate / noteFrequencies[note];
-    float stepSample = 4 * maxAmplitude / waveLength;
-    float currSample = 0;
 
-    if (waveLength <= 2) stepSample = 0;
-    LogPrint(LOG_DEBUG, "tone: msec=%d smct=%lu note=%d",
-             duration, sampleCount, note);
+    int32_t positiveShiftsPerQuarterWave = INT32_MAX / 8;
+    int32_t negativeShiftsPerQuarterWave = -positiveShiftsPerQuarterWave;
+
+    int32_t positiveShiftsPerHalfWave = 2 * positiveShiftsPerQuarterWave;
+    int32_t negativeSiftsPerHalfWave = -positiveShiftsPerHalfWave;
+
+    int32_t positiveShiftsPerFullWave = 2 * positiveShiftsPerHalfWave;
+    int32_t currentShift = 0;
+
+    int32_t shiftsPerSample = (NOTE_FREQUENCY_TYPE)positiveShiftsPerFullWave 
+                            / (NOTE_FREQUENCY_TYPE)device->sampleRate
+                            * GET_NOTE_FREQUENCY(note);
+
+    int32_t maximumAmplitude = INT16_MAX * prefs.pcmVolume / 100;
+    int32_t amplitudeGranularity = positiveShiftsPerQuarterWave / maximumAmplitude;
+
+    logMessage(LOG_DEBUG, "tone: msec=%d smct=%lu note=%d",
+               duration, sampleCount, note);
 
     while (sampleCount > 0) {
       do {
-        sampleCount--;
-        if (!writeSample(device, currSample)) return 0;
-        currSample += stepSample;
+        {
+          int32_t normalizedAmplitude = positiveShiftsPerHalfWave - currentShift;
 
-        if (currSample >= maxAmplitude) {
-          currSample = (2 * maxAmplitude) - currSample;
-          stepSample = -stepSample;
-        } else if (currSample <= -maxAmplitude) {
-          currSample = (-2 * maxAmplitude) - currSample;
-          stepSample = -stepSample;
+          if (normalizedAmplitude > positiveShiftsPerQuarterWave) {
+            normalizedAmplitude = positiveShiftsPerHalfWave - normalizedAmplitude;
+          } else if (normalizedAmplitude < negativeShiftsPerQuarterWave) {
+            normalizedAmplitude = negativeSiftsPerHalfWave - normalizedAmplitude;
+          }
+
+          {
+            int32_t actualAmplitude = normalizedAmplitude / amplitudeGranularity;
+            if (!writeSample(device, actualAmplitude)) return 0;
+            sampleCount -= 1;
+          }
         }
-      } while ((stepSample < 0) || (currSample < 0) || (currSample > stepSample));
+      } while ((currentShift += shiftsPerSample) < positiveShiftsPerFullWave);
+
+      do {
+        currentShift -= positiveShiftsPerFullWave;
+      } while (currentShift >= positiveShiftsPerFullWave);
     }
   } else {
-    LogPrint(LOG_DEBUG, "tone: msec=%d smct=%lu note=%d",
-             duration, sampleCount, note);
+    logMessage(LOG_DEBUG, "tone: msec=%d smct=%lu note=%d",
+               duration, sampleCount, note);
 
     while (sampleCount > 0) {
       if (!writeSample(device, 0)) return 0;
@@ -246,7 +266,7 @@ pcmDestruct (NoteDevice *device) {
   free(device->blockAddress);
   closePcmDevice(device->pcm);
   free(device);
-  LogPrint(LOG_DEBUG, "PCM disabled");
+  logMessage(LOG_DEBUG, "PCM disabled");
 }
 
 const NoteMethods pcmMethods = {
