@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -18,9 +18,28 @@
 
 #include "prologue.h"
 
+#include <errno.h>
+
 #include "pid.h"
 
-#if defined(__MINGW32__)
+#if defined(__MSDOS__) || defined(GRUB_RUNTIME)
+ProcessIdentifier
+getProcessIdentifier (void) {
+  return MY_PROCESS_ID;
+}
+
+int
+testProcessIdentifier (ProcessIdentifier pid) {
+  return pid == MY_PROCESS_ID;
+}
+
+int
+cancelProcess (ProcessIdentifier pid) {
+  errno = ENOSYS;
+  return 0;
+}
+
+#elif defined(__MINGW32__)
 ProcessIdentifier
 getProcessIdentifier (void) {
   return GetCurrentProcessId();
@@ -34,19 +53,17 @@ testProcessIdentifier (ProcessIdentifier pid) {
   return 1;
 }
 
-#elif defined(__MSDOS__)
-ProcessIdentifier
-getProcessIdentifier (void) {
-  return DOS_PROCESS_ID;
-}
-
 int
-testProcessIdentifier (ProcessIdentifier pid) {
-  return pid == DOS_PROCESS_ID;
+cancelProcess (ProcessIdentifier pid) {
+  errno = ENOSYS;
+  return 0;
 }
 
 #else /* Unix */
+
+#ifdef HAVE_SIGNAL_H
 #include <signal.h>
+#endif /* HAVE_SIGNAL_H */
 
 ProcessIdentifier
 getProcessIdentifier (void) {
@@ -58,4 +75,10 @@ testProcessIdentifier (ProcessIdentifier pid) {
   return kill(pid, 0) != -1;
 }
 
+int
+cancelProcess (ProcessIdentifier pid) {
+  if (kill(pid, SIGTERM) != -1) return 1;
+  return 0;
+}
+
 #endif /* pid support */
