@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -23,16 +23,16 @@
 extern "C" {
 #endif /* __cplusplus */
 
-#if defined(__MINGW32__)
-typedef DWORD ProcessIdentifier;
-#define PRIpid "lu"
-#define SCNpid "lu"
-
-#elif defined(__MSDOS__)
+#if defined(__MSDOS__) || defined(GRUB_RUNTIME)
 typedef int ProcessIdentifier;
 #define PRIpid "d"
 #define SCNpid "d"
-#define DOS_PROCESS_ID 1
+#define MY_PROCESS_ID 1
+
+#elif defined(__MINGW32__)
+typedef DWORD ProcessIdentifier;
+#define PRIpid "lu"
+#define SCNpid "lu"
 
 #else /* Unix */
 typedef pid_t ProcessIdentifier;
@@ -42,6 +42,7 @@ typedef pid_t ProcessIdentifier;
 
 extern ProcessIdentifier getProcessIdentifier (void);
 extern int testProcessIdentifier (ProcessIdentifier pid);
+extern int cancelProcess (ProcessIdentifier pid);
 
 #ifdef __cplusplus
 }
