@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -20,13 +20,15 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <limits.h>
+#include <locale.h>
 
 #ifdef ENABLE_I18N_SUPPORT
-#include <locale.h>
+#include <libintl.h>
 #endif /* ENABLE_I18N_SUPPORT */
 
 #include "program.h"
@@ -34,7 +36,6 @@
 #include "log.h"
 #include "file.h"
 #include "parse.h"
-#include "misc.h"
 #include "system.h"
 
 #ifdef WINDOWS
@@ -44,12 +45,27 @@
 const char *programPath;
 const char *programName;
 
+const char standardStreamArgument[] = "-";
+const char standardInputName[] = "<standard-input>";
+const char standardOutputName[] = "<standard-output>";
+const char standardErrorName[] = "<standard-error>";
+
+static void
+prepareLocale (void) {
+  setlocale(LC_ALL, "");
+
+#ifdef ENABLE_I18N_SUPPORT
+  bindtextdomain(PACKAGE_NAME, LOCALE_DIRECTORY);
+  textdomain(PACKAGE_NAME);
+#endif /* ENABLE_I18N_SUPPORT */
+}
+
 static char *
 testProgram (const char *directory, const char *name) {
   char *path;
 
   if ((path = makePath(directory, name))) {
-    if (access(path, X_OK) != -1) return path;
+    if (testProgramPath(path)) return path;
 
     free(path);
   }
@@ -83,15 +99,18 @@ findProgram (const char *name) {
 }
 
 void
-prepareProgram (int argumentCount, char **argumentVector) {
+beginProgram (int argumentCount, char **argumentVector) {
+#if defined(GRUB_RUNTIME)
+
+#else /* at exit */
+  atexit(endProgram);
+#endif /* at exit */
+
 #ifdef WINDOWS
   sysInit();
 #endif /* WINDOWS */
 
-#ifdef ENABLE_I18N_SUPPORT
-  setlocale(LC_ALL, "");
-  textdomain(PACKAGE_NAME);
-#endif /* ENABLE_I18N_SUPPORT */
+  prepareLocale();
 
   if (!(programPath = getProgramPath())) programPath = argumentVector[0];
 
@@ -108,9 +127,15 @@ prepareProgram (int argumentCount, char **argumentVector) {
       char *path = realpath(programPath, buffer);
 
       if (path) {
-        programPath = strdupWrapper(path);
+        char *realPath = strdup(path);
+
+        if (realPath) {
+          programPath = realPath;
+        } else {
+          logMallocError();
+        }
       } else {
-        LogError("realpath");
+        logSystemError("realpath");
       }
     }
 #endif /* defined(HAVE_REALPATH) && defined(PATH_MAX) */
@@ -127,7 +152,7 @@ prepareProgram (int argumentCount, char **argumentVector) {
   }
 
   programName = locatePathName(programPath);
-  setPrintPrefix(programName);
+  setLogPrefix(programName);
 }
 
 void
@@ -144,11 +169,11 @@ fixInstallPaths (char **const *paths) {
 
   if (!programDirectory) {
     if (!(programDirectory = getPathDirectory(programPath))) {
-      LogPrint(LOG_WARNING, gettext("cannot determine program directory"));
+      logMessage(LOG_WARNING, gettext("cannot determine program directory"));
       programDirectory = ".";
     }
 
-    LogPrint(LOG_DEBUG, "program directory: %s", programDirectory);
+    logMessage(LOG_DEBUG, "program directory: %s", programDirectory);
   }
 
   while (*paths) {
@@ -156,9 +181,9 @@ fixInstallPaths (char **const *paths) {
       char *newPath = makePath(programDirectory, **paths);
 
       if (!newPath) {
-        LogPrint(LOG_WARNING, "%s: %s", gettext("cannot fix install path"), **paths);
+        logMessage(LOG_WARNING, "%s: %s", gettext("cannot fix install path"), **paths);
       } else if (!isAbsolutePath(**paths=newPath)) {
-        LogPrint(LOG_WARNING, "%s: %s", gettext("install path not absolute"), **paths);
+        logMessage(LOG_WARNING, "%s: %s", gettext("install path not absolute"), **paths);
       }
     }
 
@@ -174,6 +199,10 @@ fixInstallPath (char **path) {
 
 int
 createPidFile (const char *path, ProcessIdentifier pid) {
+#if defined(GRUB_RUNTIME)
+  errno = EROFS;
+
+#else /* create pid file */
   if (!pid) pid = getProcessIdentifier();
 
   if (path && *path) {
@@ -195,7 +224,7 @@ createPidFile (const char *path, ProcessIdentifier pid) {
 
       if (locked || (errno == ENOSYS)) {
         char buffer[0X20];
-        int length;
+        ssize_t length;
 
         if ((length = read(file, buffer, sizeof(buffer))) != -1) {
           ProcessIdentifier oldPid;
@@ -212,12 +241,12 @@ createPidFile (const char *path, ProcessIdentifier pid) {
             if (oldPid == pid) {
               state = PFS_ready;
             } else if (testProcessIdentifier(oldPid)) {
-              LogPrint(LOG_ERR, "instance already running: PID=%" PRIpid, oldPid);
+              logMessage(LOG_ERR, "instance already running: PID=%" PRIpid, oldPid);
               state = PFS_clash;
             }
           }
         } else {
-          LogError("read");
+          logSystemError("read");
         }
 
         if (state == PFS_stale) {
@@ -225,18 +254,18 @@ createPidFile (const char *path, ProcessIdentifier pid) {
 
           if (lseek(file, 0, SEEK_SET) != -1) {
             if (ftruncate(file, 0) != -1) {
-              snprintf(buffer, sizeof(buffer), "%" PRIpid "\n%n", pid, &length);
+              length = snprintf(buffer, sizeof(buffer), "%" PRIpid "\n", pid);
 
               if (write(file, buffer, length) != -1) {
                 state = PFS_ready;
               } else {
-                LogError("write");
+                logSystemError("write");
               }
             } else {
-              LogError("ftruncate");
+              logSystemError("ftruncate");
             }
           } else {
-            LogError("lseek");
+            logSystemError("lseek");
           }
         }
 
@@ -245,9 +274,9 @@ createPidFile (const char *path, ProcessIdentifier pid) {
 
       close(file);
     } else {
-      LogPrint(LOG_WARNING, "%s: %s: %s",
-               gettext("cannot open process identifier file"),
-               path, strerror(errno));
+      logMessage(LOG_WARNING, "%s: %s: %s",
+                 gettext("cannot open process identifier file"),
+                 path, strerror(errno));
     }
 
     switch (state) {
@@ -262,10 +291,71 @@ createPidFile (const char *path, ProcessIdentifier pid) {
         break;
 
       default:
-        LogPrint(LOG_WARNING, "unexpected PID file state: %u", state);
+        logMessage(LOG_WARNING, "unexpected PID file state: %u", state);
         break;
     }
   }
+#endif /* create pid file */
 
   return 0;
 }
+
+int
+cancelProgram (const char *pidFile) {
+  int cancelled = 0;
+  FILE *file;
+
+  if ((file = fopen(pidFile, "r"))) {
+    char buffer[0X100];
+    const char *line;
+
+    if ((line = fgets(buffer, sizeof(buffer), file))) {
+      char *end;
+      long int pid = strtol(line, &end, 10);
+
+      if (!*end || isspace(*end)) {
+        if (cancelProcess(pid)) cancelled = 1;
+      }
+    }
+
+    fclose(file);
+  } else {
+    logMessage(LOG_ERR, "%s: %s: %s",
+               gettext("pid file open error"),
+               pidFile, strerror(errno));
+  }
+
+  return cancelled;
+}
+
+typedef struct ProgramExitEntryStruct ProgramExitEntry;
+static ProgramExitEntry *programExitEntries = NULL;
+
+struct ProgramExitEntryStruct {
+  ProgramExitEntry *next;
+  ProgramExitHandler *handler;
+};
+
+void
+onProgramExit (ProgramExitHandler *handler) {
+  ProgramExitEntry *pxe;
+
+  if ((pxe = malloc(sizeof(*pxe)))) {
+    pxe->handler = handler;
+    pxe->next = programExitEntries;
+    programExitEntries = pxe;
+  } else {
+    logMallocError();
+  }
+}
+
+void
+endProgram (void) {
+  while (programExitEntries) {
+    ProgramExitEntry *pxe = programExitEntries;
+    programExitEntries = pxe->next;
+
+    pxe->handler();
+    free(pxe);
+  }
+}
