@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -23,15 +23,38 @@
 extern "C" {
 #endif /* __cplusplus */
 
+#include "pid.h"
+
+typedef enum {
+  PROG_EXIT_SUCCESS  = 0,
+  PROG_EXIT_FORCE    = 1,
+  PROG_EXIT_SYNTAX   = 2,
+  PROG_EXIT_SEMANTIC = 3,
+  PROG_EXIT_FATAL    = 4
+} ProgramExitStatus;
+
 extern const char *programPath;
 extern const char *programName;
 
-extern void prepareProgram (int argumentCount, char **argumentVector);
+extern const char standardStreamArgument[];
+extern const char standardInputName[];
+extern const char standardOutputName[];
+extern const char standardErrorName[];
+
+extern void beginProgram (int argumentCount, char **argumentVector);
+extern void endProgram (void);
+
+typedef void ProgramExitHandler (void);
+extern void onProgramExit (ProgramExitHandler *handler);
+
 extern void makeProgramBanner (char *buffer, size_t size);
 
 extern void fixInstallPaths (char **const *paths);
 extern void fixInstallPath (char **path);
 
+extern int createPidFile (const char *path, ProcessIdentifier pid);
+extern int cancelProgram (const char *pidFile);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
