@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -18,16 +18,21 @@
 
 #include "prologue.h"
 
+#include <stdio.h>
+#include <stdarg.h>
 #include <string.h>
 #include <errno.h>
-#include <sys/time.h>
+
+#ifdef HAVE_SIGNAL_H
 #include <signal.h>
+#endif /* HAVE_SIGNAL_H */
 
 #ifdef SIGUSR1
 #include <sys/wait.h>
 #endif /* SIGUSR1 */
 
 #include "log.h"
+#include "program.h"
 #include "timing.h"
 #include "scr.h"
 #include "routing.h"
@@ -40,7 +45,7 @@
  * give up until the timeout has elapsed!
  */
 #define ROUTING_NICENESS	10	/* niceness of cursor routing subprocess */
-#define ROUTING_INTERVAL	0	/* how often to check for response */
+#define ROUTING_INTERVAL	1	/* how often to check for response */
 #define ROUTING_TIMEOUT	2000	/* max wait for response to key press */
 
 typedef enum {
@@ -50,7 +55,9 @@ typedef enum {
 } RoutingResult;
 
 typedef struct {
+#ifdef HAVE_SIGNAL_H
   sigset_t signalMask;
+#endif /* HAVE_SIGNAL_H */
 
   int screenNumber;
   int screenRows;
@@ -66,6 +73,56 @@ typedef struct {
   int timeCount;
 } RoutingData;
 
+typedef enum {
+  CURSOR_DIR_LEFT,
+  CURSOR_DIR_RIGHT,
+  CURSOR_DIR_UP,
+  CURSOR_DIR_DOWN
+} CursorDirection;
+
+typedef struct {
+  const char *name;
+  ScreenKey key;
+} CursorDirectionEntry;
+
+static const CursorDirectionEntry cursorDirectionTable[] = {
+  [CURSOR_DIR_LEFT]  = {.name="left" , .key=SCR_KEY_CURSOR_LEFT },
+  [CURSOR_DIR_RIGHT] = {.name="right", .key=SCR_KEY_CURSOR_RIGHT},
+  [CURSOR_DIR_UP]    = {.name="up"   , .key=SCR_KEY_CURSOR_UP   },
+  [CURSOR_DIR_DOWN]  = {.name="down" , .key=SCR_KEY_CURSOR_DOWN }
+};
+
+typedef enum {
+  CURSOR_AXIS_HORIZONTAL,
+  CURSOR_AXIS_VERTICAL
+} CursorAxis;
+
+typedef struct {
+  const CursorDirectionEntry *forward;
+  const CursorDirectionEntry *backward;
+} CursorAxisEntry;
+
+static const CursorAxisEntry cursorAxisTable[] = {
+  [CURSOR_AXIS_HORIZONTAL] = {
+    .forward  = &cursorDirectionTable[CURSOR_DIR_RIGHT],
+    .backward = &cursorDirectionTable[CURSOR_DIR_LEFT]
+  }
+  ,
+  [CURSOR_AXIS_VERTICAL] = {
+    .forward  = &cursorDirectionTable[CURSOR_DIR_DOWN],
+    .backward = &cursorDirectionTable[CURSOR_DIR_UP]
+  }
+};
+
+static void
+logRouting (const char *format, ...) {
+  va_list arguments;
+
+  va_start(arguments, format);
+  vlogMessage(LOG_CATEGORY(CURSOR_ROUTING), format, &arguments);
+  va_end(arguments);
+}
+
 static int
 readScreenRow (RoutingData *routing, ScreenCharacter *buffer, int row) {
   if (!buffer) buffer = routing->rowBuffer;
@@ -78,6 +135,7 @@ getCurrentPosition (RoutingData *routing) {
   describeScreen(&description);
 
   if (description.number != routing->screenNumber) {
+    logRouting("screen changed: num=%d", description.number);
     routing->screenNumber = description.number;
     return 0;
   }
@@ -86,15 +144,27 @@ getCurrentPosition (RoutingData *routing) {
     routing->screenRows = description.rows;
     routing->screenColumns = description.cols;
     routing->verticalDelta = 0;
-    if (!(routing->rowBuffer = calloc(routing->screenColumns, sizeof(*routing->rowBuffer)))) goto error;
+
+    if (!(routing->rowBuffer = calloc(routing->screenColumns, sizeof(*routing->rowBuffer)))) {
+      logMallocError();
+      goto error;
+    }
+
+    logRouting("screen: num=%d cols=%d rows=%d",
+               routing->screenNumber,
+               routing->screenColumns, routing->screenRows);
   } else if ((routing->screenRows != description.rows) ||
              (routing->screenColumns != description.cols)) {
+    logRouting("size changed: cols=%d rows=%d",
+               description.cols, description.rows);
     goto error;
   }
 
   routing->cury = description.posy - routing->verticalDelta;
   routing->curx = description.posx;
+
   if (readScreenRow(routing, NULL, description.posy)) return 1;
+  logRouting("read failed: row=%d", description.posy);
 
 error:
   routing->screenNumber = -1;
@@ -102,13 +172,14 @@ error:
 }
 
 static void
-insertCursorKey (RoutingData *routing, ScreenKey key) {
+moveCursor (RoutingData *routing, const CursorDirectionEntry *direction) {
 #ifdef SIGUSR1
   sigset_t oldMask;
   sigprocmask(SIG_BLOCK, &routing->signalMask, &oldMask);
 #endif /* SIGUSR1 */
 
-  insertScreenKey(key);
+  logRouting("move: %s", direction->name);
+  insertScreenKey(direction->key);
 
 #ifdef SIGUSR1
   sigprocmask(SIG_SETMASK, &oldMask, NULL);
@@ -117,12 +188,17 @@ insertCursorKey (RoutingData *routing, ScreenKey key) {
 
 static int
 awaitCursorMotion (RoutingData *routing, int direction) {
+  int oldx = (routing->oldx = routing->curx);
+  int oldy = (routing->oldy = routing->cury);
   long timeout = routing->timeSum / routing->timeCount;
-  struct timeval start;
-  gettimeofday(&start, NULL);
+  int moved = 0;
+  TimeValue start;
+
+  getCurrentTime(&start);
 
   while (1) {
     long time;
+
     approximateDelay(ROUTING_INTERVAL);
     time = millisecondsSince(&start) + 1;
 
@@ -162,24 +238,38 @@ awaitCursorMotion (RoutingData *routing, int direction) {
       routing->verticalDelta = bestRow - routing->cury;
     }
 
-    routing->oldy = routing->cury;
-    routing->oldx = routing->curx;
+    oldy = routing->cury;
+    oldx = routing->curx;
     if (!getCurrentPosition(routing)) return 0;
 
-    if ((routing->cury != routing->oldy) || (routing->curx != routing->oldx)) {
-      routing->timeSum += time * 8;
-      routing->timeCount += 1;
+    if ((routing->cury != oldy) || (routing->curx != oldx)) {
+      logRouting("moved: [%d,%d] -> [%d,%d]",
+                 oldx, oldy,
+                 routing->curx, routing->cury);
+
+      if (!moved) {
+        moved = 1;
+        timeout = time * 2;
+
+        routing->timeSum += time * 8;
+        routing->timeCount += 1;
+      }
+    } else if (time > timeout) {
+      if (!moved) {
+        logRouting("timed out");
+      }
+
       break;
     }
-
-    if (time > timeout) break;
   }
 
   return 1;
 }
 
 static RoutingResult
-adjustCursorPosition (RoutingData *routing, int where, int trgy, int trgx, ScreenKey forward, ScreenKey backward) {
+adjustCursorPosition (RoutingData *routing, int where, int trgy, int trgx, const CursorAxisEntry *axis) {
+  logRouting("to: [%d,%d]", trgx, trgy);
+
   while (1) {
     int dify = trgy - routing->cury;
     int difx = (trgx < 0)? 0: (trgx - routing->curx);
@@ -195,7 +285,7 @@ adjustCursorPosition (RoutingData *routing, int where, int trgy, int trgx, Scree
     }
 
     /* tell the cursor to move in the needed direction */
-    insertCursorKey(routing, ((dir > 0)? forward: backward));
+    moveCursor(routing, ((dir > 0)? axis->forward: axis->backward));
     if (!awaitCursorMotion(routing, dir)) return CRR_FAIL;
 
     if (routing->cury != routing->oldy) {
@@ -233,19 +323,19 @@ adjustCursorPosition (RoutingData *routing, int where, int trgy, int trgx, Scree
      * try going back to the previous position since it was obviously
      * the nearest ever reached.
      */
-    insertCursorKey(routing, ((dir > 0)? backward: forward));
+    moveCursor(routing, ((dir > 0)? axis->backward: axis->forward));
     return awaitCursorMotion(routing, -dir)? CRR_NEAR: CRR_FAIL;
   }
 }
 
 static RoutingResult
 adjustCursorHorizontally (RoutingData *routing, int where, int row, int column) {
-  return adjustCursorPosition(routing, where, row, column, SCR_KEY_CURSOR_RIGHT, SCR_KEY_CURSOR_LEFT);
+  return adjustCursorPosition(routing, where, row, column, &cursorAxisTable[CURSOR_AXIS_HORIZONTAL]);
 }
 
 static RoutingResult
 adjustCursorVertically (RoutingData *routing, int where, int row) {
-  return adjustCursorPosition(routing, where, row, -1, SCR_KEY_CURSOR_DOWN, SCR_KEY_CURSOR_UP);
+  return adjustCursorPosition(routing, where, row, -1, &cursorAxisTable[CURSOR_AXIS_VERTICAL]);
 }
 
 static RoutingStatus
@@ -266,6 +356,8 @@ doRouting (int column, int row, int screen) {
   routing.timeCount = 1;
 
   if (getCurrentPosition(&routing)) {
+    logRouting("from: [%d,%d]", routing.curx, routing.cury);
+
     if (column < 0) {
       adjustCursorVertically(&routing, 0, row);
     } else {
@@ -319,7 +411,7 @@ getRoutingStatus (int wait) {
           return ROUTING_ERROR;
         }
 
-        LogError("waitpid");
+        logSystemError("waitpid");
       }
     }
   }
@@ -365,15 +457,22 @@ startRouting (int column, int row, int screen) {
   switch (routingProcess = fork()) {
     case 0: { /* child: cursor routing subprocess */
       int result = ROUTING_ERROR;
-      nice(ROUTING_NICENESS);
-      if (constructRoutingScreen())
+      int niceness = nice(ROUTING_NICENESS);
+
+      if (niceness == -1) {
+        logSystemError("nice");
+      }
+
+      if (constructRoutingScreen()) {
         result = doRouting(column, row, screen);		/* terminate child process */
-      destructRoutingScreen();		/* close second thread of screen reading */
+        destructRoutingScreen();		/* close second thread of screen reading */
+      }
+
       _exit(result);		/* terminate child process */
     }
 
     case -1: /* error: fork() failed */
-      LogError("fork");
+      logSystemError("fork");
       routingProcess = NOT_ROUTING;
       break;
 
@@ -382,7 +481,7 @@ startRouting (int column, int row, int screen) {
         static int first = 1;
         if (first) {
           first = 0;
-          atexit(exitRouting);
+          onProgramExit(exitRouting);
         }
       }
 
