@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -19,12 +19,13 @@
 #ifndef BRLTTY_INCLUDED_SCR
 #define BRLTTY_INCLUDED_SCR
 
+#include "ktbdefs.h"
+#include "driver.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
-#include "driver.h"
-
 #define SCR_ATTR_FG_BLUE   0X01
 #define SCR_ATTR_FG_GREEN  0X02
 #define SCR_ATTR_FG_RED    0X04
@@ -34,6 +35,34 @@ extern "C" {
 #define SCR_ATTR_BG_RED    0X40
 #define SCR_ATTR_BLINK     0X80
 
+#define SCR_COLOUR_FG_BLACK 0
+#define SCR_COLOUR_FG_BLUE (SCR_ATTR_FG_BLUE)
+#define SCR_COLOUR_FG_GREEN (SCR_ATTR_FG_GREEN)
+#define SCR_COLOUR_FG_CYAN (SCR_ATTR_FG_BLUE | SCR_ATTR_FG_GREEN)
+#define SCR_COLOUR_FG_RED (SCR_ATTR_FG_RED)
+#define SCR_COLOUR_FG_MAGENTA (SCR_ATTR_FG_BLUE | SCR_ATTR_FG_RED)
+#define SCR_COLOUR_FG_BROWN (SCR_ATTR_FG_GREEN | SCR_ATTR_FG_RED)
+#define SCR_COLOUR_FG_LIGHT_GREY (SCR_ATTR_FG_BLUE | SCR_ATTR_FG_GREEN | SCR_ATTR_FG_RED)
+#define SCR_COLOUR_FG_DARK_GREY (SCR_ATTR_FG_BRIGHT)
+#define SCR_COLOUR_FG_LIGHT__BLUE (SCR_ATTR_FG_BLUE | SCR_ATTR_FG_BRIGHT)
+#define SCR_COLOUR_FG_LIGHT__GREEN (SCR_ATTR_FG_GREEN | SCR_ATTR_FG_BRIGHT)
+#define SCR_COLOUR_FG_LIGHT__CYAN (SCR_ATTR_FG_BLUE | SCR_ATTR_FG_GREEN | SCR_ATTR_FG_BRIGHT)
+#define SCR_COLOUR_FG_LIGHT__RED (SCR_ATTR_FG_RED | SCR_ATTR_FG_BRIGHT)
+#define SCR_COLOUR_FG_LIGHT__MAGENTA (SCR_ATTR_FG_BLUE | SCR_ATTR_FG_RED | SCR_ATTR_FG_BRIGHT)
+#define SCR_COLOUR_FG_YELLOW (SCR_ATTR_FG_GREEN | SCR_ATTR_FG_RED | SCR_ATTR_FG_BRIGHT)
+#define SCR_COLOUR_FG_WHITE (SCR_ATTR_FG_BLUE | SCR_ATTR_FG_GREEN | SCR_ATTR_FG_RED | SCR_ATTR_FG_BRIGHT)
+
+#define SCR_COLOUR_BG_BLACK 0
+#define SCR_COLOUR_BG_BLUE (SCR_ATTR_BG_BLUE)
+#define SCR_COLOUR_BG_GREEN (SCR_ATTR_BG_GREEN)
+#define SCR_COLOUR_BG_CYAN (SCR_ATTR_BG_BLUE | SCR_ATTR_BG_GREEN)
+#define SCR_COLOUR_BG_RED (SCR_ATTR_BG_RED)
+#define SCR_COLOUR_BG_MAGENTA (SCR_ATTR_BG_BLUE | SCR_ATTR_BG_RED)
+#define SCR_COLOUR_BG_BROWN (SCR_ATTR_BG_GREEN | SCR_ATTR_BG_RED)
+#define SCR_COLOUR_BG_LIGHT_GREY (SCR_ATTR_BG_BLUE | SCR_ATTR_BG_GREEN | SCR_ATTR_BG_RED)
+
+#define SCR_COLOUR_DEFAULT (SCR_COLOUR_FG_LIGHT_GREY | SCR_COLOUR_BG_BLACK)
+
 typedef struct {
   wchar_t text;
   unsigned char attributes;
@@ -56,7 +85,7 @@ extern int validateScreenBox (const ScreenBox *box, int columns, int rows);
 extern void setScreenMessage (const ScreenBox *box, ScreenCharacter *buffer, const char *message);
 
 extern void clearScreenCharacters (ScreenCharacter *characters, size_t count);
-extern void setScreenCharacterText (ScreenCharacter *characters, char text, size_t count);
+extern void setScreenCharacterText (ScreenCharacter *characters, wchar_t text, size_t count);
 extern void copyScreenCharacterText (ScreenCharacter *characters, const char *text, size_t count);
 extern void setScreenCharacterAttributes (ScreenCharacter *characters, unsigned char attributes, size_t count);
 
@@ -84,7 +113,32 @@ typedef enum {
   SCR_KEY_END,
   SCR_KEY_INSERT,
   SCR_KEY_DELETE,
-  SCR_KEY_FUNCTION
+  SCR_KEY_FUNCTION,
+
+  SCR_KEY_F1 = SCR_KEY_FUNCTION,
+  SCR_KEY_F2,
+  SCR_KEY_F3,
+  SCR_KEY_F4,
+  SCR_KEY_F5,
+  SCR_KEY_F6,
+  SCR_KEY_F7,
+  SCR_KEY_F8,
+  SCR_KEY_F9,
+  SCR_KEY_F10,
+  SCR_KEY_F11,
+  SCR_KEY_F12,
+  SCR_KEY_F13,
+  SCR_KEY_F14,
+  SCR_KEY_F15,
+  SCR_KEY_F16,
+  SCR_KEY_F17,
+  SCR_KEY_F18,
+  SCR_KEY_F19,
+  SCR_KEY_F20,
+  SCR_KEY_F21,
+  SCR_KEY_F22,
+  SCR_KEY_F23,
+  SCR_KEY_F24,
 } ScreenKey;
 
 /* Routines which apply to all screens. */
@@ -93,17 +147,23 @@ extern void destructSpecialScreens (void);
 
 extern int isLiveScreen (void);
 
-extern int isFrozenScreen (void);
-extern int haveFrozenScreen (void);
-extern int activateFrozenScreen (void);
-extern void deactivateFrozenScreen (void);
-
 extern int isHelpScreen (void);
 extern int haveHelpScreen (void);
 extern int activateHelpScreen (void);
 extern void deactivateHelpScreen (void);
 
+extern int isMenuScreen (void);
+extern int haveMenuScreen (void);
+extern int activateMenuScreen (void);
+extern void deactivateMenuScreen (void);
+
+extern int isFrozenScreen (void);
+extern int haveFrozenScreen (void);
+extern int activateFrozenScreen (void);
+extern void deactivateFrozenScreen (void);
+
 /* Routines which apply to the current screen. */
+extern size_t formatScreenTitle (char *buffer, size_t size);
 extern void describeScreen (ScreenDescription *);		/* get screen status */
 extern int readScreen (short left, short top, short width, short height, ScreenCharacter *buffer);
 extern int readScreenText (short left, short top, short width, short height, wchar_t *buffer);
@@ -112,11 +172,12 @@ extern int routeCursor (int column, int row, int screen);
 extern int highlightScreenRegion (int left, int right, int top, int bottom);
 extern int unhighlightScreenRegion (void);
 extern int getScreenPointer (int *column, int *row);
-extern int selectVirtualTerminal (int vt);
-extern int switchVirtualTerminal (int vt);
+extern int selectScreenVirtualTerminal (int vt);
+extern int switchScreenVirtualTerminal (int vt);
 extern int currentVirtualTerminal (void);
 extern int userVirtualTerminal (int number);
-extern int executeScreenCommand (int);
+extern int executeScreenCommand (int *);
+extern KeyTableCommandContext getScreenCommandContext (void);
 
 /* Routines which apply to the routing screen.
  * An extra `thread' for the cursor routing subprocess.
@@ -129,7 +190,13 @@ extern void destructRoutingScreen (void);
 /* Routines which apply to the help screen. */
 extern int constructHelpScreen (void);
 extern void destructHelpScreen (void);
-extern int addHelpLine (const wchar_t *line);
+extern int addHelpPage (void);
+extern unsigned int getHelpPageCount (void);
+extern unsigned int getHelpPageNumber (void);
+extern int setHelpPageNumber (unsigned int number);
+extern int clearHelpPage (void);
+extern int addHelpLine (const wchar_t *characters);
+extern unsigned int getHelpLineCount (void);
 
 typedef struct ScreenDriverStruct ScreenDriver;
 extern const char *const *getScreenParameters (const ScreenDriver *driver);
