@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -99,6 +99,12 @@ currentVirtualTerminal_BaseScreen (void) {
   return -1;
 }
 
+static size_t
+formatTitle_BaseScreen (char *buffer, size_t size) {
+  *buffer = 0;
+  return 0;
+}
+
 static void
 describe_BaseScreen (ScreenDescription *description) {
   description->rows = 1;
@@ -143,12 +149,18 @@ getPointer_BaseScreen (int *column, int *row) {
 }
 
 static int
-executeCommand_BaseScreen (int command) {
+executeCommand_BaseScreen (int *command) {
   return 0;
 }
 
+static KeyTableCommandContext
+getCommandContext_BaseScreen (void) {
+  return KTB_CTX_DEFAULT;
+}
+
 void
 initializeBaseScreen (BaseScreen *base) {
+  base->formatTitle = formatTitle_BaseScreen;
   base->selectVirtualTerminal = selectVirtualTerminal_BaseScreen;
   base->switchVirtualTerminal = switchVirtualTerminal_BaseScreen;
   base->currentVirtualTerminal = currentVirtualTerminal_BaseScreen;
@@ -160,6 +172,7 @@ initializeBaseScreen (BaseScreen *base) {
   base->unhighlightRegion = unhighlightRegion_BaseScreen;
   base->getPointer = getPointer_BaseScreen;
   base->executeCommand = executeCommand_BaseScreen;
+  base->getCommandContext = getCommandContext_BaseScreen;
 }
 
 void
