@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -42,7 +42,7 @@ construct_FrozenScreen (BaseScreen *source) {
     free(screenCharacters);
     screenCharacters = NULL;
   } else {
-    LogError("malloc");
+    logMallocError();
   }
   return 0;
 }
