@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -24,15 +24,20 @@ extern "C" {
 #endif /* __cplusplus */
 
 #include "scr_base.h"
-#include "help.h"
 
 typedef struct {
   BaseScreen base;
-  int (*construct) (const char *);
-  void (*destruct) (void);		   /* called once to close the help screen */
-  void (*setPageNumber) (short);
-  short (*getPageNumber) (void);
-  short (*getPageCount) (void);
+  int (*construct) (void);
+  void (*destruct) (void);
+
+  unsigned int (*addPage) (void);
+  unsigned int (*getPageCount) (void);
+  unsigned int (*getPageNumber) (void);
+  int (*setPageNumber) (unsigned int number);
+
+  int (*clearPage) (void);
+  int (*addLine) (const wchar_t *characters);
+  unsigned int (*getLineCount) (void);
 } HelpScreen;
 
 extern void initializeHelpScreen (HelpScreen *);
