@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2009 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -24,7 +24,6 @@
 #include <fcntl.h>
 #include <sys/ioctl.h>
 
-#include "misc.h"
 #include "system.h"
 
 #include "sys_prog_none.h"
@@ -52,5 +51,3 @@
 #endif /* ENABLE_MIDI_SUPPORT */
 
 #include "sys_ports_none.h"
-
-#include "sys_kbd_none.h"
