@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -23,6 +23,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
+#include <sys/stat.h>
 #include <linux/kd.h>
 
 #ifdef HAVE_LINUX_INPUT_H
@@ -33,7 +34,6 @@
 #include "file.h"
 #include "device.h"
 #include "timing.h"
-#include "misc.h"
 #include "system.h"
 #include "sys_linux.h"
 #include "bitmask.h"
@@ -46,14 +46,14 @@ enableUinputKeyEvents (int device) {
   int key;
 
   if (ioctl(device, UI_SET_EVBIT, EV_KEY) == -1) {
-    LogError("ioctl[UI_SET_EVBIT,EV_KEY]");
+    logSystemError("ioctl[UI_SET_EVBIT,EV_KEY]");
     return 0;
   }
 
   for (key=0; key<=KEY_MAX; key+=1) {
     if (ioctl(device, UI_SET_KEYBIT, key) == -1) {
-      LogError("ioctl[UI_SET_KEYBIT]");
-      return 0;
+      logMessage(LOG_WARNING, "ioctl[UI_SET_KEYBIT] failed for key 0X%X: %s",
+                 key, strerror(errno));
     }
   }
 
@@ -63,7 +63,7 @@ enableUinputKeyEvents (int device) {
 static int
 enableUinputAutorepeat (int device) {
   if (ioctl(device, UI_SET_EVBIT, EV_REP) == -1) {
-    LogError("ioctl[UI_SET_EVBIT,EV_REP]");
+    logSystemError("ioctl[UI_SET_EVBIT,EV_REP]");
     return 0;
   }
 
@@ -78,52 +78,77 @@ getProgramPath (void) {
   char *buffer = NULL;
 
   while (1) {
-    buffer = reallocWrapper(buffer, size<<=1);
+    {
+      char *newBuffer = realloc(buffer, size<<=1);
+
+      if (!newBuffer) {
+        logMallocError();
+        break;
+      }
+
+      buffer = newBuffer;
+    }
 
     {
       int length = readlink("/proc/self/exe", buffer, size);
 
       if (length == -1) {
-        if (errno != ENOENT) LogError("readlink");
+        if (errno != ENOENT) logSystemError("readlink");
         break;
       }
 
       if (length < size) {
         buffer[length] = 0;
-        path = strdupWrapper(buffer);
+        if (!(path = strdup(buffer))) logMallocError();
         break;
       }
     }
   }
 
-  free(buffer);
+  if (buffer) free(buffer);
   return path;
 }
 
 char *
 getBootParameters (const char *name) {
   size_t nameLength = strlen(name);
-  const char *path = "/proc/cmdline";
-  FILE *file = fopen(path, "r");
-  if (file) {
-    char *parameters = strdupWrapper("");
-    char buffer[0X1000];
-    char *line = fgets(buffer, sizeof(buffer), file);
-    if (line) {
-      char *token;
-      while ((token = strtok(line, " \n"))) {
-        line = NULL;
-        if ((strncmp(token, name, nameLength) == 0) &&
-            (token[nameLength] == '=')) {
-          free(parameters);
-          parameters = strdupWrapper(token + nameLength + 1);
+  char *parameters;
+
+  if ((parameters = strdup(""))) {
+    const char *path = "/proc/cmdline";
+    FILE *file;
+
+    if ((file = fopen(path, "r"))) {
+      char buffer[0X1000];
+      char *line = fgets(buffer, sizeof(buffer), file);
+
+      if (line) {
+        char *token;
+
+        while ((token = strtok(line, " \n"))) {
+          line = NULL;
+
+          if ((strncmp(token, name, nameLength) == 0) &&
+              (token[nameLength] == '=')) {
+            char *newParameters = strdup(token + nameLength + 1);
+
+            if (newParameters) {
+              free(parameters);
+              parameters = newParameters;
+            } else {
+              logMallocError();
+            }
+          }
         }
       }
+
+      fclose(file);
     }
-    fclose(file);
-    return parameters;
+  } else {
+    logMallocError();
   }
-  return NULL;
+
+  return parameters;
 }
 
 #include "sys_exec_unix.h"
@@ -135,7 +160,11 @@ getBootParameters (const char *name) {
 #include "sys_shlib_dlfcn.h"
 #endif /* ENABLE_SHARED_OBJECTS */
 
+#ifdef ENABLE_BEEPER_SUPPORT
 #include "sys_beep_linux.h"
+#else /* ENABLE_BEEPER_SUPPORT */
+#include "sys_beep_none.h"
+#endif /* ENABLE_BEEPER_SUPPORT */
 
 #ifdef ENABLE_PCM_SUPPORT
 #if defined(USE_PCM_SUPPORT_ALSA)
@@ -187,7 +216,7 @@ installKernelModule (const char *name, int *status) {
 
         fclose(stream);
       } else {
-        LogPrint(LOG_WARNING, "cannot open %s: %s", path, strerror(errno));
+        logMessage(LOG_WARNING, "cannot open %s: %s", path, strerror(errno));
       }
     }
 
@@ -196,7 +225,7 @@ installKernelModule (const char *name, int *status) {
       int ok = executeHostCommand(arguments) == 0;
 
       if (!ok) {
-        LogPrint(LOG_WARNING, "kernel module not installed: %s", name);
+        logMessage(LOG_WARNING, "kernel module not installed: %s", name);
         return 0;
       }
 
@@ -215,31 +244,31 @@ openCharacterDevice (const char *name, int flags, int major, int minor) {
   if (!path) {
     descriptor = -1;
   } else if ((descriptor = open(path, flags)) != -1) {
-    LogPrint(LOG_DEBUG, "device opened: %s: fd=%d", path, descriptor);
+    logMessage(LOG_DEBUG, "device opened: %s: fd=%d", path, descriptor);
   } else {
-    LogPrint(LOG_DEBUG, "cannot open device: %s: %s", path, strerror(errno));
+    logMessage(LOG_DEBUG, "cannot open device: %s: %s", path, strerror(errno));
 
     if (errno == ENOENT) {
       free(path);
       if ((path = makeWritablePath(locatePathName(name)))) {
         if ((descriptor = open(path, flags)) != -1) {
-          LogPrint(LOG_DEBUG, "device opened: %s: fd=%d", path, descriptor);
+          logMessage(LOG_DEBUG, "device opened: %s: fd=%d", path, descriptor);
         } else {
-          LogPrint(LOG_DEBUG, "cannot open device: %s: %s", path, strerror(errno));
+          logMessage(LOG_DEBUG, "cannot open device: %s: %s", path, strerror(errno));
 
           if (errno == ENOENT) {
             mode_t mode = S_IFCHR | S_IRUSR | S_IWUSR;
 
             if (mknod(path, mode, makedev(major, minor)) == -1) {
-              LogPrint(LOG_DEBUG, "cannot create device: %s: %s", path, strerror(errno));
+              logMessage(LOG_DEBUG, "cannot create device: %s: %s", path, strerror(errno));
             } else {
-              LogPrint(LOG_DEBUG, "device created: %s mode=%06o major=%d minor=%d",
-                       path, mode, major, minor);
+              logMessage(LOG_DEBUG, "device created: %s mode=%06o major=%d minor=%d",
+                         path, mode, major, minor);
 
               if ((descriptor = open(path, flags)) != -1) {
-                LogPrint(LOG_DEBUG, "device opened: %s: fd=%d", path, descriptor);
+                logMessage(LOG_DEBUG, "device opened: %s: fd=%d", path, descriptor);
               } else {
-                LogPrint(LOG_DEBUG, "cannot open device: %s: %s", path, strerror(errno));
+                logMessage(LOG_DEBUG, "cannot open device: %s: %s", path, strerror(errno));
               }
             }
           }
@@ -253,17 +282,17 @@ openCharacterDevice (const char *name, int flags, int major, int minor) {
     struct stat status;
 
     if (fstat(descriptor, &status) == -1) {
-      LogPrint(LOG_DEBUG, "cannot fstat device: %d [%s]: %s",
-               descriptor, path, strerror(errno));
+      logMessage(LOG_DEBUG, "cannot fstat device: %d [%s]: %s",
+                 descriptor, path, strerror(errno));
     } else if (!S_ISCHR(status.st_mode)) {
-      LogPrint(LOG_DEBUG, "not a character device: %s: fd=%d", path, descriptor);
+      logMessage(LOG_DEBUG, "not a character device: %s: fd=%d", path, descriptor);
     } else {
       ok = 1;
     }
 
     if (!ok) {
       close(descriptor);
-      LogPrint(LOG_DEBUG, "device closed: %s: fd=%d", path, descriptor);
+      logMessage(LOG_DEBUG, "device closed: %s: fd=%d", path, descriptor);
       descriptor = -1;
     }
   }
@@ -289,7 +318,7 @@ getUinputDevice (void) {
 
     {
       static const char *const names[] = {"uinput", "input/uinput", NULL};
-      name = resolveDeviceName(names, "uinput", W_OK);
+      name = resolveDeviceName(names, "uinput");
     }
 
     if (name) {
@@ -297,7 +326,7 @@ getUinputDevice (void) {
 
       if (device != -1) {
         struct uinput_user_dev description;
-        LogPrint(LOG_DEBUG, "uinput opened: %s fd=%d", name, device);
+        logMessage(LOG_DEBUG, "uinput opened: %s fd=%d", name, device);
         
         memset(&description, 0, sizeof(description));
         strcpy(description.name, "brltty");
@@ -308,20 +337,20 @@ getUinputDevice (void) {
               if (ioctl(device, UI_DEV_CREATE) != -1) {
                 uinputDevice = device;
               } else {
-                LogError("ioctl[UI_DEV_CREATE]");
+                logSystemError("ioctl[UI_DEV_CREATE]");
               }
             }
           }
         } else {
-          LogError("write(struct uinput_user_dev)");
+          logSystemError("write(struct uinput_user_dev)");
         }
 
         if (device != uinputDevice) {
           close(device);
-          LogPrint(LOG_DEBUG, "uinput closed");
+          logMessage(LOG_DEBUG, "uinput closed");
         }
       } else {
-        LogPrint(LOG_DEBUG, "cannot open uinput");
+        logMessage(LOG_DEBUG, "cannot open uinput");
       }
     }
   }
@@ -360,7 +389,7 @@ writeInputEvent (uint16_t type, uint16_t code, int32_t value) {
     if (write(device, &event, sizeof(event)) != -1) {
       return 1;
     } else {
-      LogError("write(struct input_event)");
+      logSystemError("write(struct input_event)");
     }
   }
 #endif /* HAVE_LINUX_INPUT_H */
