@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -47,6 +47,7 @@
 #include "sys_ports_x86.h"
 
 #include <stdarg.h>
+#include <stdio.h>
 #include <string.h>
 #include <setjmp.h>
 #include <dpmi.h>
@@ -64,7 +65,7 @@ int _stklen = 8192;
 void __crt0_load_environment_file(char *_app_name) { return; }
 char **__crt0_glob_function(char *_arg) { return 0; }
 
-static void tsr_exit(void);
+static void tsr_exit(void) NORETURN;
 /* Start undocumented way to make exception handling disappear (v2.03) */
 short __djgpp_ds_alias;
 void __djgpp_exception_processor(void) { return; }
@@ -244,7 +245,7 @@ tsr_exit(void) {
 
 /* go to background: TSR */
 void
-background(void) {
+msdosBackground(void) {
   saveState(&mainState);
   if (!setjmp(mainCtx)) {
     __dpmi_regs regs;
@@ -281,7 +282,7 @@ background(void) {
     __dpmi_int(DOS_INT, &regs);
 
     /* shouldn't be reached */
-    LogPrint(LOG_ERR,"Installation failed");
+    logMessage(LOG_ERR,"Installation failed");
     backgrounded = 0;
   }
   saveState(&intState);
