@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -21,7 +21,6 @@
 
 #include "log.h"
 #include "timing.h"
-#include "misc.h"
 
 struct PcmDeviceStruct {
   snd_pcm_t *handle;
@@ -68,7 +67,7 @@ PCM_ALSA_SYMBOL(pcm_resume);
 
 static void
 logPcmError (int level, const char *action, int code) {
-  LogPrint(level, "ALSA PCM %s error: %s", action, pcmAlsa_strerror(code));
+  logMessage(level, "ALSA PCM %s error: %s", action, pcmAlsa_strerror(code));
 }
 
 static int
@@ -93,7 +92,7 @@ configurePcmSampleFormat (PcmDevice *pcm, int errorLevel) {
     ++format;
   }
 
-  LogPrint(errorLevel, "Unsupported PCM sample format.");
+  logMessage(errorLevel, "Unsupported PCM sample format.");
   return 0;
 }
 
@@ -114,7 +113,7 @@ configurePcmSampleRate (PcmDevice *pcm, int errorLevel) {
   }
 
   if ((minimum > maximum) || (minimum < 1)) {
-    LogPrint(errorLevel, "Invalid PCM rate range: %u-%u", minimum, maximum);
+    logMessage(errorLevel, "Invalid PCM rate range: %u-%u", minimum, maximum);
     return 0;
   }
 
@@ -144,7 +143,7 @@ configurePcmChannelCount (PcmDevice *pcm, int errorLevel) {
   }
 
   if ((minimum > maximum) || (minimum < 1)) {
-    LogPrint(errorLevel, "Invalid PCM channel range: %u-%u", minimum, maximum);
+    logMessage(errorLevel, "Invalid PCM channel range: %u-%u", minimum, maximum);
     return 0;
   }
 
@@ -163,7 +162,7 @@ openPcmDevice (int errorLevel, const char *device) {
 
   if (!pcmAlsaLibrary) {
     if (!(pcmAlsaLibrary = loadSharedObject("libasound.so.2"))) {
-      LogPrint(LOG_ERR, "Unable to load ALSA PCM library.");
+      logMessage(LOG_ERR, "Unable to load ALSA PCM library.");
       return NULL;
     }
 
@@ -216,7 +215,7 @@ openPcmDevice (int errorLevel, const char *device) {
                     pcm->periodTime = pcm->bufferTime / 8;
                     if ((result = pcmAlsa_pcm_hw_params_set_period_time_near(pcm->handle, pcm->hardwareParameters, &pcm->periodTime, NULL)) >= 0) {
                       if ((result = pcmAlsa_pcm_hw_params(pcm->handle, pcm->hardwareParameters)) >= 0) {
-                        LogPrint(LOG_DEBUG, "ALSA PCM: Chan=%u Rate=%u BufTim=%u PerTim=%u", pcm->channelCount, pcm->sampleRate, pcm->bufferTime, pcm->periodTime);
+                        logMessage(LOG_DEBUG, "ALSA PCM: Chan=%u Rate=%u BufTim=%u PerTim=%u", pcm->channelCount, pcm->sampleRate, pcm->bufferTime, pcm->periodTime);
                         return pcm;
                       } else {
                         logPcmError(errorLevel, "set hardware parameters", result);
@@ -249,7 +248,7 @@ openPcmDevice (int errorLevel, const char *device) {
 
     free(pcm);
   } else {
-    LogError("PCM device allocation");
+    logSystemError("PCM device allocation");
   }
 
   return NULL;
