@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -48,14 +48,14 @@ openPcmDevice (int errorLevel, const char *device) {
       info.play.precision = 16;
       info.play.gain = AUDIO_MAX_GAIN;
       if (ioctl(pcm->fileDescriptor, AUDIO_SETINFO, &info) == -1)
-        LogPrint(errorLevel, "Cannot set audio info: %s", strerror(errno));
+        logMessage(errorLevel, "Cannot set audio info: %s", strerror(errno));
       return pcm;
     } else {
-      LogPrint(errorLevel, "Cannot open PCM device: %s: %s", device, strerror(errno));
+      logMessage(errorLevel, "Cannot open PCM device: %s: %s", device, strerror(errno));
     }
     free(pcm);
   } else {
-    LogError("PCM device allocation");
+    logSystemError("PCM device allocation");
   }
   return NULL;
 }
@@ -74,7 +74,7 @@ writePcmData (PcmDevice *pcm, const unsigned char *buffer, int count) {
 static int
 getPcmAudioInfo (PcmDevice *pcm, audio_info_t *info) {
   if (ioctl(pcm->fileDescriptor, AUDIO_GETINFO, info) != -1) return 1;
-  LogError("AUDIO_GETINFO");
+  logSystemError("AUDIO_GETINFO");
   return 0;
 }
 
