@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -98,14 +98,14 @@ WIN_PROC_STUB(freeaddrinfo);
 static void *
 loadLibrary (const char *name) {
   HMODULE module = LoadLibrary(name);
-  if (!module) LogPrint(LOG_DEBUG, "%s: %s", gettext("cannot load library"), name);
+  if (!module) logMessage(LOG_DEBUG, "%s: %s", gettext("cannot load library"), name);
   return module;
 }
 
 static void *
 getProcedure (HMODULE module, const char *name) {
   void *address = module? GetProcAddress(module, name): NULL;
-  if (!address) LogPrint(LOG_DEBUG, "%s: %s", gettext("cannot find procedure"), name);
+  if (!address) logMessage(LOG_DEBUG, "%s: %s", gettext("cannot find procedure"), name);
   return address;
 }
 
@@ -114,7 +114,7 @@ addWindowsCommandLineCharacter (char **buffer, int *size, int *length, char char
   if (*length == *size) {
     char *newBuffer = realloc(*buffer, (*size = *size? *size<<1: 0X80));
     if (!newBuffer) {
-      LogError("realloc");
+      logSystemError("realloc");
       return 0;
     }
     *buffer = newBuffer;
@@ -183,7 +183,7 @@ makeWindowsCommandLine (const char *const *arguments) {
   {
     char *line = realloc(buffer, length);
     if (line) return line;
-    LogError("realloc");
+    logSystemError("realloc");
   }
 
 error:
@@ -210,20 +210,20 @@ installService (const char *name, const char *description) {
                                         command, NULL, NULL, NULL, NULL, NULL);
 
       if (service) {
-        LogPrint(LOG_NOTICE, "service installed: %s", name);
+        logMessage(LOG_NOTICE, "service installed: %s", name);
         installed = 1;
 
         CloseServiceHandle(service);
       } else if (GetLastError() == ERROR_SERVICE_EXISTS) {
-        LogPrint(LOG_WARNING, "service already installed: %s", name);
+        logMessage(LOG_WARNING, "service already installed: %s", name);
         installed = 1;
       } else {
-        LogWindowsError("CreateService");
+        logWindowsSystemError("CreateService");
       }
 
       CloseServiceHandle(scm);
     } else {
-      LogWindowsError("OpenSCManager");
+      logWindowsSystemError("OpenSCManager");
     }
 
     free(command);
@@ -242,26 +242,26 @@ removeService (const char *name) {
 
     if (service) {
       if (DeleteService(service)) {
-        LogPrint(LOG_NOTICE, "service removed: %s", name);
+        logMessage(LOG_NOTICE, "service removed: %s", name);
         removed = 1;
       } else if (GetLastError() == ERROR_SERVICE_MARKED_FOR_DELETE) {
-        LogPrint(LOG_WARNING, "service already being removed: %s", name);
+        logMessage(LOG_WARNING, "service already being removed: %s", name);
         removed = 1;
       } else {
-        LogWindowsError("DeleteService");
+        logWindowsSystemError("DeleteService");
       }
 
       CloseServiceHandle(service);
     } else if (GetLastError() == ERROR_SERVICE_DOES_NOT_EXIST) {
-      LogPrint(LOG_WARNING, "service not installed: %s", name);
+      logMessage(LOG_WARNING, "service not installed: %s", name);
       removed = 1;
     } else {
-      LogWindowsError("OpenService");
+      logWindowsSystemError("OpenService");
     }
 
     CloseServiceHandle(scm);
   } else {
-    LogWindowsError("OpenSCManager");
+    logWindowsSystemError("OpenSCManager");
   }
 
   return removed;
@@ -363,7 +363,7 @@ win_getLocale (void) {
 
       return locale;
     } else {
-      LogWindowsError("GetLocaleInfoEx");
+      logWindowsSystemError("GetLocaleInfoEx");
     }
   }
 
@@ -508,11 +508,15 @@ win_getLocale (void) {
         DIALECT(PORTUGUESE, BRAZILIAN, "pt_BR");
         DIALECT(PORTUGUESE, PORTUGAL, "pt_PT");
         DIALECT(PUNJABI, INDIA, "pa_IN");
+#ifdef SUBLANG_PUNJABI_PAKISTAN
         DIALECT(PUNJABI, PAKISTAN, "pa_PK");
+#endif /* SUBLANG_PUNJABI_PAKISTAN */
         DIALECT(QUECHUA, BOLIVIA, "qu_BO");
         DIALECT(QUECHUA, ECUADOR, "qu_EC");
         DIALECT(QUECHUA, PERU, "qu_PE");
+#ifdef SUBLANG_ROMANIAN_MOLDOVA
         DIALECT(ROMANIAN, MOLDOVA, "ro_MD");
+#endif /* SUBLANG_ROMANIAN_MOLDOVA */
         DIALECT(ROMANIAN, ROMANIA, "ro_RO");
         DIALECT(RUSSIAN, RUSSIA, "ru_RU");
         DIALECT(SAMI, LULE_NORWAY, "smj_NO");
@@ -710,7 +714,7 @@ win_getLocale (void) {
 #undef LANGUAGE
       }
     } else {
-      LogWindowsError("GetLocaleInfo");
+      logWindowsSystemError("GetLocaleInfo");
     }
   }
 
