@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -88,8 +88,8 @@ loadContractionTable (const char *path, TableData *data) {
   if (!table) return 0;
 
   data->object = table;
-  data->bytes = table->header.bytes;
-  data->size = table->size;
+  data->bytes = table->data.internal.header.bytes;
+  data->size = table->data.internal.size;
   return 1;
 }
 
@@ -131,7 +131,7 @@ findTableEntry (const char *extension) {
     entry += 1;
   }
 
-  LogPrint(LOG_ERR, "unrecognized file extension: %s", extension);
+  logMessage(LOG_ERR, "unrecognized file extension: %s", extension);
   return NULL;
 }
 
@@ -144,7 +144,7 @@ dumpBytes (FILE *stream, const unsigned char *bytes, size_t count) {
 
   if (count) {
     char buffer[0X10];
-    snprintf(buffer, sizeof(buffer), "%X%n", (unsigned int)count-1, &digits);
+    digits = snprintf(buffer, sizeof(buffer), "%X", (unsigned int)count-1);
   } else {
     digits = 1;
   }
@@ -197,13 +197,13 @@ dumpBytes (FILE *stream, const unsigned char *bytes, size_t count) {
   return 1;
 
 outputError:
-  LogPrint(LOG_ERR, "table write error: %s", strerror(errno));
+  logMessage(LOG_ERR, "table write error: %s", strerror(errno));
   return 0;
 }
 
 int
 main (int argc, char *argv[]) {
-  int status;
+  ProgramExitStatus exitStatus;
   char *path;
 
   {
@@ -212,35 +212,42 @@ main (int argc, char *argv[]) {
       .applicationName = "tbl2hex",
       .argumentsSummary = "table-file"
     };
-    processOptions(&descriptor, &argc, &argv);
+    PROCESS_OPTIONS(descriptor, argc, argv);
   }
 
   if (argc == 0) {
-    LogPrint(LOG_ERR, "missing table file.");
-    exit(2);
+    logMessage(LOG_ERR, "missing table file.");
+    return PROG_EXIT_SYNTAX;
   }
   path = *argv++, argc--;
 
   {
-    const TableEntry *entry = findTableEntry(locatePathExtension(path));
+    const char *extension = locatePathExtension(path);
 
-    if (entry) {
-      TableData data;
-      if (entry->load(path, &data)) {
-        if (dumpBytes(stdout, data.bytes, data.size)) {
-          status = 0;
+    if (extension) {
+      const TableEntry *entry = findTableEntry(extension);
+
+      if (entry) {
+        TableData data;
+        if (entry->load(path, &data)) {
+          if (dumpBytes(stdout, data.bytes, data.size)) {
+            exitStatus = PROG_EXIT_SUCCESS;
+          } else {
+            exitStatus = PROG_EXIT_FATAL;
+          }
+
+          entry->unload(&data);
         } else {
-          status = 4;
+          exitStatus = PROG_EXIT_FATAL;
         }
-
-        entry->unload(&data);
       } else {
-        status = 3;
+        exitStatus = PROG_EXIT_SEMANTIC;
       }
     } else {
-      status = 5;
+      logMessage(LOG_ERR, "no file extension");
+      exitStatus = PROG_EXIT_SEMANTIC;
     }
   }
 
-  return status;
+  return exitStatus;
 }
