@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -126,22 +126,6 @@ getDotsOperand (DataFile *file, unsigned char *dots) {
   return 0;
 }
 
-static int
-processCharOperands (DataFile *file, void *data) {
-  TextTableData *ttd = data;
-  wchar_t character;
-
-  if (getCharacterOperand(file, &character)) {
-    unsigned char dots;
-
-    if (getDotsOperand(file, &dots)) {
-      if (!setTextTableCharacter(ttd, character, dots)) return 0;
-    }
-  }
-
-  return 1;
-}
-
 static int
 processByteOperands (DataFile *file, void *data) {
   TextTableData *ttd = data;
@@ -158,10 +142,43 @@ processByteOperands (DataFile *file, void *data) {
   return 1;
 }
 
+static int
+processCharOperands (DataFile *file, void *data) {
+  TextTableData *ttd = data;
+  wchar_t character;
+
+  if (getCharacterOperand(file, &character)) {
+    unsigned char dots;
+
+    if (getDotsOperand(file, &dots)) {
+      if (!setTextTableCharacter(ttd, character, dots)) return 0;
+    }
+  }
+
+  return 1;
+}
+
+static int
+processGlyphOperands (DataFile *file, void *data) {
+  TextTableData *ttd = data;
+  wchar_t character;
+
+  if (getCharacterOperand(file, &character)) {
+    unsigned char dots;
+
+    if (getDotsOperand(file, &dots)) {
+      if (!setTextTableGlyph(ttd, character, dots)) return 0;
+    }
+  }
+
+  return 1;
+}
+
 static int
 processTextTableLine (DataFile *file, void *data) {
   static const DataProperty properties[] = {
     {.name=WS_C("char"), .processor=processCharOperands},
+    {.name=WS_C("glyph"), .processor=processGlyphOperands},
     {.name=WS_C("byte"), .processor=processByteOperands},
     {.name=WS_C("include"), .processor=processIncludeOperands},
     {.name=NULL, .processor=NULL}
