@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -20,6 +20,8 @@
 
 #include <stdio.h>
 
+#include "log.h"
+#include "file.h"
 #include "charset.h"
 #include "ttb.h"
 #include "ttb_internal.h"
@@ -84,6 +86,24 @@ getUnicodeCellEntry (TextTable *table, wchar_t character) {
   return NULL;
 }
 
+typedef struct {
+  TextTable *const table;
+  unsigned char dots;
+} SetBrailleRepresentationData;
+
+static int
+setBrailleRepresentation (wchar_t character, void *data) {
+  SetBrailleRepresentationData *sbr = data;
+  const unsigned char *cell = getUnicodeCellEntry(sbr->table, character);
+
+  if (cell) {
+    sbr->dots = *cell;
+    return 1;
+  }
+
+  return 0;
+}
+
 unsigned char
 convertCharacterToDots (TextTable *table, wchar_t character) {
   switch (character & ~UNICODE_CELL_MASK) {
@@ -97,12 +117,25 @@ convertCharacterToDots (TextTable *table, wchar_t character) {
     }
 
     default: {
-      const unsigned char *cell;
-      if ((cell = getUnicodeCellEntry(table, character))) return *cell;
+      {
+        SetBrailleRepresentationData sbr = {
+          .table = table,
+          .dots = 0
+        };
+
+        if (handleBestCharacter(character, setBrailleRepresentation, &sbr)) {
+          return sbr.dots;
+        }
+      }
 
     unknownCharacter:
-      if ((cell = getUnicodeCellEntry(table, UNICODE_REPLACEMENT_CHARACTER))) return *cell;
-      if ((cell = getUnicodeCellEntry(table, WC_C('?')))) return *cell;
+      {
+        const unsigned char *cell;
+
+        if ((cell = getUnicodeCellEntry(table, UNICODE_REPLACEMENT_CHARACTER))) return *cell;
+        if ((cell = getUnicodeCellEntry(table, WC_C('?')))) return *cell;
+      }
+
       return BRL_DOT1 | BRL_DOT2 | BRL_DOT3 | BRL_DOT4 | BRL_DOT5 | BRL_DOT6 | BRL_DOT7 | BRL_DOT8;
     }
   }
@@ -114,3 +147,28 @@ convertDotsToCharacter (TextTable *table, unsigned char dots) {
   if (BITMASK_TEST(header->dotsCharacterDefined, dots)) return header->dotsToCharacter[dots];
   return UNICODE_REPLACEMENT_CHARACTER;
 }
+
+int
+replaceTextTable (const char *directory, const char *name) {
+  int ok = 0;
+  char *path;
+
+  if ((path = makeTextTablePath(directory, name))) {
+    TextTable *newTable;
+
+    logMessage(LOG_DEBUG, "compiling text table: %s", path);
+    if ((newTable = compileTextTable(path))) {
+      TextTable *oldTable = textTable;
+      textTable = newTable;
+      destroyTextTable(oldTable);
+      ok = 1;
+    } else {
+      logMessage(LOG_ERR, "%s: %s", gettext("cannot compile text table"), path);
+    }
+
+    free(path);
+  }
+
+  if (!ok) logMessage(LOG_ERR, "%s: %s", gettext("cannot load text table"), name);
+  return ok;
+}
