@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -29,14 +29,12 @@
 #include "tunes.h"
 #include "log.h"
 #include "parse.h"
-#include "misc.h"
 #include "defaults.h"
-#include "brltty.h"
+#include "prefs.h"
 #include "message.h"
 #include "brl.h"
 
 int updateInterval = DEFAULT_UPDATE_INTERVAL;
-Preferences prefs;
 
 static char *opt_tuneDevice;
 static char *opt_outputVolume;
@@ -135,41 +133,41 @@ validateInstrument (unsigned char *value, const char *string) {
 
 int
 main (int argc, char *argv[]) {
-  TuneDevice tuneDevice;
-  unsigned char outputVolume;
-
-#ifdef ENABLE_MIDI_SUPPORT
-  unsigned char midiInstrument;
-#endif /* ENABLE_MIDI_SUPPORT */
-
   {
     static const OptionsDescriptor descriptor = {
       OPTION_TABLE(programOptions),
       .applicationName = "tunetest",
       .argumentsSummary = "{note duration} ..."
     };
-    processOptions(&descriptor, &argc, &argv);
+    PROCESS_OPTIONS(descriptor, argc, argv);
   }
 
+  resetPreferences();
+  prefs.alertMessages = 0;
+  prefs.alertDots = 0;
+  prefs.alertTunes = 1;
+
   if (opt_tuneDevice && *opt_tuneDevice) {
     unsigned int device;
+
     if (!validateChoice(&device, opt_tuneDevice, deviceNames)) {
-      LogPrint(LOG_ERR, "%s: %s", "invalid tune device", opt_tuneDevice);
-      exit(2);
+      logMessage(LOG_ERR, "%s: %s", "invalid tune device", opt_tuneDevice);
+      return PROG_EXIT_SYNTAX;
     }
-    tuneDevice = device;
-  } else {
-    tuneDevice = getDefaultTuneDevice();
+
+    prefs.tuneDevice = device;
   }
 
 #ifdef ENABLE_MIDI_SUPPORT
   if (opt_midiInstrument && *opt_midiInstrument) {
-    if (!validateInstrument(&midiInstrument, opt_midiInstrument)) {
-      LogPrint(LOG_ERR, "%s: %s", "invalid musical instrument", opt_midiInstrument);
-      exit(2);
+    unsigned char instrument;
+
+    if (!validateInstrument(&instrument, opt_midiInstrument)) {
+      logMessage(LOG_ERR, "%s: %s", "invalid musical instrument", opt_midiInstrument);
+      return PROG_EXIT_SYNTAX;
     }
-  } else {
-    midiInstrument = 0;
+
+    prefs.midiInstrument = instrument;
   }
 #endif /* ENABLE_MIDI_SUPPORT */
 
@@ -177,92 +175,93 @@ main (int argc, char *argv[]) {
     static const int minimum = 0;
     static const int maximum = 100;
     int volume;
+
     if (!validateInteger(&volume, opt_outputVolume, &minimum, &maximum)) {
-      LogPrint(LOG_ERR, "%s: %s", "invalid volume percentage", opt_outputVolume);
-      exit(2);
+      logMessage(LOG_ERR, "%s: %s", "invalid volume percentage", opt_outputVolume);
+      return PROG_EXIT_SYNTAX;
+    }
+
+    switch (prefs.tuneDevice) {
+      case tdPcm:
+        prefs.pcmVolume = volume;
+        break;
+
+      case tdMidi:
+        prefs.midiVolume = volume;
+        break;
+
+      case tdFm:
+        prefs.fmVolume = volume;
+        break;
+
+      default:
+        break;
     }
-    outputVolume = volume;
-  } else {
-    outputVolume = 50;
   }
 
   if (!argc) {
-    LogPrint(LOG_ERR, "missing tune.");
-    exit(2);
+    logMessage(LOG_ERR, "missing tune.");
+    return PROG_EXIT_SYNTAX;
   }
 
   if (argc % 2) {
-    LogPrint(LOG_ERR, "missing note duration.");
-    exit(2);
+    logMessage(LOG_ERR, "missing note duration.");
+    return PROG_EXIT_SYNTAX;
   }
 
   {
     unsigned int count = argc / 2;
-    TuneElement *elements = mallocWrapper((sizeof(*elements) * count) + 1);
-    TuneElement *element = elements;
+    TuneElement *elements = malloc((sizeof(*elements) * count) + 1);
 
-    while (argc) {
-      int note;
-      int duration;
+    if (elements) {
+      TuneElement *element = elements;
 
-      {
-        static const int minimum = 0X01;
-        static const int maximum = 0X7F;
-        const char *argument = *argv++;
-        if (!validateInteger(&note, argument, &minimum, &maximum)) {
-          LogPrint(LOG_ERR, "%s: %s", "invalid note number", argument);
-          exit(2);
+      while (argc) {
+        int note;
+        int duration;
+
+        {
+          static const int minimum = 0X01;
+          static const int maximum = 0X7F;
+          const char *argument = *argv++;
+          if (!validateInteger(&note, argument, &minimum, &maximum)) {
+            logMessage(LOG_ERR, "%s: %s", "invalid note number", argument);
+            return PROG_EXIT_SYNTAX;
+          }
+          --argc;
         }
-        --argc;
-      }
 
-      {
-        static const int minimum = 1;
-        static const int maximum = 255;
-        const char *argument = *argv++;
-        if (!validateInteger(&duration, argument, &minimum, &maximum)) {
-          LogPrint(LOG_ERR, "%s: %s", "invalid note duration", argument);
-          exit(2);
+        {
+          static const int minimum = 1;
+          static const int maximum = 255;
+          const char *argument = *argv++;
+          if (!validateInteger(&duration, argument, &minimum, &maximum)) {
+            logMessage(LOG_ERR, "%s: %s", "invalid note duration", argument);
+            return PROG_EXIT_SYNTAX;
+          }
+          --argc;
+        }
+
+        {
+          TuneElement te = TUNE_NOTE(duration, note);
+          *(element++) = te;
         }
-        --argc;
       }
 
       {
-        TuneElement te = TUNE_NOTE(duration, note);
-        *(element++) = te;
+        TuneElement te = TUNE_STOP();
+        *element = te;
       }
+    } else {
+      logMallocError();
+      return PROG_EXIT_FATAL;
     }
 
-    {
-      TuneElement te = TUNE_STOP();
-      *element = te;
+    if (!setTuneDevice(prefs.tuneDevice)) {
+      logMessage(LOG_ERR, "unsupported tune device: %s", deviceNames[prefs.tuneDevice]);
+      return PROG_EXIT_SEMANTIC;
     }
 
-    if (!setTuneDevice(tuneDevice)) {
-      LogPrint(LOG_ERR, "unsupported tune device: %s", deviceNames[tuneDevice]);
-      exit(3);
-    }
-
-    memset(&prefs, 0, sizeof(prefs));
-    prefs.alertMessages = 0;
-    prefs.alertDots = 0;
-    prefs.alertTunes = 1;
-    switch (tuneDevice) {
-      default:
-        break;
-      case tdPcm:
-        prefs.pcmVolume = outputVolume;
-        break;
-      case tdMidi:
-        prefs.midiVolume = outputVolume;
-        break;
-      case tdFm:
-        prefs.fmVolume = outputVolume;
-        break;
-    }
-#ifdef ENABLE_MIDI_SUPPORT
-    prefs.midiInstrument = midiInstrument;
-#endif /* ENABLE_MIDI_SUPPORT */
     {
       TuneDefinition tune = {NULL, 0, elements};
       playTune(&tune);
@@ -271,11 +270,11 @@ main (int argc, char *argv[]) {
 
     free(elements);
   }
-  return 0;
+  return PROG_EXIT_SUCCESS;
 }
 
 int
-message (const char *mode, const char *string, short flags) {
+message (const char *mode, const char *text, short flags) {
   return 1;
 }
 int
