@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -42,7 +42,7 @@ typedef struct {
       Queue *pending;
       void *completed;
       unsigned char *buffer;
-      int length;
+      size_t length;
     } input;
 
     struct {
@@ -56,7 +56,8 @@ typedef struct UsbDeviceExtensionStruct UsbDeviceExtension;
 struct UsbDeviceStruct {
   UsbDeviceDescriptor descriptor;
   UsbDeviceExtension *extension;
-  const UsbSerialOperations *serial;
+  const UsbSerialOperations *serialOperations;
+  const void *serialData;
   UsbConfigurationDescriptor *configuration;
   const UsbInterfaceDescriptor *interface;
   Queue *endpoints;
@@ -72,37 +73,44 @@ extern UsbDevice *usbTestDevice (
 extern UsbEndpoint *usbGetEndpoint (UsbDevice *device, unsigned char endpointAddress);
 extern UsbEndpoint *usbGetInputEndpoint (UsbDevice *device, unsigned char endpointNumber);
 extern UsbEndpoint *usbGetOutputEndpoint (UsbDevice *device, unsigned char endpointNumber);
-extern int usbApplyInputFilters (UsbDevice *device, void *buffer, int size, int *length);
+extern int usbApplyInputFilters (UsbDevice *device, void *buffer, size_t size, ssize_t *length);
+
+extern int usbSetSerialOperations (UsbDevice *device);
 
 extern int usbSetConfiguration (
   UsbDevice *device,
   unsigned char configuration
 );
+
 extern int usbClaimInterface (
   UsbDevice *device,
   unsigned char interface
 );
+
 extern int usbReleaseInterface (
   UsbDevice *device,
   unsigned char interface
 );
+
 extern int usbSetAlternative (
   UsbDevice *device,
   unsigned char interface,
   unsigned char alternative
 );
-extern int usbControlTransfer (
+
+extern ssize_t usbControlTransfer (
   UsbDevice *device,
-  unsigned char direction,
-  unsigned char recipient,
-  unsigned char type,
-  unsigned char request,
-  unsigned short value,
-  unsigned short index,
+  uint8_t direction,
+  uint8_t recipient,
+  uint8_t type,
+  uint8_t request,
+  uint16_t value,
+  uint16_t index,
   void *buffer,
-  int length,
+  uint16_t length,
   int timeout
 );
+
 extern int usbReadDeviceDescriptor (UsbDevice *device);
 extern int usbAllocateEndpointExtension (UsbEndpoint *endpoint);
 extern void usbDeallocateEndpointExtension (UsbEndpointExtension *eptx);
