@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -38,14 +38,14 @@ usbResetDevice (UsbDevice *device) {
   if (result >= 0) return 1;
 
   errno = -result;
-  LogError("USB device reset");
+  logSystemError("USB device reset");
   return 0;
 }
 
 int
 usbDisableAutosuspend (UsbDevice *device) {
   errno = ENOSYS;
-  LogError("USB device autosuspend disable");
+  logSystemError("USB device autosuspend disable");
   return 0;
 }
 
@@ -59,7 +59,7 @@ usbSetConfiguration (
   if (result >= 0) return 1;
 
   errno = -result;
-  LogError("USB configuration set");
+  logSystemError("USB configuration set");
   return 0;
 }
 
@@ -73,7 +73,7 @@ usbClaimInterface (
   if (result >= 0) return 1;
 
   errno = -result;
-  LogError("USB interface claim");
+  logSystemError("USB interface claim");
   return 0;
 }
 
@@ -87,7 +87,7 @@ usbReleaseInterface (
   if (result >= 0) return 1;
 
   errno = -result;
-  LogError("USB interface release");
+  logSystemError("USB interface release");
   return 0;
 }
 
@@ -102,7 +102,7 @@ usbSetAlternative (
   if (result >= 0) return 1;
 
   errno = -result;
-  LogError("USB alternative set");
+  logSystemError("USB alternative set");
   return 0;
 }
 
@@ -116,21 +116,21 @@ usbClearEndpoint (
   if (result >= 0) return 1;
 
   errno = -result;
-  LogError("USB endpoint clear");
+  logSystemError("USB endpoint clear");
   return 0;
 }
 
-int
+ssize_t
 usbControlTransfer (
   UsbDevice *device,
-  unsigned char direction,
-  unsigned char recipient,
-  unsigned char type,
-  unsigned char request,
-  unsigned short value,
-  unsigned short index,
+  uint8_t direction,
+  uint8_t recipient,
+  uint8_t type,
+  uint8_t request,
+  uint16_t value,
+  uint16_t index,
   void *buffer,
-  int length,
+  uint16_t length,
   int timeout
 ) {
   UsbDeviceExtension *devx = device->extension;
@@ -139,7 +139,7 @@ usbControlTransfer (
   if (result >= 0) return result;
 
   errno = -result;
-  LogError("USB control transfer");
+  logSystemError("USB control transfer");
   return -1;
 }
 
@@ -148,7 +148,7 @@ usbSubmitRequest (
   UsbDevice *device,
   unsigned char endpointAddress,
   void *buffer,
-  int length,
+  size_t length,
   void *data
 ) {
   errno = ENOSYS;
@@ -161,7 +161,7 @@ usbCancelRequest (
   void *request
 ) {
   errno = ENOSYS;
-  LogError("USB request cancel");
+  logSystemError("USB request cancel");
   return 0;
 }
 
@@ -173,16 +173,16 @@ usbReapResponse (
   int wait
 ) {
   errno = ENOSYS;
-  LogError("USB request reap");
+  logSystemError("USB request reap");
   return NULL;
 }
 
-int
+ssize_t
 usbReadEndpoint (
   UsbDevice *device,
   unsigned char endpointNumber,
   void *buffer,
-  int length,
+  size_t length,
   int timeout
 ) {
   UsbDeviceExtension *devx = device->extension;
@@ -191,7 +191,7 @@ usbReadEndpoint (
   if ((endpoint = usbGetInputEndpoint(device, endpointNumber))) {
     const UsbEndpointDescriptor *descriptor = endpoint->descriptor;
     UsbEndpointTransfer transfer = USB_ENDPOINT_TRANSFER(descriptor);
-    int result = -1;
+    ssize_t result = -1;
 
     switch (transfer) {
       case UsbEndpointTransfer_Bulk:
@@ -205,7 +205,7 @@ usbReadEndpoint (
         break;
 
       default:
-        LogPrint(LOG_ERR, "USB endpoint input transfer not supported: 0X%02X", transfer);
+        logMessage(LOG_ERR, "USB endpoint input transfer not supported: 0X%02X", transfer);
         result = -ENOSYS;
         break;
     }
@@ -228,16 +228,16 @@ usbReadEndpoint (
   if (errno == ETIMEDOUT) errno = EAGAIN;
 #endif /* ETIMEDOUT */
 
-  if (errno != EAGAIN) LogError("USB endpoint read");
+  if (errno != EAGAIN) logSystemError("USB endpoint read");
   return -1;
 }
 
-int
+ssize_t
 usbWriteEndpoint (
   UsbDevice *device,
   unsigned char endpointNumber,
   const void *buffer,
-  int length,
+  size_t length,
   int timeout
 ) {
   UsbDeviceExtension *devx = device->extension;
@@ -260,7 +260,7 @@ usbWriteEndpoint (
         break;
 
       default:
-        LogPrint(LOG_ERR, "USB endpoint output transfer not supported: 0X%02X", transfer);
+        logMessage(LOG_ERR, "USB endpoint output transfer not supported: 0X%02X", transfer);
         result = -ENOSYS;
         break;
     }
@@ -269,7 +269,7 @@ usbWriteEndpoint (
     errno = -result;
   }
 
-  LogError("USB endpoint write");
+  logSystemError("USB endpoint write");
   return -1;
 }
 
@@ -334,13 +334,14 @@ usbFindDevice (UsbDeviceChooser chooser, void *data) {
 
                   usb_close(devx->handle);
                 } else {
-                  LogPrint(LOG_ERR, "USB open error: vendor=%X product=%X",
-                           dev->descriptor.idVendor, dev->descriptor.idProduct);
+                  logMessage(LOG_ERR, "USB open error: vendor=%X product=%X",
+                             getLittleEndian16(dev->descriptor.idVendor),
+                             getLittleEndian16(dev->descriptor.idProduct));
                 }
 
                 free(devx);
               } else {
-                LogError("USB device extension allocate");
+                logSystemError("USB device extension allocate");
               }
 
               if ((dev = dev->next) == dev0) dev = NULL;
@@ -352,11 +353,11 @@ usbFindDevice (UsbDeviceChooser chooser, void *data) {
       }
     } else {
       errno = -result;
-      LogError("USB devices find");
+      logSystemError("USB devices find");
     }
   } else {
     errno = -result;
-    LogError("USB busses find");
+    logSystemError("USB busses find");
   }
 
   return device;
