@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2010 by The BRLTTY Developers.
+ * Copyright (C) 1995-2012 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -55,13 +55,13 @@ typedef struct {
 
   void *context;
   void *buffer;
-  int length;
+  size_t length;
 } UsbAsynchronousRequest;
 
 static int
 usbOpenStatusFile (const char *path, int *status) {
   if ((*status = open(path, O_RDONLY)) != -1) return 1;
-  LogError("USB status file open");
+  logSystemError("USB status file open");
   return 0;
 }
 
@@ -82,13 +82,13 @@ usbOpenEndpointFiles (
     if (usbOpenStatusFile(path, status)) {
       return 1;
     } else {
-      LogError("USB endpoint status open");
+      logSystemError("USB endpoint status open");
     }
 
     close(*data);
     *data = -1;
   } else {
-    LogError("USB endpoint data open");
+    logSystemError("USB endpoint data open");
   }
 
   return 0;
@@ -97,14 +97,14 @@ usbOpenEndpointFiles (
 int
 usbResetDevice (UsbDevice *device) {
   errno = ENOSYS;
-  LogError("USB device reset");
+  logSystemError("USB device reset");
   return 0;
 }
 
 int
 usbDisableAutosuspend (UsbDevice *device) {
   errno = ENOSYS;
-  LogError("USB device autosuspend disable");
+  logSystemError("USB device autosuspend disable");
   return 0;
 }
 
@@ -155,21 +155,21 @@ usbClearEndpoint (
   unsigned char endpointAddress
 ) {
   errno = ENOSYS;
-  LogError("USB endpoint clear");
+  logSystemError("USB endpoint clear");
   return 0;
 }
 
-int
+ssize_t
 usbControlTransfer (
   UsbDevice *device,
-  unsigned char direction,
-  unsigned char recipient,
-  unsigned char type,
-  unsigned char request,
-  unsigned short value,
-  unsigned short index,
+  uint8_t direction,
+  uint8_t recipient,
+  uint8_t type,
+  uint8_t request,
+  uint16_t value,
+  uint16_t index,
   void *buffer,
-  int length,
+  uint16_t length,
   int timeout
 ) {
   UsbDeviceExtension *devx = device->extension;
@@ -177,22 +177,22 @@ usbControlTransfer (
 
   setup.bRequestType = direction | recipient | type;
   setup.bRequest = request;
-  putLittleEndian(&setup.wValue, value);
-  putLittleEndian(&setup.wIndex, index);
-  putLittleEndian(&setup.wLength, length);
+  putLittleEndian16(&setup.wValue, value);
+  putLittleEndian16(&setup.wIndex, index);
+  putLittleEndian16(&setup.wLength, length);
 
   switch (direction) {
     case UsbControlDirection_Input: {
-      int size = sizeof(setup);
-      int count;
+      size_t size = sizeof(setup);
+      ssize_t count;
 
       if ((count = write(devx->data, &setup, size)) == -1) {
-        LogError("USB control request");
+        logSystemError("USB control request");
       } else if (count != size) {
-        LogPrint(LOG_ERR, "USB truncated control request: %d < %d", count, size);
+        logMessage(LOG_ERR, "USB truncated control request: %d < %d", count, size);
         errno = EIO;
       } else if ((count = read(devx->data, buffer, length)) == -1) {
-        LogError("USB control read");
+        logSystemError("USB control read");
       } else {
         return count;
       }
@@ -201,8 +201,8 @@ usbControlTransfer (
 
     case UsbControlDirection_Output: {
       unsigned char packet[sizeof(setup) + length];
-      int size = 0;
-      int count;
+      size_t size = 0;
+      ssize_t count;
 
       memcpy(&packet[size], &setup, sizeof(setup));
       size += sizeof(setup);
@@ -211,9 +211,9 @@ usbControlTransfer (
       size += length;
 
       if ((count = write(devx->data, packet, size)) == -1) {
-        LogError("USB control write");
+        logSystemError("USB control write");
       } else if (count != size) {
-        LogPrint(LOG_ERR, "USB truncated control write: %d < %d", count, size);
+        logMessage(LOG_ERR, "USB truncated control write: %d < %d", count, size);
         errno = EIO;
       } else {
         return size;
@@ -222,7 +222,7 @@ usbControlTransfer (
     }
 
     default:
-      LogPrint(LOG_ERR, "USB unsupported control direction: %02X", direction);
+      logMessage(LOG_ERR, "USB unsupported control direction: %02X", direction);
       errno = ENOSYS;
       break;
   }
@@ -235,7 +235,7 @@ usbSubmitRequest (
   UsbDevice *device,
   unsigned char endpointAddress,
   void *buffer,
-  int length,
+  size_t length,
   void *context
 ) {
   UsbEndpoint *endpoint;
@@ -255,24 +255,24 @@ usbSubmitRequest (
       switch (direction) {
         case UsbEndpointDirection_Input:
           if (aioread(eptx->data, request->buffer, request->length, 0, SEEK_CUR, &request->result) != -1) return request;
-          LogError("USB asynchronous read");
+          logSystemError("USB asynchronous read");
           break;
 
         case UsbEndpointDirection_Output:
           if (request->buffer) memcpy(request->buffer, buffer, length);
           if (aiowrite(eptx->data, request->buffer, request->length, 0, SEEK_CUR, &request->result) != -1) return request;
-          LogError("USB asynchronous write");
+          logSystemError("USB asynchronous write");
           break;
 
         default:
-          LogPrint(LOG_ERR, "USB unsupported asynchronous direction: %02X", direction);
+          logMessage(LOG_ERR, "USB unsupported asynchronous direction: %02X", direction);
           errno = ENOSYS;
           break;
       }
 
       free(request);
     } else {
-      LogError("USB asynchronous request allocate");
+      logSystemError("USB asynchronous request allocate");
     }
   }
 
@@ -291,7 +291,7 @@ usbCancelRequest (
   if (!deleteItem(eptx->requests, req)) {
     if (aiocancel(&req->result) == -1) {
       if ((errno != EINVAL) && (errno != EACCES)) {
-        LogError("USB asynchronous cancel");
+        logSystemError("USB asynchronous cancel");
         return 0;
       }
     }
@@ -326,7 +326,7 @@ usbReapResponse (
         if (errno == EINTR) goto doWait;
 
         if (errno != EINVAL) {
-          LogError("USB asynchronous wait");
+          logSystemError("USB asynchronous wait");
           return NULL;
         }
 
@@ -343,7 +343,7 @@ usbReapResponse (
         UsbEndpoint *ep = request->endpoint;
         UsbEndpointExtension *epx = ep->extension;
         if (!enqueueItem(epx->requests, request)) {
-          LogError("USB asynchronous enqueue");
+          logSystemError("USB asynchronous enqueue");
         }
       }
     }
@@ -356,7 +356,7 @@ usbReapResponse (
 
     if (response->count == -1) {
       errno = response->error;
-      LogError("USB asynchronous completion");
+      logSystemError("USB asynchronous completion");
     } else {
       switch (USB_ENDPOINT_DIRECTION(endpoint->descriptor)) {
         case UsbEndpointDirection_Input:
@@ -374,24 +374,24 @@ usbReapResponse (
   return NULL;
 }
 
-int
+ssize_t
 usbReadEndpoint (
   UsbDevice *device,
   unsigned char endpointNumber,
   void *buffer,
-  int length,
+  size_t length,
   int timeout
 ) {
   UsbEndpoint *endpoint;
 
   if ((endpoint = usbGetInputEndpoint(device, endpointNumber))) {
     UsbEndpointExtension *eptx = endpoint->extension;
-    int count;
+    ssize_t count;
 
   doRead:
     if ((count = read(eptx->data, buffer, length)) == -1) {
       if (errno == EINTR) goto doRead;
-      LogError("USB endpoint read");
+      logSystemError("USB endpoint read");
     } else if (!usbApplyInputFilters(device, buffer, length, &count)) {
       errno = EIO;
     } else if (!count) {
@@ -404,26 +404,26 @@ usbReadEndpoint (
   return -1;
 }
 
-int
+ssize_t
 usbWriteEndpoint (
   UsbDevice *device,
   unsigned char endpointNumber,
   const void *buffer,
-  int length,
+  size_t length,
   int timeout
 ) {
   UsbEndpoint *endpoint;
 
   if ((endpoint = usbGetOutputEndpoint(device, endpointNumber))) {
     UsbEndpointExtension *eptx = endpoint->extension;
-    int count;
+    ssize_t count;
 
   doWrite:
     if ((count = write(eptx->data, buffer, length)) == -1) {
       if (errno == EINTR) goto doWrite;
-      LogError("USB endpoint write");
+      logSystemError("USB endpoint write");
     } else if (count != length) {
-      LogPrint(LOG_ERR, "USB truncated endpoint write: %d < %d", count, length);
+      logMessage(LOG_ERR, "USB truncated endpoint write: %d < %d", count, length);
       errno = EIO;
     } else {
       return count;
@@ -438,15 +438,11 @@ usbReadDeviceDescriptor (UsbDevice *device) {
   int count = usbGetDeviceDescriptor(device, &device->descriptor);
 
   if (count == UsbDescriptorSize_Device) {
-    device->descriptor.bcdUSB = getLittleEndian(device->descriptor.bcdUSB);
-    device->descriptor.idVendor = getLittleEndian(device->descriptor.idVendor);
-    device->descriptor.idProduct = getLittleEndian(device->descriptor.idProduct);
-    device->descriptor.bcdDevice = getLittleEndian(device->descriptor.bcdDevice);
     return 1;
   }
 
   if (count != -1) {
-    LogPrint(LOG_ERR, "USB short device descriptor (%d).", count);
+    logMessage(LOG_ERR, "USB short device descriptor (%d).", count);
     errno = EIO;
   }
 
@@ -502,7 +498,7 @@ usbAllocateEndpointExtension (UsbEndpoint *endpoint) {
               break;
   
             default:
-              LogPrint(LOG_ERR, "USB unsupported endpoint direction: %02X", direction);
+              logMessage(LOG_ERR, "USB unsupported endpoint direction: %02X", direction);
               goto nameError;
           }
   
