@@ -1,7 +1,7 @@
 /*
  * XBrlAPI - A background process tinkering with X for proper BrlAPI behavior
  *
- * Copyright (C) 2003-2010 by Samuel Thibault <Samuel.Thibault@ens-lyon.org>
+ * Copyright (C) 2003-2012 by Samuel Thibault <Samuel.Thibault@ens-lyon.org>
  *
  * XBrlAPI comes with ABSOLUTELY NO WARRANTY.
  *
@@ -25,7 +25,10 @@
 #include <stdarg.h>
 #include <signal.h>
 #include <string.h>
+
+#ifdef HAVE_LANGINFO_H
 #include <langinfo.h>
+#endif /* HAVE_LANGINFO_H */
 
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
@@ -48,7 +51,7 @@
 #include <X11/extensions/XTest.h>
 #define CAN_SIMULATE_KEY_PRESSES
 #else /* HAVE_X11_EXTENSIONS_XTEST_H && HAVE_X11_EXTENSIONS_XKB_H */
-#warning key press simulation not supported on this installation
+#warning key press simulation not supported by this build - check that libxtst has been installed
 #endif /* HAVE_X11_EXTENSIONS_XTEST_H && HAVE_X11_EXTENSIONS_XKB_H */
 
 #define BRLAPI_NO_DEPRECATED
@@ -70,6 +73,7 @@
 static char *auth;
 static char *host;
 static char *xDisplay;
+static int quiet;
 
 BEGIN_OPTION_TABLE(programOptions)
   { .letter = 'b',
@@ -92,6 +96,12 @@ BEGIN_OPTION_TABLE(programOptions)
     .setting.string = &xDisplay,
     .description = strtext("X display to connect to")
   },
+
+  { .letter = 'q',
+    .word = "quiet",
+    .setting.flag = &quiet,
+    .description = strtext("Do not write any text to the braille device")
+  },
 END_OPTION_TABLE
 
 /******************************************************************************
@@ -106,7 +116,7 @@ void fatal_brlapi_errno(const char *msg, const char *fmt, ...) {
     vfprintf(stderr,fmt,va);
     va_end(va);
   }
-  exit(1);
+  exit(PROG_EXIT_FATAL);
 }
 
 void fatal_errno(const char *msg, const char *fmt, ...) {
@@ -117,7 +127,7 @@ void fatal_errno(const char *msg, const char *fmt, ...) {
     vfprintf(stderr,fmt,va);
     va_end(va);
   }
-  exit(1);
+  exit(PROG_EXIT_FATAL);
 }
 
 void fatal(const char *fmt, ...) {
@@ -127,7 +137,7 @@ void fatal(const char *fmt, ...) {
     vfprintf(stderr,fmt,va);
     va_end(va);
   }
-  exit(1);
+  exit(PROG_EXIT_FATAL);
 }
 
 /******************************************************************************
@@ -143,7 +153,7 @@ static int brlapi_fd;
 
 void api_cleanExit(int foo) {
   close(brlapi_fd);
-  exit(0);
+  exit(PROG_EXIT_SUCCESS);
 }
 
 void tobrltty_init(char *auth, char *host) {
@@ -288,7 +298,7 @@ static int ErrorHandler(Display *dpy, XErrorEvent *ev) {
     fatal("XGetErrorText");
   fprintf(stderr,gettext("X Error %d, %s on display %s\n"), ev->type, buffer, XDisplayName(Xdisplay));
   fprintf(stderr,gettext("resource %#010lx, req %u:%u\n"),ev->resourceid,ev->request_code,ev->minor_code);
-  exit(1);
+  exit(PROG_EXIT_FATAL);
 }
 
 static int getXVTnb(void) {
@@ -451,10 +461,12 @@ static void setFocus(Window win) {
   curWindow=win;
   api_setFocus((uint32_t)win);
 
-  if (!(window=window_of_Window(win))) {
-    fprintf(stderr,gettext("didn't grab window %#010lx but got focus\n"),win);
-    api_setName("unknown");
-  } else setName(window);
+  if (!quiet) {
+    if (!(window=window_of_Window(win))) {
+      fprintf(stderr,gettext("didn't grab window %#010lx but got focus\n"),win);
+      api_setName("unknown");
+    } else setName(window);
+  }
 }
 
 #ifdef CAN_SIMULATE_KEY_PRESSES
@@ -637,6 +649,7 @@ void toX_f(const char *display) {
       case ConfigureRequest:
       case CirculateNotify:
       case CirculateRequest:
+      case ClientMessage:
 	break;
 
       /* "shouldn't happen" events */
@@ -647,7 +660,8 @@ void toX_f(const char *display) {
     if (haveXTest && FD_ISSET(brlapi_fd,&readfds)) {
       while ((res = brlapi_readKey(0, &code)==1)) {
 	if (((code & BRLAPI_KEY_TYPE_MASK) != BRLAPI_KEY_TYPE_SYM)) {
-	  fprintf(stderr,gettext("unexpected block type %" BRLAPI_PRIxKEYCODE "\n"),code);
+	  fprintf(stderr, "%s: %" BRLAPI_PRIxKEYCODE "\n",
+                  gettext("unexpected block type"), code);
 	  continue;
 	}
 
@@ -703,18 +717,19 @@ void toX_f(const char *display) {
  * main 
  */
 
-int main(int argc, char *argv[]) {
+int
+main (int argc, char *argv[]) {
   {
     static const OptionsDescriptor descriptor = {
       OPTION_TABLE(programOptions),
       .applicationName = "xbrlapi"
     };
-    processOptions(&descriptor, &argc, &argv);
+    PROCESS_OPTIONS(descriptor, argc, argv);
   }
 
   tobrltty_init(auth,host);
 
   toX_f(xDisplay);
 
-  return 0;
+  return PROG_EXIT_SUCCESS;
 }
